
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                get-values-dialog
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun get-values-dialog (prompt &rest args &key (initial "" supplied))
  "Args: (s &key initial)

	Opens a modal dialog with prompt S, a text field and OK, Cancel
buttons. INITIAL is converted to a string with ~S format directive. On Cancel
returns NIL. ON OK Returns list of result of reading and eval'ing the text
field's content.

;; Examples:

 (get-values-dialog \"domain (e.g. 2 4):\")
"
  (let* ((initial (if supplied
                      (format nil "~s" initial)
                      initial))
         (s (apply #'get-string-dialog prompt :initial initial args)))
    (if s (string-to-list s))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;   Wanted to be able to adjust the text-length, so adapted this....
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth get-string-dialog-proto :isnew 
  (s &rest args &key (initial nil has-init) (text-length 40))
  "
(defmeth get-string-dialog-proto :isnew 
  (s &rest args &key (initial nil has-init) (text-length 40))
"
  (if initial (setq text-length (max (length initial) text-length)))
      
  (let* ((prompt-item (send text-item-proto :new s))
         (edit-item (send edit-text-item-proto :new 
                          (if has-init (format nil "~a" initial) "")
                          :text-length text-length)))
    (apply #'call-next-method 
           (list prompt-item edit-item)
           :ok-action #'(lambda () (send edit-item :text))
           args)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                get-string-dialog
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun get-string-dialog (&rest args)
  "
Args: (s &key initial (text-length 40))

	Opens a modal dialog with prompt string S, a text field and OK,
Cancel buttons. INITIAL is converted to a string with ~A format directive.
Returns string of text field content on OK, NIL on cancel.

	The text-length will be automatically adjusted if it is too small
for an initial string.

;; Examples:
 (get-string-dialog \"hi\" :text-length 10)
 (get-string-dialog \"hi\"
    :initial \"hidklskjfaplskdfalksdjfaksdlfaksdlkaskdf sksld fkl\")
"
  (let ((d (apply #'send get-string-dialog-proto :new args)))
    (send d :modal-dialog))
  )
