#|
			Geoeas data file prototype

	It's so simple! I give the usual data set parameters, in the usual order.

|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                geoeas-data-proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto geoeas-data-proto '(number active-list))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                geoeas-data-set-proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto geoeas-data-set-proto
  '(name title nvar varnames data var-info site-info documentation) '()
  geoeas-data-proto)

(defmeth geoeas-data-set-proto :isnew (&rest e)
  "
;; Creates a new geoeas data set, with the usual information:

;; Examples:
  (setf x
	(send geoeas-data-set-proto :new
	      \"fdata\" ;; name of prefix
	      \"fake geoeas data\" ;; prefix.title
	      5 ;; prefix.nvar
	      '(v1 v2 v3 v4 v5) ;; prefix.varnames
	      (matrix '(10 5) (rand.u 50)) ;; prefix.data
	      \"random noise in the variables\" ;; prefix.var-info
	      \"random noise in the samples\" ;; prefix.site-info
	      \"just an example\" ;; prefix.documentation
	      )
	)
  (send x :show)
  (ask x name)
  (ask x title)
  fdata.site-info
"
  (let* (
	 (k 0)
	 piece
	 (pieces
	  '(name title nvar varnames data var-info site-info documentation))
	 (name (first e))
	 val
	 )
    (dolist
     (i pieces)
     (setf piece (nth k e))
     (setf (slot-value i) piece)

     (print (strcat name "." (symbol-name (nth k pieces))))
     
     (setq val (read-from-string
		(strcat name "." (symbol-name (nth k pieces)))))
     (eval `(def ,val ',piece))
     (setf k (1+ k))
     )
    
    ;; send info to the folks:
    (send geoeas-data-proto :slot-value 'number
	  (if (send geoeas-data-proto :slot-value 'number)
	      (1+ (send geoeas-data-proto :slot-value 'number))
	    1
	    )
	  )
    (send geoeas-data-proto :slot-value 'active-list
	  (append (send geoeas-data-proto :slot-value 'active-list)
		  (list self)))
    )
  )

(defmeth geoeas-data-set-proto :save ()
  "
;; writes out a geoeas data set, with the usual information:

;; Examples:
"
  (let ()
    (list
     'send
     'geoeas-data-set-proto
     ':new
     (send self :name)
     (send self :title)
     (send self :nvar)
     (combine 'list (send self :varnames))
     (send self :data)
     (send self :var-info)
     (send self :site-info)
     (send self :documentation)
     )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                standard methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth geoeas-data-set-proto :name (&optional (name nil set))
  (if set (setf (slot-value 'name) name))
  (slot-value 'name))

(defmeth geoeas-data-set-proto :title (&optional (title nil set))
  (if set (setf (slot-value 'title) title))
  (slot-value 'title))

(defmeth geoeas-data-set-proto :data (&optional (data nil set))
  (if set (setf (slot-value 'data) data))
  (slot-value 'data))

(defmeth geoeas-data-set-proto :varnames (&optional (varnames nil set))
  (if set (setf (slot-value 'varnames) varnames))
  (slot-value 'varnames))

(defmeth geoeas-data-set-proto :nvar (&optional (nvar nil set))
  (if set (setf (slot-value 'nvar) nvar))
  (slot-value 'nvar))

(defmeth geoeas-data-set-proto :var-info (&optional (var-info nil set))
  "
Give information about the variables. For 

;; Example:
 (test-geoeas)
 (send test.geoeas :var-info)
 (send test.geoeas :var-info \"chemicals in a USGS groundwater study.\")
 (send test.geoeas :var-info)
"
  (if set (setf (slot-value 'var-info) var-info))
  (slot-value 'var-info))

(defmeth geoeas-data-set-proto :site-info (&optional (site-info nil set))
  "
Give information about the sites. For 

;; Example:
 (test-geoeas)
 (send test.geoeas :site-info)
 (send test.geoeas :site-info \"chemicals in a USGS groundwater study.\")
 (send test.geoeas :site-info)
"
  (if set (setf (slot-value 'site-info) site-info))
  (slot-value 'site-info))

(defmeth geoeas-data-set-proto :documentation (&optional (documentation nil set))
  (if set (setf (slot-value 'documentation) documentation))
  (slot-value 'documentation))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :print method
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth geoeas-data-set-proto :print (&optional (stream t))
  (format stream "#<~a>" (send self :title)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :describe method
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth geoeas-data-set-proto :describe (&optional (stream t))
  (format stream "This is ~a, with ~d variables.~%~%"
	  (slot-value 'title) (slot-value 'nvar))
  (dotimes (i (send self :nvar))
	   (format stream "Variable ~a:~%" (nth i (slot-value 'varnames)))
	   (setq j (1+ i))
	   (format stream "~t Mean of ~g~%" (mean (column j (slot-value 'data))))
	   (format stream "~t S.D. of ~g~%" (sd (column j (slot-value 'data))))
	   )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :missing method
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth geoeas-data-set-proto :missing (&optional value)
  "
		geoeas-data-set-proto :missing (&optional value)

Eliminates missing values (rows) from a geoeas data set.
"
  (let* (
	 (data (send self :data))
	 (nrows (size data))
	 (ncols (second nrows))
	 (nrows (first nrows))
	 (choose (iseq 0 (1- nrows)))
	 )
    (if value
	(dotimes
	 (i nrows)
	 (dotimes
	  (j ncols)
	  (if (equalp value (aref data i j)) (setf (elt choose i) nil))
	  ))
      (dotimes
       (i nrows)
       (dotimes
	(j ncols)
	(if (not (numberp (aref data i j))) (setf (elt choose i) nil))
	)
       )
      )
    (setf choose (remove-if #'(lambda (x) (eq x nil)) choose))
    (send self
	  :data
	  (matrix (list (length choose) ncols)
		  (combine
		   (select (row-list data) choose)
		   )
		  )
	  )
    )
  nil
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :selecting method
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth geoeas-data-set-proto :selecting (column value)
  "
selecting values (rows) from a data set with a certain variable value.
The arguments are column and value (where column is indexed from 1).
"
  (let* (
	 (data (send self :data))
	 (nrows (size data))
	 (ncols (second nrows))
	 (nrows (first nrows))
	 (choose (iseq 0 (1- nrows)))
	 (col (1- column))
	 )
    (dotimes (i nrows)
	     (if (/= value (aref data i col))
		 (setf (elt choose i) nil)
	       )
	     )
    (setf choose (remove-if #'(lambda (x) (eq x nil)) choose))
    (select-rows data choose)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :plot method
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth geoeas-data-set-proto :plot ()
  (title (send self :title))
  (plot (send self :data))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                               create-geoeas-data
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun create-geoeas-data (&optional (file (open-file-dialog t)))
  "
;; Description: function create-geoeas-data
			 (&optional (file (open-file-dialog t)))

	Used to read in a geoeas data file. A geoeas data file has the
following format: on the first line of the file is the title; on the
second line, the number of variables n; the next n lines are the variable
names; and following that is a stream of data values (usually arranged
in columns, but this is not necessary).

	Here's a simple example (the following 8 non-empty lines):

Sample data file
3
up
down
over
1 2 3
2 3 4
3 4 5

	If those 8 lines form the body of the file sample.dat, then I can
execute the command

 (def sample.dat (create-geoeas-data \"sample.dat\"))

to create the geoeas-data object sample.dat. It simultaneously defines the
variables

sample.dat.NAME			- \"sample.dat\"
sample.dat.TITLE		- \"Sample data file\"
sample.dat.NVAR			- 3
sample.dat.VARNAMES		= '(\"up\" \"down\" \"over\")
sample.dat.DATA			= #2a((1 2 3)(2 3 4)(3 4 5))
sample.dat.VAR-INFO		= \"from file sample.dat\"
sample.dat.SITE-INFO		= \"from file sample.dat\"
sample.dat.DOCUMENTATION	= \"from file sample.dat\"

	I find the geoeas data file format to be rather useful, and this is a
good way to read in ascii data.

;; Notes: reworked after read-data-file, defined in file stats.lsp of the
;; standard distribution. Much faster now!

;; See Also: read-data-columns, read-data-file

"
  (if file
      (let (
	    (oldtable *readtable*)
            (oldbreak *breakenable*)
            (eof (gensym))
	    )
        (setq *readtable* *xlisptable*)
        (setq *breakenable* nil)
        (with-open-file
	 (f file)
	 (if f
	     (unwind-protect
		 (let* (
			(title (read-line f))
			(num-vars (read-from-string (read-line f)))
			varnames
			(varnames
			 (mapcar (lambda(x) (read-line f)) (iseq num-vars)))
			(data
			 (transpose
			  (split-list 
			   (do*
			    (
			     (r (read f nil eof) (read f nil eof))
			     (x (list nil))
			     (tail x (cdr tail)))
			    ((eq r eof) (cdr x))
			    (setf (cdr tail) (list r)))
			   num-vars
			   )
			  )
			 )
			)
		   (send geoeas-data-set-proto :new
			 file ;; name of prefix
			 title ;; title
			 num-vars ;; number of variables
			 varnames ;; variable names
			 (apply #'bind-columns data);; the data
			 (strcat "from file " file) ;; prefix.var-info
			 (strcat "from file " file) ;; prefix.site-info
			 (strcat "from file " file) ;; prefix.site-info
			 (strcat "from file " file) ;; prefix.documentation
			 )
		   )
               (setq *breakenable* oldbreak)
               (setq *readtable* oldtable)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                               write-geoeas-data
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun write-geoeas-data (data wfile)
  "
 (write-geoeas-data data \"geo.dat\")
"
  (let* (
	 (file (open wfile :direction :output))
	 )
    (format file "~a~%~d~%" (send data :title) (send data :nvar))
    (mapcar #'(lambda(x) (format file "~a~%" x)) (send data :varnames))
    (dolist
     (i (row-list (send data :data)))
     (mapcar #'(lambda(x) (format file "~g " x)) (coerce i 'list))
     (format file "~%")
     )     
    (close file)
    )     
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                               matrix-to-geoeas-data
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun matrix-to-geoeas-data (data name &optional wfile)
  "
	Creates a geoeas data set of a matrix. If given an optional
filename, it will create a file for use with geoeas software.

;; Examples:
 (matrix-to-geoeas-data (id 4) 'geo.dat \"geo.dat\")
"
  (setf name
	(send geoeas-data-set-proto :new
	      (symbol-name name) ;; name of prefix
	      (strcat "geoeas data from matrix-to-geoeas-data "
		      (symbol-name name)
		      )
	      (second (size data)) ;; prefix.nvar
	      (mapcar #'(lambda(x) (strcat "v" (num-to-string x)))
		      (iseq (second (size data)))) ;; prefix.varnames
	      data ;; prefix.data
	      "" ;; prefix.var-info
	      "" ;; prefix.site-info
	      "" ;; prefix.documentation
	      )
	)
  (if wfile (write-geoeas-data name wfile))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                test-geoeas
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun test-geoeas ()
  (setf test.geoeas
	(send geoeas-data-set-proto :new
	      "fdata" ;; name of prefix
	      "fake geoeas data" ;; prefix.title
	      5 ;; prefix.nvar
	      '("v1" "v2" "v3" "v4" "v5") ;; prefix.varnames
	      (matrix '(10 5) (rand.u 50)) ;; prefix.data
	      "random noise in the variables" ;; prefix.var-info
	      "random noise in the samples" ;; prefix.site-info
	      "just an example" ;; prefix.documentation
	      )
	)
  (send test.geoeas :show)
  (print (ask test.geoeas name))
  (print fdata.name)
  (print (ask test.geoeas title))
  (print fdata.title)
  (send test.geoeas :missing)
  )


