;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 postscript
;;
(defun postscript(filename
		  &key plot (bulsize 1) width height no-axes no-box axes-box
		  no-squish
		  (fmt "~,3,3g")
		  )
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; postscript Doc:    Creation Date:1/27/97        Author: Andy Long
;;
;; Description/Arguments:

       (postscript filename
		  &key plot (bulsize 1) width height no-axes no-box axes-box
  		       no-squish
		       (fmt \"~,3,3g\")
		  )
  
	Requires:
		filename (a string).

	Options:
		plot is a plot-name, if you don't want to use the most recent;
		bulsize is the size of points;
		width is the width of the postscript window;
		height is the height of the postscript window;
		no-axes is true if you want to can the axes;
		no-box is true if you want to can the box;
		axes-box is true if you want the axes part of the box;
		fmt is a format for the axis labels (defaults to \"~,3,3g\").

;; Notes:

;; See Also:

;; Examples:
 (new-plot)
 (plot sin)
 (postscript \"postscript.example0.ps\")
 (plot (sample (iseq 100) 10) (sample (iseq 100) 10))
 (postscript \"postscript.example1.ps\")
 (plot (rand.u 10) (rand.u 10))
 (postscript \"postscript.example2.ps\")
 
 (def mat (outer-product (rseq 0 2 20) (rseq 0 2 20) {x*y :v '(x y)}))
 (contour-matrix mat)
 (send my-plot :range 0 5 15)
 (send my-plot :range 1 5 15)
 (postscript \"postscript.example3.ps\")
"
  (let* (
	 (plot (if plot plot (first (agw))))
	 (f (open filename :direction :output))

	 (variable-labels (send plot :variable-label '(0 1)))
	 (width (if width width 400))
	 (100+width (+ 100 width))
	 (height (if height height 400))
	 (100+height (+ 100 height))
	 (domain (send plot :range 0))
	 (dmin (first domain))
	 (dmax (second domain))
	 (ddiff (- dmax dmin))
	 (range (send plot :range 1))
	 (rmin (first range))
	 (rmax (second range))
	 (rdiff (- rmax rmin))
	 (tinyx (/ ddiff 100))
	 (tinyy (/ rdiff 100))
	 (labels-showing (send plot :showing-labels))
	 (no-squish (if no-squish no-squish nil))

	 lineson linewidth linetype point-labels plot-symbol
	 stuff started do-it-to-it text turn-me-off
	 )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                header:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (format f "%!PS 
%%BoundingBox: 0 0 ~d ~d
/m {moveto} def
/l {lineto} def
/r {rlineto} def
/s {stroke} def
/n {newpath} def
/c {closepath} def
2 setlinejoin
/rtext{dup stringwidth pop -1 div 0 rmoveto show} def
/ltext{show} def
/ctext{dup stringwidth pop -2 div 0 rmoveto show} def
/1b{1 0 360 arc c fill} def
/2b{2 0 360 arc c fill} def
/3b{3 0 360 arc c fill} def
/4b{4 0 360 arc c fill} def
/5b{5 0 360 arc c fill} def
/6b{6 0 360 arc c fill} def
1.000000 1.000000 scale
/Courier-Bold findfont 14 scalefont setfont
"
	    100+width 100+height
	    )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                Boxes:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (if no-box
	(if axes-box
	    (format f "
s n
100 100 m
100 ~d l
100 100 m
~d 100 l
s n
" 
		    100+width 100+height
		    )
	  )
      (format f "
s n
100 100 m
100 ~d l
100 ~d m
~d ~d l
~d ~d m
~d 100 l
~d 100 m
100 100 l
s n
" 
	      100+width 100+width
	      100+width 100+height 100+width 100+height
	      100+height 100+height
	      )
      )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;             Create do-it-to-it, to actually write the stuff:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (setq
     do-it-to-it
     (lambda(i)
       (let* (
	      (bullet-size bulsize)
	      (lineson *lines*)
	      (linewidth (if lineson *line-width*))
	      (linetype (if lineson *line-type*))
	      point-labels text
	     )
	 (cond
	  ((or (functionp i) (listp i))
	   (cond
	    ((functionp i)
	     (setq x (rseq dmin dmax *points*)
		   y (mapcar i x)
		   ))
	    ((listp (first i))
	     (setq x (first i)
		   y (second i)
		   )
	     (if (and *point-labels* (not lineson) labels-showing
		      (= (length *point-labels*) (length x))
		      )
		 (setq point-labels *point-labels*
		       angle nil
		       ))
	     )

	    ((equal (first i) 'lines)
	     (setq x (second i)
		   y (third i)
		   lineson t
		   linewidth (if (fifth i) (fifth i) linewidth)
		   linetype (if (sixth i) (sixth i) linetype)
		   ))

	    ((equal (first i) 'points)
	     (setq x (second i)
		   y (third i)
		   lineson nil
		   plot-symbol (if (fifth i) (fifth i))
		   bullet-size (if (seventh i) (seventh i) bulsize)
		   ))

	    ((equal (first i) 'function)
	     ;; These y values can be nil, so I'm going to remove any that are:
	     (setq x (rseq dmin dmax *points*))
	     (setq y (mapcar (second i) x))
	     (setq x (remove-if (lambda(x) (null (second x))) (weave x y)))
	     (setq y (mapcar #'second x))
	     (setq x (mapcar #'first x))
	     (setq ;; third is color, can't use it yet
		   lineson (if (fourth i) (fourth i) lineson)
		   linewidth (if (fifth i) (fifth i) linewidth)
		   linetype (if (sixth i) (sixth i) linetype)
		   ))

	    ((stringp (first i))
	     (setq x (list (second i))
		   y (list (third i))
		   text (first i)
		   lineson t
		   angle (if (fourth i) (fourth i) nil)
		   ))

	    (t
	     (print "Search me! I don't know how to deal with this:")
	     (print i)
	     )
	    )
	   
	   (setq started nil)

	   (mapcar
	    (lambda(j1 j2)

	      (if no-squish ()
		(setq
		 j1 (if (< j1 100) 100 (if (> j1 100+width) 100+width j1))
		 j2 (if (< j2 100) 100 (if (> j2 100+height) 100+height j2))
		 ))

	      (if (and lineson (not started) linewidth)
		  (if (or (equal linetype 'solid) (equal linetype 'normal))
		      (let ()
			(setq turn-me-off t)
			(format f "~%~d setlinewidth~%0 setgray" linewidth)
			)
		    (let ()
		      (setq turn-me-off t)
		      (format f "~%~d setlinewidth~%.5 setgray" linewidth)
		      )
		    )
		)
	      (if started 
		  (if lineson
		      (format f "~%~d ~d l~%s n ~d ~d m" j1 j2 j1 j2)
		    (let ()
		      (format f "~%s n ~d ~d ~db" j1 j2 bullet-size)
		      (if point-labels
			  (let((text (first point-labels)))
			    (format f "~%s n ~d ~d m" j1 j2)
			    (setq point-labels (circulate point-labels))
			    (if angle
				(format f
					"~%~g rotate~%(~a) ltext~%-~g rotate~%"
					angle text angle)
			      (format f "~%(~a) ltext" text))))))
		(let ()
		  (if lineson
		      (format f "~%s n ~d ~d m" j1 j2)
		    (format f "~%s n ~d ~d ~db" j1 j2 bullet-size))
		  (if (or text point-labels)
		      (let()
			(if point-labels
			    (let()
			      (setq text (first point-labels))
			      (setq point-labels (circulate point-labels))
			      ))
			(format f "~%s n ~d ~d m" j1 j2)
			(if angle
			    (format
			     f "~%~g rotate~%(~a) ltext~%-~g rotate~%"
			     angle text angle)
			  (format f "~%(~a) ltext" text)
			  ))))
		)
	      (setq started t)
	      )
	    (mapcar
	     #'(lambda(x) (+ 100 (floor (* width (/ (- x dmin) ddiff))))) x)
	    (mapcar
	     #'(lambda(x) (+ 100 (floor (* height (/ (- x rmin) rdiff))))) y)
	    )
	   )
	  (t
	   (print "I can't handle this yet:")
	   (print i)
	   )
	  )
	 (if turn-me-off
	     (let ()
	       (setq turn-me-off nil)
	       (format f "~%~d setlinewidth~%0 setgray" *line-width*)
	       )
	   )
	 )
       )
     )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                Usual plot-item stuff, now, with no-squish false:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (mapcar do-it-to-it (send plot :plot-item))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;             Plus the axes, labels, etc., with no-squishing:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (setq no-squish t)
    (if (send plot :has-slot 'jason-horiz-vert)
	(mapcar do-it-to-it 
		(let* (
		       (v-h (send plot :jason-horiz-vert))
		       (angles
			(mapcar
			 (lambda (x) (if (= x 0) 0 90))
			 v-h))
		       (labels (send plot :jason-plot-labels))
		       (coords
			(mapcar
			 (lambda(x)
			   (send plot :canvas-to-real (first x) (second x)))
			 (send plot :jason-label-coordinates)))
		       )
		  (mapcar
		   (lambda(text coords angle)
		     (list text (first coords) (second coords) angle))
		   labels coords angles
		   )
		  )))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                axes:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (if no-axes () ;; do nothing! Otherwise,

      (let ()
	;; Plot the y-axis if visible:
	(if (and (< 0 dmax) (> 0 dmin))
	    (funcall do-it-to-it (list 'lines '(0 0) (list rmin rmax))))

	;; Plot the x-axis if visible:
	(if (and (< 0 rmax) (> 0 rmin))
	    (funcall do-it-to-it (list 'lines (list dmin dmax) '(0 0))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                             box axis tic marks:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	(mapcar do-it-to-it 
			    (mapcar
			     #'(lambda(x)
				 (list 'lines (list dmin (- dmin tinyx))
				       (list x x))
				 )
			     (rseq rmin rmax *y-tics*)
			     ))
	(mapcar do-it-to-it 
		      (mapcar
		       #'(lambda(x)
			   (list 'lines (list x x) (list rmin (- rmin tinyy)))
			   )
		       (rseq dmin dmax *x-tics*)
		       ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                axis tic labels:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	(mapcar do-it-to-it 
			    (mapcar
			     #'(lambda(x)
				 (list
				  (format nil fmt x)
				  (- dmin (* 5 tinyx))
				  (- x (* tinyy (size (format nil fmt x))))
				  90
				  ))
			     (rseq rmin rmax *y-tics*)
			     ))

	(mapcar do-it-to-it 
			    (mapcar
			     #'(lambda(x)
				 (list
				  (format nil fmt x)
				  (- x (* tinyx (size (format nil fmt x))))
				  (- rmin (* 5 tinyy))
				  ))
			     (rseq dmin dmax *x-tics*)
			     ))
	)
      )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                          variable labels:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (if variable-labels
	(mapcar do-it-to-it 
		(list
		 (list (first variable-labels)
		       (+ dmin (half ddiff)
			  (- (* tinyx (size (first variable-labels)))))
		       (- rmin (* 10 tinyy))
		       )
		 (list (second variable-labels)
		       (- dmin (* 10 tinyx))
		       (+ rmin (half rdiff)
			  (- (* tinyy (size (second variable-labels))))
			  )
		       90
		       )
		 )
		)
      )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                wrap it up:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (format f "~%showpage~%")
    (close f)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 postscript-3d
;;
(defun postscript-3d(filename &key plot)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; postscript-3d Doc:    Creation Date:2/14/97        Author: Andy Long
;;
;; Description/Arguments:

       (postscript-3d filename &key plot)
  
;; Notes:

;; See Also: postscript

;; Examples:
 (new-plot)
 (plot (lambda(x y) (* x y)))
 (postscript-3d \"postscript-3d.example0.ps\")
"
  (let* (
	 (plot (if plot plot (first (agw))))
	 (lines (send plot :3d-2d))
	 (points (send plot :3d-2d-points))
	 (myplot (send scatterplot-proto :new 2 :show nil))
	 )
    (send myplot :add-slot 'andy-options)
    (send myplot :add-slot 'plot-item)
    (mapcar
     (lambda(x)
       (send myplot
	     :add-plot-item
	     (list 'lines (coerce (first x) 'list) (coerce (second x) 'list)
		   'black)))
     lines)
    (mapcar
     (lambda(x)
       (send myplot
	     :add-plot-item
	     (list (third x) (first x) (second x))
	     ))
     points)
    (send myplot :range 0 (min (xs lines)) (max (xs lines)))
    (send myplot :range 1 (min (ys lines)) (max (ys lines)))
    (postscript filename :plot myplot :no-axes t :no-box t)
    (send myplot :close)
    )
  )
;;
