;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                read-file
;;
;; new and improved!?
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
;; Here's the old one:
(defun read-file (filename &optional n)
  (with-open-file
   (f filename :direction :input)
   (let (fileall)
     (if n
	 (do
	  ((i 0 (1+ i))
	   (filebit (read-line f nil nil) (read-line f nil nil))
	   )
	  ((not (and filebit (< i n))) fileall)
	  (setq fileall (combine fileall filebit))
	  )
       (do
	((filebit (read-line f nil nil) (read-line f nil nil)))
	((not filebit) fileall)
	(setq fileall (combine fileall filebit))
	)
       )
     (if fileall (rest fileall))
     )
   )
)
|#
(defun read-file (filename &optional n)
  "
;; Description:

      (read-file filename &optional n)

	Read-file will fill a list with the lines (or the first n lines) of any
 (non-empty) file.

;; old time:
 ;; in ~/tmp/numrec
 (time (def ctmp (read-file \"numrec.f\")))
 > The evaluation took 350.86 seconds; 277.79 seconds in gc.

;; Examples:
 ;; this creates a file crumcake in the current working directory:
 (with-open-file
   (stream \"crumcake\" :direction :output)
   (dotimes (i 10) (format stream \"~d~%\" i)))
 (read-file \"crumcake\")
"
  (let* (
	 (fileall 
	  (coerce 
	   (read-file-as-vector filename) 
	   'list))
	 ;; need to add these in so that the first and last lines are handled
	 ;; properly:
	 (fileall
	  (if (eq (first fileall) #\Newline) fileall
	    (combine #\Newline fileall)))
	 (fileall
	  (if (eq (final fileall) #\Newline) fileall
	    (combine fileall #\Newline)))

	 (lines (where fileall #\Newline))
	 (lines (transpose (list (butlast lines) (rest lines))))
	 )
    (mapcar
     (lambda(line)
       ;; line is a pair of indices marking newline characters
       ;; between these is found the line. If non-empty is nil, the lines are
       ;; right next to each other - return an empty string; otherwise,
       ;; select the part between them:
       (let ((non-empty (rest (butlast (select fileall (apply #'iseq line))))))
	 (if non-empty (coerce non-empty 'string) "")
	 )
       )
     lines
     )))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                read-file-as-vector
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun read-file-as-vector (filename)
  "
 ;; Description: 
 ;; this creates a file crumcake in the current working directory:

;; Examples:
 (with-open-file
   (stream \"crumcake\" :direction :output)
   (dotimes (i 10) (format stream \"~d~%\" i)))
 (read-file \"crumcake\")

 (def crum (read-file-as-vector \"crumcake\"))
 (def cruml (coerce crum 'list))
 (def crumstring (coerce cruml 'string))
"
  (let* (
	 (f (open filename :direction :input))
	 (charfile (make-array (file-length f)))
	 )
    (dotimes (i (file-length f))
	     (setf (aref charfile i) (read-char f))
	     )
    (close f)
    charfile
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                matrix-from-file-of-lines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun matrix-from-file-of-lines (filename m n &optional types)
  "
;; Description: (matrix-from-file-of-lines filename m n)
;; matrix m by n will be initialized with values read from filename. values
;; should appear one per line.

;; Examples:
 (let ((f (open (strcat *default-path* \"tmpola\") :direction :output)))
   (mapcar (lambda(x) (format f \"~a~%\" x)) '(1 2 3 \"hi you\" 3 \"4 7\"))
   (close f)
   (mprint
     (matrix-from-file-of-lines
       (strcat *default-path* \"tmpola\") 3 2 '(1 2))
     )
   (setq f
   (open (strcat *default-path* \"tmpola\") :direction :output))
   (close f)
 )
"
  (let* (
	 (f (open filename :direction :input))
	 (total-length (* m n))
	 (charfile (make-array total-length))
	)
    (dotimes
     (i total-length)
     (setf (aref charfile i) (read-line f nil nil))
     )
    (close f)
    (setq charfile (matrix (list m n) charfile))
    (if types
	(let (
	      (j 1)
	      )
	  (dolist
	   (i types)
	   (if (eq i 1)
	       (set-column j charfile (vcoerce (column j charfile) 'float))
	     )
	   (setq j (1+ j))
	   )
	  )
      )
    charfile
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                sas2ascii
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun sas2ascii(name variables &key (src ".") (dir ".") (where ""))
  "
;; Description:
sas2ascii(name variables
		&key
		(src \".\")
		(dir \".\")
		(where \"\"))

;; Examples:
;; (sas2ascii \"ballmen\" \"age incpov death\" :src \"/pme4/scratch\")
;; (sas2ascii \"ballmen\" \"age incpov death\"
;;      :where \" -where 'death .ne. .' \")
"
  (let ((nvars (length (string-to-list variables))) data datmat)
    ;; Some system commands that use pme:
    (system
     (strcat
      "pme -format -ascii -noshow -o ctmp -s " dir
      " -a " src " "
      where
      name " " variables)
     )
    (system "grep years ctmp.ascii")
    (system "change_word \"+ years\" \" \" ctmp.ascii")
    (system "grep years ctmp.ascii")
    
    (setq data (read-data-file "ctmp.ascii"))
    (setq datmat (matrix (list (/ (length data) nvars) nvars) data))
    (send geoeas-data-set-proto :new
	  name ;; name of prefix
	  "data" ;; prefix.title
	  nvars ;; prefix.nvar
	  (mapcar #'symbol-name (string-to-list variables))
	  datmat
	  "" ;; prefix.var-info
	  "" ;; prefix.site-info
	  "" ;; prefix.documentation
	  )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         find-nasty-character
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun find-nasty-character(string &key (n 127))
  "
;; Description: (find-nasty-character string &key (n 127))

	Finds nasty characters in strings, like rubout (ASCII character
127). It will find any character in the first 

	Won't handle control-c, control-d as well as other characters. But the
example below shows that it can (theoretically, anyway) handle any ASCII
character: we try them from 0 to 255. 

;; Examples:
 ;; An ASCII character is inserted in the string, and it's position is
 ;; identified by find-nasty-character:
 (let ()
   (mapcar #'(lambda(x y) (format t \"~d~15t~a~30t~a~45t~a~%\"
				y (string (code-char y)) x
				(find-nasty-character x :n y)))
     (mapcar (lambda(x) (strcat \"hello\" (string (code-char x)) \"!\"))
	     (iseq 0 127))
     (iseq 0 127)
     )
   nil
   )
 (find-nasty-character \" (if (= j 0) (go tag-1)\" :n 102) ;; f
 (find-nasty-character \" (if (= j 0) (go tag-1)\" :n 105) ;; i
 (find-nasty-character \" (if (= j 0) (go tag-1)\"))
"
  (let ((m (grep (string (code-char n)) string)))
    (if m (subseq string 0 m))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                  find-and-eliminate-nasty-character
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun find-and-eliminate-nasty-character(string &key (n 127))
  "
;; Description: (find-and-eliminate-nasty-character string &key (n 127))

	Finds and eliminates nasty characters from strings, like rubout.

;; Examples:
 ;; An ASCII character is inserted in the string, and it's position is
 ;; identified and eliminated by find-and-eliminate-nasty-character:
 (let ()
   (mapcar #'(lambda(x y) (format t \"~a:~20t~a~40t~a~%\"
				(string (code-char y)) x
				(find-and-eliminate-nasty-character x :n y)))
     (mapcar (lambda(x) (strcat \"hello\" (string (code-char x)) \"!\"))
	     (iseq 0 127))
     (iseq 0 127)
     )
   nil
   )
 (find-and-eliminate-nasty-character
    \" (if (= j 0) (go tag-1)\")
 (find-and-eliminate-nasty-character
    \" (if (= j 0) (go tag-1)\" :n 102)
"
  (let ((char (string (code-char n))))
    (do
     (
      (m (grep char string) (grep char string))
      (l (length string) (length string))
      )
     ((not m))
     (setq string (strcat (subseq string 0 m) (subseq string (1+ m) l)))
     )
    string
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 open-append
;;
(defun open-append(filename)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          open-append Doc:
;;
;; Author: Andy Long
;; Created: Fri Dec 12 09:37:21 1997
;;
;; Description/Arguments:

       (open-append filename)
  
;; Returns: an open output stream positioned at the end of the file filename,
  ready to be appended to. 

;; Notes: too bad function open doesn't have an append option.

;; See Also: open, read-file-as-vector, read-file

;; Examples:
 (def f (open \"openatry.bye\" :direction :output))
 (format f \"We want to append\")
 (close f)
 (read-file \"openatry.bye\")

 (def f (open-append \"openatry.bye\") )
 (format f \" to that line.\")
 (close f)
 (read-file \"openatry.bye\")

 (delete-file \"openatry.bye\")
"
  (let* (
	 (f (read-file-as-vector filename))
	 (len (length f))
	 (stream (open filename :direction :output))
	 )
    (dotimes
     (i len)
     (format stream "~a" (elt f i))
     )
    stream
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                with-open-file
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro with-open-file (stream-file-args &rest body)
  "
;; Description/Arguments:
  
Syntax: (with-open-file (stream filename {options}*) {form}*)
Opens file stream for FILENAME with specified options, binds to 
STREAM and evaluates FORMs with this binding. Closes stream regardless
of errors.

;; Returns: nil

;; Notes: ael: I've modified this (originally defined in common.lsp) to make
  use of function open-append when the direction is given as append. 12/12/97

;; See Also: open, read-file-as-vector, read-file

;; Examples:
 (with-open-file (f \"openatry.bye\" :direction :output)
		 (format f \"We want to append\")
		 )
 (read-file \"openatry.bye\")

 (with-open-file (f \"openatry.bye\" :direction :append)
		 (format f \" to this line.\")
		 )

 (with-open-file (f \"openatry.bye\" :direction :input)
		 (format t \"~%~A\" (read-line f))
		 )

 (delete-file \"openatry.bye\")
"
  (let* (
	 (stream (first stream-file-args))
	 (file-args (rest stream-file-args))
	 (filename (first file-args))
	 (append (first (catch-keys '(:direction) file-args)))
	 )
    (if (eq append :append)
	`(let ((,stream (open-append ,filename)))
	   (unwind-protect 
	       (progn ,@body)
	     (when ,stream (close ,stream))))
      `(let ((,stream (open ,@file-args)))
	 (unwind-protect 
	     (progn ,@body)
	   (when ,stream (close ,stream)))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
