;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          Readtable changes
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	These changes to the readtable were suggested by Betz in his xlisp-plus
documentation. I first came upon them while examining the WebMaker version of
the documentation, in html format.

							Andy Long, 5/10/96
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;             To switch back and forth between readtables:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun old-readtable()
  "
	Sets the *readtable* to the readtable as Luke meant it to be.

;; See Also: new-readtable (sets it to Andy's)

;; Examples:
 (old-readtable)
"
  (setq *readtable* (copy-vector *old-readtable*))
  t
  )

(defun new-readtable()
  "
	Sets the *readtable* to the readtable as Andy likes it to be.

;; See Also: old-readtable (sets it to Luke's)

;; Examples:
 (new-readtable)
"
  (setq *readtable* (copy-vector *new-readtable*))
  t
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;     [] constitutes the matrix command: For example, [[1 2][1 2]] is the
;; two-by-two matrix #2A((1 2) (1 2)), whereas [1 2] is #(1 2).
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setf (aref *readtable* (char-int #\[)) ; #\[ table entry
      (cons :tmacro
            (lambda (f c &aux ex) ; second arg is not used
	      (do
	       ()
	       ((eq (peek-char t f) #\]))
	       (setf ex (append ex (list (read f))))
	       )
	      (print ex)
	      (read-char f) ; toss the trailing #\]
	      (if (apply #'and-function (mapcar #'listp ex))
		  (let ((sizes (mapcar #'size ex)))
		    (print sizes)
		    (if (= 0 (sum (abs (- sizes (mean sizes)))))
			;; that is, if they're all the same size, then we've
			;; got a bunch of rows of a matrix:
			(cons
			 #| - Tue Feb 17 19:26:44 1998
			  (matrix (list (length ex) (1- (first sizes)))
				  (combine (mapcar #'rest (mapcar #'eval ex))))
|#
			 (apply #'bind-rows (mapcar #'eval ex))
			 nil)
		      ;; Otherwise they'll be treated as a vector of vectors of
		      ;; differing lengths, etc.:
		      (cons (cons 'vector ex) NIL)
		      )
		    )
		;; same deal here: if they weren't all lists, we'll just turn
		;; it into a vector:
		(cons (cons 'vector ex) NIL)
		))))

(setf (aref *readtable* (char-int #\]))
      (cons :tmacro
            (lambda (f c)
                    (error "misplaced right bracket: ]."))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                    {} constitutes the func command:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setf (aref *readtable* (char-int #\{)) ; #\{ table entry
      (cons :tmacro
            (lambda (f c &aux ex) ; second arg is not used
	      (let (bing new-that dash (vars '(x)))
		(do ()
		    ((eq (peek-char t f) #\}))
		    (setf ex (append ex (list (read f)))))
		(read-char f) ; toss the trailing #\}
		(if (first (catch-keys '(:dash) ex))
		    (let ()
		      (setq dash t)
		      (setq ex (remove-keys '(:dash) ex))
		      )
		  )
		(if (first (catch-keys '(:v) ex))
		    (let ()
		      (setq vars (first (rest (first (catch-keys '(:v) ex)))))
		      (setq ex (remove-keys '(:v) ex))
		      )
		  )
		(setq new-that (eval `(list-to-string ',ex)))
		(setq new-that (eval `(no-space ,new-that :dash ,dash)))
		(setq bing (eval `(parser ',new-that)))
		(cons `(lambda ,vars ,bing) NIL)))))

;; this becomes an accessory to the previous function:
(setf (aref *readtable* (char-int #\}))
      (cons :tmacro
            (lambda (f c)
                    (error "misplaced right brace: }."))))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			c-a : the apropos command:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setf (aref *readtable* (char-int #\c-a)) ; #\c-a table entry
      (cons :tmacro
            (lambda (f c) ; second arg is not used
	      (let ((g (read-line f)))
		(setq g (read-from-string g))
		(eval `(apropos ',g))
		(format t "> ")
		)
	      )
	    )
      )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         c-b load command:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setf (aref *readtable* (char-int #\c-b)) ; #\c-b table entry
      (cons :tmacro
            (lambda (f c) ; second arg is not used
	      (let (
		    (g (read-line f))
		    stuff
		    )
		(setq g (string-to-list g))
		(eval `(load-calc ,@g))
		(format t "> ")
		)
	      )
	    )
      )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          c-k compile command:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setf (aref *readtable* (char-int #\c-k)) ; #\c-k table entry
      (cons :tmacro
            (lambda (f c) ; second arg is not used
	      (let ((g (read-line f)))
		(compile-file g)
		(format t "> ")
		)
	      )
	    )
      )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         c-l load command:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setf (aref *readtable* (char-int #\c-l)) ; #\c-l table entry
      (cons :tmacro
            (lambda (f c) ; second arg is not used
	      (let ((g (read-line f)))
		(load g)
		(format t "> ")
		)
	      )
	    )
      )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         c-o option command:
;;
#| - Tue Feb 17 19:49:21 1998
	Thanks once again to the always interesting 
---|  Steven D. Majewski   (804-982-0831)  <sdm7g@Virginia.EDU>  |---
---|  Department of Molecular Physiology and Biological Physics  |---
---|  University of Virginia             Health Sciences Center  |---
---|  P.O. Box 10011            Charlottesville, VA  22906-0011  |---
All power corrupts and obsolete power corrupts obsoletely." - Ted Nelson
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(set-macro-character
 #\c-o ; #\c-o table entry
 (lambda (s c)
   (list
    (case (read-char s nil nil nil )
	  (#\i  (list #'iseq (read s t nil nil )))
	  (#\n  (list #'normal-rand (read s t nil nil )))
	  (#\u  (list #'uniform-rand (read s t nil nil )))
	  (t
	   (format t " Command-O is the option command. It takes a number of
different SECOND characters, with the following meanings:

  next character | followed by  | yields
	i  	 | an integer n | a sequence of integers (iseq n)
	n  	 | an integer n | a list of n random normal numbers
	u  	 | an integer n | a list of n uniform normal numbers

;; Examples:

	i10 ;; same as (iseq 10)
	n10 ;; same as (rand.n 10)
	u10 ;; same as (rand.u 10)
")
	   )
	  )
    )
   )
 )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                   c-n puts in parentheses for you:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setf (aref *readtable* (char-int #\c-n)) ; #\c-n table entry
      (cons :tmacro
            (lambda (f c) ; second arg is not used
	      (let ((g (string-to-list (read-line f))))
		(cons g nil)
		)
	      )
	    )
      )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                      C-f - display help (hulp)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setf (aref *readtable* (char-int #\c-f)) ; #\c-f table entry
      (cons :tmacro
            (lambda (f c) ; second arg is not used
	      (let ((g (read-line f)))
		(setq g (read-from-string g))
		(eval (helpless g))
		(format t "> ")
		)
	      )
	    )
      )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 ? help
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun unset-question-mark ()
  "
	If you don't want the symbol ? to stand for the help command, then
execute the unset-question-mark command:

;; Examples:
 (unset-question-mark)
"
  (setf (aref *readtable* (char-int #\?))
	(aref *readtable* (char-int #\a))))

(defun reset-question-mark ()
  "
	If you want the symbol ? to stand for the help command, then execute
the reset-question-mark command:

;; Examples:
 (reset-question-mark)
"
  (setf (aref *readtable* (char-int #\?)) ; #\? table entry
	(cons :tmacro
	      (lambda (f c) ; second arg is not used
		(let ((g (read-line f)))
		  (setq
		   g 
		   (string-downcase
		    (string-right-trim " " (string-left-trim " " g))))
		  (if (/= (length g) 0) (x11-display-help g) (hulp))
		  (format t "> ")
		  )
		)
	      )
	)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                               $ calc
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun unset-dollar ()
  "
	If you don't want the symbol $ to stand for the calc command, then
execute the unset-dollar command:

;; Examples:
 (unset-dollar)
"
  (setf (aref *readtable* (char-int #\$))
	(aref *readtable* (char-int #\a))))
(defun reset-dollar ()
  "
	If you want the symbol $ to stand for the calc command, then execute
the reset-dollar command:

;; Examples:
 (reset-dollar)
"
  (setf (aref *readtable* (char-int #\$)) ; #\$ table entry
	(cons :tmacro
	      (lambda (f c) ; second arg is not used
		(let ((g (read-line f)))
		  (setq g (string-to-list g))
		  (princ (eval `(calc ,g)))
		  (format t "~%> ")
		  )
		)
	      )
	)
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                ! system call:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun unset-exclamation-mark ()
  "
	If you don't want the symbol ! to stand for a system command, then
execute the unset-exclamation-mark command:

;; Examples:
 (unset-exclamation-mark)
"
  (setf (aref *readtable* (char-int #\!))
	(aref *readtable* (char-int #\a))))
(defun reset-exclamation-mark ()
  "
	If you want the symbol ! to stand for a system command, then execute
the reset-exclamation-mark command:

;; Examples:
 (reset-exclamation-mark)
"
  (setf (aref *readtable* (char-int #\!)) ; #\! table entry
	(cons :tmacro
	      (lambda (f c) ; second arg is not used
		(let ((g (read-line f)))
		  (system g)
		  (format t "> ")
		  )
		)
	      )
	)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                       c-r is the calc command:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 (setf (aref *readtable* (char-int #\c-r)) ; #\c-r table entry
      (cons :tmacro
            (lambda (f c) ; second arg is not used
	      (let ((g (string-to-list (read-line f))))
		(princ (eval `(calc ,g)))
		(terpri)
		(princ "> ")
		)
	      )
	    )
      )
;;
;; set-macro-character is defined in common.lsp, in the Lisp Sources directory:
(set-macro-character #\C-e #'(lambda (&rest f) (print (again))))
(set-macro-character #\C-p #'(lambda (&rest f) (history)))
;;
;; (set-macro-character #\C-t #'(lambda (&rest f) (tutorials)))
;; c-t : the try-out command:
(setf (aref *readtable* (char-int #\c-t)) ; #\c-t table entry
      (cons :tmacro
            (lambda (f c) ; second arg is not used
	      (let ((g (read-line f)))
		(setq g (read-from-string g))
		(eval `(helplass ',g))
		(format t "~%> ")
		)
	      )
	    )
      )

(set-macro-character #\C-u #'(lambda (&rest f) (his)))
(set-macro-character #\C-x #'(lambda (&rest f)
			       (princ "
       Here's the scoop on the control (and other) keys (note: some of these
only work in one environment or another, unfortunately): c-a = control-a

  {} : constitutes the func command - e.g., (plot {sinc x})
  [] : send command - e.g. [scatterplot-proto :help]
   ! : system call - e.g. !ls
   ? : display help (hulp) for command - e.g., ?help will generate help on help
   $ : calc - e.g., $1+2
 c-a : apropos a command
 c-b : load-calc
 c-e : again - executes the previous command
 c-f : help - e.g., ?help will generate help on the help command
 c-k : compiles a file
 c-n : puts in the parentheses for you
 c-o : option command (c-o? gets help on this baby)
 c-l : loads a file
 c-p : calls history
 c-r : the calc command
 c-t : try-out
 c-u : adds the previous command to the history
 c-x : tells you what the control keys do!
")
			       (format t "> ")
			       )
		     )

#|
(setf (aref *readtable* (char-int #\{)) ; #\{ table entry
      (cons :tmacro
            (lambda (f c &aux ex) ; second arg is not used
	      (let (
		    bing
		    (g (string-right-trim
			"}"
			(string-left-trim "{" (read-line f))))
		    )
		(print g)
		(setq g (string-to-list g))
		(print g)
		(if (listp (first g))
		    (let ()
		      (setq bing (rest g))
		      (setq bing (eval `(parser ',bing)))
		      (print (list (first g) bing))
		      (cons `(lambda '(first ,g) ,bing) NIL)
		      )
		  (let ()
		    (setq bing (eval `(parser ',g)))
		    (print bing)
		    (cons `(lambda (x) ,bing) NIL)
		    )
		  )
		)
	      )
	    )
      )
(plot {x^2})
{(y)y^2
|#
