#|
				Mancala

	This is file bele.lsp, created to play one version of mancala and beat
the old men of Cotonou. I created this file for Rock, when we got to talking
about how the old men like to sit around and play. I told him that it would be
fun to program a computer to beat them (I wondered what they'd say to that!).

							Andy Long
							November, 1997
|#
#|
Version change log:
	1) add a :moves slot to the game, permitting one to reproduce a game to
a given point. Check.
	2) I replaced various versions of mancala with play-mancala (the other
two are now subsets).
	3) I fixed a problem with interact - wasn't catching all bad inputs.

I'm now moving to mancala.lsp: I've taken care of the problem below, and hope
that from here on out I can stick to one file (rather than bele.lsp and
adji.lsp, implementing them rather using the type slot):

	?) I should be able to use type, finally to distinguish between the two
games, and so consolidate even further.
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                player proto:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(uber-proto game (players pits winnings moves type) nil nil "

			Game of Mancala.

;; Examples:
 (setq new-game (send game :new (iseq 2) (repeat 4 12) '(0 0)))
 (ask new-game pits)
 (send new-game :move 0 0 :check t)
 (send new-game :maximize 0)
 (send new-game :move 0 5 :check t)
 (mapcar #'(lambda(n) (send new-game :move 0 n :check t)) (iseq 6))
 (mapcar #'(lambda(n) (send new-game :move 1 n :check t)) (iseq 6 11))

;; A tough sequence: caused 'error: argument stack overflow'
;; 4
8
;; 5
8
;; 2
9
;; 2
5
")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
 	(send game :help 'proto) 

 retrieves this help, and 

 	(try-out proto :proto game)

 tries it out!
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Initialize game:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(send game 'players (iseq 2))
(send game 'pits (repeat 4 12))
(send game 'winnings '(0 0))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 methods:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :move (player pit &key pits check started print)
  "
pits are labelled 0 to 11: 0-5 are for player 0, 6-11 for player 1.
"
  ;; First we check to make sure that the entry is valid:
  (if (not started) (let () (setq started t) (check-start player pit)))
  (let* (
	 (pits (if pits pits (copy-list (send self 'pits))))
	 (result (bele-move pits pit))
	 (pits (first result))
	 (final-pit (second result))
	 )

    (print-move print player pits pit)

    ;; we're done, and should tally up:
    (let* ((other-pit (- 11 final-pit))
	   (winnings (elt pits other-pit))
	   )
      (if (or	      
	   (and (= player 1) (< other-pit 6))
	   (and (= player 0) (>= other-pit 6))
	   )
	  ;; then there are winnings!
	  (let ((winnings (elt pits other-pit)))
	    (setf (elt pits other-pit) 0)
	    (if (/= winnings 0) 
		(let ()
		  (setq winnings (1+ winnings))
		  (setf (elt pits final-pit) 0)
		  ))
	    (if (not check)
		(let ()
		  (send self 'pits pits)
		  (send self 'winnings
			(+ (send self 'winnings)
			   (* winnings (list (- 1 player) player))))
		  ))
	    (list pits winnings)
	    )
	(let ()
	  (if (not check) (send self 'pits pits))
	  (list pits 0))
	)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :one-round
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :one-round (&key print
			       interactive
			       (order '(0 1))
			       (names '("r1" "r2"))
			       )
  (let* (
	 (zero (first order))
	 (one (second order))
	 (m1 (send self :maximize zero))
	 (m1 (if interactive
		 (if (= interactive zero) (interact zero self) m1)
	       m1))
	 (r1 (if m1 (send self :move zero m1 :print (if print (elt names zero)))))
	 (m2 (if m1 (send self :maximize one)))
	 (m2 (if m2
		 (if interactive
		     (if (= interactive one) (interact one self) m2)
		   m2)))
	 (r2 (if m2 (send self :move one m2 :print (if print (elt names one)))))
	 )
    (format t "~%Next round!~%")
    (if (not (and r1 r2)) ;; game over!
	(let (
	      (pits (send self 'pits))
	      (winnings (send self 'winnings))
	      )
	  (send self 'winnings
		(+ winnings (list (sum (select pits (iseq 6)))
				  (sum (select pits (iseq 6 11))))))
	  (values nil (list m1 m2))
	  )
      (values t (list m1 m2))
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                interact
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun interact(interactive game)
  (let (
	(pits (send game 'pits))
	(score (send game 'winnings))
	ans
	)
    (print-pits pits)
    (format t 
	    (if (= 1 interactive)
		"~%The score is computer ~d, bad guys ~d."
	      "~%The score is bad guys ~d, computer ~d.")
	    (first score) (second score))
    (format t "~%Make your move: you have pits~{ ~d~}: "
	    (if (= interactive 0) (iseq 1 6) (iseq 12 7)))
    (tagbody
     read
     (setq ans (read))
     (if (not (integerp ans))
	 (let ()
	   (format t "Need an integer! Make your move:~%")
	   (go read)))
     (setq ans (mod (1- ans) 12))
     (if (>= (mod (- ans (* interactive 6)) 12) 6)
	 (let ()
	   (format t "~% You chose an illegal pit (~d)! Try again: " (1+ ans))
	   (go read)))
     (if (= 0 (elt pits ans))
	 (let ()
	   (format t "~% Pit ~d is empty. Choose a non-empty pit: "
		   (1+ ans))
	   (go read)))
     )
    ans
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :report-winnings
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :report-winnings (interactive &key (names '("computer" "bad guys")))
  (let (
	(pits (send self 'pits))
	(score (send self 'winnings))
	)
    (if interactive
	(let ()
	  (format t "~2%~{	~d~}~2%" (iseq 12 7))
	  (format t "~{	~d~}~%" (select pits (iseq 11 6)))
	  (format t "~{	~d~}~2%" (select pits (iseq 6)))
	  (format t "~{	~d~}~%" (iseq 1 6))
	  (format t "~%The final score is ~a ~d, ~a ~d."
		  (elt names 0) (first score)
		  (elt names 1) (second score))
	  )
      )
    score
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :possibilities
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :possibilities (player)
  "Makes all possible next moves, and gives the winnings.

;; Examples:
 (send new-game :possibilities 0)
"
  (let* (
	 ;; All the pits:
	 (pits (send self 'pits))
	 ;; Let's select only the player's pits to consider:
	 (player-pits (+ (* player 6) (iseq 6)))
	 ;; the bead values of those pits:
	 (player-pit-vals (select pits player-pits))
	 ;; if the player has any beads in front of him, then he can
	 ;; play - otherwise, he'll pass:
	 (vals
	  (if (/= 0 (sum player-pit-vals))
	      (mapcar
	       #'(lambda(n) (append (send self :move player n :check t) (list n)))
	       (select player-pits (where player-pit-vals 0 #'/=)))))
	 )
    (if vals vals) ;; otherwise nil - no possibilities, i.e. player passes!
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :maximize
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :maximize (player)
  "Maximize maximizes the player's gain, given that he can look ahead at his
opponent's next move and minimize the maximum.

;; Examples:
 (send new-game :maximize 0)
"
  (let* (
	 (other-player (- 1 player))
	 ;; keep the true pits to put back later:
	 (true-pits (send self 'pits))
	 ;; Let's see what player can do:
	 (vals (send self :possibilities player))
	 ;; Now for each one of these possibilities, we want to see what the
	 ;; other player can do:
	 (result
	  (mapcar
	   (lambda(val)
	     (send self 'pits (first val))
	     (mapcar
	      (lambda(y)
		(list (- (second val) (second y)) (third val) (third y)))
	      (send self :possibilities other-player))
	     )
	   vals)
	  )
	 ;; The minimum of this is the worst they can do to us,
	 (win-loss (mapcar (lambda(x) (mapcar #'first x)) result))
	 (win-loss (if (some (lambda(x) x) win-loss)
		       (mapcar #'min win-loss)))
	 ;; and we want to take the maximum of the minima:
	 (max-win-loss (remove-if #'null win-loss))
	 (max-win-loss (if max-win-loss (max max-win-loss)))
	 (minimax (if max-win-loss 
		      (select result
			      (where win-loss max-win-loss))))
	 )
    ;; make sure that we put the pits back right:
    ;; (break)
    (send self 'pits true-pits)
    (if minimax
	(if (length=1 minimax)
	    (values (second (first (first minimax))) max-win-loss)
	  ;; There may be ties: in that case, we take the one with the highest
	  ;; average as well:
	  (let* (
		 (avg
		  (mapcar
		   #'mean
		   (mapcar (lambda(x) (mapcar #'first x)) minimax)))
		 (maxiavg (select minimax (where avg (max avg))))
		 )	    
	    (values (second (first (first maxiavg))) max-win-loss)
	    )
	  )
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                print-pits
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun print-pits(pits)
  "
;; Examples:
 (PRINT-PITS '(2 1 9 2 0 4 3 0 2 1 5 1))
"
  (format t "~%~{~s~}~%" (repeat '- 80))
  (eval
   `(format t  "~18t \#~d~28t \#~d~38t \#~d~48t \#~d~58t \#~d~68t \#~d~%"
	    ,@(iseq 12 7)))
  (eval `(format t    "~20t~a~30t~a~40t~a~50t~a~60t~a~70t~a~%"
		 ,@(repeat "_____ " 6)))
  (eval `(format t   "~19t~a~29t~a~39t~a~49t~a~59t~a~69t~a~%"
		 ,@(repeat "/    /" 6)))
  (eval
   `(format t
	    "~18t/ ~d ~22t/~28t/ ~d ~32t/~38t/ ~d ~42t/~48t/ ~d ~52t/~58t/ ~d ~
~62t/~68t/ ~d ~72t/~%"
	    ,@(select pits (iseq 11 6))))
  (eval `(format t "~17t~a~27t~a~37t~a~47t~a~57t~a~67t~a~2%" 
		 ,@(repeat "/____/" 6)))
  (eval `(format t    "~20t~a~30t~a~40t~a~50t~a~60t~a~70t~a~%"
		 ,@(repeat "_____ " 6)))
  (eval `(format t   "~19t~a~29t~a~39t~a~49t~a~59t~a~69t~a~%"
		 ,@(repeat "/    /" 6)))
  (eval `(format t  "~18t/ ~d ~22t/~28t/ ~d ~32t/~38t/ ~d ~42t/~48t/ ~d ~52t/~58t/ ~d ~
~62t/~68t/ ~d ~72t/~%"
		 ,@(select pits (iseq 6))))
  (eval `(format t "~17t~a~27t~a~37t~a~47t~a~57t~a~67t~a~2%"
		 ,@(repeat "/____/" 6)))
  (eval
   `(format t  "~18t \#~d~28t \#~d~38t \#~d~48t \#~d~58t \#~d~68t \#~d~%"
	    ,@(iseq 1 6)))
  (format t "~{~s~}~%" (repeat '- 80))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                bele-move
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun bele-move (pits pit)
  "
pits are labelled 0 to 11: 0-5 are for player 0, 6-11 for player 1.

;; Examples:
 (bele-move '(1 0 0 0 0 0 1 2 3 4 5 6) 0)
 (bele-move '(1 0 0 0 0 0 1 2 3 4 5 6) 6)
"
  (do*
   (
    (pebbles (elt pits pit) (elt pits pit))
    (pits-to-add
     (mapcar (lambda(x) (mod x 12)) (iseq (+ pit 1) (+ pit pebbles)))
     (mapcar (lambda(x) (mod x 12)) (iseq (+ pit 1) (+ pit pebbles)))
     )
    (temp (setf (elt pits pit) 0) (setf (elt pits pit) 0))
    (temp
     (mapcar
      (lambda(pit) (setf (elt pits pit) (incf (elt pits pit))))
      pits-to-add)
     (mapcar
      (lambda(pit) (setf (elt pits pit) (incf (elt pits pit))))
      pits-to-add)
     )
    (pit (final pits-to-add) (final pits-to-add))
    )
   ((= 1 (elt pits pit)) (list pits pit))
   )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                check-start
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun check-start (player pit)  
  (let ()
    (unless
     (and
      (integerp pit)
      (< pit 12)
      (< (+ (* player (- 11 pit)) (* (- 1 player) pit)) 6)
      )
     (error (format nil "Bad input:~%Player: ~d~%Pit: ~d" player (1+ pit))))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                print-move
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun print-move (print player pits pit)
  (if print 
      (let ()
	(if (stringp print) (format t "~%~a pit: ~d" print (1+ pit))
	  (format t "~%pit: ~d" (1+ pit)))
	(print-pits pits)))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         play-mancala
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun play-mancala (
		     &key (print t)
			  (names '("Ordinateur" "Bad Guys"))
			  (order '(0 1))
			  (player 0) 
			  (pits (repeat 4 12))
			  (score '(0 0)))
  "
;; Examples:
 (play-mancala :player nil)
 (play-mancala :print t)
 (play-mancala :print t :player 1) ;; to play second

 ;; to play a game from a point at which it was stopped:
 (play-mancala :order '(1 0) :names '(\"Rock\" \"Ord\")
    :pits '(2 1 9 2 0 4 3 0 2 1 5 1) :score '(7 11))
"
  (let ((new-game (send game :new (iseq 2) pits score)))
    (unwind-protect
	(let ()
	  (print-pits pits)
	  (do
	   ((i 
	     (cmv (send new-game :one-round :print print :interactive player
		       :order order :names names))
	     (cmv (send new-game :one-round :print print :interactive player
		       :order order :names names))
	     ))
	   ((not (first i)))
	   (if (/= 48 (sum (combine (send new-game 'winnings) 
				    (send new-game 'pits))))
	       (let ()
		 (printall 48 (send new-game 'winnings) (send new-game 'pits))
		 (break)
		 ))
	   (send new-game 'moves (append (send new-game 'moves) (second i)))
	   (format t "The score is: ~s" (send new-game 'winnings))
	   )
	  (send new-game :report-winnings player :names names)
	  )
      ;; we want to make sure to dispose of the game, after showing how to
      ;; reproduce it:
      (format t "~%Moves: ~{ ~d~}~%" (1+ (send new-game 'moves)))
      (send game :dispose new-game)
      new-game
      )
    )
  )