;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	This is work on a paper by L. P. Bos, J. E. Grabenstetter, and K.
;; Salkauskas, Pseudo-tensor product interpolation and blending with families
;; of univariate schemes, Computer-Aided Geometric Design, 13 (1996) 429-440.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                example:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(func f(x y) 1/(1+(x^2+y^2)^10))
(def m 11)
(title (num-to-string m))
(def xs (rseq -2 2 m))
(def ys xs)
(def A (outer-product xs ys #'f))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Now I'm going to create the interpolating function:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun G(x) (mapcar (lambda(y) (f x y)) ys))
(defun H(y) (mapcar (lambda(x) (f x y)) xs))
(def inv (pseudo-inverse A))
(mprint inv)
(defun skin(x y) (matmult (g x) inv (h y)))
(spin-domain -2 2 -2 2)
(def nplot 41)
(spin-points nplot)
(plot skin)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			On to function two:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(func f(x y) (abs (y-x^2+.4317)))
(def xs (rseq -1 1 m))
(def ys xs)
(def A (outer-product xs ys #'f))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Now I'm going to create the interpolating function:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun G(x) (mapcar (lambda(y) (f x y)) ys))
(defun H(y) (mapcar (lambda(x) (f x y)) xs))
(def inv (pseudo-inverse A))
(mprint inv)
(defun skin(x y) (matmult (g x) inv (h y)))
(spin-domain -1 1 -1 1)
(plot skin)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                    linear-connector-function
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun linear-connector-function (x1 y1 x2 y2)
  "
;; Description: (linear-connector-function x1 y1 x2 y2)

	Returns the linear function which joins points (x1 y1) and (x2 y2).

;; Examples:
 (new-plot)
 (plot (linear-connector-function 0 0 1 1))
 (plot (linear-connector-function 0 1 1 0))
"
  (let* (
	 (m (/ (- y2 y1) (- x2 x1)))
	 (b (- y1 (* m x1)))
	 )
    (lambda(x) (+ b (* m x)))
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;              linear-connector-function-parameters
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun linear-connector-function-parameters (x1 y1 x2 y2)
  "
;; Description: (linear-connector-function-parameters x1 y1 x2 y2)

	Returns the parameters of the linear function which joins points
 (x1 y1) and (x2 y2). 

;; Examples:
 (new-plot)
 (plot (linear-connector-function-parameters 0 0 1 1))
 (plot (linear-connector-function-parameters 0 1 1 0))
"
  (let* (
	 (m (/ (- y2 y1) (- x2 x1)))
	 (b (- y1 (* m x1)))
	 )
    (list m b)
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; On to patching two matroid function meshes together:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def n 11)
(func auxi(x) x^2-.4317)
(def xs1 (rseq -1 1 n))
(def ys1 (rseq -1 .5683 n))
(def ys3 (rseq .5683 1 n))
(def lc1 (linear-connector-function -1 -1 .5683 -.4317))
(def lc2 (linear-connector-function .5683 -.4317 1 1))
(def lc1-parameters (linear-connector-function-parameters -1 -1 .5683 -.4317))
(def lc2-parameters (linear-connector-function-parameters .5683 -.4317 1 1))

(def xmat [[-1 -1 -1][0 0 0][1 1 1]])
(def ymat [[-1 -.7 .5683][-1 -.7 -.4317][-1 -.7 .5683]])
(def ymat2 [[.5683 .7 1][-.4317 .7 1][.5683 .7 1]])
(def yinv (pseudo-inverse ymat))
(def yinv2 (pseudo-inverse ymat2))
(def zinv (pseudo-inverse (f xmat ymat)))
(def zinv2 (pseudo-inverse (f xmat ymat2)))

;; Here we create the functions x(a,b), y(a,b) and z(a,b) of the parameters:
(defun xf(a b) a)
(defun yf(a b)
  (matmult (list -1 -.7 (auxi a)) yinv (list b (funcall lc1 b) b)))
(defun yf2(a b)
  (matmult (list (auxi a) .7 1) yinv2 (list b (funcall lc2 b) b)))
(defun zf(a b)
  (matmult (list (f a -1) (f a -.7) (f a (auxi a))) zinv
	   (list (f -1 b) (f 0 (funcall lc1 b)) (f 1 b))))
(defun zf2(a b)
  (matmult (list 0 (f a .7) (f a 1)) zinv2
	   (list (f -1 b) (f 0 (funcall lc2 b)) (f 1 b))))

(def xm1 (outer-product xs1 ys1 #'xf))
(def ym1 (outer-product xs1 ys1 #'yf))
(def zm1 (outer-product xs1 ys1 #'zf))
(def ym2 (outer-product xs1 ys3 #'yf2))
(def zm2 (outer-product xs1 ys3 #'zf2))

(def my-plot
     (matroid
      (bind-columns xm1 xm1) (bind-columns ym1 ym2) (bind-columns zm1 zm2)))
(send my-plot :add-function #'f -1 1 -1 1 :num-points n :spline nil)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	Trying to make a function of (x,y) out of the function of the
parameters (a,b):
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun bf(x y)
  (let* (
	 (beta (first lc1-parameters))
	 (alpha (second lc1-parameters))
	 (tmp (matmult (list -1 -.7 (auxi x)) yinv))
	 )
    (/ 
     (- y (matmult tmp (list 0 alpha 0)))
     (matmult tmp (list 1 beta 1))
     )
    )
  )
(defun bf2(x y)
  (let* (
	 (beta (first lc2-parameters))
	 (alpha (second lc2-parameters))
	 (tmp (matmult (list (auxi x) .7 1) yinv2))
	 )
    (/ 
     (- y (matmult tmp (list 0 alpha 0)))
     (matmult tmp (list 1 beta 1))
     )
    )
  )
(defun newz(x y)
  (if (> y (auxi x))
      (zf2 x (bf2 x y))
    (zf x (bf x y))))
(plot newz)
(send my-plot :add-function #'f -1 1 -1 1 :num-points n :spline nil)
(plot (lambda(x y) (- (f x y) (newz x y))))
(max (outer-product (rseq -1 1 *spin-points*) (rseq -1 1 *spin-points*)
		    (lambda(x y) (- (f x y) (newz x y)))))
