;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
Dad,

	I've realized that, in fact, the whole thing boils down to something
that doesn't even require all the SVD gyrations: we could just proceed directly
to the inverse of the matrix. So I do!

	Start with some complicated function (which generates a full matrix a):
|#
(defun skin(n)
  "
;; Description:
	The function returns the maximum difference in the function f and the
 interpolating function axy on the region [0,1]x[0,1]:

;; Examples:
 (func f(x y) sin(6*ln(1+x)*y) + y*x + (exp y*(1-x)))
 (spin-domain 0 1 0 1)
 (spin-points 21)
 (skin 2) ;; 11.574895163992332
 (skin 3) ;; 0.14839850183330983
 (skin 5) ;; 5.245574898382621E-5
 (skin 7) ;; 1.4801850412027306E-8
"
  ;; Now we define the grid, and compute the svd:
  (let* (
	 (xs (rseq 0 1 n))
	 (ys xs)
	 (A (outer-product xs ys #'f))
	 (A-1 (inverse A))

	 (G (lambda(x) (mapcar (lambda(y) (f x y)) ys)))
	 (H (lambda(y) (mapcar (lambda(x) (f x y)) xs)))
	 (Axy (lambda(x y) (matprod (funcall G x) A-1 (funcall H y))))

	 (diff (outer-product (rseq 0 1 21) (rseq 0 1 21)
			      (lambda(x y) (- (funcall axy x y) (f x y)))))
	 )

    (title (strcat "Compare the two functions: " (num-to-string n) " grid"))
    (plotter axy #'f)
    (max diff)
    )
  )
