;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
Hi Dad,
 
        Here's a kludgy way to get an m x n case to work out (in the sense of
getting curve interpolation with as much differentiability as existed in the
original curves):
 
        Assume m < n: step over the n dimension m curves at a time, and do an
mxm case (n-m+1 times). You now have n-m+1 patch functions, which you can
multiply by bump functions (infinitely differentiable, finitely supported) and
add together to maintain interpolation and differentiability.
 
        It's kludgy, but would work. I'm trying to recall what the bump
functions look like (I've got my analysis notes at home), but it seems to me
that the basic form is:
 
                {exp(-1/(1-x^2))        |x| < 1
        f(x) =  {
                {0                      |x| >= 1
 
 
        Now if you used a pair of these for each patch, essentially only using
each patch at its ends, and put them together in just the right way....

	Well here, let me show you in xlispstat:
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Start with the canonical bump-function we'll need:
(func
 canonical-bump-function(x)
 (if ((abs x) < 1)
     (exp(1-1/(1-x^2)))
   (0))
 )
(plot canonical-bump-function -2 2)

(def half-area-under-canonical-bump-function
     (simpson canonical-bump-function -1 0 1e-6))
(def area-under-canonical-bump-function
     (* 2 half-area-under-canonical-bump-function))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	I need a bump function which possesses more symmetry than the canonical
bump function. So I integrate it (from -infinity to x) to get an infinitely
differentiable function from 0 to alpha (which I renormalize to 1).

	I paste two of these together back-to-back, and I have a more symmetric
bump function, called the cbf. Then these are combined to form bi-humped
animals, built up of pairs of cbf functions.
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun integral-canonical-bump-function(X)
  (cond
   ((<= x -1) 0)
   ((<= x 0)
    (/
     (eval `(simpson canonical-bump-function -1 ,x .001))
     area-under-canonical-bump-function)
    )
   ((<= x 1)
    (- 1 (integral-canonical-bump-function (- x)))
    )
   (t 1)
   )
  )
(plot integral-canonical-bump-function -2 2)

(defun cbf(x)
  (if (<= x 0)
      (integral-canonical-bump-function (+ 1 (* x 2)))
    (cbf (- x))
    ))
(plot cbf -2 2)
(defun new-bf(x a b c d)
  (cond
   ((<= x a)
    0
    )
   ((<= x b)
    (integral-canonical-bump-function (- (/ (* 2 (- x a)) (- b a)) 1))
    )
   ((<= x c)
    1
    )
   ((<= x d)
    (integral-canonical-bump-function (+ (/ (* -2 (- x c)) (- d c)) 1))
    )
   (t 0)
   )
  )
(plot (lambda(x) (new-bf x -1.5 -1 0 1)) -2 2)

(defun bump-function(x m xstart xend)
  (let (
	(width (/ (- xend xstart) (1- m)))
	)
    (+
     (cbf (/ (- x xstart) width))
     (cbf (/ (- x xend) width))
     )
    )
  )
(defun front-humped-bump-function(x m xstart xend)
  (let (
	(width (/ (- xend xstart) (1- m)))
	)
    (+
     (cbf (/ (- x xstart) width))
     (* .5 (cbf (/ (- x xend) width)))
     )
    )
  )
(defun back-humped-bump-function(x m xstart xend)
  (let (
	(width (/ (- xend xstart) (1- m)))
	)
    (+
     (* .5 (cbf (/ (- x xstart) width)))
     (cbf (/ (- x xend) width))
     )
    )
  )
      
(x-tics 7)
(plot (lambda(x) (bump-function x 5 -2 2)) -3 3)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	Start with some complicated function (which generates a full matrix a):
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(func f(x y) sin(6*ln(1+x)*y) + y*x + (exp y*(1-x)))
(def m 3)
(def n 8)
(title (num-to-string n))
(def xs (rseq 0 1 m))
(def ys (rseq 0 1 n))
(def A (outer-product xs ys #'f))
(spin-domain 0 1 0 1)
(spin-points n)

(def m-1 (1- m))
(def n-m+1 (1+ (- n m)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	This one demonstrates that a mesh will fit a 3x3 sub-function grid of
the entire 3x5 function-grid. These individual functions have to be multiplied
by bump functions to give us full interpolation:
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(dotimes
 (i n-m+1)
 (let ((some (iseq i (+ i m-1))))
   (def suba (submatrix a 0 m-1 i (+ i m-1)))
   (def A-1 (inverse suba))
   (defun G(x) (mapcar (lambda(y) (f x y)) (select ys some)))
   (defun H(y) (mapcar (lambda(x) (f x y)) xs))
   (defun Axy(x y) (matprod (G x) A-1 (H y)))
   (plot axy f)
   )
 )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	This one shows each function getting two curves exactly right, and the
others are set at zero:
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(dotimes
 (i n-m+1)
 (let ((some (select ys (iseq i (+ i m-1)))))
   (def suba (submatrix a 0 m-1 i (+ i m-1)))
   (def A-1 (inverse suba))
   (defun G(x) (mapcar (lambda(y) (f x y)) some))
   (defun H(y) (* (bump-function y m (first some) (final some))
		  (mapcar (lambda(x) (f x y)) xs)))
   (defun Axy(x y) (matprod (G x) A-1 (H y)))
   (plot axy f)
   )
 )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	Now I'm going to create the monster function, which is the functions
from the previous example times the appropriate grid functions.

|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def matrices
     (mapcar
      (lambda(i)
	(let (
	      (suba (submatrix a 0 m-1 i (+ i m-1)))
	      (some (select ys (iseq i (+ i m-1))))
	      )
	  (inverse suba)
	  )
	)
      (iseq n-m+1)
      )
     )

(def G-functions
     (mapcar
      (lambda(i)
	(let ((some (select ys (iseq i (+ i m-1)))))
	  (lambda(x) (mapcar (lambda(y) (f x y)) some))
	  )
	)
      (iseq n-m+1)
      )
     )

;; types: 'f (front), 'n (normal), 'h (half), 'b (back):
(def types 
     (cond
      ((= n 4)
       '(n n)
       )
      ((= n 5)
       '(f n b)
       )
      ((= n 6)
       '(f f b b)
       )
      ((= n 7)
       '(f f h b b)
       )
      ((= n 8)
       '(f f h h b b)
       )
      ))

(def H-functions
     (mapcar
      (lambda(i)
	(let (
	      (some (select ys (iseq i (+ i m-1))))
	      (type (elt types i))
	      )
	  (lambda(y) (* 
		      (cond
		       ((equalp type 'f)
			(front-humped-bump-function
			 y m (first some) (final some))
			)
		       ((equalp type 'b)
			(back-humped-bump-function
			 y m (first some) (final some))
			)
		       ((equalp type 'h)
			(* .5 (bump-function y m (first some) (final some)))
			)
		       (t
			(bump-function y m (first some) (final some))
			)
		       )
		      (mapcar (lambda(x) (f x y)) xs)
		      )
	    )
	  )
	)
      (iseq n-m+1)
      )
     )

(defun skin(x y)
	(sum
	 (mapcar
	  (lambda(G A-1 H)
	    (matprod (funcall G x) A-1 (funcall H y))
	    )
	  G-functions
	  matrices
	  H-functions
	  )
	 )
	)

(plot f skin)

(spin-points (+ n (1- n)))
(plot f skin)

(plot (lambda(x y) (- (f x y) (skin x y))))
(print 
 (max (outer-product
       (rseq 0 1 (spin-points (+ n (1- n))))
       (rseq 0 1 (spin-points (+ n (1- n))))
       (lambda(x y) (- (f x y) (skin x y)))))
 )

(defun type-check(n)
  (let (
	(types 
	 (cond
	  ((= n 4)
	   '(n n)
	   )
	  ((= n 5)
	   '(f n b)
	   )
	  ((= n 6)
	   '(f f b b)
	   )
	  ((= n 7)
	   '(f f h b b)
	   )
	  ((= n 8)
	   '(f f h h b b)
	   )
	  ))
	(ys (rseq 0 1 n))
	)
    
    (flet ((lam(x)
	       (sum
		(mapcar
		 (lambda(type xstart xend)
		   (cond
		    ((equalp type 'f)
		     (front-humped-bump-function
		      x m xstart xend)
		     )
		    ((equalp type 'b)
		     (back-humped-bump-function
		      x m xstart xend)
		     )
		    ((equalp type 'h)
		     (* .5 (bump-function x m xstart xend))
		     )
		    (t
		     (bump-function x m xstart xend)
		     )
		    )
		   )
		 types
		 (select ys (iseq (length types)))
		 (select-from-end ys (iseq (length types)))
		 ))))
	  (max (abs (- 1 (mapcar #'lam (rseq 0 1 100)))))
	  )
    )
  )
;; (mapcar #'type-check (iseq 4 8))
