
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	Here are most of the examples and exercises from the book "Applied
Regression Analysis", by N. R. Draper and H. Smith., second edition.

	I start by entering some of the data from the appendices:
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	The table (Table 1.1) of p. 616 and Appendix B, p. 629 are contained
in the file drapedat.lsp in the examples directory: loading it allows these
functions to establish the data needed.
|#
(load-calc drapedat)
(table1.1)
(appendixB)
(def appB (regression-model (list x1) x5))
(send appB :anova)
(square (partial-correlation (bind-columns x1 x2 x3 x4) x5 1))
(square (partial-correlation (bind-columns x1 x2 x3 x4) x5 1 2))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                page 9
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(load-calc drapedat)
(table1.1)
(mapcar #'mean (list x1 x2 x3 x4 x5 x6  x7 x8 x9 x10))
(mapcar #'sd (list x1 x2 x3 x4 x5 x6  x7 x8 x9 x10))
(mprint (correlation x1 x2 x3 x4 x5 x6 x7 x8 x9 x10))

(def x x8)
(def y x1)
(size y x)
(mapcar #'mean (list y x))
(mapcar #'sd (list y x))
(lines off)

(def page9 (regression-model x y))
(send page9 :plot 25 85)
(send page9 :coef-estimates)
(send page9 :resss)
(send page9 :regss)
(send page9 :tss)

(send page9 :sse)

(send page9 :s-squared)
(send page9 :n-p)

(send page9 :f)
(send page9 :prob-f)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	This is the repeated measurements example, from page 38:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def page38 '((1.3 (2.3 1.8)) (2 (2.8 1.5)) (2.7 (2.2)) (3.3 (3.8 1.8))
	      (3.7 (3.7 1.7)) (4 (2.8 2.8 2.2)) (4.7 (5.4 3.2 1.9)) (5.0 (1.8))
	      (5.3 (3.5 2.8 2.1)) (5.7 (3.4)) (6 (3.2 3)) (6.3 (3))
	      (6.7 (5.9))))
(def xs (xs page38))
(def ys (ys page38))
(def xs
     (combine
      (mapcar {(repeat (first x) (second x))} (weave xs (mapcar #'length ys)))
      )
     )
(def ys (combine ys))
(def page38 (regression-model xs ys))
(send page38 :plot)
(send page38 :anova)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise A, page 55
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise A, page 55")
(def x (iseq -5 5))
(def y (list 1 5 4 7 10 8 9 13 14 13 18))
(def exA (regression-model x y))
(send exA :plot)
(send exa :anova)
;; 9.27273 + 1.43636x
(send exa :coef-confidence-intervals)
(plot x (send exa :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise C, page 57
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise C, page 55")
(def x (/ (list 1 48 71 95  119  1  48 144 71  196 1 144 196) 100))
(def y
     (/ (list 1276 1240 1108 1039 1015 1301 1220 923 1131 837 1280 914 862) 10))
(def exc (regression-model x y))
(send exc :plot)
(send exc :anova)
(send exc :coef-confidence-intervals)
(plot x (send exc :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise E, page 59
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise E, page 59")
(def x1 (/ (list 35 41 44 50 55 61 66) 10))
(def y1 (list 24 32 37 40 43 51 62))
(def x2 (/ (list 32 39 49 61 70 81) 10))
(def y2 (list 22 33 39 44 53 57))
(def x3 (/ (list 30 40 50 60 65 70 73 74) 10))
(def y3 (list 32 36 47 49 55 59 64 64))
(def x (combine x1 x2 x3))
(def y (combine y1 y2 y3))
;; Wrote this function to do the test:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; test-bs Doc:    Creation Date:3/2/97        Author: Andy Long
;;
;;
(defun test-bs(list-of-groups)
  "
;; Description/Arguments:

       (test-bs list-of-groups)
  
	This is described in Draper and Smith, Applied Regression Analysis,
2nd Edition, exercise E, page 59. It is a test for the equality of slopes of
several regression equations. It is equivalent to a test of hypothesis using
lots of indicators, etc., but may be easier if the regression equations are
already in hand.

;; Notes:

;; See Also:

;; Examples:
 (title \"Exercise E, page 59\")
 (def x1 (/ (list 35 41 44 50 55 61 66) 10))
 (def y1 (list 24 32 37 40 43 51 62))
 (def x2 (/ (list 32 39 49 61 70 81) 10))
 (def y2 (list 22 33 39 44 53 57))
 (def x3 (/ (list 30 40 50 60 65 70 73 74) 10))
 (def y3 (list 32 36 47 49 55 59 64 64))
 (test-bs (weave (list x1 x2 x3) (list y1 y2 y3)))

 ;; Now I'm going to try this another way: I'm going to use the :hypothesis
 ;; method which I rigged up, to see if I get the same response:
 (def x (combine x1 x2 x3))
 (def y (combine y1 y2 y3))
 (def x1-ind (combine (repeat 1 (length x1))
 		     (repeat 0 (+ (length x2) (length x3)))
 		     ))
 (def x2-ind (combine (repeat 0 (length x1))
 		     (repeat 1 (length x2))
 		     (repeat 0 (length x3))
 		     ))
 (def x3-ind (combine (repeat 0 (length x1))
 		     (repeat 0 (length x2))
 		     (repeat 1 (length x3))
 		     ))
 (def x1-l (combine x1
 		     (repeat 0 (+ (length x2) (length x3)))
 		     ))
 (def x2-l (combine (repeat 0 (length x1))
 		     x2
 		     (repeat 0 (length x3))
 		     ))
 (def x3-l (combine (repeat 0 (length x1))
 		     (repeat 0 (length x2))
 		     x3
 		     ))
 ;; Here we go:
 (def exe (regression-model (list x1-ind x2-ind x3-ind x1-l x2-l x3-l) y
 			   :intercept nil))
 (send exe :hypothesis #2a((0 0 0 -2 1 1)(0 0 0 1 -2 1)) :verbose t)
 
 ;; Yep! That's good! Here it is another (simpler) way:
 (def exe (regression-model (list x2-ind x3-ind x x2-l x3-l) y))
 (send exe :hypothesis #2a((0 0 0 0 1)(0 0 0 1 0)) :verbose t)
"
 (let* (
	(m (length list-of-groups))
	(sum-n (sum (mapcar #'(lambda(x) (length (first x))) list-of-groups)))

	(regs (mapcar
	       (lambda(x) (regression-model (first x) (second x) :print nil))
	       list-of-groups
	       )
	      )

	(bs (mapcar (lambda(x) (second (send x :coef-estimates))) regs))

	(ss (mapcar (lambda(x) (send x :regss)) regs))
	(tss (sum (mapcar (lambda(x) (send x :tss)) regs)))
	(xdiffsquares (/ ss (square bs)))

	(nums (* bs xdiffsquares))
	(b (/ (sum nums) (sum xdiffsquares)))

	(gdf (1- m))
	(rdf (- sum-n (* 2 m)))
	(tdf (- sum-n m))

	(s1 (* (square b) (sum xdiffsquares)))
	(s2 (- (sum ss) s1))
	(s3 (- tss s2 s1))

	(m1 s1)
	(m2 (/ s2 gdf))
	(m3 (/ s3 rdf))

	(f1 (/ m1 m3))
	(f2 (/ m2 m3))
	(prob (- 1 (f-cdf f2 gdf rdf)))

	(fmt1 "~%  ~a ~14t~a ~29t~a ~49t~a ~64t~a~2%")
	(fmt2 "~1t~a ~14t~d ~25t~8,8g ~45t~8,8g ~60t~8,8g~%")
	(fmt3 "~1t~a ~14t~d ~25t~8,8g ~45t~8,8g~%")
	)
      (format t fmt1 "Source" "df" "SS" "MS" "F ratio")
      (format t fmt2 "b" 1 s1 m1 f1)
      (format t fmt2 "All bi|b" gdf s2 m2 f2)
      (format t fmt3 "Residual" rdf s3 m3)
      (format t "~% Total: ~14t~d ~25t~8,8g~2%" tdf tss)
      (format t "Probability of Ho: b's equal: ~8,5f~%" prob)
      prob
      )
 )   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(test-bs (weave (list x1 x2 x3) (list y1 y2 y3)))

;; Now I'm going to try this another way: I'm going to use the :hypothesis
;; method which I rigged up, to see if I get the same response.
(def x1-ind (combine (repeat 1 (length x1))
		     (repeat 0 (+ (length x2) (length x3)))
		     )
     )
(def x2-ind (combine (repeat 0 (length x1))
		     (repeat 1 (length x2))
		     (repeat 0 (length x3))
		     )
     )
(def x3-ind (combine (repeat 0 (length x1))
		     (repeat 0 (length x2))
		     (repeat 1 (length x3))
		     )
     )
(def x1-l (combine x1
		     (repeat 0 (+ (length x2) (length x3)))
		     )
     )
(def x2-l (combine (repeat 0 (length x1))
		     x2
		     (repeat 0 (length x3))
		     )
     )
(def x3-l (combine (repeat 0 (length x1))
		     (repeat 0 (length x2))
		     x3
		     )
     )
;; Here we go:
(def exe (regression-model (list x1-ind x2-ind x3-ind x1-l x2-l x3-l) y
			   :intercept nil))
(send exe :hypothesis #2a((0 0 0 -2 1 1)(0 0 0 1 -2 1)) :verbose t)

;; Yep! That's good! Here it is another way:
(def exe (regression-model (list x2-ind x3-ind x x2-l x3-l) y))
(send exe :hypothesis #2a((0 0 0 0 1)(0 0 0 1 0)) :verbose t)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise F, page 60
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise F, page 60")
(def x (/ (list 47 50  52 52 59 47 59 52 53 59 56 50) 10))
(def y (list 3 3 4 5 10 2 9 3  7  6 6 4))

(def exf (regression-model x y))
(send exf :plot)
(send exf :anova)
(send exf :coef-confidence-intervals)
(plot x (send exf :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise G, page 61
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def xs (list .5 1 4 4.5 5 5.5 6))
(def ys '((163 182) (978 466 549) (495 723 681) (619 1049 1033) (890 1522 1194)
	  (987) (764 1373)))
(def xs
     (combine
      (mapcar {(repeat (first x) (second x))} (weave xs (mapcar #'length ys)))
      )
     )
(def ys (combine ys))

(def exg (regression-model xs ys))
(send exg :plot)
(send exg :anova)
(plot xs (send exg :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise H, page 62
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise H, page 62")
(def x (/ (list 30 31 30 36 38 27 31 27 27 33 32 21 30 26) 10))
(def y (/ (list 31 39 34 40 36 36 31 36 29 36 41 26 31 28) 10))

(def exh (regression-model x y))
(send exh :plot)
(send exh :anova)
(send exh :coef-confidence-intervals)
(plot x (send exh :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise I, page 62
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise I, page 62")
(def x (list 4 3 5 8 4 3 3 4 3 5 7 3 8))
(def y (list 27 54 86 136 65 109 28 75 53 33 168 47 52))

(def exi (regression-model x y))
(send exi :plot)
(send exi :anova)
(send exi :coef-confidence-intervals)
(plot x (send exi :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise J, page 63
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise J, page 63")
(def x (* 10 (combine (iseq 46 41) (iseq 45 40) (iseq 42 40))))
(def y (/ (list 3 3 4 4 6 5 5 6 6 6 7 6 6 6 6) 10))

(def exj (regression-model x y))
(send exj :plot)
(send exj :anova)
(send exj :coef-confidence-intervals)
(plot x (send exj :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise K, page 63
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise K, page 63")
(def x (/ (list 30 47 83 93 99 110 123 125 126 159 167 188 188 189 217 219 228
		242 258 306 362 398 443  468 468 581 623 706 711 713 832 836
		995 1112) 10))
(def y (/ (list 971 979 982 971 957 961 956 972 889 961 982 975 942 932 908 970
		985 933 858 987 958 909 859 863 811 877 798 855 788 821 830 718
		642 658) 1000))

(def exk (regression-model x y))
(send exk :plot)
(send exk :anova)
(send exk :coef-confidence-intervals)
(plot x (send exk :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise L, page 64
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	I'm faking this by averaging values of x as per the directions in the
text. The answer is close, but not exactly right.
|#
(title "Exercise L, page 63")
(def x (/ (list 30 47 83 96 96 110 125 125 125 159 167 188 188 188 218 218 228
		242 258 306 362 398 443  468 468 581 623 709 709 709 834 834
		995 1112) 10))
(def y (/ (list 971 979 982 971 957 961 956 972 889 961 982 975 942 932 908 970
		985 933 858 987 958 909 859 863 811 877 798 855 788 821 830 718
		642 658) 1000))

(def exl (regression-model x y))
(send exl :plot)
(send exl :anova)
(send exl :coef-confidence-intervals)
(plot x (send exl :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise N, page 63
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise N, page 63")
 (def x (* 10 (list 1 1 2 2 3 3 4 4 4 5 5 5)))
 (def y (list -2 -4 1 3 2 5 0 1 2 -2 -3 -4))

 (def exn (regression-model x y))
 (def my-plot (send exn :plot))
 (send my-plot :add-plot-item (list 'function {1.692 - x * .0546} 'red t))

 (send exn :anova)
 ;; This is the true model, according to the regression above:
 (anova x y (send exn :regression-function))

 ;; compare it to this model, which inspired this function (one other than the
 ;; actual regression model):
 (anova x y {1.692 - x * .0546})

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise O, page 63
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise O, page 63")
(def x (/ (list 133 169 199 232 263 301 426) 10))
(def y (/ (list 35 51 48 67 60 95 81) 10))

(def exo (regression-model x y))
(send exo :plot)
(send exo :anova)
(send exo :coef-confidence-intervals)
(plot x (send exo :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise R, page 66
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise R, page 66")
(def x (list 50 60 70 80 90 95 100 105 110 115 120 125 130 135 140 145 150 155
	     160 165 170 180))
(def y '((19)(20 21)(17 22)(25 28)(21 25 31)(25)(30 29 33)(35 32)(30 28 30)
	 (31 36 30)(36 25 28)(28)(31 32)(34 25)(26 33)(31)(36 33)(41 33)
	 (40 30 37)(32)(35)(38)))
(def exr (weave x y))
(def xs x)
(def ys (ys exr))
(def xs
     (combine
      (mapcar {(repeat (first x) (second x))} (weave xs (mapcar #'length ys)))
      )
     )
(def ys (combine ys))

(def exr (regression-model xs ys))
(send exr :plot)
(send exr :anova)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise S, page 66
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise S, page 66")
(load-calc drapedat)
(table1.1)
(def x x8)
(def y x1)
(def exs (regression-model x y))
(secant2 {10 - (funcall (send exs :top-ci-function) x) :dash t}
	(min x) (max x))
(secant2 {10 - (funcall (send exs :bottom-ci-function) x) :dash t}
	(min x) (max x))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise V, page 67
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise V, page 67")
(def x (list 19 40 42 42 47 49 50 54 56 56 57 57 58 61 62 62 65 65 65 66 66 66
	     67 68 68 68 69 69 70 71 71 71 72 73 74 74 75 75 75 76 77 80 82 82
	     82 83 86 88 88 94))
(def y (/ (list 975 900 960 975 1125 945 1125 900 795 1200 810 1020 855 720 795
		885 825 885 975 885 915 1020 915 795 885 900 780 1005 1050 915
		945 945 945 810 885 960 645 975 1020 600 885 900 975 1065 1320
		795 795 915 975 900) 100))

(def exv (regression-model x y))
(send exv :plot)
(send exv :anova)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise W, page 68
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise W, page 68")
(def x (+ 1900 (list -10 0 20 31 34 35 40 41 44 48 50 52 55 60)))
(def y (list 50 35 25 11.98 15 13 6.98 10 5.99 8.98 6.98 4.99 5.98 4.98))
(def exw (regression-model x y))
(lines off)
(plot x (send exw :residuals))
(send exw :plot)
(send exw :anova)

(def y (ln y))
(def x (- 1972 x))
(def exw (regression-model x y))
(plot x (send exw :residuals))
(send exw :plot)
(send exw :anova)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                exercise X, page 69
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise X, page 69")
(def x (list 10 8 13 9 11 14 6 4 12 7 5))
(def y1 (/ (list 804 695 758 881 833 996 724 426 1084 482 568) 100))
(def y2 (/ (list 914 814 874 877 926 810 613 310 913 726 474) 100))
(def y3 (/ (list 746 677 1274 711 781 884 608 539 815 642 573) 100))
(def x4 (combine (repeat 8 10) 19))
(def y4 (/ (list 658 576 771 884 847 704 525 556 791 689 1250) 100))
(def groups (weave (list x x x x4) (list y1 y2 y3 y4)))
(def regs (mapcar {(regression-model (first x) (second x)) :dash t} groups))
(mapcar {send x :plot} regs)
(tile (select (agw) (iseq 4)))

;; Note: all these regression parameters are the same!
(mapcar {send x :r-squared :dash t} regs)
(mapcar {send x :coef-estimates :dash t} regs)
(mapcar {send x :coef-standard-errors :dash t} regs)
(mapcar {send x :regss} regs)
(mapcar {send x :resss} regs)
(mapcar {send x :tss} regs)
(mapcar {send x :anova} regs)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Chapter 2
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                      page 106: general hypotheses.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def y (list 1 4 8 9 3 8 9))
(def x (matrix '(7 4) (list 1 -1 -1 1 1 1 -1 1 1 -1 1 1 1 1 1 1 1 0 0 0 1 0 1 0
			    1 0 2 0)))
(def x (select-columns x '(1 2 3)))
(def page106 (regression-model x y))
(send page106 :anova)

(def c (list 1 1 0))
(def page107 (regression-model (matmult x c) y))
(send page107 :anova)

(def p (send page106 :num-coefs))
(def q (send page107 :num-coefs))
(def df (send page106 :n-p))
(def ndf (- p q))

(def stat (/ 
	   (/ (- (send page107 :resss) (send page106 :resss)) ndf)
	   (/ (send page106 :resss) df)
	   ))
(- 1 (f-cdf stat ndf df))

#|
	Created :hypothesis to take care of doing this:
|#
(send page106 :hypothesis (matrix (list 1 3) (list 1 1 0)) :verbose t)
(send page106 :hypothesis (matrix (list 1 3) (list 0 1 0)) :verbose t)
(send page106 :hypothesis (matrix (list 1 3) (list 1 0 0)) :verbose t)
(send page106 :hypothesis (matrix (list 1 3) (list 0 0 1)) :verbose t)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        Weighted Least Squares:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Table 2.1: page 113
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def x (combine 1.15 1.9 (repeat 3 5) 5.34 5.38 5.4 5.4 5.45 7.7 7.8 7.81 7.85
		7.87 7.91 7.94 9.03 9.07 9.11 9.14 9.16 9.37 10.17 10.18
		(repeat 10.22 3) 10.18 10.50 10.23 10.03 10.23))
(def y (combine 99 98 260 267 266 278 280 592 535 433 489 521 768 981 652 971
		982 981 850 947 1145 1214 1150 1065 1064 978 1239 1103 800 1190
		868 725 1346 1019 993))
(def y (/ y 100))
(def z (combine 124028 218224 (repeat 784930 5) 743652 699309 (repeat 678574 2)
		630514 89204 84420 83963 82171 81296 79588 78342 47385 46621
		45878 45327 44968 41435 31182 31079 (repeat 30672 3) 31079
		28033 30571 32680 30571))
(def z (/ z 100000))

(def page113-unweighted (regression-model x y))
(def page113 (regression-model x y :weights z))
(send page113 :plot)
(send page113 :s-squared)
(send page113 :tss)
(send page113 :regss)
(send page113 :resss)
(send page113 :anova)

(lines off)
(y-tics 6)
(x-tics 6)

(title "Plots recreate page 116")
(def residuals (send page113 :residuals))
(def estimates (send page113 :estimates))
(variable-labels "w^.5*x" "Residuals versus predictor values")
(plot (* (sqrt z) x) residuals)
(send my-plot :range 0 0 15)
(variable-labels "w^.5*yhat" "Residuals versus fitted values")
(plot (* (sqrt z) estimates) residuals)
(send my-plot :range 0 0 20)
(mapcar (lambda(x) (let () (send x :plot-item) nil)) (select (agw) (iseq 2)))

(title "Plots recreate page 114")
(def residuals (send page113-unweighted :residuals))
(def estimates (send page113-unweighted :estimates))
(variable-labels "x" "Residuals versus predictor values")
(plot x residuals)
(send my-plot :range 0 1 13.5)
(send my-plot :range 1 -4.5 3)
(variable-labels "yhat" "Residuals versus fitted values")
(plot estimates residuals)
(send my-plot :range 0 0 15)
(send my-plot :range 1 -4.5 3)
(mapcar (lambda(x) (let () (send x :plot-item) nil)) (select (agw) (iseq 2)))

(tile (select (agw) (iseq 4)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                      Bias in Regression Estimates, p. 117
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	b = (X^tX)^{-1}X^tY

and since

	E(Y) = X\beta,

	E(b) = \beta.

|#
(defun bias-matrix (x y)
  "Computes the matrix (X^tX)^{-1}X^tY for matrices X and Y"
  (matprod (inverse (xtx x)) (transpose x) y)
  )

;; Example 1, page 118:
(mprint
 (bias-matrix (matrix '(3 2) '(1 -1 1 0 1 1)) (matrix '(3 1) '(1 0 1))))

;; Example 2, page 119:
(mprint
 (bias-matrix
  (matrix '(7 2) (combine (weave (repeat 1 7) (iseq -3 3))))
  (matrix '(7 2) (combine (weave (square (iseq -3 3)) (cube (iseq -3 3)))))
  )
 )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise D, page 137
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise D, page 137")
(def x (* 10 (list 3 4 5 8 3 4 6 7 7 7 3 8 7 7)))
(def y (list 13 17 20 29 12 15 22 25 23 27 15 27 24 26))
(def exd (regression-model x y))
(send exd :plot)
(send exd :anova)
(send exd :coef-confidence-intervals)
(plot x (send exd :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise E, page 138
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise E, page 138")
(def x (list 1 1 2 3 4 4 4 5 6 6))
(def y (/ (list 42 38 30 23 18 20 22 20 25 27) 10))
(def exe (regression-model x y))
(send exe :plot)
(send exe :anova)
(send exe :coef-confidence-intervals)
(plot x (send exe :residuals))
;; Significant lack of fit.

(def exe (regression-model (list x (square x)) y))
(send exe :anova)
(send exe :coef-confidence-intervals)
(plot x (send exe :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise F, page 137
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise F, page 137")
(def x (list 0 2  2 5 5 9 9 9 9 10))
(def y (list -2 0 2 1 3 1 0 0 1 -1))
(def exf (regression-model x y))
(send exf :plot)
(send exf :anova)
(plot (send exf :estimates) (send exf :residuals))

(def exf (regression-model (list x (square x)) y))
(send exf :anova)
(plot x (send exf :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            Exercise G, page 139
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise G, page 139")
(load-calc drapedat)
(table1.1)
(def x x8)
(def y x1)
(def exg (regression-model x y))
(send exg :plot)
(send exg :anova)
(plot (send exg :estimates) (send exg :residuals))
(def exg (regression-model x y))
(def pf (send exg :joint-coef-probs-function :prob .9))
(spin-domain 12 15 -.11 -.04)
(spin-points 20)
(plot (lambda(x y) (funcall pf (list x y))))
(contour (lambda(x y) (funcall pf (list x y)))
	 :xmin 12 :xmax 15 :ymin -.11 :ymax -.04
	 :levels '(.8 .9 .95 1) :labels t)
	 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                Chapter 3: The Examination of Residuals
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	Some references for outliers: I'll just put them into my diss.bib
file. There are additional references on page 174. Also on that page are

	Recommendations: that I print out

1. Q_k
2. Cook's statistic, and
3. The second factor of the AP statistic.
|#


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Example, page 175
#|
	Are these "imputed values"?
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def x0 (repeat 1 16))
(def x1 (combine (weave (repeat -1 8) (repeat 1 8))))
(def x2 (list -1 -1 1 1))
(def x2 (combine x2 x2 x2 x2))
(def x3 (combine (repeat -1 4) (repeat 1 4)))
(def x3 (combine x3 x3))
(def x4 (combine (repeat -1 8) (repeat 1 8)))
(def x1x2 (* x1 x2))
(def x1x3 (* x1 x3))
(def x1x4 (* x1 x4))
(def x2x3 (* x2 x3))
(def x2x4 (* x2 x4))
(def x3x4 (* x3 x4))
(def y (list nan 45 18 64 28 48 nan 63 35 53 67 66 36 72 73 101))

(def mat (bind-columns x0 x1 x2 x3 x4 x1x2 x1x3 x1x4 x2x3 x2x4 x3x4))
(def X_2 (select-rows mat '(0 6)))
(def X_1 (select-rows mat (remove 0 (remove 6 (iseq 16) ))))
(size mat x_2 x_1)
(def xtxinv (inverse (xtx mat)))
(def R12 (matprod X_1 xtxinv (transpose X_2)))
(def R22 (matprod X_2 xtxinv (transpose X_2)))
(def R21 (matprod X_2 xtxinv (transpose X_1)))
(def Y_1 (remove-missing y :test #'not-a-numberp))

(def Y_2 (matprod (inverse (- (id 2) R22)) R21 Y_1))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Page 183, ex. A
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(load-calc drapedat)
(table1.1)
(def y x1)
#|
	I did the following to figure out that the regression was actually
variables x5 x6 and x8 against x1:

(def res (list x2 x3 x4 x5 x6 x7 x8 x9 x10))
(def residuals (list .12 .65 .72 -.33))
(def pure 1e31)
(dotimes
 (i (- (length res) 2))
 (setq x1 (elt res i))
 (dolist
  (j (iseq (1+ i) (- (length res) 2)))
  (setq x2 (elt res j))
  (dolist
   (k (iseq (1+ j) (- (length res) 1)))
   (setq x3 (elt res k))
   (def resid (send (regression-model (list x1 x2 x3) y :print nil) :residuals))
   (setq met (sum (square (- residuals (select resid (iseq 4))))))
   (if (< met pure)
       (let ()
	 (setq pure met)
	 (setq nums (list i j k))
	 )
     )
   )
  )
 )
(print nums)
(print pure)
|#
(def exa (regression-model (list x5 x6 x8) y))
(def residuals (send exa :residuals))
(def estimates (send exa :estimates))
(mapcar
 (lambda(x) (format t (if (< x 0) "~19t~,2f~%" "~20t~,2f~%") x))
 residuals
 )
(lines off)
(histo residuals)
(plot residuals)
(plot (butlast residuals) (rest residuals))
(plot (send exa :cooks-distances))
(plot estimates residuals)

(durbin-watson residuals)
(runs-test residuals)

(title "Small values are suspect!")
(plot (andrews-pregibon (list x5 x6 x8) y))

(new-plot)
(prob-plot residuals)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise B, p. 184
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def x (rseq 4 7 7))
(def y (list 33 42 45 51 53 61 62))
(def exb (regression-model x y))
(send exb :anova)
(send exb :plot)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise C, p. 185
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def x (rseq 3.5 8 10))
(def y (/ (list 244 321 371 404 433 514 619 661 772 792) 10))
(def exc (regression-model x y))
(send exc :anova)
(send exc :plot)
(def exc (regression-model x y :intercept nil))
(send exc :anova)
(send exc :plot)
(plot (send exc :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise D, p. 186
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def x (* 10 (list 4 5 6 7 8 9 4 6 8 5)))
(def y (list 69 175 272 335 490 415 72 265 492 180))
(def exd (regression-model x y))
(send exd :anova)
(send exd :plot)
(def residuals (send exd :residuals))
(def estimates (send exd :estimates))
(lines off)
(histo residuals)
(plot residuals)
(plot (butlast residuals) (rest residuals))
(plot (send exd :cooks-distances))
(plot estimates residuals)
(durbin-watson residuals)
(runs-test residuals)
(title "Small values are suspect!")
(plot (andrews-pregibon (list x) y))
(new-plot)
(prob-plot residuals)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise E, p. 186
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def x (* (list 28 25 30 32 31 28 32 30 32) 10))
(def y (/ (list 21 30 32 14 26 39 13 34 28) 10))
(def exe (regression-model x y))
(send exe :anova)
(send exe :plot)
(send exe :coef-confidence-intervals)

(def residuals (send exe :residuals))
(def estimates (send exe :estimates))
(mapcar
 (lambda(x y)
   (format t (if (< x 0) "~5t~,2f~19t~,2f~%" "~5t~,2f~20t~,2f~%") y x))
 residuals estimates
 )

(funcall (send exe :ci-function) 315)

;; Value:
(def value (funcall (send exe :regression-function) 315))
;; variance? See Draper and Smith, p. 83:
(def variance (send exe :estimation-variance (list 315)))
(list (- value (* 1.96 (sqrt variance))) (+ value (* 1.96 (sqrt variance))))
;; (1.6439675046218465 3.007739812451323)
;; Supposed to be .193 < yo < 4.453
(funcall (send exe :ci-function :obs 1) 315) ;;okay! Got it...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise F, p. 187
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; uses the results of exercise C:
(def c (list 6 6 8 7 9 8 5 9 11))
(lines off)
(plot c residuals)

(def x1 (/ (- x 300) 10))
(def x2 (- c 8))

(def exf (regression-model (list x1 x2) y))
(send exf :anova)
(send (regression-model (list x1) y) :anova)

(send exf :plot)
(def residuals (send exf :residuals))
(def estimates (send exf :estimates))
(plot x1 x2 residuals)

(mapcar
 (lambda(x y)
   (format t (if (< x 0) "~d~19t~,2f~%" "~d~20t~,2f~%") y x))
 residuals (iseq (length residuals))
 )

;; Value:
(funcall (send exf :regression-function) (list (/ (- 315 300) 10) (- 8 8)))
;; variance? See Draper and Smith, p. 83; created the following method:
(send exf :estimation-variance (list (/ (- 315 300) 10) (- 8 8)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise G, p. 187
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(load-calc drapedat)
(table1.1)
(def exg (regression-model (list x8 x6) x1))
(def residuals (send exg :residuals))
(def estimates (send exg :estimates))
(plot x1 x2 residuals)
(runs-test residuals)
(durbin-watson residuals)
(- 4 (durbin-watson residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise H, p. 187
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def x (list 1 2 2 3 3 3 4 6 6 7 8 10 10))
(def y (* (list 5 11 9 15 14 18 19 31 33 34 36 38 36) 10))
(def exh (regression-model x y))
(send exh :anova)
(send exh :plot)
(def residuals (send exh :residuals))
(def estimates (send exh :estimates))
(plot residuals)
(plot x residuals)
(plot y residuals)
(plot estimates residuals)
(runs-test residuals)
(durbin-watson residuals)

(def exh (regression-model (list x (square x)) y))
(send exh :anova)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise I, p. 188
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise I, page 188")
(def x (/ (list 30 47 83 93 99 110 123 125 126 159 167 188 188 189 217 219 228
		242 258 306 362 398 443  468 468 581 623 706 711 713 832 836
		995 1112) 10))
(def y (/ (list 971 979 982 971 957 961 956 972 889 961 982 975 942 932 908 970
		985 933 858 987 958 909 859 863 811 877 798 855 788 821 830 718
		642 658) 1000))
(def exi (regression-model x y))
(send exi :plot)
(send exi :anova)
(send exi :coef-confidence-intervals)
(def residuals (send exi :residuals))
(def estimates (send exi :estimates))
(plot residuals)
(plot x residuals)
(plot y residuals)
(plot (butlast residuals) (rest residuals))
(plot estimates residuals)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise K, p. 188
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise K, page 188")
(def y (list 968 43 588 640 4743 566 325 118 115 1545 1302 262 2207 1410 833
	     669 911 1037 196 616 766 2120 841 648 1289 249 450 215 158 1071
	     387 2745 1580 185 2096 785 575 1889 100 870 270 1059 3006 295 131
	     1050 730 467 1059 148))
(def x1 (list 158 11 91 92 952 109 167 30 35 298 203 41 544 254 150 136 147 146
	      46 157 255 403 189 85 234 38 89 23 37 329 54 744 226 38 530 137
	      108 570 46 122 40 177 515 57 20 208 160 88 207 22))
(def exk (regression-model x1 y))
(send exk :plot)
(send exk :anova)
(send exk :coef-confidence-intervals)

(def exk (regression-model (log x1) (log y)))
(send exk :plot)
(send exk :anova)
(send exk :coef-confidence-intervals)

(def residuals (send exk :residuals))
(def estimates (send exk :estimates))
(plot residuals)
(plot x1 residuals)
(plot y residuals)
(plot (butlast residuals) (rest residuals))
(plot estimates residuals)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                      Chapter 4: Two Predictor Variables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(load-calc drapedat)
(table1.1)
(def y x1)

(title "Page 194")
(def page194 (regression-model (list x8 x6) y))
(send page194 :plot-2d)

(title "Page 196")
(def page196 (regression-model (list x8) y))
(def yhat (send page196 :estimates))
(send page196 :plot)

(title "Figure 4.1")
(def page197 (regression-model (list x8) x6))
(def x6hat (send page197 :estimates))
(send page197 :plot)

(title "Figure 4.2")
(def page200 (regression-model (- x6 x6hat) (- y yhat)))
(def my-plot (send page200 :plot -10 10))
(send my-plot :range 1 -3 4)

(send page194 :anova)
(title "Figure 4.6")
(plot)
(def yhat (send page194 :estimates))
(send my-plot :add-plot-item
      (list 'points (iseq 1 (length y)) y 'black 'plus)
      (list 'lines (iseq 1 (length y)) y)
      (list 'points (iseq 1 (length y)) yhat 'black 'cross)
      (list 'lines (iseq 1 (length y)) yhat 'black 1 'dashed)
      )
(send my-plot :update-plot)

(send page196 :r-squared)
(send page194 :r-squared)

(sqrt (send page196 :s-squared))
(sqrt (send page194 :s-squared))

(send page196 :anova)
(send page194 :anova)

(send page194 :anova-sequential)
(send (regression-model (list x6 x8) x1) :anova-sequential)

(send page194 :se)
(funcall (send page194 :ci-function) (list 32 22))

(def estimates (send page194 :estimates))
(def residuals (send page194 :residuals))
(title "Figure 4.7, p. 212")
(plot estimates residuals)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise A, p. 212
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def x1 (list 1 4 9 11 3 8 5 10 2 7 6))
(def x2 (list 8 2 -8 -10 6 -6 0 -12 4 -2 -4))
(def y (list 6 8 1 0 5 3 2 -4 10 -3 5))
(def exa (regression-model (list x1 x2) y))
(send exa :anova)
(square (send exa :se))
(send exa :estimation-variance (list 3 5))
(funcall (send exa :ci-function) (list 3 5))

(send exa :anova-sequential)
(send (regression-model (list x1 x2) y) :anova-sequential)
(send (regression-model (list x2 x1) y) :anova-sequential)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise B, p. 213
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def x (/ (list 152 141 116 145 124 121 163 138 137 136 140 103) 100))
(def y (list 98 76 58 94 73 57 97 91 79 92 92 54))
(def z (list 77 139 179 95 142 186 82 100 125 96 99 190))
(def exb (regression-model (list y z) x))
(send exb :anova)
(send exb :anova-sequential)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise C, p. 214
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise C, p. 214")
(def y  (list 476 457 540 551 575 698 545 574 645 556 634 637 390 562 560))
(def x1 (list 111 92 90 107 98 150 118 110 117 94 130 118 91 118 109))
(def x2 (list 68 46 50 59 50 66 54 51 59 97 57 51 44 61 66))
(def exc (regression-model (list x1 x2) y))
(send exc :anova)
(send exc :anova-sequential)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise D, p. 214
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise D, p. 214")
(def y  (list 66 43 36 23 22 14 12 7.6))
(def x1 (list 38 41 34 35 31 34 29 32))
(def x2 (/ (list 475 213 365 180 295 142 210 100) 10))
(def exd (regression-model (list x1 x2) y))
(send exd :anova)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise E, p. 215
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise E, p. 215")
(def x1 (list 130 174 134 191 165 194 143 186 139 188 175 156 190 178 132 148))
(def x2 (list 190 176 205 210 230 192 220 235 240 230 200 218 220 210 208 225))
(def y  (/ (list 350 817 425 983 527 820 345 954 567 844 943 443 833 914 435
		 517) 10))
(def exe (regression-model (list x1 x2) y))
(send exe :anova)
(send exe :anova-sequential)
(send (regression-model (list x2 x1) y :print nil) :anova-sequential)
(sqrt (send exe :s-squared))
(/ (sqrt (send exe :s-squared)) (mean y))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise F, p. 215
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise F, p. 215")
(def x1 (list 17 19 19 20 20 20 20 21 21 21 21 25 25 27 28 30 30))
(def x2 (list 42 45 45 29 29 29 29 93 93 93 93 34 34 98 9 73 73))
(def y  (list 90 71 76 63 63 80 80 80 64 82 66 75 82 99 73 67 74))
(def exf (regression-model (list x1 x2) y))
(send exf :anova)
(send exf :anova-sequential)
(send (regression-model (list x2 x1) y :print nil) :anova-sequential)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise G, p. 216
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise G, p. 216")
(def x1 (/ (list 26 28 28 29 29 29 29 30 30 30 30 34 34 36 37 39 39) 10))
(def x2 (/ (list 39 42 42 26 26 26 26 90 90 90 90 31 31 95 06 70 70) 10))
(def y  (list 83 64 69 56 56 73 73 57 59 73 75 68 75 92 66 60 67))
(def exg (regression-model (list x1 x2) y))
(send exg :s-squared)

(send exg :anova)
(send exg :anova-sequential)
(send (regression-model (list x2 x1) y :print nil) :anova-sequential)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise H, p. 216
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise H, p. 216")
(def x (list 1 2 2 3 3 3 4 6 6 7 8 10 10))
(def y (* (list 5 11 9 15 14 18 19 31 33 34 36 38 36) 10))
(def z (list 0 0 0 0 0 0 1 0 0 1 3 6 6))
(def exh (regression-model (list x z) y))
(send exh :anova)
(send exh :plot-2d)
(send exh :hypothesis #2a((0 1)) :verbose t)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise I, p. 216
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise I, p. 216")
(def x1 (list -5 -4 -1 2 2 3 3))
(def x2 (list 5 4 1 -3 -2 -2 -3))
(def y (list 11 11 8 2 5 5 4))
(def exi (regression-model (list x1 x2) y :predictor-names (list "x1" "x2")))
(send exi :plot-2d)
(send exi :anova)
(send exi :anova-sequential)
(send exi :hypothesis #2a((0 1)) :verbose t)
(send (regression-model (list x2 x1) y :print nil
			:predictor-names (list "x2" "x1"))
      :anova-sequential)
(send exi :hypothesis #2a((1 0)) :verbose t)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                    Chapter 5: More Complicated Models
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;               5.1: Polynomial Models of various orders in the x_j
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;       5.2: Models involving transformations other than integer powers
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	Some references are provided to the literature of transformations.
Intrinsically and non-intrinsically Non-linear models.
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        5.3: Families of Transformation
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Page 225:
(defun w(lambda y) (if (= lambda 0) (ln y) (/ (- (^ y lambda) 1) lambda)))
(defun section5.3(x y &optional list)
  (let (
	(n (length y))
	results
	(list (if list list (rseq -2 2 20)))
	)
    (dolist
     (lambda list)
     (let ((resss (send (regression-model x (w lambda y) :print nil) :resss)))
       (setq
	results
	(append
	 results
	 (list
	  (- (* (1- lambda) (sum (ln y))) (* .5 n (ln (/ resss n))))
	  )
	 )
	)
       )
     )
    results
    )
  )
(def oil (combine (repeat 0 6) (repeat 10 6) (repeat 20 6) (repeat 30 5)))
(def phr (rseq 0 60 6))
(def phr (combine phr phr phr (rest phr)))
(def vis (list 26 38 50 76 108 157 17 26 37 53 83 124 13 20 27 37 57 87 15 22
	       27 41 63))
(def lambda
     (combine (rseq -1 -.2 5)
	      (list -.15 -.1 -.08 -.06 -.05 -.04 -.02)
	      (rseq 0 .1 3)
	      (rseq .2 1 5)
	      ))
(def table5.2 (section5.3 (list oil phr) vis lambda))
(pprint table5.2)	  
(lines on)
(title "Figure 5.1, p. 231")
(plot lambda table5.2)

;; Choose lambda=0: then
(def page230 (regression-model (list oil phr) (ln vis)))
(new-plot)
(lines off)
(send page230 :plot-2d)
(send page230 :anova)

;; Choose lambda=1 (i.e. effectively no transformation): then
(def page231 (regression-model (list oil phr) vis))
(new-plot)
(lines off)
(send page231 :plot-2d)
(send page231 :anova)

(digits 5)
(title "residuals of log transformation")
(plot oil phr (send page230 :residuals))

(defun example2(list1 list2 y lambda)
  (let* (
	 (w (w lambda y))
	 (reg1 (regression-model list1 w :print nil))

	 (w (send reg1 :residuals))
	 (reg2 (regression-model (smoosh (list list1 list2)) w :print nil))

	 (ms1 (/ (send reg1 :regss) (length list1)))
	 (ms2 (/ (send reg2 :regss) (length list2)))
	 )
    (list ms1 ms2 (/ ms2 ms1))
    )
  )

(defun table5.7(list1 list2 y)
  (let (
	(reg (regression-model (smoosh (list list1 list2)) (ln y)))
	result results
	)
    (dolist
     (lambda lambda)
     (setq result (example2 list1 list2 y lambda))
     (setq results (append results (list (third result))))
     (apply
      (lambda(x y z) (format t "~,4f ~15t~,4f ~30t~,4f ~%" x y z))
      result
      )
     )
    (send reg :anova)
    (send reg :anova-sequential :sets (list list1 list2))
    (setq reg (regression-model (smoosh (list list1 list2)) y))
    (send reg :anova-sequential :sets (list list1 list2))
    results
    )
  )
(def table5.7
     (table5.7 (list phr oil) (list (square phr) (* oil phr) (square oil)) vis)
     )
(new-plot)
(lines on)
(title "Figure 5.3, p. 233")
(plot lambda table5.7)

(def x (list 28 20 32 22 29 27 28 26 21 27 29 23 25))
(def y (/ (list 133 89 151 104 131 124 132 118 115 142 154 131 138) 10))
(def origin (combine (repeat 'g 4) (repeat 'v 4) (repeat 'w 5)))
(def z1 (indicator origin (where origin 'g)))
(def z2 (indicator origin (where origin 'v)))
(def page243 (regression-model (list x) y))

(new-plot)
(lines off)
(title "Figure 5.4, p. 244")
(plot (send page243 :residuals) (- z1 z2))

(def page243 (regression-model (list x z1 z2) y))
(lines on)
(plot)
(title "Figure 5.5, p. 245")
(def beta (send page243 :beta))
(send my-plot :add-plot-item
      (list 'function {(first beta) + (second beta)*x} 'black)
      (list 'points (select x (where origin 'g)) (select y (where origin 'g))
	    'black 'cross)
      (list 'function {(third beta)+(first beta)+(second beta)*x} 'blue)
      (list 'points (select x (where origin 'v)) (select y (where origin 'v))
	    'blue 'x)
      (list 'function {(fourth beta)+(first beta)+(second beta)*x} 'red)
      (list 'points (select x (where origin 'w)) (select y (where origin 'w))
	    'red 'disk)
      )
(send my-plot :update-plot)
(send page243 :anova-sequential :sets (list (list z1 z2) (list x)))

(def page248 (regression-model (list x z1 (* x z1) z2 (* x z2)) y))
;; p. 249:
(send page248 :hypothesis
      #2a((0 1 0 0 0)(0 0 1 0 0)(0 0 0 1 0)(0 0 0 0 1)) :verbose t)
(send page248 :hypothesis #2a((0 0 1 0 0)(0 0 0 0 1)) :verbose t)

;; page 250:
(def x (iseq 1 21))
(def y (/ (list 291 290 286 281 286 287 282 286 286 281 287 286 269 270 268 257
		259 256 251 252 251) 10))
(title "Table 5.14")
(send (regression-model x y) :plot)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise A, p. 280
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise A, p. 280")
(def x (iseq 1 9))
(def y (list 1 4 6 7 9.5 11 11.5 13 13.5))
(def x1 (list 1 2 3 4 5 5 5 5 5))
(def x2 (list 0 0 0 0 0 1 2 3 4))
(def x3 (list 0 0 0 0 1 1 1 1 1))
(def exa (regression-model (list x1 x2 x3) y))
(send exa :anova)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise B, p. 280
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise B, p. 280")
(def y (list 81 88 94 95 123))
(def x1 (list 1 2 4 7 8))
(def x2 (list -2 -7 3 1 -1))
(def x3 (list 4 11 5 13 17))
;; One variable will be aliased in the following:
(def exb (regression-model (list x1 x2 x3) y))
(send exb :anova)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise E, p. 281
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise E, p. 281")
(def x (iseq 1950 1964))
(def y (/ (list 64 60 56 73 92 104 113 93 99 111 133 152 160 147 133) 100))
(def exe (regression-model (list x) y))
(send exe :anova)
(send exe :plot)
(lines off)
(plot (send exe :estimates) (send exe :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Might be a fifth degree trend there, but I think that we can stop!
;; Orthogonal polynomials (up to constants)
;; (Note that these are also a function of n, the length of the vector.):
;;
(func phi1(x) x)
(func phi2(x) x^2 - (n^2-1)/12)
(func phi3(x) x^3 - (3*n^2-7)*x/20)
(func phi4(x) x^4 - (3*n^2-13)*x^2/14 + 3*(n^2-1)*(n^2-9)/560)
(func phi5(x) x^5 - 5*(n^2-7)*x^3/18 + (15*n^4- 230*n^2+407)*x/1008)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(def x (- x 1957))
(def n (length x))
(def exe (regression-model (list (phi1 x) (phi2 x) (phi3 x) (phi4 x) (phi5 x))
			   y))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise F, p. 281
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise F, p. 281")
(def x (iseq 1 20))
(def y (list 141 144 148 150 158 161 166 170 175 181 189 194 196 206 218 229
	     234 242 247 257))
(def x (- x (mean x)))
(def n (length x))
(def exf (regression-model (list x) y))
(send exf :anova)
(send exf :plot)
(lines off)
(plot (send exf :estimates) (send exf :residuals))

(def exf (regression-model (list x (square x)) y))
(send exf :anova)
(plot (send exf :estimates) (send exf :residuals))

(def exf (regression-model (list x (square x) (cube x)) y))
(send exf :anova)
(plot (send exf :estimates) (send exf :residuals))

(def exf (regression-model (list (phi1 x) (phi2 x) (phi3 x) (phi4 x) (phi5 x))
			   y))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise G, p. 282
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise G, p. 282")
(def operator (list 1 1 1 2 2 2 3 3 3))
(def speed (list 150 175 200 150 175 200 150 175 200))
(def y (list 255 246 249 260 223 231 265 247 256))
(def exg (regression-model (list operator speed) y))
(def i1 (indicator operator (where operator 1)))
(def i2 (indicator operator (where operator 2)))
(def exg (regression-model (list i1 i2 speed) y))

(def exg (regression-model (list speed) y))
(send exg :anova)

(def exg (regression-model (list i1 i2) y))
(send exg :anova)

(def exg (regression-model (list speed (square speed)) y))
(send exg :anova)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise H, p. 282
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise H, p. 282")
(def x (rseq 0 4.5 10))
(def y (- (/ (list -21 146 304 321 504 537 603 721 746 796) 100)))
(def x (- x (mean x)))
(def n (length x))
(def exh (regression-model (list (phi1 x) (phi2 x) (phi3 x)) y))

(def exh (regression-model (list (phi1 x) (phi2 x)) y))
(plot (send exh :estimates) (send exh :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise I, p. 283
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise I, p. 283")
(def x (combine (iseq 9) (* 2 (iseq 6)) (* 3 (iseq 4))))
(def y (list 221 245 260 268 282 289 300 304 314 219 261 285 303 315 331 228
	     273 298 318))
(def y (/ y 10))
(def page283 (regression-model (list x (square x)) y))
(send page283 :anova)
(send page283 :beta)

(def page283-linear (regression-model x y))
(send page283-linear :anova)
(send page283-linear :beta)
(plot (send page283-linear :estimates) (send page283-linear :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise J, p. 283
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise J, p. 283")
(def x (list 1830 1905 1930 1947 1952 1969))
(def y (list 30 130 400 760 1500 25000))
(def mean-x (mean x))
(def x (- x mean-x))
(plot x y)
(plot x (ln y))
(plot x (ln (ln y)))
(def exj (regression-model x (ln (ln y))))
(send exj :plot)
(send exj :anova)
;; When were we only able to go 0 mph?
(+ mean-x (first (secant (send exj :regression-function) 0 200)))
;; 1667.9
;; When will we be able to go the speed of light?
(+ mean-x
   (first 
    (secant
     (lambda(x)
       (- (funcall (send exj :regression-function) x) (ln (ln (* 3600 186000))))
       )
     0 200
     )))
;; 2093.6

(def exj (regression-model (list x (square x)) (ln (ln y))))
(send exj :plot-2d)
(send exj :anova)
(send exj :hypothesis #2a((0 1)) :verbose t)
;; When were we only able to go 0 mph?
(+ mean-x (first (secant (send exj :regression-function) 0 200)))
;; 1747.8
;; When will we be able to go the speed of light?
(+ mean-x
   (first 
    (secant
     (lambda(x)
       (- (funcall (send exj :regression-function) x) (ln (ln (* 3600 186000))))
       )
     0 200
     )))
;; 2057.9


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example K, p. 284
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example K, p. 284")
(def x (list -5 -3 -1 1 3 5))
(def y (list 13 4 3 4 10 22))
(def exk (regression-model (list x) y))
(send exk :anova)
(send exk :plot)

(def exk (regression-model (list x (square x)) y))
(send exk :anova)
(lines off)
(send exk :plot-2d)

(def exk (regression-model (list x (square x) (cube x)) y))
(send exk :anova)
(send exk :hypothesis #2a((0 1 1)) :verbose t)
(send exk :hypothesis #2a((0 0 1)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example M, p. 284
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example M, p. 284")
(def x1
     (list 47.1 72.9 47.1 60 60 72.9 47.1 72.9 60 72.9 47.1 60 60 60 39 60 60
	   60 81 60))
(def x2 (/ (list 339 339 81 210 210 81 81 339 210 81 339 210 210 210 210 0 210
		 420 210 210) 10))
(def x3 (/ (list 75 7500 7500 750 750 75 75 75 750 7500 7500 750 30000 30 750
		 750 750 750 750 750) 10))
(def y (/ (list 1197 863 880 1073 1069 1312 1258 1224 1064 909 846 1065 760
		1306 1051 1122 1067 1024 1074 1069) 100))
(def z (ln (1+ x3)))

(def exm
     (regression-model
      (list z (square z) x1 (* x1 z ) (* x1 (square z))
	    x2 (* x2 z ) (* x2 (square z))
	    )
      y))
(send exm :anova)

(def exm
     (regression-model
      (list z (square z) x1 (* x1 z ) (* x1 (square z))
	    x2 (* x2 z ) (* x2 (square z))
	    (* x1 x2) (* (* x1 x2) z ) (* (* x1 x2) (square z))
	    )
      y))

(send exm :anova-sequential
      :sets 
      (list
       (list z (square z) x1 (* x1 z ) (* x1 (square z))
	     x2 (* x2 z ) (* x2 (square z))
	     )
       (list (* x1 x2) (* (* x1 x2) z))
       )
      )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example N, p. 284
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Requires:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                  Section 5.8: Regression Analysis of Summary Data
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(title "Example N, p. 284")
(def x (iseq 16 23))
(def x (combine x x x))
(def wt (/ (list 10382 10539 10750 10312 10583 10750 10250 11750 9917 10767
		10357 11219 11000 11313 11200 11321 10083 9932 10083 9639 10167
		10000 nan 11250) 100))
(def groups (combine (repeat 'a 8) (repeat 'b 8) (repeat 'c 8)))
(def i1 (indicator groups (where groups 'a)))
(def i2 (indicator groups (where groups 'b)))

(def exn (regress (list x) wt :missing nan))
(send exn :plot)
(def exn (regress (list i1 i2 x) wt :missing nan))

(def students
     (list 19 19 16 8 6 6 1 1 12 29 28 32 22 8 4 7 18 33 24 18 6 4 nan 1))
;; In truth the following 0 sd's are missing:
(def sd (/ (list 1270 1250 1530 768 1675 577 0 0 920 1955 1305 1760 1204
		 1487 1118 728 1675 1450 1967 1329 1812 1521 nan 0) 100))
(def exn2 (send exn :summary-data students sd))
(send exn2 :anova)
(def xmat (send exn2 :x))
(send exn2 :anova-sequential
      :sets
      (list
       (list (column 1 xmat) (column 2 xmat))
       (list (column 3 xmat))
       ))
(send exn2 :anova-summary students sd)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example O, p. 286
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example O, p. 286")
(def h-w (/ (list 46 47 56 61 61 67 68 78 69 74 77 78 75 80 78 82 77 80 81 78
		  87 80 83 81 88 81 83 82 82 86 82 85 88 86 91 87 87 87 85 90
		  87 91 90 93 89 89 92 89 92 96 92 91 95 93 93 98 95 97 97 96
		  97 94 96 103 99 101 99 99 97 101 99 104) 100))
(def age (+ .5 (iseq 0 71)))
(def exo (regression-model (list age) h-w))
(send exo :anova)
(send exo :plot)

(def x1 (combine (iseq 1 7) (repeat 8 65)))
(def x2 (combine (repeat 0 7) (iseq 0 64)))
(def x3 (combine (repeat 0 7) (repeat 1 65)))
(def exo (regression-model (list x1 x2 x3) h-w))
(send exo :anova)

;; I prefer it this way: 
(def exo (regression-model (list x3 age (* x3 age)) h-w))
(send exo :anova)
(def beta (send exo :beta))
(func line1(x) (first beta) + (third beta)*x)
(func line2(x) (first beta) + (second beta) + ((third beta)+(fourth beta))*x)
(lines on)
(plot line1 line2 0 72)
(secant (lambda(x) (- (line1 x) (line2 x))) 0 72)

(def residuals (send exo :residuals))
(def estimates (send exo :estimates))
(histogram residuals)
(lines off)
(plot age residuals)
(plot estimates residuals)
(plot (rest residuals) (butlast residuals))
(durbin-watson residuals)
(runs-test residuals)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example P, p. 286
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example P, p. 286")
(def h-w (select h-w (iseq 32)))
(def age (select age (iseq 32)))
(def exp (regression-model (list age) h-w))
(send exp :anova)
(send exp :plot)

(def x1 (combine (iseq 1 7) (repeat 8 25)))
(def x2 (combine (repeat 0 7) (iseq 0 24)))
(def x3 (combine (repeat 0 7) (repeat 1 25)))
(def exp (regression-model (list x1 x2 x3) h-w))
(send exp :anova)

(def exp (regression-model (list x3 age (* x3 age)) h-w))
(send exp :anova)
(def beta (send exp :beta))
(func line1(x) (first beta) + (third beta)*x)
(func line2(x) (first beta) + (second beta) + ((third beta)+(fourth beta))*x)
(lines on)
(plot line1 line2 0 32)
(secant (lambda(x) (- (line1 x) (line2 x))) 0 32)

(def residuals (send exp :residuals))
(def estimates (send exp :estimates))
(histogram residuals)
(lines off)
(plot age residuals)
(plot estimates residuals)
(plot (rest residuals) (butlast residuals))
(durbin-watson residuals)
(runs-test residuals)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example R, p. 287
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example R, p. 287")
(def y (list 11 12 10 12 11 14 16 22 28))
(def n (size y))
(def x (iseq n))
;; We need the orthogonal polynomials defined above: phi1, etc.
(def exr (regression-model (list (phi1 x)) y))
(send exr :anova)
(send exr :plot)

;; Appears to be a quadratic trend in the residuals:
(def exr (regression-model (list (phi1 x) (phi2 x)) y))
(send exr :anova)
(send exr :plot-2d)

;; By third degree, the coefficients are no longer significant:
(def exr (regression-model (list (phi1 x) (phi2 x) (phi3 x)) y))
(send exr :anova-sequential)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example S, p. 287
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example S, p. 287")
(def oil (combine (repeat 0 6) (repeat 10 6) (repeat 20 6) (repeat 30 5)))
(def phr (rseq 0 60 6))
(def phr (combine phr phr phr (rest phr)))
(def vis (list 26 28 30 32 34 37 18 19 20 21 24 24 12 14 14 16 17 17   12 12 13
	       14 14))

;; Define functions w, example2, etc. from above, section 5.3.
(def lambda (rseq -3 2 40))
(def exs
     (funcall
      (lambda(list1 list2 y)
	(let (
	      (reg (regression-model (smoosh (list list1 list2)) (ln y)))
	      result results
	      )
	  (dolist
	   (lambda lambda)
	   (setq result (example2 list1 list2 y lambda))
	   (setq results (append results (list (third result))))
	   (apply
	    (lambda(x y z) (format t "~,4f ~15t~,4f ~30t~,4f ~%" x y z))
	    result
	    )
	   )
	  results
	  )
	)
      (list phr oil) (list (square phr) (* oil phr) (square oil)) vis
      )
     )
(setq exponent (first (select lambda (where exs (min exs)))))
(def exs (regression-model (list oil phr) (w exponent vis)))
(send exs :anova)
(send exs :plot-2d)

;; Second set of data:
(def oil (combine (repeat 0 6) (repeat 10 6) (repeat 20 6) (repeat 30 6)))
(def phr (rseq 0 60 6))
(def phr (combine phr phr phr phr))
(def vis (list 25 30 35 40 50 60 18 21 24 28 33 41 13 15 17 20 24 29 11 14 15
	       17 18 25))
(def exs
     (funcall
      (lambda(list1 list2 y)
	(let (
	      (reg (regression-model (smoosh (list list1 list2)) (ln y)))
	      result results
	      )
	  (dolist
	   (lambda lambda)
	   (setq result (example2 list1 list2 y lambda))
	   (setq results (append results (list (third result))))
	   (apply
	    (lambda(x y z) (format t "~,4f ~15t~,4f ~30t~,4f ~%" x y z))
	    result
	    )
	   )
	  results
	  )
	)
      (list phr oil) (list (square phr) (* oil phr) (square oil)) vis
      )
     )
(setq exponent (first (select lambda (where exs (min exs)))))
(def exs (regression-model (list oil phr) (w exponent vis)))
(send exs :anova)
(send exs :plot-2d)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example T, p. 288
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example T, p. 288")
(def x1 (list -3 -1 -1 -3 -3 -1 1 -1 1 -1 1))
(def x2 (list -3 -1 -1 -1 1 -3 -3 1 -1 -1 1 ))
(def x3 (list -3 -1 -1 -1 1 -1 1 -3 -3 1 -1))
(def x4 (list -3 -1 -1 1 -1 -1 1 1 -1 -3 -3))
(def y1 (list 51 61 64 36 39 55 50 88 124 54 133))
(def y2 (list 29 34 35 20 21 30 27 49 68 30 74))
(mprint (bind-columns x1 x2 x3 x4 y1 y2))
#|
;; wrong approach, as we can't do the regressions using all four variables.
(def ext
     (let* (
	    (list1 (list x1 x2 x3 x4))
;;	    (list2 (all-pairs (list x1 x2 x3 x4) :apply #'*))
	    (list2 (all-pairs (list x1 x2 x3) :apply #'*))
	    (y y1)
	    (reg (regression-model (smoosh (list list1 list2)) (ln y)))
	    result results
	    )
       (dolist
	(lambda lambda)
	(setq result (example2 list1 list2 y lambda))
	(setq results (append results (list (third result))))
	(apply
	 (lambda(x y z) (format t "~,4f ~15t~,4f ~30t~,4f ~%" x y z))
	 result
	 )
	)
       results
       )
     )
|#
(defun w(lambda y) (if (= lambda 0) (ln y) (/ (- (^ y lambda) 1) lambda)))
(defun section5.3(x y &optional list)
  (let (
	(n (length y))
	results
	(list (if list list (rseq -2 2 20)))
	)
    (dolist
     (lambda list)
     (let ((resss (send (regression-model x (w lambda y) :print nil) :resss)))
       (setq
	results
	(append
	 results
	 (list
	  (- (* (1- lambda) (sum (ln y))) (* .5 n (ln (/ resss n))))
	  )
	 )
	)
       )
     )
    results
    )
  )
(def lambda (combine -1 (rseq -.5 .5 11) 1))
(def ext
     (section5.3
      (list x1 x2 x3 x4)
      y1
      lambda))
(plot lambda ext)
(setq exponent (first (select lambda (where ext (max ext)))))
(def reg (regression-model (list x1 x2 x3 x4) (w exponent y1)))
(send reg :anova)
(send my-plot
      :add-plot-item
      (list
       'function
       (lambda(x) (- (max ext) (* .5 (CHISQ-QUANT .95 1))))))
;; I get (-.218, .216) for a 95% CI.



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example V, p. 289
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example V, p. 289")
(def x (list 50 60 60 70 70 80 80 90 90 90 95 100 100 100 105 105 110 110 110
	     115 115 115 120 120 120 125 130 130 135 135 140 140 145 150 150
	     155 155 160 160 160 165 170 180))
(def y (/ (list 114 82 115 141 172 335 288 156 244 335 388 477 580 361 477 655
		580 477 335 695 695 477 872 510 335 477 920 872 580 477 732 580
		477 1189 580 1439 872 1439 732 737 970 1123 1132) 10))
(setq x (ln x))
(def lambda (rseq -1 1 21))
(def exv (section5.3 (list x) y lambda))
(plot lambda exv)
(send my-plot
      :add-plot-item
      (list
       'function
       (lambda(x) (- (max exv) (* .5 (CHISQ-QUANT .95 1))))))
(send my-plot :update-plot)

(setq exponent (first (select lambda (where exv (max exv)))))
(def reg (regression-model (list x) (w exponent y)))
(send reg :anova)
(send reg :plot)

(def reg (regression-model (list x) (ln y)))
(send reg :anova)
(send reg :plot)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example W, p. 290
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example W, p. 290")
(def x (list 1520 1620 1660 1708))
(def y (list 1953 2135 2471 4727 6143 6314 1190 1286 1550 2125 2557 2845 651
	     837 848 1038 1361 1543 511 651 651 652 688 729))
(def x (combine (mapcar (lambda(x) (repeat x 6)) x)))
(lines off)
(plot x y)

(def exw (regression-model (/ 1000 (+ x 460)) (log y 10)))
(send exw :plot)
(plot x (send exw :residuals))
(send exw :anova)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example Z, p. 292
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example Z, p. 292")
(def x1 (list 10 100 1000 10000))
(def x2 (combine (mapcar (lambda(x) (repeat x 4)) x1)))
(def x1 (repeat x1 4))
(def y (list 2040 7350 12210 23580 18200 10 2960 108040 10370 1150 23580 296120
	     9040 1960 96980 1004020))
(def exz (regression-model (list (log x1 10) (log x2 10)) (log y 10)))
(send exz :plot-2d)
(send exz :anova)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example AA, p. 292
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example AA, p. 292")
(def x (/ (list 42 72 98 36 76 103 31 52 78 28 60 77 20 28 36 27 31 44) 10))
(def y (list 370 659 935 390 753 733 182 417 686 188 632 538 27 141 262 34 222
	     242))
(def n (repeat (list 0 50 270) 6))
(def wc (repeat (list 'w 'w 'w 'c 'c 'c) 3))
(def ts (combine (repeat 1 6) (repeat 2 6) (repeat 3 6)))
(def ind-wc (indicator wc (where wc 'c)))
(def ind-n1 (indicator wc (where n 50)))
(def ind-n2 (indicator wc (where n 270)))
(def ind-ts1 (indicator wc (where ts 2)))
(def ind-ts2 (indicator wc (where ts 3)))

(def exaa
     (regress (list x (square x) ind-wc ind-ts1 ind-ts2 ind-n1 ind-n2) y))
(send exaa :anova)

(def exaa (regress (list x (square x) n) y))
(send exaa :anova)

(def exaa (regress (list x (square x)) y))
(send exaa :anova)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                  Chapter 6: Selecting the "Best" Equation
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(lines off)
(load-calc drapedat)
(appendixb)
(def y x5)
(def vars (list x1 x2 x3 x4))

;; Avoid the empty set!
(def subsets (rest (all-subsets vars)))
(def sizes (mapcar #'size subsets))
(def subset-names
     (mapcar #'list-to-string (rest (all-subsets (list 'x1 'x2 'x3 'x4)))))
(point-labels subset-names)

;; Table, p. 297:
(def rsq
     (mapcar
      (lambda(x) (send (regress x y :print nil) :r-squared))
      subsets
      ))
(plot sizes rsq)

;; Table, p. 298:
(def ssq
      (mapcar
       (lambda(x) (send (regress x y :print nil) :s-squared))
       subsets
       ))
(plot sizes ssq)

;; Table, p. 298:
(def mallows (send (regress vars y :print nil) :mallows))
(point-labels (butlast subset-names))
(plot (butlast sizes) mallows)
(mprint (bind-columns (butlast subset-names) mallows))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 backward-step-wise
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(load-calc drapedat)
(appendixB)
(backward-step-wise (list x1 x2 x3 x4) x5)
(backward-step-wise (list x1 x2 x3 x4) x5 :alpha .1)
(backward-step-wise (list x1 x2 x3 x4) x5 :alpha .5)

 ;; When you'll take any old alpha, everyone gets into the act:
(backward-step-wise (list x1 x2 x3 x4) x5 :alpha 1)

 ;; you can turn off the intercept term:
(backward-step-wise (list x1 x2 x3 x4) x5 :intercept nil)
(backward-step-wise (list (repeat 1 (length x1)) x1 x2 x3 x4) x5
		    :intercept nil)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 step-wise
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(load-calc drapedat)
(appendixB)
(step-wise (list x1 x2 x3 x4) x5)
(step-wise (list x1 x2 x3 x4) x5 :alpha .1)
;; this is just a trial to show that I get the same thing using a vector of
;; ones and the :intercept nil option:
(step-wise (list (repeat 1 (length x1)) x1 x2 x3 x4) x5
	   :intercept nil :alpha .1)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Ridge-regression
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This reproduce results of pages 316-317, D&S:
(load-calc drapedat)
(appendixb)
(lines on)
(def xs (rseq 0 1 100))
(def ridges (ridge-regression (list x1 x2 x3 x4) x5 :ridge xs))
(title "Page 316")
(plot xs (mapcar #'first ridges))
(plot xs (mapcar #'second ridges))
(send my-plot :add-plot-item
   (list 'lines xs (mapcar #'third ridges) 'black)
   (list 'lines xs (mapcar #'fourth ridges) 'black)
   (list 'lines xs (mapcar #'fifth ridges) 'black)
   )

(def xs (rseq 0 .03 100))
(def ridges (ridge-regression (list x1 x2 x3 x4) x5 :ridge xs))
(title "Page 318")
(plot xs (mapcar #'second ridges))
(send my-plot :add-plot-item
   (list 'lines xs (mapcar #'third ridges) 'black)
   (list 'lines xs (mapcar #'fourth ridges) 'black)
   (list 'lines xs (mapcar #'fifth ridges) 'black)
   )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                     PRESS, Section 6.8, p. 325
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Hald data from Appendix B of Draper and Smith:
(load-calc drapedat)
(appendixb)
(title "Plot of results in Table 6.2, p. 325")
(def result
  (mapcar
    (lambda(x) (sum (press x x5)))
    (rest (all-subsets (list x1 x2 x3 x4)))
    )
  )
(mprint (bind-columns (rest (all-subsets '(x1 x2 x3 x4))) result))
(plot result)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 pca-regression
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(load-calc drapedat)
(appendixB)
 (def reg (pca-regression (list x1 x2 x3 x4) x5))

 ;; You can catch the eigenvectors, eigenvalues, and scores, since they are
 ;; returned using the values statement. I use cmv to catch them to a list,
 ;; then parcel out the list values:
 (def stuff (cmv (pca-regression (list x1 x2 x3 x4) x5)))
 (def reg (first stuff))
 (def eigvecs (first (second stuff)))
 (def scores (second (second stuff)))
 (def eigvals (third (second stuff)))
 (send reg :anova-sequential)

 ;; Using all the vectors (i.e. percent 100):
 (def reg (pca-regression (list x1 x2 x3 x4) x5 :percent 100))
 (def z-times-sqrt-lambda (send reg :x))
 (mprint (matmult z-times-sqrt-lambda (diagonal (/ (sqrt eigvals)))))
 ;; equals the scores:
 (mprint scores)

 ;; Reproduces the results of p. 331 (note that W_2 in 6.9.10 should be W_3):
 (def reg (pca-regression (list x1 x2 x3 x4) x5
 		:percent 100
		:use-variables '(0)))
 (def reg (pca-regression (list x1 x2 x3 x4) x5
 		:percent 100
		:use-variables '(0 2)))
 (def reg (pca-regression (list x1 x2 x3 x4) x5
 		:percent 100
		:use-variables '(0 1 2)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                     Chapter 7: Two Specific Problems
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The following is the data for the first problem, from Table 7.1, p. 381:
(def unit (iseq 1 24))
(def temp (repeat 0 24))
(setf (elt temp 0) -75)
(setf (elt temp 1) 175)
(setf (elt temp 9) -75)
(setf (elt temp 10) 175)
(setf (elt temp 13) -75)
(setf (elt temp 14) 175)
(def vibrations (list 0 0 -75 175 -75 175 0 0 0 -75 175 -75 175 -75 175 -75
                      175 0 0 -75 175 -75 175 0))
(def drop (combine (repeat 0 6) -65 165 0 0 0 -65 165 -65 165 0 0 -65 165
		   -65 165 -65 165 0))
(def static (list -65 150 150 -65 150 -65 150 -65 150 150 -65 150 -65 150
		  -65 150 -65 -65 150 -65 150 150 -65 -65))
(def pressure (/ (list 14 263 265 58 234 74 294 97 329 264 84 288 118 284 115
		       265 58 13 214 4 229 264 114 37) 10))
(mvsq '(x1 x2 x3 x4 y) (list temp vibrations drop static pressure))

;; Identify the duplicates in the four predictors:
(def dups
     (find-duplicates (row-list (bind-columns x1 x2 x3 x4)) :test #'equalp))

(print "Table 7.1, p. 381")
(mprint (bind-columns x1 x2 x3 x4 y))

(print "Correlations, p. 383")
(mprint (correlation (bind-columns x1 x2 x3 x4 y)))

(def reg (regress (list x4) y))
(send reg :anova :dups dups)
(def residuals (send reg :residuals))
(print "Table 7.3, p. 384")
(mprint (bind-columns y (send reg :estimates) residuals))

(lines off)
(title "Figure 7.1, p. 386")
(plot x1 residuals)
(title "Figure 7.2, p. 385")
(plot x2 residuals)
(title "Figure 7.3, p. 385")
(plot x3 residuals)
(title "Figure 7.4, p. 386")
(plot (* x4 x3) residuals)
 
(send (regress (list x4 (* x4 x3)) y) :anova :dups dups)
 
(def reg (regress (list x4 (* x4 x3) x2) y))
(send reg :anova :dups dups)
(send reg :coef-confidence-intervals)
(def estimates (send reg :estimates))
(def residuals (send reg :residuals))
(def s-squared (send reg :s-squared))
(mprint (bind-columns unit y estimates residuals (/ residuals (sqrt
							       s-squared))))
 
;; Example 2: p. 390.
(def x1 (combine -1 1 -1 1 -1 1 -1 1 -5/3 5/3 (repeat 0 10)))
(def x2 (combine -1 -1 1 1 -1 -1 1 1 0 0 -5/3 5/3 (repeat 0 8)))
(def x3 (combine -1 -1 -1 -1  1 1 1 1 0 0 0 0 -5/3 5/3 (repeat 0 6)))
(def y (/ (list 528 679 554 642 751 816 738 795 681 912 806 775 368 780 746 759
                769 723 759 798) 10))
(def y4 (/ (list 695 590 710 708 564 518 684 567 600 567 552 647 717 536 648
                 591 639 599  586 596) 100))
(def dups
     (find-duplicates (row-list (bind-columns x1 x2 x3)) :test #'equalp))

(def
 reg
 (regress
  (list x1 x2 x3 (* x1 x1) (* x2 x2) (* x3 x3) (* x1 x2) (* x1 x3) (* x2 x3))
  y)
 )
(send reg :anova :dups dups)
(send reg :anova-sequential :sets
      (list (list x1 x2 x3)
            (list (* x1 x1) (* x2 x2) (* x3 x3) (* x1 x2) (* x1 x3) (* x2 x3))
            ))
(send reg :hypothesis
      [[0 1 0 0 0 0 0 0 0]
       [0 0 0 0 1 0 0 0 0]
       [0 0 0 0 0 0 1 0 0]
       [0 0 0 0 0 0 0 0 1]
       ]
       :verbose t)

(def reg (regress (list x1 x3 (* x1 x1) (* x3 x3) (* x1 x3)) y))
(send reg :anova :dups dups)
(send reg :anova-sequential :sets
      (list (list x1 x3) (list (* x1 x1) (* x3 x3) (* x1 x3))))

(def f (send reg :regression-function))
(spin-domain -2 2 -2 2)
(title "Figure 7.5, p. 399")
(contour (lambda(x y) (funcall f (list x y (* x x) (* y y) (* x y))))
	 :xmin -2 :xmax 2
	 :ymin -2 :ymax 3
	 :levels (list 70 75 80 85 90)
	 )

(print "Table 7.11, p. 400")
(mprint (bind-columns x1 x3 y (send reg :estimates) (send reg :residuals)))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                  Chapter 9: analysis of variance problems
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(title "Table 9.1, p. 425")
(def c0 (list 242 245 244 248 247 248 242 244 246 242))
(def c100 (list 248 246 245 247 248 250 247 246 243 244))
(def c200 (list 246 248 250 252 248 250 246 248 245 250))

(one-way-anova (list c0 c100 c200) :compare '(1 0 -1))
(one-way-anova (list c0 c100 c200) :compare '(1 -2 1))
 
(send 
 (regress (combine (repeat -1 10) (repeat 0 10) (repeat 1 10))
	  (combine c0 c100 c200))
 :anova)
(send 
 (regress (combine (repeat 1 10) (repeat -2 10) (repeat 1 10))
	  (combine c0 c100 c200))
 :anova)

(send 
 (regress 
  (list
   (combine (repeat -1 10) (repeat 0 10) (repeat 1 10))
   (combine (repeat 1 10) (repeat -2 10) (repeat 1 10))
   )
  (combine c0 c100 c200))
  :anova)

(title "Table 9.3, p. 439")
(def x (combine (repeat 'A 6) (repeat 'B 6) (repeat 'C 6) (repeat 'D 6)))
(def y (list 1 1 2 2 3 3))
(def y (repeat y 4))
(def z (list 4 6 11 7 5 9 6 4 13 15 9 7 13 15 15 9 13 13 12 12 12 14 7 9))
(def reg (two-way-regression (list x y z)))

(title "Example 9.10, p. 451")
(def x (combine (repeat 1 12) (repeat 2 12)))
(def y (list 1 1 1 1 2 2 2 2 3 3 3 3))
(def y (repeat y 2))
(def z (list 17 21 49 54 64 48 34 63 62 72 61 91
	     33 37 40 16 41 64 34 64 56 62 57 72))
(def reg (two-way-regression (list x y z)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example A, p. 454
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example A, part a, p. 454")
(def x (list 1 1 2 2 3 3))
(def x (repeat x 3))
(def y (combine (repeat 10 6) (repeat 20 6) (repeat 30 6)))
(def z (/ (list 452 460 400 390 359 341 418 206 278 190 225 177 235 331 446 522
		427 486) 10))
(def reg (two-way-regression (list x y z)))
(mprint (send reg :matrix-estimate))

(def indicators (send reg :x))
;; Their indicators are not my indicators:
(def x1 (- (- 1 (column 3 indicators)) (* 2 (column 4 indicators))))
(def x2 (- (column 3 indicators) (column 4 indicators)))
(def x3 (- (- 1 (column 1 indicators)) (* 2 (column 2 indicators))))
(def x4 (- (column 1 indicators) (column 2 indicators)))
(def indicators (bind-columns x1 x2 x3 x4))
(def newreg (regress
	     (bind-columns
	      indicators
	      (* (column 1 indicators) (column 3 indicators))
	      (* (column 1 indicators) (column 4 indicators))
	      (* (column 2 indicators) (column 3 indicators))
	      (* (column 2 indicators) (column 4 indicators))
	      )
	     z))
(send newreg :anova-sequential
      :pure-error t
      :sets (list (list x1 x2) (list x3 x4)
		  (list
		   (* (column 1 indicators) (column 3 indicators))
		   (* (column 1 indicators) (column 4 indicators))
		   (* (column 2 indicators) (column 3 indicators))
		   (* (column 2 indicators) (column 4 indicators))
		   )))

(title "Example A, part b, p. 454")
(def x (list 1 1 2 2 3 3))
(def x (repeat x 3))
(def y (combine (repeat 10 6) (repeat 20 6) (repeat 30 6)))
(def z (list 4 -3 3 2 -1 -5 3 10 2 2 0 0 -1 -10 1 2 7 9))
(def reg (two-way-regression (list x y z)))
(mprint (send reg :matrix-estimate))

(def indicators (send reg :x))
;; Their indicators are not my indicators:
(def x1 (- (- 1 (column 3 indicators)) (* 2 (column 4 indicators))))
(def x2 (- (column 3 indicators) (column 4 indicators)))
(def x3 (- (- 1 (column 1 indicators)) (* 2 (column 2 indicators))))
(def x4 (- (column 1 indicators) (column 2 indicators)))
(def indicators (bind-columns x1 x2 x3 x4))
(def newreg (regress
	     (bind-columns
	      indicators
	      (* (column 1 indicators) (column 3 indicators))
	      (* (column 1 indicators) (column 4 indicators))
	      (* (column 2 indicators) (column 3 indicators))
	      (* (column 2 indicators) (column 4 indicators))
	      )
	     z))
(send newreg :anova-sequential
      :pure-error t
      :sets (list (list x1 x2) (list x3 x4)
		  (list
		   (* (column 1 indicators) (column 3 indicators))
		   (* (column 1 indicators) (column 4 indicators))
		   (* (column 2 indicators) (column 3 indicators))
		   (* (column 2 indicators) (column 4 indicators))
		   )))

(mprint (matrix '(3 3) (mapcar #'mean (split-list z 2))))
(plot (send reg :matrix-estimate))
(plot (matrix '(3 3) (mapcar #'mean (split-list z 2))))
(plot (- (send reg :matrix-estimate)
	 (matrix '(3 3) (mapcar #'mean (split-list z 2)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example D, p. 455
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example D, p. 456")
(one-way-anova
 (- 
  (list
   (list 1690 1580 1745 1685)
   (list 1550 1445 1645 1545)
   (list 1625 1450 1510)
   (list 1725 1550 1430 1445)
   (list 1530 1545 1565 1520)
   )
  1430
  ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;             Chapter 10: An Introducton to Non-linear Estimation
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Table 10.2")
(def xs (* 2 (iseq 4 21)))
(def ys '((49 49)(48 47 48 47)(46 46 45 43)(45 43 43)(44 43 43)(46 45)
	  (42 42 43)(41 41 40)
	  (42 40 40)(41 40 41)(41 40)(40 40 38)(41 40)(40)(41 38)(40 40)(39)(39))
     )
(def xs
     (combine
      (mapcar {(repeat (first x) (second x))} (weave xs (mapcar #'length ys)))
      )
     )
(def ys (/ (combine ys) 100))
(minimize-sum-of-squares
 ys
 (lambda(alpha beta) (+ alpha (* (- .49 alpha) (exp (- (* beta (- xs 8)))))))
 :guess (list .3 .02)
 )

(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta))
	      (beta (second theta)))
	  (+ alpha (* (- .49 alpha) (exp (- (* beta (- xs 8))))))))
      ys
      (list .3 .02)
      )
     )
(def theta (send nreg :beta))
(lines on)
(plot (lambda(xs)
	(let ((alpha (first theta))
	      (beta (second theta)))
	  (+ alpha (* (- .49 alpha) (exp (- (* beta (- xs 8))))))))
      (min xs) (max xs)
      )
(send my-plot :add-plot-item (list 'points xs ys))

(spin-domain 0 .2 .3 .9)
(contour
 (lambda(beta alpha)
   (sum (square (- ys (+ alpha (* (- .49 alpha) (exp (- (* beta (- xs 8)))))))
		 )))
 :levels '(.0055 .007 .026 .113 .225 .377 .640 1.27 2.84 4.49 5.26)
 :labels nil
 )
(send nreg :anova)
(def pf (send nreg :joint-coef-probs-function))
(spin-points 20)
(title "Figure 10.8")
(spin-domain .06 .16 .36 .41)
(contour (lambda(y x) (funcall pf (list x y))) :levels '(.75 .95 .99 .995))

;; Singular values (p. 485):
(def xtx (send nreg :xtx))
(def svd (svd xtx))
(sqrt (second svd))
(mprint (first svd))


;; See p. 487, D&S for se: we agree!
(sqrt (* (send nreg :ss) (diagonal (inverse xtx))))
(diagonal (sqrt (* (send nreg :ss) (send nreg :xtxinv))))
(def se (send nreg :se))
(mprint
 (apply #'bind-rows
	(weave (- theta (* 2 se))
	       (+ theta (* 2 se)))
	)
 )
(mapcar pf (list
	    (list (first theta) (second (- theta (* 2 se))))
	    (list (first theta) (second (+ theta (* 2 se))))
	    (list (first (- theta (* 2 se))) (second theta))
	    (list (first (+ theta (* 2 se))) (second theta))
	    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Exercise A, p. 517
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise A, p. 517")
(def x (list 1 4 16))
(def y (list .8 .45 .04))
(minimize-sum-of-squares y #'(lambda(theta) (exp (- (* theta x)))))
(def nreg (nreg-model (lambda(theta) (exp (- (* (first theta) x)))) y (list .1)))
(def pf (send nreg :joint-coef-probs-function :prob .9))
(lines on)
(plot (lambda(x) (funcall pf (list x))) 0.1 0.3)

(secant (lambda(x) (- .95 (funcall pf (list x)))) .2 0.25 :verbose nil)
(secant (lambda(x) (- .95 (funcall pf (list x)))) .17 0.18 :verbose nil)
(def theta (send nreg :beta))
(def se (send nreg :se))
(list (- theta (* 2 se)) (+ theta (* 2 se)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Exercise B, p. 517
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise B, p. 517")
(def x (list .5 1 2 4 8 16))
(def x (combine (weave x x)))
(def y (/ (list 96 91 86 79 63 62 48 42 17 21 3 5) 100))
(def nreg
     (nreg-model (lambda(theta) (exp (- (* (first theta) x)))) y (list .1)))
(def pf (send nreg :joint-coef-probs-function :prob .9))
(lines on)
(plot (lambda(x) (funcall pf (list x))) 0.17 0.24)

(secant (lambda(x) (- .95 (funcall pf (list x)))) .22 0.23 :verbose nil)
(secant (lambda(x) (- .95 (funcall pf (list x)))) .18 0.2 :verbose nil)
(def theta (send nreg :beta))
(def se (send nreg :se))
(list (- theta (* 2 se)) (+ theta (* 2 se)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise C, p. 518
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise C, p. 518")
(def xs (* 10 (iseq 1 4)))
(def ys (/ (list 48 42 40 39) 100))
(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta))
	      (beta (second theta)))
	  (+ alpha (* (- .49 alpha) (exp (- (* beta (- xs 8))))))))
      ys
      (list .3 .02)
      )
     )
(def theta (send nreg :beta))
(def se (send nreg :se))
(plot (lambda(xs)
	(let ((alpha (first theta))
	      (beta (second theta)))
	  (+ alpha (* (- .49 alpha) (exp (- (* beta (- xs 8))))))))
      (min xs) (max xs)
      )
(send my-plot :add-plot-item (list 'points xs ys))

(def intervals
     (apply #'bind-rows
	    (weave (- theta (* 2 se))
		   (+ theta (* 2 se)))
	    )
     )
(apply #'spin-domain (combine intervals))
(contour
 (lambda(alpha beta)
   (sum (square (- ys (+ alpha (* (- .49 alpha) (exp (- (* beta (- xs 8)))))))
		 )))
 :levels '(.0055 .007 .026 .113 .225 .377 .640 1.27 2.84 4.49 5.26)
 :labels nil
 )
(def pf (send nreg :joint-coef-probs-function))
(spin-points 20)
(contour (lambda(x y) (funcall pf (list x y))) :levels '(.75 .95 .99 .995))
(mprint intervals)
(mapcar pf (list
	    (list (first theta) (second (- theta (* 2 se))))
	    (list (first theta) (second (+ theta (* 2 se))))
	    (list (first (- theta (* 2 se))) (second theta))
	    (list (first (+ theta (* 2 se))) (second theta))
	    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise D, p. 518
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise D, p. 518")
(def xs (iseq 5))
(def ys (/ (list 444 546 638 657 689) 10))
(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta))
	      (beta (second theta))
	      (gamma (third theta))
	      )
	  (- alpha (* beta (exp (* xs (ln gamma)))))
	  ))
      ys
      (list 44.4 0 .5)
      )
     )
(def theta (send nreg :beta))
(def se (send nreg :se))
(plot (lambda(xs)
	(let ((alpha (first theta))
	      (beta (second theta))
	      (gamma (third theta))
	      )
	  (- alpha (* beta (exp (* xs (ln gamma)))))
	  ))
      (min xs) (max xs)
      )
(send my-plot :add-plot-item (list 'points xs ys))

(def intervals
     (apply #'bind-rows
	    (weave (- theta (* 2 se))
		   (+ theta (* 2 se)))
	    )
     )
(mprint intervals)
(apply #'spin-domain (combine (select-rows intervals '(0 1))))
(contour
 (lambda(alpha beta)
   (sum (square (- ys (- alpha (* beta (exp (* xs (ln (third theta))))))))))
 :levels 10
 :labels nil
 )
(def pf (send nreg :joint-coef-probs-function))
(spin-points 20)
(contour (lambda(x y) (funcall pf (list x y (third theta))))
	 :levels '(.75 .95 .99 .995))
(mapcar pf (list
	    (list (first theta) (second (- theta (* 2 se))) (third theta)) 
	    (list (first theta) (second (+ theta (* 2 se))) (third theta))
	    (list (first (- theta (* 2 se))) (second theta) (third theta))
	    (list (first (+ theta (* 2 se))) (second theta) (third theta))
	    ))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise E, p. 518
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise E, p. 518")
(def xs (list 0 10 20 30 40 50 60 70 80 85 90 95 100 105))
(def ys (/ (list 414 852 1631 3218 6462 9876 15113 22474 34135 42336 52278
		 67432 78204 92001) 100))
(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta))
	      (beta (second theta))
	      (gamma (third theta))
	      )
	  (* alpha (^ 10 (/ (* beta xs) (+ gamma xs))))
	  ))
      ys
      (list 4.14 9 300)
      )
     )
(def theta (send nreg :beta))
(def se (send nreg :se))
(plot (lambda(xs)
	(let ((alpha (first theta))
	      (beta (second theta))
	      (gamma (third theta))
	      )
	  (* alpha (^ 10 (/ (* beta xs) (+ gamma xs))))
	  ))
      (min xs) (max xs)
      )
(send my-plot :add-plot-item (list 'points xs ys))

(def intervals
     (apply #'bind-rows
	    (weave (- theta (* 2 se))
		   (+ theta (* 2 se)))
	    )
     )
(mprint intervals)
(apply #'spin-domain (combine (select-rows intervals '(0 1))))
(contour
 (lambda(alpha beta)
   (sum (square (- ys (* alpha (^ 10 (/ (* beta xs) (+ (third theta) xs)))))))
   )
 :levels 10
 :labels nil
 )
(def pf (send nreg :joint-coef-probs-function))
(spin-points 20)
(contour (lambda(x y) (funcall pf (list x y (third theta))))
	 :levels '(.75 .95 .99 .995))
(mapcar pf (list
	    (list (first theta) (second (- theta (* 2 se))) (third theta)) 
	    (list (first theta) (second (+ theta (* 2 se))) (third theta))
	    (list (first (- theta (* 2 se))) (second theta) (third theta))
	    (list (first (+ theta (* 2 se))) (second theta) (third theta))
	    ))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise G, p. 519
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise G, p. 519")
(def xs (list 40 80 160 320 640))
(def ys (/ (list 4902 5853 6737 7592 8375) 10))
(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta))
	      (beta (second theta))
	      (gamma (third theta))
	      )
	  (* alpha (ln (+ (* beta xs) gamma)))
	  ))
      ys
      (list (/ (first ys) (ln 40)) 1 0)
      )
     )
(send nreg :resss)
(def theta (send nreg :beta))
(def se (send nreg :se))
(plot (lambda(xs)
	(let ((alpha (first theta))
	      (beta (second theta))
	      (gamma (third theta))
	      )
	  (* alpha (ln (+ (* beta xs) gamma)))
	  ))
      (min xs) (max xs)
      )
(send my-plot :add-plot-item (list 'points xs ys))

(def intervals
     (apply #'bind-rows
	    (weave (- theta (* 2 se))
		   (+ theta (* 2 se)))
	    )
     )
(mprint intervals)
(apply #'spin-domain (combine (select-rows intervals '(0 1))))

(contour
 (lambda(alpha beta)
   (sum (square (- ys (* alpha (ln (+ (* beta xs) (third theta)))))))
   )
 :levels 10
 :labels nil
 )
(def pf (send nreg :joint-coef-probs-function))
(spin-points 20)
(contour (lambda(x y) (funcall pf (list x y (third theta))))
	 :levels '(.75 .95 .99 .995))
(mapcar pf (list
	    (list (first theta) (second (- theta (* 2 se))) (third theta)) 
	    (list (first theta) (second (+ theta (* 2 se))) (third theta))
	    (list (first (- theta (* 2 se))) (second theta) (third theta))
	    (list (first (+ theta (* 2 se))) (second theta) (third theta))
	    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise H, p. 519
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise H, p. 519")
(def x1 (* 10 (list 12 6 6 12 12 6 6 3 1.5 6 4.51 9 15 6 6 6 3 9 15 6 6 6 6 6 6 3
	      4.51 4 3 4.5 1.5 3 9 2.5 6 6 3 3)))
(def x2 (combine 600 600 (repeat 612 4) (repeat 620 19) (repeat 631 5)
		 (repeat 639 8)))
(def ys (/ (list 900 949 886 785 791 890 787 877 938 782 827 696 582 795 800
		 790 883 712 576 802 802 804 794 804 799 764 688 717 802 695
		 808 655 309 689 437 425 638 659) 1000))
(mprint (bind-columns x1 x2 ys))
(def reg (regress (- (- (/ x2) 1/620)) (- (ln (- (ln ys))) (ln x1))))
(def beta (send reg :beta))
(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta))
	      (beta (second theta))
	      )
	  (+ (ln alpha) (ln x1) (- (* beta (- (/ x2) 1/620))))
	  ))
      (ln (- (ln ys)))
      (list (exp (first beta)) (second beta))
      )
     )
(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta))
	      (beta (second theta))
	      )
	  (exp (- (* alpha x1 (exp (- (* beta (* (- (/ x2) 1/620) 10000)))))))
	  ))
      ys
      (list (exp (first beta)) (/ (second beta) 10000))
      :epsilon 1e-10
      )
     )
(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta))
	      (beta (second theta))
	      )
	  (exp (- (* alpha x1 (exp (- (* beta (- (/ x2) 1/620)))))))
	  ))
      ys
      (list (exp (first beta)) (second beta))
      :epsilon 1e-10
      )
     )
(send nreg :resss)
(def theta (send nreg :beta))
(def se (send nreg :se))
(spin-domain (min x1) (max x1)(min x2) (max x2))
(plot (lambda(x1 x2)
	(let ((alpha (first theta))
	      (beta (second theta))
	      )
	  (exp (- (* alpha x1 (exp (- (* beta (- (/ x2) 1/620)))))))
	  ))
      )
(send my-plot :add-plot-item (list 'points x1 x2 ys :color 'blue))

(def intervals
     (apply #'bind-rows
	    (weave (- theta (* 2 se))
		   (+ theta (* 2 se)))
	    )
     )
(mprint intervals)
(apply #'spin-domain (combine intervals))
(contour
 (lambda(alpha beta)
   (sum (square (- ys (exp (- (* alpha x1 (exp (- (* beta (- (/ x2) 1/620))))))))))
   )
 :levels 10
 :labels nil
 )
(def pf (send nreg :joint-coef-probs-function))
(spin-points 20)
(contour (lambda(x y) (funcall pf (list x y)))
	 :levels '(.75 .95 .99 .995))
(mapcar pf (list
	    (list (first theta) (second (- theta (* 2 se))))
	    (list (first theta) (second (+ theta (* 2 se))))
	    (list (first (- theta (* 2 se))) (second theta))
	    (list (first (+ theta (* 2 se))) (second theta))
	    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise I, p. 520
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise I, p. 520")
(def x1 (list 109 65 1180 66 1270 69 1230 68))
(def x2 (combine 600 640 600 640 600 640 600 640))
(def ys (/ (list 912 382 397 376 342 358 348 376) 1000))
(mprint (bind-columns x1 x2 ys))
(def reg (regress (- (- (/ x2) 1/620)) (- (ln (- (ln ys))) (ln x1))))
(def beta (send reg :beta))
(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta))
	      (beta (second theta))
	      )
	  (exp (- (* alpha x1 (exp (- (* beta (* (- (/ x2) 1/620) 10000)))))))
	  ))
      ys
      (list (exp (first beta)) (/ (second beta) 10000))
      :epsilon 1e-10
      )
     )
#|
(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta))
	      (beta (second theta))
	      )
	  (exp (- (* alpha x1 (exp (- (* beta (- (/ x2) 1/620)))))))
	  ))
      ys
      (list (exp (first beta)) (second beta))
      :epsilon 1e-10
      )
     )
|#
(send nreg :resss)
(def theta (send nreg :beta))
(def se (send nreg :se))
(spin-domain (min x1) (max x1)(min x2) (max x2))
(plot (lambda(x1 x2)
	(let ((alpha (first theta))
	      (beta (second theta))
	      )
	  (exp (- (* alpha x1 (exp (- (* beta (- (/ x2) 1/620)))))))
	  ))
      )
(send my-plot :add-plot-item (list 'points x1 x2 ys :color 'blue))

(def intervals
     (apply #'bind-rows
	    (weave (- theta (* 2 se))
		   (+ theta (* 2 se)))
	    )
     )
(mprint intervals)
(apply #'spin-domain (combine intervals))
(contour
 (lambda(alpha beta)
   (sum (square (- ys (exp (- (* alpha x1 (exp (- (* beta (- (/ x2) 1/620))))))))))
   )
 :levels 10
 :labels nil
 )
(def pf (send nreg :joint-coef-probs-function))
(spin-points 20)
(contour (lambda(x y) (funcall pf (list x y)))
	 :levels '(.75 .95 .99 .995))
(mapcar pf (list
	    (list (first theta) (second (- theta (* 2 se))))
	    (list (first theta) (second (+ theta (* 2 se))))
	    (list (first (- theta (* 2 se))) (second theta))
	    (list (first (+ theta (* 2 se))) (second theta))
	    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                    Compare linear and non-linear SE's:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise F, page 60")
(def x (/ (list 47 50  52 52 59 47 59 52 53 59 56 50) 10))
(def y (list 3 3 4 5 10 2 9 3  7  6 6 4))
(def lreg (regress x y))
(def nreg (nreg-model 
	   (lambda(theta)
	     (let ((alpha (first theta))
		   (beta (second theta))
		   )
	       (+ alpha (* beta x))))
	   y
	   (send lreg :beta)
	   )
     )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise J, p. 521
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise J, p. 521")
(def x1 (/ (list 10 20 10 20 1 30 2 30 3 30 30 2 30) 10))
(def x2 (/ (list 10 10 20 20 0 0 0 0 0 8 0 0 8) 10))
(def ys (/ (list 126 219 76 126 186 606 268 614 318 298 509 247 319) 1000))
(mprint (bind-columns x1 x2 ys))
(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta))
	      (beta (second theta))
	      (gamma (third theta))
	      )
	  (/ (* alpha gamma x1) (+ 1 (* alpha x1) (* beta x2)))
	  ))
      ys
      (list 2.9 12.2 .69)
      )
     )
(send nreg :resss)
(def theta (send nreg :beta))
(def se (send nreg :se))
(spin-domain (min x1) (max x1) (min x2) (max x2))
(plot (lambda(x1 x2)
	(let ((alpha (first theta))
	      (beta (second theta))
	      (gamma (third theta))
	      )
	  (/ (* alpha gamma x1) (+ 1 (* alpha x1) (* beta x2)))
	  ))
      )
(send my-plot :add-plot-item (list 'points x1 x2 ys :color 'blue))

(def intervals
     (apply #'bind-rows
	    (weave (- theta (* 2 se))
		   (+ theta (* 2 se)))
	    )
     )
(mprint intervals)
(apply #'spin-domain (combine (select-rows intervals '(0 1))))
(contour
 (lambda(alpha beta)
   (sum
    (square (- ys (/ (* alpha (third theta) x1) (+ 1 (* alpha x1) (* beta x2))))))
   )
 :levels 10
 :labels nil
 )
(def pf (send nreg :joint-coef-probs-function))
(spin-points 20)
(contour (lambda(x y) (funcall pf (list x y (third theta))))
	 :levels '(.75 .95 .99 .995))
(mapcar pf (list
	    (list (first theta) (second (- theta (* 2 se))) (third theta)) 
	    (list (first theta) (second (+ theta (* 2 se))) (third theta))
	    (list (first (- theta (* 2 se))) (second theta) (third theta))
	    (list (first (+ theta (* 2 se))) (second theta) (third theta))
	    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Exercise K, p. 521
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Exercise K, p. 521")
(def xs (list 4 10 17 22 25))
(def ys (list 5 20 45 66 85))
(def lreg (regress (ln xs) (ln ys)))
(def beta (send lreg :beta))
(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta))
	      (beta (second theta))
	      )
	  (* alpha (^ xs beta))
	  ))
      ys
      (list (exp (first beta)) (second beta))
      )
     )
(def theta (send nreg :beta))
(def se (send nreg :se))
(plot (lambda(xs)
	(let ((alpha (first theta))
	      (beta (second theta))
	      )
	  (* alpha (^ xs beta))
	  ))
      (min xs) (max xs)
      )
(send my-plot :add-plot-item (list 'points xs ys))

(def intervals
     (apply #'bind-rows
	    (weave (- theta (* 2 se))
		   (+ theta (* 2 se)))
	    )
     )
(mprint intervals)
(apply #'spin-domain (combine (select-rows intervals '(0 1))))
(contour
 (lambda(alpha beta)
   (sum (square (- ys (* alpha (^ xs beta)))))
   )
 :levels 10
 :labels nil
 )
(def pf (send nreg :joint-coef-probs-function))
(spin-points 20)
(contour (lambda(x y) (funcall pf (list x y)))
	 :levels '(.75 .95 .99 .995))
(mapcar pf (list
	    (list (first theta) (second (- theta (* 2 se)))) 
	    (list (first theta) (second (+ theta (* 2 se))))
	    (list (first (- theta (* 2 se))) (second theta))
	    (list (first (+ theta (* 2 se))) (second theta))
	    ))

(plot (lambda(x) (- .95 (funcall pf (list x (second theta)))))
      (aref intervals 0 0) (aref intervals 0 1)
      )
(bisection (lambda(x) (- .95 (funcall pf (list x (second theta)))))
	   (aref intervals 0 0) 
	   (first theta)
	   :eps 1e-6
	   )
(bisection (lambda(x) (- .95 (funcall pf (list x (second theta)))))
	   (aref intervals 0 1) 
	   (first theta)
	   :eps 1e-6
	   )
(bisection (lambda(x) (- .95 (funcall pf (list (first theta) x))))
	   (aref intervals 1 0) 
	   (second theta)
	   :eps 1e-6
	   )
(bisection (lambda(x) (- .95 (funcall pf (list (first theta) x))))
	   (aref intervals 1 1) 
	   (second theta)
	   :eps 1e-6
	   )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example L, p. 522
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example L, p. 522")
(def x
     (list
      (iseq 1 7)
      (list 1 2 3 4 5 7 9 11)
      (list 1 2 3 5 6 7)
      (list 1 2 3 4 5 7)
      (list 1 2 3 4 5 7)
      (list 1 2 3 4 5 7)
      (list 1 2 3 5 7 10)
      (list 1 2 3 4 5 7)
      (list 1 2 3 4)
      )
     )
(def y
     (list
      (list 82 112 153 163 176 192 200)
      (/ (list 47 74 117 142 160 184 219 217) 100)
      (list 168 336 468 660 708 696)
      (list 9 9 16 20 21 22)
      (/ (list 43 82 95 104 121 131) 10)
      (/ (list 68 127 148 154 170 199) 10)
      (list 109 149 149 191 213 224)
      (/ (list 83 103 190 160 156 198) 10)
      (list 4710 7080 8460 9580)
      )
     )

(def results
     (mapcar
      (lambda(x y)
	(let ((nreg
	       (nreg-model
		(lambda(theta)
		  (let ((alpha (first theta)) (beta (second theta)))
		    (* alpha (- 1 (exp (- (* x beta)))))
		    )
		  )
		y
		(list (mean y) 0)
		)
	       )
	      )
	  (combine (send nreg :beta) (send nreg :resss) (send nreg :s-theta))
	  ))
      x y)
     )
(mprint (apply #'bind-rows results))
     

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example M, p. 524
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example M, p. 524")
(def x (list 50 60 60 70 70 80 80 90 90 90 95 100 100 100 105 105 110 110 110
	     115 115 115 120 120 120 125 130 130 135 135 140 140 145 150 150
	     155 155 160 160 160 165 170 180))
(def y (/ (list 114 82 115 141 172 335 288 156 244 335 388 477 580 361 477 655
		580 477 335 695 695 477 872 510 335 477 920 872 580 477 732 580
		477 1189 580 1439 872 1439 732 737 970 1123 1132) 10))
(def exm (regress (ln x) (ln y)))
(def beta (send exm :beta))
(def nreg
     (nreg-model
      (lambda(theta)
	(let ((alpha (first theta)) (beta (second theta))) (* alpha (^ x beta)))
	)
      y
      (list (exp (first beta)) (second beta))
      )
     )

(send exm :anova)
(send exm :plot)

(lines off)
(plot (send exm :residuals) (send nreg :residuals))

;; I wanted to compare residuals in some meaningful manner, so I exponentiate
;; the residuals of the ln/ln variables, to get y/y^*, and divide the residuals
;; y-y^* in the un-transformed regression by y^* and add 1 to get "the same
;; thing". Then I throw the plot of y=x in there, to see how well we've done:
(plot
 (exp (send exm :residuals))
 (1+ (/ (send nreg :residuals) (send nreg :estimates)))
 )
(send my-plot :add-plot-item (list 'function {x} 'black 'lines))
(send my-plot :update-plot)

(plot x (send exm :residuals))
(plot x (send nreg :residuals))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example N, p. 524
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example N, p. 524")
(def x (list 118 484 664 1004 1231 1372 1582))
(def x (combine (mapcar {repeat x 5} x)))
(def y (list 30 33 30 32 30 58 69 51 62 49 87 111 75 112 81 115 156 108 167 125
	     120 172 115 179 142 142 203 139 209 174 145 203 140 214 177))
(def lambdas
     (list
      (lambda(theta) ;; 10.7.2, p. 505
	(let ((alpha (first theta)) (beta (second theta)) (gamma (third theta)))
	  (* alpha (- 1 (* beta (exp (- (* x gamma))))))))
      (lambda(theta) ;; 10.7.5, p. 508
	(let ((alpha (first theta)) (beta (second theta)) (gamma (third theta)))
	  (/ alpha (+ 1 (* beta (exp (- (* x gamma))))))))
;      (lambda(theta) ;; 10.7.8, p. 509
;	(let ((alpha (first theta)) (beta (second theta)) (gamma (third theta)))
;	  (- alpha (ln (+ 1 (* beta (exp (- (* x gamma)))))))))
      (lambda(theta) ;; 10.7.10, p. 511
	(let ((alpha (first theta)) (beta (second theta)) (gamma (third theta)))
	  (* alpha (exp (- (* beta (exp (- (* x gamma)))))))))
      ))	
(def initials
     (list 
      (list 250 1 .001)
      (list 150 5 .001)
      ;; (list 5 5 .001)
      (list 150 4 .001)
      ))
(def initials
     (list 
      (list 268.3 .9478 .000474)
      (list 154.1 5.643 .002759)
      ;; (list 5.032 5.792 .002814)
      (list 172.2 2.813 .001626)
      ))
(def nregs
     (mapcar
      (lambda(f i) (nreg-model f y i :count-limit 20))
      lambdas
      initials
      )
     )
(mapcar (lambda(nreg) (send nreg :plot '(x))) nregs)
