(princ "
;; farouki: skins the farouki matroid
(try-out farouki)

;; surface-curvature-analysis: computes the curvature of a function of two
;; variables at a given point.
(try-out surface-curvature-analysis)

;; surface-curvature-analysis-matroid: curvature in the case not of a function,
;; but rather of a matroid interpolator, using approximate derivatives.
(try-out surface-curvature-analysis-matroid)
")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun farouki-data()
  "
;; Description: loads in the farouki-data.
;; Examples:
 (farouki-data)
"
  (def
   farouki
   (matrix
    '(6 3)
    (split-list
     '(
       -3.0 -1.2  0.0
	    -3.0  0.0  0.1
	    -2.7  1.3 -0.1
	    -2.0 -1.0 -0.2
	    -1.7  0.2  0.3
	    -1.5  1.6  0.0
	    -1.1 -1.5  1.0
	    -0.6 -0.1  1.1
	    -0.8  1.2  0.3
	    -0.4 -0.8 -0.3
	    0.0  0.1  0.1
	    0.2  1.0  0.5
	    0.8 -1.0  0.3
	    1.5 -0.3  0.4
	    2.2  1.5  0.6
	    2.3 -1.7  0.2
	    3.0 -0.1  0.3
	    3.3  1.2  0.5)
     3))))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                               surface-curvature-analysis 
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun surface-curvature-analysis(f a b)
  "
;; Description: (surface-curvature-analysis f a b)

;; Notes: Not sure where this definition came from: need to ask Dad. Added
;; realpart to alleviate problem in curvature of sphere.

;; Examples:
 (new-plot)
 (spin-points 10)
 (func f(u v) u*v)
 (surface-curvature-analysis #'f 0 0)
 (title \"Curvature for u*v\")
 (plot f (lambda(x y) (first (surface-curvature-analysis #'f x y))))

 (func f(x y) (sqrt (4  -  x^2 - y^2)))
 (title \"A sphere has constant curvature\")
 (spin-domain -1 1 -1 1)
 (plot f (lambda(x y) (first (surface-curvature-analysis #'f x y))))
"
  (let* (
	 (newvar (second (get-lambda-expression f)))
	 (x (first newvar))
	 (y (second newvar))
	 
	 (fx (lambda-deriv f x))
	 (fy (lambda-deriv f y))
	 (fxx (lambda-deriv fx x))
	 (fyy (lambda-deriv fy y))
	 (fxy (lambda-deriv fx y))

 	 (ru (list 1 0 (funcall fx a b)))
	 (rv (list 0 1 (funcall fy a b)))
	 (ruu (list 0 0 (funcall fxx a b)))
	 (ruv (list 0 0 (funcall fxy a b)))
	 (rvv (list 0 0 (funcall fyy a b)))

	 (e (dot ru ru))
	 (f (dot ru rv))
	 (g (dot rv rv))

	 (n (normalize (cross ru rv)))
	 (l (dot n ruu))
	 (m (dot n ruv))
	 (nn (dot n rvv))

	 (k (/ (- (* l nn) (* m m))
	       (- (* e g) (* f f))
	       ))
	 (h (/ (- (* 2 f m) (+ (* e nn) (* g l)))
	       (* 2 (- (* e g) (* f f)))
	       ))

	 (sqrt (realpart (sqrt (- (* h h) k))))
	 (kmax (+ h sqrt))
	 (kmin (- h sqrt))
	 )
    (list kmax kmin)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                farouki
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun farouki(matroid
	       &key
	       (fu #'hermite-spline-function)
	       (fv #'hermite-spline-function)
	       )
  "
	(farouki matroid
	       &key
	       (fu #'hermite-spline-function)
	       (fv #'hermite-spline-function)
	       )

;; Examples:
 (farouki-data)
 (def stuff (farouki farouki))
 (funcall stuff 0 0)
 (def hmm 
      (outer-product 
           (iseq (first (size farouki)))
           (iseq (second (size farouki)))
           #'(lambda(u v) (funcall stuff u v))
           )
     )
 (max (abs (- hmm farouki))) ;; tiny difference
 (def hmm (matroid-to-matrices hmm))
 (def xvals (first hmm))
 (def yvals (second hmm))
 (def zvals (third hmm))
 (matroid xvals yvals zvals)

 ;; Now on a little grander scale:
 (def hmm 
      (outer-product 
           (rseq 0 (1- (first (size farouki))) 18)
           (rseq 0 (1- (second (size farouki))) 18)
           #'(lambda(u v) (funcall stuff u v))
           )
     )
 (def hmm (matroid-to-matrices hmm))
 (def my-plot (matroid (first hmm) (second hmm) (third hmm)))
 ;; Now tack in the original points:
 (send my-plot :add-slot 'plot-item)
 (send my-plot :add-points
               (list
                 (matrix-to-list xvals)
                 (matrix-to-list yvals)
                 (matrix-to-list zvals)
                 )
               :color 'yellow)
"
  (let* (
	 (matrices (matroid-to-matrices matroid))

	 (x (first matrices))
	 (y (second matrices))
	 (z (third matrices))

	 (size (size x))
	 (xs (iseq (first size)))
	 (ys (iseq (second size)))

	 (svd-x (svd x))
	 (svd-y (svd y))
	 (svd-z (svd z))

	 (lambda-inv 
	  (mapcar
	   (lambda(x)
	     (let* (
		    (x (coerce x 'vector))
		    (result (aref x 0))
		    (total (sum x))
		    (sum (/ result total))
		    lambda
		    )
	       (do
		((i 1 (1+ i)))
		((> sum (- 1 1e-6)))
		(setq lambda (aref x i))
		(setq sum (+ sum (/ lambda total)))
		(setf result (combine result lambda))
		)
	       result
	       ))
	   (mapcar #'second (list svd-x svd-y svd-z))
	   )
	  )

	 (gxs
	  (mapcar (lambda(x) (funcall fu xs x))
		  (select (column-list (first svd-x))
			  (iseq (length (first lambda-inv))))))

	 (gys
	  (mapcar (lambda(x) (funcall fu xs x))
		  (select (column-list (first svd-y))
			  (iseq (length (second lambda-inv))))))
	 (gzs
	  (mapcar (lambda(x) (funcall fu xs x))
		  (select (column-list (first svd-z))
			  (iseq (length (third lambda-inv))))))


	 (gfns (list
		(lambda(u) (mapcar (lambda(x) (funcall x u)) gxs))
		(lambda(u) (mapcar (lambda(x) (funcall x u)) gys))
		(lambda(u) (mapcar (lambda(x) (funcall x u)) gzs))
		))	   


	 (hxs
	  (mapcar (lambda(x) (funcall fv ys x))
		  (select (column-list (third svd-x))
			  (iseq (length (first lambda-inv))))))
	 (hys
	  (mapcar (lambda(x) (funcall fv ys x))
		  (select (column-list (third svd-y))
			  (iseq (length (second lambda-inv))))))
	 (hzs
	  (mapcar (lambda(x) (funcall fv ys x))
		  (select (column-list (third svd-z))
			  (iseq (length (third lambda-inv))))))
	 (hfns (list
		(lambda(u) (mapcar (lambda(x) (funcall x u)) hxs))
		(lambda(u) (mapcar (lambda(x) (funcall x u)) hys))
		(lambda(u) (mapcar (lambda(x) (funcall x u)) hzs))
		))	   
	 )

    ;; I've defined:
    ;; lambda-inv - a list of three lists of the singular value inverses.
    ;; gfns - a list of three list functions 
    ;; hfns - a list of three list function

    ;; So what I'm going to do is create a function which evaluates them
    ;; appropriately, and returns a three-list of (x,y,z) coordinates.
    (lambda(u v)
      (mapcar 
       (lambda(x y z) (sum (* (funcall x u) y (funcall z v))))
       gfns lambda-inv hfns
       )
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                     surface-curvature-analysis-matroid
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun surface-curvature-analysis-matroid(matroid)
  "
;; Description: (surface-curvature-analysis-matroid matroid)

	Computes the surface-curvature-analysis for the case where we use
approximate derivatives (calculated using hermite by default) to estimate the
derivatives.

;; Examples:
 (farouki-data)
 (def stuff (surface-curvature-analysis-matroid farouki))
 (def results
   (outer-product
     (rseq 0 (1- (first (size farouki))) 18)
     (rseq 0 (1- (second (size farouki))) 18)
     (lambda(x y) (funcall stuff x y))
     )
   )

 (new-plot)
 (spin-points 18)
 (title \"Mean Curvature\")
 (def mean-curvature (inside-nth results :apply #'mean))
 (plot mean-curvature)
 (title \"Gaussian Curvature\")
 (def gauss-curvature (inside-nth results :apply #'(lambda (x) (apply #'* x))))
 (plot gauss-curvature)

 ;; Here's a little trial to plot farouki x and y coordinates against the
 ;; z-coordinates of curvature:
 (title \"(far-x,far-y,gauss-curve)\")
 (def far (farouki farouki))
 (def hmm 
      (outer-product 
           (rseq 0 (1- (first (size farouki))) 18)
           (rseq 0 (1- (second (size farouki))) 18)
           #'(lambda(u v) (funcall far u v))
           )
     )
 (def hmm (matroid-to-matrices hmm))
 (def my-plot (matroid (first hmm) (second hmm) gauss-curvature))
"
  (let (
	(ruf (farouki matroid :fu #'hermite-spline-derivative))
	(rvf (farouki matroid :fv #'hermite-spline-derivative))
	(ruuf (farouki matroid :fu #'hermite-spline-second-derivative))
	(ruvf (farouki matroid
		       :fu #'hermite-spline-derivative
		       :fv #'hermite-spline-derivative))
	(rvvf (farouki matroid :fv #'hermite-spline-second-derivative))
	)
    (lambda(a b)
      (let* (
	     (ru (funcall ruf a b))
	     (rv (funcall rvf a b))
	     (ruu (funcall ruuf a b))
	     (ruv (funcall ruvf a b))
	     (rvv (funcall rvvf a b))

	     (e (dot ru ru))
	     (f (dot ru rv))
	     (g (dot rv rv))

	     (n (normalize (cross ru rv)))
	     (l (dot n ruu))
	     (m (dot n ruv))
	     (nn (dot n rvv))

	     (k (/ (- (* l nn) (* m m))
		   (- (* e g) (* f f))
		   ))
	     (h (/ (- (* 2 f m) (+ (* e nn) (* g l)))
		   (* 2 (- (* e g) (* f f)))
		   ))

	     (kmax (+ h (sqrt (- (* h h) k))))
	     (kmin (- h (sqrt (- (* h h) k))))
	     )
	(list kmax kmin)
	)
      )
    )
  )
