(princ "
	Some functions to help with Formal operations:

(try-out formal-op)
(try-out formal-matrix-outer-product)
(try-out formal-matrix-product)
(try-out formal-matrix-product-pair)
(try-out formal-matrix-sum)
(try-out formal-matrix-sum-pair)
")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          formal-op
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun formal-op(op a b)
  "
;; Description: (formal-op op a b)

	Creates a formal operation using infix notation.

;; Examples:
 (formal-op '- 'x 'y)
 (apply #'formal-op '(/ x y))
 (apply #'formal-op '(/ x (- 1 y)))
"
  `(,a ,op ,b))
;;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                       formal-matrix-outer-product
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun formal-matrix-outer-product(a b)
  "
;; Examples:
 (formal-matrix-outer-product '(1 x y) '(2 y))
 (mprint
   (formal-matrix-outer-product #2a((x x x)(1 2 3)(y y y)) '(2 x y)))
"
  (outer-product
   a b
   #'(lambda(x y) (simplifier (formal-op '* x y)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          formal-matrix-product
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun formal-matrix-product(&rest matrices)
  "
;; Generates a formal matrix product of the matrices given as arguments. They
;; MUST be matrices (not as forgiving as matmult), and the dimensions must be
;; correct.

;; Examples:
 (mprint
   (formal-matrix-product #2a((x x x)(1 2 3)(y y y)) #2a((2)(x)(y))))
 (mprint
   (formal-matrix-product #2a((x y)(1 z)) #2a((2)(y))))
"
  (let* (
	 (xsize (size (first matrices)))
	 (m (first xsize))
	 (n (second xsize))
	 (v (first matrices))
	 )
    (dolist
     (i (rest matrices))
     (setq v (formal-matrix-product-pair v i))
     )
    v
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          formal-matrix-product-pair
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun formal-matrix-product-pair
  (x y
     &optional
     (f #'(lambda(x y) (simplifier (formal-op '* x y)))))
  "
;; Examples:
 (mprint
   (formal-matrix-product-pair #2a((x x x)(1 2 3)(y y y)) #2a((2)(x)(y))))
 (mprint
   (formal-matrix-product-pair #2a((x y)(1 z)) #2a((2)(y))))
"
  (let* (
	 (xsize (size x))
	 (ysize (size y))
	 (m (first xsize))
	 (n (second ysize))
	 (x (if (matrixp x) (column-list x) (list x)))
	 (y (if (matrixp y) (row-list y) (list y)))
	 v
	 )
    (apply
     #'formal-matrix-sum
     (mapcar #'(lambda(x y) (formal-matrix-outer-product x y)) x y)
     )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          formal-matrix-sum
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun formal-matrix-sum(&rest matrices)
  "
;; Examples:
 (mprint
   (formal-matrix-sum #2a((x x x)(1 2 3)(y y y)) (id 3)))
 (mprint
   (formal-matrix-sum #2a((x x x)(1 2 3)(y y y)) (id 3) (id 3)))
"
  (let* (
	 (xsize (size (first matrices)))
	 (m (first xsize))
	 (n (second xsize))
	 (v (first matrices))
	 )
    (dolist
     (i (rest matrices))
     (setq v (formal-matrix-sum-pair v i))
     )
    v
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         formal-matrix-sum-pair
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun formal-matrix-sum-pair
  (x y
     &optional (f #'(lambda(x y) (simplifier (formal-op '+ x y)))))
  "
;; Examples:
 (mprint
   (formal-matrix-sum-pair #2a((x x x)(1 2 3)(y y y)) (id 3)))
"
  (let* (
	 (xsize (size x))
	 (ysize (size y))
	 (error (if (not (equalp xsize ysize))
		    (error
		     "In formal-matrix-sum-pair:different matrices sizes.")))
	 (m (first xsize))
	 (n (second xsize))
	 (v (make-array (list m n)))
	 )
    (dotimes
     (i m)
     (dotimes
      (j n)
      (setf (aref v i j) (funcall f (aref x i j) (aref y i j)))))
    v)
  )

