;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Part I
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
From majordom@stat.umn.edu Wed Oct 15 11:50 CDT 1997
From: "Steven D. Majewski" <sdm7g@Virginia.EDU>
To: Isabelle REGINSTER <isabelle.reginster@fundp.ac.be>

Re: Hashtables: 

Hashtables are a data structure that is like an array but is 
accessable by an arbitrary non-integer key. ( Other languages,
like Icon, Python, Perl, etc. have these by other names --
associative arrays or dictionaries -- and the syntax is more
array like:   dict["key"] = value, for example. ) 

The main functions you need are #'make-hash-table (to create one),
#'gethash (which can be used with setf to set the key,value pairs),
#'maphash (to do something to all of the key,value pairs), and maybe
#'hash-table-count (The number of entries -- #'hash-table-size is
the size allocated for the table.) 

|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                just to generate something to hash:
;;
;; s is strings from A-Z, a-z,  c is char-codes of those strings:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setf c (append (iseq (char-code #\A) (char-code #\Z))
		(iseq (char-code #\a) (char-code #\z))))
(setf s (mapcar #'string (mapcar #'code-char c)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        make a hash table:
;;
;; the default :test is #'eq which is fine for symbols, which are unique
;; but not for string values. Use #'equal or #'string= 
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setf tab1 (make-hash-table :test #'string=))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          fill the hashtable:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(mapcar #'(lambda (key value) (setf (gethash key tab1) value)) s c)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                trial
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(gethash "E" tab1)
;; 69
;; T
(gethash "3" tab1)
;; NIL
;; NIL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                maphash
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(maphash #'(lambda (key val) (format T "~S : ~S~%" key val)) tab1)
#|
"H" : 72
"G" : 71
"F" : 70
"E" : 69
"D" : 68
"C" : 67
"B" : 66
"A" : 65
"z" : 122
"y" : 121
"x" : 120
"w" : 119
"v" : 118
"u" : 117

... 

Note that you typically can't predict the order entries come out
of the table. And since #'maphash returns nil and does not collect
it's results like #'mapcar, to iterate over the table by sorted 
keys you need to do something like:

 (let ((lis ()))
    (maphash #'(lambda (key value) (setf lis (cons key lis))) tab1)
    (sort-data lis))

to get a sorted keylist to feed to #'gethash. 

Try: (apropos "hash") and help on those names for more info, or
look at one of the on-line lisp references. 

If you're a bigger to both Lisp and LispStat, you might want to
get a good basic Lisp book -- CLtL (Common Lisp the Language)
or the ANSI reference make poor tutorials. I like Paul Graham's
books -- look at the Lisp FAQs for others. 

(I ought to put some online references here, but I don't have
  them at hand right now. Some are reachable from ucla's site.
  I'll followup later with some URLs, unless someone else does
  it first.) 

---|  Steven D. Majewski   (804-982-0831)  <sdm7g@Virginia.EDU>  |---
---|  Department of Molecular Physiology and Biological Physics  |---
---|  University of Virginia             Health Sciences Center  |---
---|  P.O. Box 10011            Charlottesville, VA  22906-0011  |---
All power corrupts and obsolete power corrupts obsoletely." - Ted Nelson
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Part II
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
From majordom@stat.umn.edu Fri Oct 17 16:32 CDT 1997
To: stat-lisp-news@umnstat.stat.umn.edu
From: "Dom. Early" <Dom.Early@tcd.ie>
Subject: Re: concatenate - variable with one or two indexes
>Hello,
>
>I still have two questions from a beginner in xlispstat.  I have looked
>for the answers in the book, but I have found nothing.
>
>-----------------------------------------------------------------
>- The first question :
>
>I have two lists :
>(def a (list a b c d e f))
>(def b (list u v w x y z))
>

This question seems to have been dealt with pretty well by other replies, I
point out Johannes Mueller-Gerking's reply where he observes that the above
code does not work and I adopt his interpretation of what you are trying to
do by working with strings.

>- The second question is the indexed variables :
>for example :
>(def var1 (list a b c d e))
>(def var2 (list l m n o p))
>
>I want define the variable toto like this : toto(a)=q, toto(b)=w, ...
>
>and for two indexes :
>(def var3 (list v w x y z))
>I want : toto(a,l) = v, toto(b,m)= w, ...

For the second part of your question, hash-tables have been suggested as a
solution.  I provide code to solve your problems below.  The first is a
straight-forward implementation using hashtables.  I provide a utility
function that populates a hash table - it takes two equal length lists as
an argument and a hash table.  The first list should contain indices, the
second should contain corresponding values.
Also provided is a 'toto function which does the hash table lookup so as to
provide a solution close to your specifications.
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          init-hashtable
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun init-hashtable (lista listb hashtable)
  "
;; Examples:
 (def a (list \"a\" \"b\" \"c\" \"d\" \"e\" \"f\"))
 (def b (list \"l\" \"m\" \"n\" \"o\" \"p\"))
 (def toto-ht (make-hash-table :test #'equal))
 (init-hashtable a b toto-ht)
 (hash-lookup \"a\")
"
  (when lista
	(setf (gethash (car lista) hashtable) (car listb))
	(init-hashtable (cdr lista) (cdr listb) hashtable)
	)
  hashtable
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                hash-lookup
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun hash-lookup(index) (gethash index toto-ht))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                trial
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(try-out init-hashtable)

#|
For the more general version which allows both single and double indexing,
I re-implemented the solution as follows.  As we access the hashtable
through our own function calls, namely init-hashtable and hash-lookup, the
internal representation of the index is hidden from the user.  This allows
us to implement the index any way we like.  I have now implemented it as a
list so when we do (hash-lookup "a") we actually index the hash table with the
indice ("a") .  Similarly when we index it with (hash-lookup "a" "l") we actually
index it internally as ("a" "l")

I couple this with variable length argument lists to produce a neat solution.
|#

(defun init-hashtable (lista listb hashtable &optional(listc nil set))
  "
;; Examples:
 (def a (list \"a\" \"b\" \"c\" \"d\" \"e\" \"f\"))
 (def b (list \"l\" \"m\" \"n\" \"o\" \"p\"))
 (def c (list \"v\" \"w\" \"x\" \"y\" \"z\"))
 (def toto-ht (make-hash-table :test #'equal))

 (init-hashtable a b toto-ht)
 (init-hashtable a b toto-ht c)

 (hash-lookup \"a\")
 (hash-lookup \"a\" \"l\")
 (hash-lookup \"a\" (hash-lookup \"a\"))
"
  (if set
      (when lista
	    (setf (gethash (list (car lista) (car listb)) hashtable) (car listc))
	    (init-hashtable (cdr lista) (cdr listb) hashtable (cdr listc))
	    )
    (when lista
	  (setf (gethash (list (car lista)) hashtable) (car listb))
	  (init-hashtable (cdr lista) (cdr listb) hashtable)
	  )
    )
  hashtable
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                hash-lookup
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun hash-lookup(&rest index) (gethash index toto-ht))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                trial
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(try-out init-hashtable)

#|
I recommend you get a good lisp reference if you plan to do any serious
work with lisp, things can be a little confusing if you aren't too sure
what it is all about.

Also try the excellent 'Common Lisp HyperSpec' HTML common lisp reference
(see  http://www.harlequin.com/ )

+--------------------------------------------------------------------------+
| Dominic Early                                  e-mail: Dom.Early@tcd.ie  |
| A.31, Department of Statistics,                Phone:  +353-1-608 1917   |
| Trinity College,                               Fax:    +353-1-661 5046   |
| Dublin 2, Ireland.                                                       |
|                                                                          |
|              http://www2.tcd.ie/Statistics/staff/dbearly.html            |
+--------------------------------------------------------------------------+
|#


