(message "
It would be nice if missing-data capabilities were built into lisp-stat, but
it's not to hard to do this kind of thing.  Here's a simple function that
gets rid of missing data in a list of variables. I'm sure that someone else
could program this more elegantly.
") 

(message "
(defun delete-missing (variables &key (miss '#.not-a-number) (test #'equal))
  \"Args: (variables &key (miss '#.not-a-number) (test #'equal))
VARIABLES is a list of sequences, containing the variables; MISS gives the
missing data code (default '#.not-a-number); TEST gives the equality test (default
#'equal). The function returns a list a sequences containing only
observations with valid data for all variables.
Example: (delete-missing (list x y z) :miss '*)\"
  (let* ((bad (remove-duplicates 
               (combine (mapcar 
                         #'(lambda (x) (which (map-elements test miss x)))
                         variables))))
         (all (iseq (length (first variables))))
         (good (sort-data (set-difference all bad))))
    (mapcar #'(lambda (x) (select x good)) variables)))
")

(defun delete-missing (variables &key (miss '#.not-a-number) (test #'equal))
  "Args: (variables &key (miss '#.not-a-number) (test #'equal))
VARIABLES is a list of sequences, containing the variables; MISS gives the
missing data code (default '#.not-a-number); TEST gives the equality test (default
#'equal). The function returns a list a sequences containing only
observations with valid data for all variables.
Example: (delete-missing (list x y z) :miss '*)"
  (let* ((bad (remove-duplicates 
               (combine (mapcar 
                         #'(lambda (x) (which (map-elements test miss x)))
                         variables))))
         (all (iseq (length (first variables))))
         (good (sort-data (set-difference all bad))))
    (mapcar #'(lambda (x) (select x good)) variables)))

(message "What is the difference between (def x) and (defvar x)?")

(princ
 "Def defines variables that are lexically scoped (the default in lisp);
defvar defined variables that are dynamically scoped. Here's a lisp-stat
dialog that illustrates the difference:
 
(defvar x 2) ;; X
(def y 1) ;; Y
(defun f (x) (g)) ;; F
(defun g () x) ;; G
(g) ;; 2
(f 3) ;; 3
(defun h (y) (i)) ;; H
(defun i () y) ;; I
(i) ;; 1
(h 3) ;; 1
") 

(defvar x 2) ;; X
(def y 1) ;; Y
(defun f (x) (g)) ;; F
(defun g () x) ;; G
(g) ;; 2
(f 3) ;; 3
(defun h (y) (i)) ;; H
(defun i () y) ;; I
(i) ;; 1
(h 3) ;; 1

(message
 "So, calling g from f shadows the global value of the dynamically scoped
variable x, but calling i from h doesn't shadow the global value of y
because y is a lexically scoped variable defined in the global environment
(or something like that).
")


(message "
I've used XLS #.NOT-A-NUMBER to encode missing values. 
 
I don't know if that and the POSITYVE and NEGATIVE-INFINITY
values are supported on all platforms or just where there is
existing IEEE FP support. It is on the Mac. ( 3.47 )
 
I also often use a reciprocal function, that doesn't give an
error on divide by zero, but yields infinity. I had qualms 
about just changing the bahaviour of '/' : I find that typically,
a divide by zero of a single number is usually an error I 
want to catch, but something of the form ( / 1 vector ) I
usually expect to pass, zeros or not. 
 
Much better than just using an arbitrary large number.
 
You can also modify the :adjust-to-data methods to take this
into account so that the graph functions don't bother to
include those asymptotic or missing values in scaling the
graph to the window. 
  
 
 
---|  Steven D. Majewski   (804-982-0831)  <sdm7g@Virginia.EDU>  |---
---|  Department of Molecular Physiology and Biological Physics  |---
---|  University of Virginia             Health Sciences Center  |---
---|  P.O. Box 10011            Charlottesville, VA  22906-0011  |---
         By doing just a little every day, you can gradually 
                let the task completely overwhelm you.
")
