;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                 Making a knot, using matroids and flesh...
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
Hi Dad,
	I just put a gif of my first pre-knot matroid on my home page: it's not
spectacular, but it will get better, I'm sure!
	This is a trio of matroids, which together constitute the sphere with
three holes in it. It's called matroid.gif, and it's in my home directory, in
the dad subdirectory. Have a look.
	It's fleshed out (doubled) from six sections per matroid.
	I made it using circles in the plane that have their derivatives
perpendicular to the equator (so that when they are projected onto the sphere
there will be derivative continuity). I see that I need to do some geometry: I
should really use the orthogonal family of these circles to define the points
in the other direction, so as to assure continuity of derivatives in the other
direction (or something like that!).
	I'm anxious to see what you've been up to. I'll work on twisting my
matroids into knots later: I've spent enough time on this for today. Hope that
your chemo left you alone this time. Take care,
							Love,
								Andy
(defun matroid(xmat ymat zmat &key plot)
  "
;; Description: (matroid xmat ymat zmat &key plot)

	Matroid takes three matrices, which are essentially x, y, and z
components of a 'matroid' - a twisted grid - and recreates the grid.

;; See also: matroid-to-matrices

;; Examples:
 (def xmat (matrix '(4 3) '(1 1 1 2 2 2 3 3 3 4 4 4)))
 (def ymat (matrix '(4 3) '(1 2 3 1 2 3 1 2 3 1 2 3)))
 (def zmat (matrix '(4 3) '(1 1 1 2 2 2 1 1 1 -2 -2 -2)))
 (matroid xmat ymat zmat)
 (matroid (flesh xmat) (flesh ymat) (flesh zmat))

 (def eps .1)
 (def xmat (+ xmat (* eps (matrix '(4 3) (rand.n (* 4 3))))))
 (def ymat (+ ymat (* eps (matrix '(4 3) (rand.n (* 4 3))))))
 (def zmat (+ zmat (* eps (matrix '(4 3) (rand.n (* 4 3))))))
 (matroid xmat ymat zmat)
 (matroid (flesh xmat) (flesh ymat) (flesh zmat))

 (def eps .4)
 (def xmat (+ xmat (* eps (matrix '(4 3) (rand.n (* 4 3))))))
 (def ymat (+ ymat (* eps (matrix '(4 3) (rand.n (* 4 3))))))
 (def zmat (+ zmat (* eps (matrix '(4 3) (rand.n (* 4 3))))))
 (matroid xmat ymat zmat)
 (matroid (flesh xmat) (flesh ymat) (flesh zmat))
 (matroid (flesh xmat :increase 4) (flesh ymat :increase 4) (flesh zmat :increase 4))

 ;; this is the case of degenerate vertices:
 (def xmat (matrix '(3 2) '(1 1 1 2 1 3)))
 (def ymat (matrix '(3 2) '(1 2 1 2 1 2)))
 (def zmat (matrix '(3 2) '(1 1 1 2 2 1)))
 (matroid xmat ymat zmat)
 (matroid (flesh xmat :increase 4) (flesh ymat :increase 4) (flesh zmat :increase 4))
 (matroid (flesh xmat :increase 4 'cubic)
   (flesh ymat :increase 4 'cubic) (flesh zmat :increase 4 :type 'cubic))
"
  (let* (
	 (my-plot (if plot plot (send spin-proto :new 3)))
	 (nrows (first (size xmat)))
	 (ncols (second (size xmat)))
	 )
    (dotimes
     (j ncols)
     (let (
	   (xp (aref xmat 0 j))
	   (yp (aref ymat 0 j))
	   (zp (aref zmat 0 j))
	   )
       (dotimes
	(i (1- nrows))
	(let (
	      (xnext (aref xmat (1+ i) j))
	      (ynext (aref ymat (1+ i) j))
	      (znext (aref zmat (1+ i) j))
	      )
	  (send my-plot :add-lines (list
				    (list xp xnext)
				    (list yp ynext)
				    (list zp znext)
				    )
		)
	  (setq xp xnext)
	  (setq yp ynext)
	  (setq zp znext)
	  )
	)
       )
     )
    (dotimes
     (i nrows)
     (let (
	   (xp (aref xmat i 0))
	   (yp (aref ymat i 0))
	   (zp (aref zmat i 0))
	   )
       (dotimes
	(j (1- ncols))
	(let (
	      (xnext (aref xmat i (1+ j)))
	      (ynext (aref ymat i (1+ j)))
	      (znext (aref zmat i (1+ j)))
	      )
	  (send my-plot :add-lines (list
				    (list xp xnext)
				    (list yp ynext)
				    (list zp znext)
				    )
		)
	  (setq xp xnext)
	  (setq yp ynext)
	  (setq zp znext)
	  )
	)
       )
     )

    (send my-plot :mouse-mode 'hand-rotate)
    (send my-plot :depth-cuing nil)
    (send my-plot :adjust-to-data)
    my-plot
    )
  )

|#

(def matrix nil)

;; these are the values along the equator where we want bands to pass:
(def xs (rseq .5 (/ (sqrt 3) 2) 10))
(def fineness 5)

(dolist
 (xnot xs)

 ;; For each band, we create the circle function which will give the slope
 ;; perpendicular at the equator:
 (def r (/ xnot (sqrt (- 1 (square xnot)))))
 (def c (/ (sqrt (- 1 (square xnot)))))

 ;; Now we construct the points in the plane:
 (def x (rseq xnot 0 fineness))
 (def y (mapcar #'(lambda(z) (- c (sqrt (- (square r) (square z))))) x))

 ;; These are the projections onto the sphere:
 (def r-square (+ (square x) (square y)))
 (def z (/ (1- r-square) (1+ r-square)))
 (def x (/ (* 2 x) (1+ r-square)))
 (def y (/ (* 2 y) (1+ r-square)))

 ;; We can use symmetry to make the rest of the matroid:
 (def x (combine x (rest (reverse (- x)))))
 (def y (combine y (rest (reverse y))))
 (def z (combine z (rest (reverse z))))
 (def x (combine x (rest (reverse x))))
 (def y (combine y (rest (reverse y))))
 (def z (combine z (rest (reverse (- z)))))
 (def matrix (if matrix (bind-columns matrix (weave x y z)) (weave x y z)))
 )

(def x (rseq (/ (sqrt 3) 2) 0 fineness))
(def y (/ x (sqrt 3)))
(def r-square (+ (square x) (square y)))
(def z (/ (1- r-square) (1+ r-square)))
(def x (/ (* 2 x) (1+ r-square)))
(def y (/ (* 2 y) (1+ r-square)))
(def x (combine x (rest (reverse (- x)))))
(def y (combine y (rest (reverse y))))
(def z (combine z (rest (reverse z))))
(def x (combine x (rest (reverse x))))
(def y (combine y (rest (reverse y))))
(def z (combine z (rest (reverse (- z)))))
(def matrix (if matrix (bind-columns matrix (weave x y z)) (weave x y z)))

;; this is one patch:
(def matrices (matroid-to-matrices matrix))
(matroid (first matrices) (second matrices) (third matrices))

;; Here's the second:
(def matrices-2 (matrix-multiply
		 (x-y-rotation (/ 2pi 3))
		 (matrix '(3 1) matrices)))
(def matrices-2 (matrix-to-list matrices-2))
(matroid (first matrices-2) (second matrices-2) (third matrices-2))

;; and the third.
(def matrices-3 (matrix-multiply
		 (x-y-rotation (/ 4pi 3))
		 (matrix '(3 1) matrices)))
(def matrices-3 (matrix-to-list matrices-3))
(matroid (first matrices-3) (second matrices-3) (third matrices-3))

;; Now I want to patch them all together, and get my super matroid!
(def matroids (list matrices matrices-2 matrices-3))
(matroid
 (apply #'bind-columns (mapcar #'first matroids))
 (apply #'bind-columns (mapcar #'second matroids))
 (apply #'bind-columns (mapcar #'third matroids))
 )

;; Wow, that's cool! But it's not what I want: I want to do several matroids
;; at a time. I'll just add a key to matroid, allowing the plot to stay
;; constant:
(def my-plot
     (matroid (first matrices) (second matrices) (third matrices)))
(matroid (first matrices-2) (second matrices-2) (third matrices-2)
	 :plot my-plot)
(matroid (first matrices-3) (second matrices-3) (third matrices-3)
	 :plot my-plot)

(def my-plot
     (matroid
      (flesh (first matrices) :type 'linear :increase 2)
      (flesh (second matrices) :type 'linear :increase 2)
      (flesh (third matrices) :type 'linear :increase 2)))
(matroid 
 (flesh (first matrices-2) :type 'linear :increase 2)
 (flesh (second matrices-2) :type 'linear :increase 2)
 (flesh (third matrices-2) :type 'linear :increase 2)
 :plot my-plot)
(matroid 
 (flesh (first matrices-3) :type 'linear :increase 3)
 (flesh (second matrices-3) :type 'linear :increase 3)
 (flesh (third matrices-3) :type 'linear :increase 3)
 :plot my-plot)

	 
