(defun newtonmax (f start &key 
                    scale 
                    (derivstep -1.0)
                    (count-limit -1)
                    (verbose 1)
                    return-derivs
		    trace)
"Args:(f start &key scale derivstep (verbose 1) return-derivs)
Maximizes F starting from START using Newton's method with backtracking.
If RETURN-DERIVS is NIL returns location of maximum; otherwise returns
list of location, unction value, gradient and hessian at maximum.
SCALE should be a list of the typical magnitudes of the parameters.
DERIVSTEP is used in numerical derivatives and VERBOSE controls printing
of iteration information. COUNT-LIMIT limits the number of iterations"
  (let ((verbose (if verbose (if (integerp verbose) verbose 1) 0))
        (minfo (send minfo-proto :new f start 
                     :scale scale :derivstep derivstep)))
    (send minfo :maxiter count-limit)
    (send minfo :derivscale)
    (send minfo :maximize
	  verbose
	  (if trace #'(lambda () (funcall trace minfo))))
    (if return-derivs
        (cons (send minfo :x) (- (send minfo :fvals)))
        (send minfo :x))))

(defmeth minfo-proto :derivscale ()
  (let* ((x (send self :x))
	 (step (^ machine-epsilon (/ 1 6)))
	 (hess (numhess (send self :f) x (send self :scale) step))
	 (scale (pmax (abs x) (sqrt (abs (/ (diagonal hess)))))))
    (setf hess (numhess (send self :f) x scale step))
    (setf scale (pmax (abs x) (sqrt (abs (/ (diagonal hess))))))
    (setf (mode-info-scale) (coerce scale '(vector float)))
    (setf (dpars-derivstep (mode-info-dpars)) step)))

