;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				non-linear-system-root
;;
(defun non-linear-system-root(f v &key x0 (eps 1e-6) (maxits 100) jack)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			non-linear-system-root Doc:
;;
;; Author: Andy Long
;; Created: Wed Feb 25 20:51:44 1998
;;
;; Description/Arguments: non-linear-system-root
  
;; Arguments: f v &key x0 (eps 1e-6) (maxits 100) jack

;; Returns: vector

;; Notes: For DESS class at the IMSP

;; See Also: newtonmax

;; Examples:
 (non-linear-system-root
  (list (func g1(x y) x*y^2) (func g2(x y) x^2+y^3)) '(x y)
  :x0 '(.01 .01)
  )

 ;; Strang, p. 379:
 (spin-points 21)
 (spin-domain -2 2 -2 2)
 (plot (func p(x y) .25*(x^4-4*x*y+y^4)))
 (diff p x g1)
 (diff p y g2)

 (non-linear-system-root (list #'g1 #'g2) '(x y)
  :x0 (* .01 (rand.n 2))
  )
 (non-linear-system-root (list #'g1 #'g2) '(x y)
  :x0 (* .1 (rand.n 2))
  )
 (non-linear-system-root (list #'g1 #'g2) '(x y)
  :x0 (* 1 (rand.n 2))
  )
 (non-linear-system-root (list #'g1 #'g2) '(x y)
  :x0 (* 10 (rand.n 2))
  )

 ;; How about trying to solve linear systems?
 (def A #2a((2 4)(5 2)))
 (def b #(2 1))
 ;; now we set up the two functions, Ax-b:
 (defun g1(x y) (- (+ (* 2 x) (* 4 y)) 2))
 (defun g2(x y) (- (+ (* 5 x) (* 2 y)) 1))
 ;; and compare solutions:
 (non-linear-system-root (list #'g1 #'g2) '(x y)
  :eps 1e-12
  )
 (solve A b)
"
  (let (
	(jack (if jack jack (jacobian f v :matrix-function t)))
	(f (coerce f 'list))
	)
    (do*
     (
      (i 1 (1+ i))
      (x0 (if x0 x0 (repeat 0 (length v))) x1)
      (x1 
       (+ x0 (solve (apply jack x0) (- (mapcar (lambda(x) (apply x x0)) f))))
       (+ x0 (solve (apply jack x0) (- (mapcar (lambda(x) (apply x x0)) f))))
       )
      )
     ((or (> i maxits) (< (norm (- x1 x0)) eps)) (print i) x1)
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
