;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			 Snedecor and Cochrane, 6th edition
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;          Chapter 10: One-way classifications. Analysis of Variance
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; I wrote the function one-way-anova to attempt to do these examples of
;; theirs.

;; Page 259:
(def g1 (list 64 72 68 77 56 95))
(def g2 (list 78 91 97 82 85 77))
(def g3 (list 75 93 78 71 63 76))
(def g4 (list 55 66 49 64 70 68))
(one-way-anova (list g1 g2 g3 g4))

;; Example 10.3.1, p. 260.
(def g (list 11 13 21 10 4 9 18 4 6 14 15 19))
(one-way-anova (transpose (split-list g 4)))

;; Example 10.3.2, p. 262.
(def g1 (list 152 156 154))
(def g2 (list 163 157 154))
(def g3 (list 144 152 163))
(one-way-anova (/ (list g1 g2 g3) 100))

;; Example 10.3.3, p. 263.
(def g (list 11 4 4 5 6 4 3 6 8 6 4 11 14 27 8 18 7 4 9 14))
(one-way-anova (split-list g 4))

;; Example 10.3.4, p. 263.
(def g (list 418 389 361 330 375 331 385 359 339 437 389 363 342 386 402 326
	     384 348 362 334 379))
(one-way-anova (/ (split-list g 3) 10))

;; Example 10.5.1, p. 267.
(def g (list 55 61 42 169 49 112 97 137 42 30 81 169 21 89 95 85 52 63 92 154))
(one-way-anova (split-list g 4))


;; Table 10.6.1, using 4.9.1: p. 268.
(def g (list 57 120 101 137 119 117 104 73 53 68 118 89 30 82 50 39 22 57 32 96
	     31 88))
(one-way-anova (split-list g 11))

;; Example 10.6.1
(def g (list 19 26 22 13 26 25 38 40 36 12 16 8 36 42 20 43 47 49 59 37 28 49
	     31 39))
(one-way-anova (split-list g 12))

;; Examples 10.12.1, 10.12.2, p. 278-279.
(def days (iseq 2 14))
(def vars
     (mapcar
      (lambda(x)
	(combine
	 (mapcar
	  (lambda(x y) (repeat y x))
	  x (select days (iseq (length x)))
	  )
	 )
	)
      (list
       (list 6 4 9 8 3 1)
       (list 1 3 3 6 6 14 11 4 6 2 3 1)
       (list 3 5 5 8 19 23 22 14 14 7 8 4 1)
       )
      ))

(one-way-anova vars)
;; Here's the standard error calculation for the difference in means (top of
;; p. 279):
(one-way-anova vars :compare '(-1 1 0))
(one-way-anova vars :compare '(0 -1 1))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        Random Effects, p. 279
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Table 10.13.1, p. 281.
(def g (/ (list 328 309 303 303 352 348 338 338 288 280 281 276 334 338 323
		326) 100))
(one-way-anova (split-list g 4))

;; Example 10.13.1, p. 282.
(def g (/ (list 118 111 136 165 137 140 107 90) 100))
(def nmeasures 2)
(def reg (one-way-anova (split-list g nmeasures)))
(/
 (- 
  (/ (send reg :regss) (1- (length (send reg :beta))))
  (send reg :s-squared)
  )
 nmeasures
 )



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Two-way Anova
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Table 11.2.1, p. 300:
(def a (matrix '(5 5)
	       (list
		8 10 12 13 11
		2 6 7 11 5
		4 10 9 8 10
		3 5 9 10 6
		9 7 5 5 3)))
(two-way-anova a)

;; Example 11.2.1, p. 301:
(def a (matrix '(3 3) (list 112 90 123 86 73 89 80 62 81)))
(two-way-anova a)

;; Example 11.2.2, p. 301, using 4.3, p. 95:
(def a (matrix '(2 8) (list 31 20 18 17 9 8 10 7 18 17 14 11 10 7 5 6)))
(two-way-anova a)

;; Table 11.5.1, p. 304:
(def a (matrix '(4 3) (list 30 29 46 35 34 33 31 30 32 16 11 21)))
(two-way-anova a)
(def reg (two-way-regression a))
(send reg :display)
(chi-square a)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	Question:

	Why doesn't the chi-square give the same probability? Chi-square is
supposed to determine how close to rank-one the matrix is, which is essentially
the same question as the f-test.

	Answer:

	The chi-square is looking to see how close to rank-one a matrix is:
i.e., whether

	M = a*(outer u v) + eps

Two way anova is seeing how close a matrix is to

	M = a*(outer u 1) + b*(outer 1 v) + eps

where the u and v are mean vectors of the rows and columns respectively.

(def a (+ 
	(outer-product (rand.u 4) (ones 1 2))
	(outer-product (ones 1 4) (rand.u 2))
	))
(two-way-anova a)
(two-way-regression a)
(chi-square a)

;; Rank one: all agree that there is some dependence:
(def a (outer-product (rand.u 4) (ones 1 2)))
(two-way-anova a)
(two-way-regression a)
(chi-square a)

;; I guess the following represents a multiplicative effect:
(def a (matrix '(3 2) '(1 2 2 4 3 6)))
(two-way-anova a)
(two-way-regression a)
(chi-square a)

(def a (ln (matrix '(3 2) '(1 2 2 4 3 6))))
;; The log turns it into an additive model.
(two-way-anova a)
(two-way-regression a)
(chi-square a)
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Example 11.5.2, continued:
(def a (matrix '(4 3) (list 34 32 39 33 31 38 30 28 35 15 13 20)))
(two-way-anova a)
(chi-square a)

;; Example 11.5.1, p. 307:
(def a (matrix '(2 2) (list 1 1.1 1.2 1.32)))
(two-way-anova a)
(def reg (two-way-regression a))
(send reg :display)
(mprint (send reg :matrix-estimates))
(chi-square a)

;; Table 11.19.1, p. 333:
(def a (/ (matrix '(5 3) (list 191 501 1230 234 1661 4074 295 2239 3981 234 589
			       2291 166 646 2512)) 10))
(two-way-anova a)
(def reg (two-way-regression a))
(send reg :display)
(mprint (send reg :matrix-estimates))
(chi-square a)

