#|
Date: 13 Mar 97 11:57:52 -0500
Subject: Re: HOW-TO? Problem relating to spin-plots
From: "Steven D. Majewski" <sdm7g@Virginia.EDU>
To: stat-lisp-news@umnstat.stat.umn.edu, "Dom. Early" <Dom.Early@tcd.ie>
Mime-Version: 1.0
Content-Transfer-Encoding: 7bit
 
You can get/set the current transformation matrix from a spin plot 
with the :transformation method. 
 
The following ALMOST draws a line into the plane. 
Sometimes I get a point visible and sometimes I get a short line segment. 
Perhaps this is due to round off or some other truncation. 
But when you rotate or spin the plot, it looks approximately correct. 
( I just hacked this out and minimally tested it manually -- you can
  hook it into do-click to get it to work automatically. I just made
  :do-click report the canvas (x,y) and I typed them in manually. ) 
 
Change the -1, 1  Z values to make it longer. I was testing it on a 
range ( -3 3 ) spin plot, so that was a convenient test value. 
[ Does a really large z value get you into greater roundoff problems? ] 

|#
(defmeth spin-proto :zline ( cx cy &key (color 'red))
  "
 (plot {x*y :v '(x y)})
 (send my-plot :zline 30 20)
 (send my-plot :zline 30 30)
 (send my-plot :zline 30 60)
"
  ( let* (( real-xy ( send self :canvas-to-real cx cy ))
          ( real-x ( car real-xy ))
          ( real-y ( cadr real-xy ))
          ( transmat ( send self :transformation ))
          ( v1 ( matmult ( vector real-x real-y -1 ) transmat ))
          ( v2 ( matmult ( vector real-x real-y 1 ) transmat )))
    ( send self :add-lines 
           ( map 'cons #'list v1 v2 ) :color color )))
#|
The more general question about how to pick or input 3-d points or lines 
is an interesting one. Gestural input would be nice to experiment with,
but this may be difficult with the current XLispStat graphics
implementation. 
 
- Steve Majewski 
- <sdm7g@Virginia.EDU>
|#
#|
Sorry I haven't got back earlier, thanks to Steve Majewski and Ted Harding
for their input. I have been playing around with Steve Majewski's code
which works nicely for spin-plots with 'no scaling' selected in the options
menu item.  I've rewritten Steve's code slightly to make investigation a
little easier (below).  I've noticed that when you switch from 'variable
scaling' to 'No scaling', the code is prone to some sort of errors, i.e.
the lines are placed exactly at the required point, and somtimes are not
completely orthogonal to the viewing plane.
 
However if you switch from 'fixed scaling' to 'no scaling', the code seems
to work fine.  In fact if you switch from 'Variable scaling' to 'fixed
scaling' and then to 'no scaling' it works fine also, STRANGE!  Maybe this
might shine some light on what is going on internally?
 
Thanks to Steve, I will continue to tinker with this problem on my 'long
coffee breaks' and let you know if I come up with anything!  I hope you
will do likewise?
 
        Thanks for the help,
                                Dom.
------------------------------------------------------------------------------
( defmeth spin-proto :zline ( cx cy &key (color 'red))
  (let*(  (real-xy (send self :canvas-to-real cx cy ))
          (real-x (car real-xy))
          (real-y (cadr real-xy))
          (transmat (send self :transformation))
          ;; When spin plots are first created the transformation is nil
          ;; which causes problems below with matmult, so we simply
          ;; initialise the transformation to the identity matrix
          (transmat (if transmat transmat #2a((1 0 0) (0 1 0) (0 0 1))))
          (v1 ( matmult (vector real-x real-y -1) transmat))
          (v2 ( matmult (vector real-x real-y 1) transmat)))
       (send self :add-lines (map 'cons #'list v1 v2) :color color))
)
 
;; Here we extend do-select-click functionality to add a ZLINE, to make
;; testing a bit easier
(defmeth spin-proto :do-select-click(pX pY pM1 pM2)
    (send self :zline pX pY)
    (call-next-method pX pY pM1 pM2)
)
 
(def s (spin-plot '((1 2 3) (3 4 5) (4 5 6))))

|#
#|
From majordom@stat.umn.edu Thu Mar 20 13:20 CST 1997
Received: from umnstat.stat.umn.edu (umnstat.stat.umn.edu [128.101.51.1]) by maya.medctr.luc.edu (8.7.3/8.6.9) with SMTP id NAA19402 for <ael@maya.medctr.luc.edu>; Thu, 20 Mar 1997 13:20:13 -0600 (CST)
Received: by umnstat.stat.umn.edu
	(1.38.193.4/16.2) id AA28740; Thu, 20 Mar 1997 12:11:50 -0600
Received: from mail.Virginia.EDU by umnstat.stat.umn.edu with SMTP
	(1.38.193.4/16.2) id AA28735; Thu, 20 Mar 1997 12:11:33 -0600
Received: from elvis.med.virginia.edu by mail.virginia.edu id aa15279;
          20 Mar 97 13:11 EST
Received: from slip-2-11.acc.Virginia.EDU by elvis.med.Virginia.EDU (5.65c/1.34)
	id AA14020; Thu, 20 Mar 1997 13:10:34 -0500
Date: 20 Mar 97 13:13:07 -0500
Subject: Re: HOW-TO? Problem relating to spin-plots
From: "Steven D. Majewski" <sdm7g@Virginia.EDU>
To: "Steven D. Majewski" <sdm7g@Virginia.EDU>, "Dom. Early" <Dom.Early@tcd.ie>
Cc: stat-lisp-news@umnstat.stat.umn.edu
X-Mailer: Cyberdog/2.0b1
Mime-Version: 1.0
Message-Id: <AF56E8B1-139288@128.143.7.211>
Content-Transfer-Encoding: 7bit
Sender: owner-stat-lisp-news@stat.umn.edu
Precedence: bulk
Content-Type: text/plain; charset=US-ASCII
Content-Length: 2257
Status: R


OK -- Try the following code.

In addition to a :scale, there is also a :shift that has to be corrected
for.
I found this by hunt and peck experimenting, with a bit of peeking at the 
sources, however, I haven't figured out the whole flow of control for these
modules from the sources, so it's mostly what works experimentally. I can't
for the life of me imagine why I have to *both* divide and multiply by the 
scales in 2 different places.  

Shift option clicking will draw a red line segment orthogonal to the 
viewing plane. 

- Steve Majewski

;;; --------------------------
;;; test 3-d line drawing ...
;;; --------------------------
;; scale factor for test -- set b to any number 
(setf b 2)

(setf S (spin-function 
           #'(lambda (x y) (+ -1 (* x x) (* y y)))
           (- b) b (- b) b :color 'green))


(defmeth s :do-click (x y a b)
  (let ((scaled (send self :canvas-to-scaled x y)))
    (format T "CANVAS: ~A    SCALED: ~A   C->Real: ~A\n"
             (list x y)
             scaled 
             (send self :canvas-to-real x y)))
  (when (and a b)
         (send self :zline x y))
  (call-next-method x y a b))


(defmeth s :zline (cx cy &key (color 'red))
  (let* ((real-xy (send self :canvas-to-real cx cy))
          (real-x (car real-xy))
          (real-y (cadr real-xy))
          (transmat (send self :transformation))
          (shifts (send self :shift (send self :current-variables)))
          (scales (send self :scale (send self :current-variables))))
    (if (null transmat) 
	(setf transmat 
	       (send self
		      :transformation
		      #2A((1. 0. 0.) (0. 1. 0.) (0. 0. 1.)))))
								 
    (let ((v1 (matmult  (/ (vector real-x real-y (- b)) scales) 
			   transmat))
           (v2 (matmult  (/ (vector real-x real-y    b) scales) 
			   transmat)))
      (send s :add-lines
             (map 'cons  #'list 
                   (+ (* v1 scales) shifts)
                   (+ (* v2 scales) shifts))
	     :color color))))


(let* ((n 8))
  (send s :add-surface 
         (rseq (- b) b n)
         (rseq (- b) b n)
         (make-array (list n n) 
		      :initial-contents (repeat 0.0 (* n n)))
         :color 'blue))
|#
