;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	Here will ultimately be many of the examples and exercises from the
book "introduction to applied mathematics", by Gilbert Strang, first edition.

						Andy Long
						Fearless Programming
						IMSP
						Cotonou, Benin, W. Africa
						1997-1998
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	Strang begins by saying that there are three fundamental triple
products which will be studied: 

	LDL'	- Lower triangular L, Diagonal D (elimination);
	A'CA	- rectangular A, square C (equilibrium problems);
	QLQ'	- orthogonal Q, diagonal L

	the first comes from the solution of a linear problem (gaussian
elimination); the second from equilibrium problems, whether discrete or
continuous; and the third arises in the eigenvalue problem.
|#

(plot
 (lambda(x) (/ (- 2 (* 2 x)) 4))
 (lambda(x) (/ (- 1 (* 4 x)) 11))
 -1 4
 )

#|
box 1a: 
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                      section 1.2: gaussian elimination
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; simple lu-decomp:
(defun simple-lu-decomp(matrix)
  (let* (
	 (size (size matrix))
	 (length (first size))
	 (seq (iseq length))
	 (u (matrix size (combine matrix)))
	 (l (make-array size :initial-element 0))
	 )
    (setf (aref l (1- length) (1- length)) 1)
    (dotimes
     (i (1- length))
     (let (
	   (pivot (aref u i i))
	   (row (select u i seq))
	   lval
	   )
       (setf (aref l i i) 1)
       (dolist
	(j (iseq (1+ i) (1- length)))
	(setf
	 lval (/ (aref u j i) pivot)
	 (select u j seq) (- (select u j seq) (* row lval))
	 (aref l j i) lval
	 ))))
    (mapcar #'mprint (list l u (matmult l u)))
    (list l u)
    )
  )

(mvsq '(l u)
      (simple-lu-decomp #2a((2 1 0 0) (1 2 1 0) (0 1 2 1) (0 0 1 2))))
(def diag (diagonal u))
(mprint (setq l-transpose (matmult (diagonal-to-full (/ diag)) u)))

;; thus we see the triple factorization a = ldl'
(mprint (matmult l (diagonal-to-matrix diag) l-transpose))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.2.1, p. 12
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.2.1, p. 12")
(def a #2a((1 1 1)(1 3 3)(1 3 5)))
(def b (list 2 0 2))
(def 1.2.1 (solve a b))
(matmult a 1.2.1)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.2.3, p. 12
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.2.3, p. 12")
(def a #2a((2 1 0 0) (1 2 1 0) (0 1 2 1) (0 0 1 2)))
(def b (list 2 7 12 11))
(def 1.2.3 (solve a b))
(matmult a 1.2.3)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.2.6, p. 13
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.2.6, p. 13")
(mprint (setq a #2a((1 2 2)(2 7 7)(2 7 9))))
(simple-lu-decomp a)
(solve a a)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.2.10, p. 14
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.2.10, p. 14")
(def a #2a((1 -1 0)(-1 2 -1)(0 -1 2)))
(def b (id 3))
(def 1.2.10 (solve a b))
;; id matrix:
(mprint (matmult a 1.2.10))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.2.12, p. 14
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.2.12, p. 14")
(def a #2a((2 3 2)(0 4 0)(6 9 7)))
(def b (list 2 -8 7))
(def 1.2.12 (solve a b))
(matmult a 1.2.12)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.2.13, p. 14
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.2.13, p. 14")
(print "same as 1.2.12!")


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.2.15, p. 14
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.2.15, p. 14")
(def a #2a((1 1 1)(3 3 4)(1 2 1)))
(def b (list 0 2 -4))
(def 1.2.15 (solve a b))
(matmult a 1.2.15)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;            section 1.3: positive definite matrices and a=ldl'
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; pd:
(cholesky (id 2))
(cholesky #2a((2 4) (4 11)))

;; the following two aren't positive definite, as can be seen by examining the
;; second part of each answer (which is non-zero):
(cholesky #2a((1 5) (5 1)))
(cholesky #2a((-2 1)(1 -2)))

(title "positive definite")
(plot (lambda(x y) (matmult (list x y) #2a((2 4) (4 11)) (list x y))))
(title "indefinite")
(plot (lambda(x y) (matmult (list x y) #2a((1 5) (5 1)) (list x y))))
(title "negative definite")
(plot (lambda(x y) (matmult (list x y) #2a((-2 1)(1 -2)) (list x y))))

(defun l-d-lt(matrix)
  (let* (
	 (chol (cholesky matrix))
	 (mat (first chol))
	 (diag (diagonal mat))
	 (l (matmult mat (diagonal-to-matrix (/ diag))))
	 (d (diagonal-to-matrix (square diag)))
	 )
    (if (/= (second chol) 0) ;; not positive definite
	(format
	 t
	 "~%matrix not positive definite: will approximate by pd matrix.~%")
      )
    (mapcar #'mprint (list matrix l d (matmult l d (transpose l))))
    (list l d)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.3.1, p. 28
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.3.1, p. 28")
(def a #2a((3 -3)(-3 5)))
(setq 1.3.1 (l-d-lt a))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.3.2, p. 28
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.3.2, p. 28")
(def a #2a((3 6)(6 8)))
(setq 1.3.2 (l-d-lt a))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.3.3, p. 28
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.3.3, p. 28")
(def a #2a((1 1 0 0)(1 2 1 0)(0 1 2 1)(0 0 1 2)))
(simple-lu-decomp a)
(solve a (list 1 0 0 0))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.3.5, p. 28
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.3.5, p. 28")
(def a #2a((1 5)(5 1)))
(simple-lu-decomp a)
(print "(x + 5y)^2 - 24y^2")

(def a #2a((1 5)(5 30)))
(simple-lu-decomp a)
(print "(x + 5y)^2 + 5y^2")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.3.7, p. 28
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.3.7, p. 28")
(def a #2a((1 1 1)(1 1 1)(1 1 1)))
(print "against, as it is rank one, or has zero pivots:")
(l-d-lt a)

(def a #2a((1 1 1)(1 2 2)(1 2 3)))
(print "for, as it has positive pivots:")
(l-d-lt a)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.3.8, p. 28
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.3.8, p. 28")
(print "we just saw that c=1 is close - anything bigger than one will do!")


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.3.9, p. 29
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.3.9, p. 29")

(format t "example: (- (id 2))~%")

(def a #2a((1 1 0 0)(1 2 1 0)(0 1 2 1)(0 0 1 2)))
(l-d-lt a) ;; product of pivots is 1; or else,
(determinant a) ;; 1


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.3.15, p. 30
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.3.15, p. 30")
(def a #2a((4 12)(12 45)))
(l-d-lt a)
(mprint (first (cholesky a)))
(format t "the connection is fairly obvious, from looking at ll'.~%")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.3.18, p. 30
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.3.18, p. 30")
(defun symmetric-factorization-l-d-lt-for-positive-definite-a(a)
  "
;; examples:
 (symmetric-factorization-l-d-lt-for-positive-definite-a
   (matrix '(2 2) '(4 12 12 45)))
 (l-d-lt (matrix '(2 2) '(4 12 12 45)))

 (symmetric-factorization-l-d-lt-for-positive-definite-a
   #2a((1 1 0 0)(1 2 1 0)(0 1 2 1)(0 0 1 2)))
 (l-d-lt #2a((1 1 0 0)(1 2 1 0)(0 1 2 1)(0 0 1 2)))
"
  (let* (
	 (n (array-dimension a 0)) 
	 (r (iseq n))
	 (d (diagonal a))
	 )
    (dolist
     (j (iseq n))
     (dolist
      (p (iseq (- j 1)))
      (setf (elt r p) (* (elt d p) (aref a j p)))
      )
     (if (= 0 
	    (let ()
	      (setf (elt d j) 
		    (- (aref a j j)
		       (if (iseq (- j 1))
			   (sum (* (select a j (iseq (- j 1)))
				   (select r (iseq (- j 1)))))
			 0))))
	    )
	 (return "failed! not positive definite....")
       (dolist
	(i (iseq j (1- n)))
	(setf (aref a i j) (/
			    (- (aref a i j)
			       (if (iseq (- j 1))
				   (sum (* (combine (select a i (iseq (- j 1))))
					   (select r (iseq (- j 1)))))
				 0))
			    (elt d j)
			    )
	      )
	)
       )
     )
    )
  a
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.3.19, p. 31
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.3.19, p. 31")
(defun solve-for-positive-definite-tridiagonal-matrix(d l b)
  "
destructive! will overwrite your d, l and b!

;; examples:
 (solve-for-positive-definite-tridiagonal-matrix
   (repeat 2 4) (repeat 1 3) (list 2 1 0 0))
 (solve-for-positive-definite-tridiagonal-matrix
   (repeat 2 4) (repeat 1 3) (list 1 2 1 0))
 (solve-for-positive-definite-tridiagonal-matrix
   (repeat 2 4) (repeat 1 3) (list 0 1 2 1))
 (solve-for-positive-definite-tridiagonal-matrix
   (repeat 2 4) (repeat 1 3) (list 0 0 1 2))
 (def d (repeat 2 4))
 (def l (repeat 1 3))
 (def b (list 0 0 1 2))
 (solve-for-positive-definite-tridiagonal-matrix d l b)
 (printall d l b)
 nil
"
  (let ((n (length d)) tt)
    (dolist
     (k (rest (iseq n)))
     (setf
      tt (elt l (1- k))
      (elt l (1- k)) (/ tt (elt d (1- k)))
      (elt d k) (- (elt d k) (* tt (elt l (1- k))))
      ))
    (dolist
     (k (rest (iseq n)))
     (setf (elt b k) (- (elt b k) (* (elt b (1- k)) (elt l (1- k)))))
     )
    (dolist
     (k (iseq n))
     (setf (elt b k) (/ (elt b k) (elt d k)))
     )
    (dolist
     (k (reverse (butlast (iseq n))))
     (setf (elt b k) (- (elt b k) (* (elt l k) (elt b (1+ k)))))
     )
    )
  b
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                      section 1.4: minimum principles
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
box 1e:

	if a is positive definite, then the quadratic p(x) = x'ax/2 - x'b is
minimized at the point where ax = b. the minimum value is 

			p(a^(-1)b)= -b'a^(-1)b/2
|#

#|
	if a is not positive definite, then we can turn it into a positive
(semi-)definite problem: and this is often definite!

box 1g:

	the vector x that minimizes ||ax - b|| is the solution to the normal
equations

			a'ax = a'b.

this vector x = (a'a)^(-1)a'b is the least squares solution to ax=b.
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example p.38, p. 38
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example p.38, p. 38")
(def a (transpose #2a((1 1 1 1)(0 1 3 4))))
(def b (list 0 8 8 20))
(def p.38 (solve (matmult (transpose a) a) (matmult (transpose a) b)))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example remark1, p. 43
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example remark1, p. 43")
(def atca #2a((2 -1 0)(-1 2 -1)(0 -1 2)))
(def remark1 (l-d-lt atca))

(mprint (setq l-inverse (inverse (first remark1))))
(mprint (inverse atca))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.4.1, p. 45
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.4.1, p. 45")
(format t "~%(i) equivalent to the system x=b!~%")
(format t "~%(ii) equivalent to the system ax=b, where~%")
(setq a #2a((1 1)(1 2)))
(setq b (list 1 -1))
(def 1.4.1 (solve a b))
(func f(x y) (x^2 + 2*x*y + 2*y^2)/2 - x + y)
(apply #'f 1.4.1)

(defun check-min(a b) (* -.5 (matmult b (inverse a) b)))
(check-min a b)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.4.2, p. 45
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.4.2, p. 45")
(format t "~%(i)\tax=b~%(ii)\ta'ax=a'b~%(iii)\ta'ax=a'b or pseudo-inverse.~%")


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.4.3, p. 45
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.4.3, p. 45")
(def a #2a((2 -1)(2 2)(-1 2)))
(def b (bind-columns '(1 1 1) '(2 -1 2) '(2 2 -1)))
(def 1.4.3 (solve (matmult (transpose a) a) b))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.4.4, p. 45
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.4.4, p. 45")
(def a (iseq 1 3))
(def b (list 0 4 2))
(def 1.4.4 (regress (list a) b))
(send 1.4.4 :plot)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.4.6, p. 45
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.4.6, p. 45")
(def a (list 1 2 3))
(def b (list 4 5 8))
(def 1.4.6 (regress (list a) b :intercept nil))
(send 1.4.6 :plot)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.4.7, p. 45
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.4.7, p. 45")
(def a (list 0 1 2))
(def b (list 0 3 12))
(def 1.4.7 (regress (list (repeat 1 3)) b :intercept nil))
;; gives 5, as it should.

(def 1.4.7 (regress (list a) b))
(send 1.4.7 :plot)

(def 1.4.7 (regress (list a (square a)) b))
(solve (bind-columns (repeat 1 3) a (square a)) b)
;; gives correct solution: 3x^2.


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.4.8, p. 45
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.4.8, p. 45")
(def a #2a((0 0)(0 1)(1 0)(1 1)))
(def b (list 2 2 1 5))
(def 1.4.8 (regress a b))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.4.9, p. 46
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.4.9, p. 46")
(def a #2a((1 1 0)(2 2 1)))
(def ata (matmult (transpose a) a))
;; one singular value is effectively zero:
(second (svd ata))

(def a #2a((1 2)))
(def ata (matmult (transpose a) a))
;; one singular value is effectively zero:
(second (svd ata))




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                     section 1.5: eigenvalues and dynamical systems
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun eigenmatrix(a &key verbose)
  "
;; examples:
 (eigenmatrix #2a((2 1)(1 2)))
"
  (let ((eigen (eigenvectors a :verbose verbose)))
    (apply #'bind-columns (second eigen))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5, p. 47
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "section 1.5, p. 47")
(def a #2a((3 0 0)(0 2 0)(0 0 0)))
(def s1.5 (eigenvectors a))
(matmult a '(2 4 6))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example ex1, p. 48
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example ex1, p. 48")
(def a #2a((2 1)(1 2)))
(def ex1 (eigenvectors a))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                remark, page 50:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(message "

	for a symmetric matrix, the singular value decomposition provides the
eigenstructure: that is, 

			a = s^{-1}ls = qlq'

the signs still have to be worked out (if u /= v, then some sign changes are
necessary to give the eigenvalues the right sign).
")
(title "example remark1, p. 50")
(def atca #2a((2 -1 0)(-1 2 -1)(0 -1 2)))
(def remark1 (eigenvectors atca))
(mprint (first (svd atca)))
(mprint (third (svd atca)))
(second (svd atca))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example, p. 52
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example, p. 52")
(def a #2a((2 -1)(-1 2)))
(def lambda (eigenvalues a))
(def ex (eigenmatrix a))
(def exinv (inverse ex))
(mprint (matmult ex (diagonal-to-matrix lambda) exinv))
(def initial (matmult exinv '(5 0)))

(format
 t
 "coefficients of the eigenfunctions exp(* lambda_1 t), exp(* lambda_2 t):")
(mprint (matmult ex (diagonal-to-full initial)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example, p. 58
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example, p. 58")

(def a (matrix '(2 2) (list 0 1 -8 -6)))
(defun u(x)
  (list
   (+ (* 2 (exp (* -2 x))) (exp (* -4 x)))
   (* -4 (+ (exp (* -2 x)) (exp (* -4 x))))))
(u 0)

(def solution (linear-first-order-ode a '(3 -8)))
(funcall solution 0)
(u 1)
(funcall solution 1)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                     hunt for generalized eigenvectors:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; here's a "nasty" matrix from Strang:
(def a #2a((0 1)(-9 6)))

;; we get our eigenstructure, but it fails in this case as the eigenvector is
;; repeated:
(setq evals (eigenvalues a))
(mprint (setq evecs (eigenmatrix a :verbose t)))

;; going after the general ones now: we pick off a distinct eigenvalue, and
;; its associated eigenvector (in this case we only have one):
(setq lambda (first evals))
(def x (column 1 evecs))

;; we create the shift matrix, then solve for y
(setq ashift (- a (* lambda (id 2))))
(setq y (matmult (pseudo-inverse ashift) x))
(matmult ashift y)

;; don't normalize! (mprint (setq s (bind-columns x (normalize y))))
(mprint (setq s (bind-columns x y)))
(mprint (setq j (matrix '(2 2) (list lambda 1 0 lambda))))
;; checks out: compare the following with a:
(mprint (matmult s j (inverse s)))
(mprint a)

(message "
page 61:

	if a is real and symmetric, then
 1) its eigenvalues are all real
 2) we may choose a set of  orthogonal eigenvectors
 3) a = qlq^-1 = qlq'

	if a is also positive definite, then 

 4) its eigenvalues are also positive.

page 63:

	4) is also sufficient for positive definiteness, as is the positiveness
of the pivots. the usual defining condition is x'ax > 0 for all non-zero x.
furthermore, 

 1) if a and b are pd, then so is a+b
 2) if a is pd, then so are a inverse and all positive powers of a (integer or
    no).
") 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.3, p. 64
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.3, p. 64")
(def a (ones 3 3))
(def 1.5.3 (eigenvectors a))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.4, p. 64
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.4, p. 64")
(def a #2a((3 4)(4 -3)))
(def 1.5.4 (eigenvectors (transpose a)))
(matrix-trace a)
(determinant a)
(def a #2a((1 -1 0)(-1 2 -1)(0 -1 1)))
(def 1.5.4 (eigenvectors (transpose a)))
(matrix-trace a)
(determinant a)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.5, p. 64
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.5, p. 64")
(def a #2a((3 4)(4 -3)))
(def lambda (eigenvalues a))
(def ex (eigenmatrix a))
(def exinv (inverse ex))
(mprint (matmult ex (diagonal-to-matrix lambda) exinv))
(def initial (matmult exinv '(0 6)))
(format
 t
 "coefficients of the eigenfunctions exp(* lambda_1 t), exp(* lambda_2 t):")
(mprint (setq constants (matmult ex (diagonal-to-full initial))))

(defun u(tt)
  (let ((expos (exp (* lambda tt))))
    (mapcar (lambda(constant) (sum (* constant expos))) (row-lists constants))))
(u 0)
		       
;; check our answers with the back of the book, page 742:
(u 1)
(+
 (* (exp 5) (/ 6 5) '(2 1))
 (* (exp -5) (/ -12 5) '(1 -2))
 )

(def solution (linear-first-order-ode a '(0 6)))
(funcall solution 1)

;; verify also with rk-4:
(defun f (vals) (matmult a vals))
(def points (rk-4 #'f '(0 6) 0 1 200))
(final points)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.6, p. 64
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.6, p. 64")
;; here's a higher order system with matrices:
;; 		u'' + a u = 0, with u(0)= (2 -1 -1) and u'(0)= (0 0 0):
(def a #2a((1 -1 0)(-1 2 -1)(0 -1 1)))
(def solution (linear-higher-order-ode (list (id 3) (zeros 3 3) a) 
				       '((2 -1 -1) (0 0 0)) :real t))
(funcall solution 0)
(funcall solution 1)

;; compare with the numerical solution obtained using runga-kutta:
(def bigmat
     (combine-matrices
      (list (zeros 3 3)  (id 3)   )
      (list    (- a)    (zeros 3 3))
      ))
(def points (rk-4 #'(lambda(vals) (matmult bigmat vals))
		  '(2 -1 -1 0 0 0) 0 1 200))
(final points)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.7, p. 64
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.7, p. 64")
(def a #2a((3 -1 0)(-2 2 -1)(-1 -1 1)))
(def 1.5.7 (eigenvectors (transpose a)))
(def 1.5.7 (eigenvectors a))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.8, p. 65
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.8, p. 65")
(def a #2a((3 -1 0)(-2 2 -1)(-1 -1 1)))
(def lambda (eigenvalues a))
(setq ex (eigenmatrix a :verbose t))
(def exinv (inverse ex))
(mprint (matmult ex (diagonal-to-matrix lambda) exinv))
(load-calc formal)
(def initial (matrix-to-list (formal-matrix-product exinv #2a((a)(b)(c)))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.10, p. 65
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.10, p. 65")
(def a #2a((0 1)(-1 0)))

;; part a:
(def lambda (eigenvalues a))
(def evecs (eigenmatrix a :verbose t))
(apply 'inner-product (row-lists evecs))

;; part b:
(def initial (matmult (inverse evecs) '(3 4)))
(mprint (setq constants (matmult evecs (diagonal-to-full initial))))

(defun u(tt)
  (let ((expos (exp (* lambda tt))))
    (mapcar (lambda(constant) (realpart 
			       (sum (* constant expos))))
	    (row-lists constants))))
(defun uprime(tt)
  (let ((expos (* lambda (exp (* lambda tt)))))
    (mapcar (lambda(constant) (realpart 
			       (sum (* constant expos))))
	    (row-lists constants))))

(u 0)
(uprime 1)
(matmult a (u 1))
(plot (lambda(x) (first (u x))) 0 10)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.14, p. 66
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.14, p. 66")
(def a #2a((3 1)(0 3)))
(setq evals (eigenvalues a))
(setq evecs (generalized-eigenvectors a :verbose t))

(setq initial '(1 1))
(defun v(tt)
  (let ((mat (* (exp (* tt (first evals))) (matrix '(2 2) (list 1 tt 0 1)))))
    (matmult mat initial)
    )
  )
(v 0)
(plot (mapcar #'v (rseq 0 1 100)))
(plot (mapcar #'first (mapcar #'v (rseq 0 1 100))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.16, p. 66
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.16, p. 66")
(def a #2a((10 -6)(-6 10)))
(def solution
     (linear-higher-order-ode
      (list (id 2) (zeros 2 2) a) 
      '((0 6) (0 0)) :real t :verbose t))
(funcall solution 0)
(funcall solution 1)

(def points 
     (transpose
      (mapcar (lambda(x) (select x (iseq 2)))
	      (mapcar solution (rseq 0 4 100)))))
(send (plot (rseq 0 4 100) (second points))
      :add-plot-item (list 'lines (rseq 0 4 100) (first points) 'red))

(message "
	from the eigenvalues, we see that the frequencies are (sqrt 2) and
2=(sqrt 4).
")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.19, p. 66
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.19, p. 66")
(def solution
     (linear-higher-order-ode (list 1 0 -1) '(2 0) :real t :verbose t))
(def points (transpose (mapcar solution (rseq 0 2 100))))
(send (plot (rseq 0 2 100) (second points))
      :add-plot-item 
      (list 'lines (rseq 0 2 100) (first points) 'red)
      ;; true solution:
      (list 'function (lambda(x) (+ (exp x) (exp (- x)))) 'blue)
      )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.23, p. 67
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.23, p. 67")
(def a #2a((10 -6)(-6 10)))
(def svd (svd a))
(def u (first svd))
(def lambda (second svd))
(def v (third svd))
(mprint
 (setq a1/2 (matmult u (diagonal-to-full (sqrt lambda)) (transpose v))))
(mprint (matmult a1/2 a1/2))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.24, p. 67
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.24, p. 67")
(def m #2a((1 0)(0 2)))
(def k #2a((2 -1)(-1 2)))
(mprint (setq a (matmult (inverse m) k)))
(def evals (eigenvalues a))
(def evecs (eigenmatrix a :verbose t))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example 1.5.25, p. 67
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example 1.5.25, p. 67")
(def m #2a((1 0)(0 2)))
(def k #2a((2 -1)(-1 2)))
(mprint (setq a (matmult (inverse m) k)))
(def evals (eigenvalues a))
(def evecs (eigenmatrix a :verbose t))
(def solution
     (linear-higher-order-ode
      (list (id 2) (zeros 2 2) a) 
      '((0 0) (0 0)) :real t :verbose t))
(def v1 (column 1 evecs))
(def v2 (column 2 evecs))
(matmult v1 m v2)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;               section 1.6: a review of matrix theory
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 example debut, p. 68
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "example debut, p. 68")
(mprint (setq a #2a((1 -1 0)(0 1 -1)(1 0 -1))))
(setq b (list 2 3 5))
(setq debut (pseudo-solve a b))

;; There's more than one vector which maps into b:
(matmult A debut)
(matmult A (list 5 3 0))
;; and the difference is always a multiple of (1 1 1):
(- debut (list 5 3 0))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example 77a, p. 77
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example 77a, p. 77")
(def A #2a((1 2)(0 3)))
(def es (generalized-eigenvectors A))
(def eigs (first es))
(def eigvs (second es))
(matmult eigvs (diagonal-to-full eigs) (inverse eigvs))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example 77b, p. 77
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example 77b, p. 77")
(def A #2a((3 2)(0 3)))
(def es (generalized-eigenvectors A))
;; es has a third component in this case, as there is an eigenvalue with
;; multiplicity 2.
(def eigs (first es))
(def eigvs (second es))
(matmult eigvs (third es) (inverse eigvs))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example 79a, p. 79
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example 79a, p. 79")
(def A #2a((1 -1)(0 1)(1 0)))
(mapcar #'mprint (qr-decomp a))
(mapcar #'mprint (butlast (svd a))) ;; last thing is t or nil - can't mprint
				    ;; that!

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example 80r, p. 80
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example 80r, p. 80")
(def A #2a((1 2 1)(3 6 3)(2 4 2)))
(eigenvalues A)
(mapcar #'mprint (butlast (svd A)))
(mprint (final (generalized-eigenvectors A)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example 82r, p. 82
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example 82r, p. 82")
(mprint (setq A #2a((1 -1 1)(3 -3 3)(2 -2 2))))
(eigenvalues A)
(mapcar #'mprint (butlast (svd A)))
(mprint (final (generalized-eigenvectors A :verbose t)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(print "
	Interesting remark (p. 82): (A^{-1}-(A-uv')^{-1}) is rank one!
Mentions the Sherman-Morrison formula, which I should know.
")
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example 1.6.2, p. 83
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example 1.6.2, p. 83")
(mprint (setq A1 #2a((-1 1 0 0)(0 -1 1 0)(0 0 -1 1)(1 0 0 -1))))
(mprint (setq A2
	      #2a((-1 1 0 0 0)
		  (0 -1 1 0 0)
		  (0 0 -1 1 0)
		  (0 0 0 -1 1)
		  (0 0 -1 0 1)
		  (-1 0 1 0 0))))
;; each is once rank-deficient (has a zero singular value):
(second (svd a1)) ;; rank 3
(second (svd a2)) ;; rank 4
									 
;; vectors proportional to '(1 1 1 1) are in the nullspace:
(final (column-lists (first (svd a1))))
(final (column-lists (third (svd a1))))

;; Gives the zero vector:
(matmult A1 (final (column-lists (third (svd a1)))))

;; Since the matrix is rank one, any vector orthogonal to the nullspace (or any
;; vector in the column space) has a solution: i.e., '(1 -1 0 0).

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example 1.6.3, p. 83
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example 1.6.3, p. 83")
(mprint (setq A2
	      #2a((-1 1 0 0 0)
		  (0 -1 1 0 0)
		  (0 0 -1 1 0)
		  (0 0 0 -1 1)
		  (0 0 -1 0 1)
		  (-1 0 1 0 0))))
;; m=6, n=5: 
(format t "
4 independent rows which form a basis:	2 from {1,2,6} (loop one),
					2 from {3,4,5} (loop two).
2 Independent vectors in the nullspace:
~{~s~%~}" '((1 1 0 0 0 1) (0 0 1 1 1 0))
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example 1.6.4, p. 83
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example 1.6.4, p. 83")
(mprint (setq A1 #2a((-1 1 0 0)(0 -1 1 0)(0 0 -1 1)(1 0 0 -1))))
(mprint (setq A2
	      #2a((-1 1 0 0 0)
		  (0 -1 1 0 0)
		  (0 0 -1 1 0)
		  (0 0 0 -1 1)
		  (0 0 -1 0 1)
		  (-1 0 1 0 0))))
;; m=4+6, n=4+5: 
(format t "
The incidence matrix is a block matrix of A1, A2, and some zero matrices:~%")
(mprint
 (setq
  incidence
  (combine-matrices
   (list           A1            (zeros (first (size A1)) (second (size A2))))
   (list (zeros (first (size A2)) (second (size A1)))    A2))))
(pseudo-inverse incidence)

(format t "
Rank is the sum of the number of columns - p: n-p;
There are p and m-n+p solutions of Ax=0 and A'y=0.~%")



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example 1.6.8, p. 83
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example 1.6.8, p. 83")
(setq A #2a((1 0)(4 1)))
(format t "
 Two matrices are equal if their difference is the zero matrix:")
(mprint (- (inverse (transpose A)) (transpose (inverse A))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example 1.6.10, p. 84
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example 1.6.10, p. 84")
(def A #2a((0 1 0 0)(0 0 1 0)(0 0 0 1)(1 0 0 0)))
(def b (* 1/3 (ones 3 3))) ;; (* 1/3 (outer-product '(1 1 1) '(1 1 1)))
(eigenvalues A)
(eigenvalues B)
(format t "
A is orthogonal, invertible, permutation, and diagonalizable.
B is symmetric, close to a projection (if we had (sqrt 3) rather than 3),
 and diagonalizable.
")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 Example 1.6.11, p. 84
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "Example 1.6.11, p. 84")
(def A1 #2a((1 -1 0)(0 1 -1)(1 0 -1)))
(def A2 #2a((-1 1 0 0)(-1 0 1 0)(0 -1 1 0)(0 -1 0 1)(0 0 -1 1)))
(def p (permutation-matrix '(0 1 3 2 4)))

(lu-decomp A1)
(lu-decomp (matmult p A2))
