;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                survive.lsp
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	This is a repackaged version of a file that Thomas Lumley shipped out
to do some survival analysis. I changed pieces here and there. 

	He's incorporated some slider stuff from B. Narasimhan, so this is
getting to be a real melange.

							Andy Long
|#
;;
;;  Packaged applet with a dynamic graphics demonstration of the 
;;  effect of covariates on the fitted survival curve in cox
;;  regression.
;;
;;-------------------
;;;;
;;;;  sliders.lsp                    B. Narasimhan
;;;;
;;;;  This is a tool that can help a person design sliders in a
;;;;  jiffy.  (I found myself creating sliders so often that I
;;;;  decided to automate the process.)  It writes the code
;;;;  for creating sliders for inclusion in your program as is.
;;;;
;;;; Modified by thomas lumley to send output to a stream so that it can
;;;; be captured and used to create sliders dynamically.
;;;;

(provide "sliders")
(require "statistics")
(require "graphics")
(require "help")

;;;<TSL>
(defun make-sliders (&optional specs &key cols fname)
  (let* ((fh (if fname
		 (open fname :direction :output)
	       (open (get-string-dialog "Output file name for code?")
		     :direction :output)))
	 )
    (make-sliders-1 specs fh :cols cols)
    (close fh)
    )
  )

(defun make-and-run-sliders (specs &optional cols)
  (let* ((fh (make-string-output-stream))
	 (junk (make-sliders-1 specs fh :cols cols))
	 (rval (eval (read-from-string (get-output-stream-string fh))))
	 )
    (close fh)
    rval
    )
  )

(defun make-and-return-sliders (specs &optional cols)
  (let* ((fh (make-string-output-stream))
	 (junk (make-sliders-1 specs fh :cols cols))
	 (rval (read-from-string (get-output-stream-string fh)))
	 )
    (close fh)
    rval
    )
  )

;;;</TSL>
(defun make-sliders-1 (&optional specs fh &key cols )
  (let* (
	 (n (if specs
		(length specs)
	      (select (get-value-dialog "Enter # of sliders required"
					:initial 1) 0)))
	 (c (if cols
		cols
	      1))
	 (r (floor (/ n c)))
	 ;;	 (fh (if fname
	 ;;		 (open fname :direction :output)
	 ;;	       (open (get-string-dialog "Output file name for code?")
	 ;;		     :direction :output)))
	 )
    (format fh "(let* (~%")
    (dotimes (j n)
	     (let* ((alist (if specs
			       (select specs j)
			     nil))
		    (id (format nil "Slider ~d" j))
		    (name (if alist
			      (select alist 0)
			    (get-string-dialog
			     (format nil "Title for ~s" id))))
		    (obj (if alist
			     (select alist 1)
			   (select
			    (get-value-dialog
			     (format nil
				     "Sequence or Interval for ~s?" id)) 0)))
		    (action (if alist
				(select alist 2)
			      (get-string-dialog
			       (format nil "Action for ~s" id))))
		    (prot-name (if (< (length obj) 3)
				   "interval-scroll-item-proto"
				 "sequence-scroll-item-proto")))
	       (format fh "       (to~d (send text-item-proto :new~%" j)
	       (format fh "                   ~s))~%" name)
	       (format fh "       (vo~d (send text-item-proto :new~%" j)
	       (format fh "                   \"\" :text-length 10))~%")
	       (format fh "       (so~d (send ~d :new~%" j prot-name)
	       (format fh "                   '~d~%" obj)
	       (format fh "                   :text-item vo~d~%" j)
	       (format fh "                   :action~%")
	       (if (eql j (- n 1))
		   (format fh "                      #'~d)))~%" action)
		 (format fh "                      #'~d))~%" action))))
    (format fh "  (send dialog-proto :new~%")
    (format fh "        (list~%")
    (dotimes (j r)
	     (let* ((tmp (iseq (* j c) (- (* (+ j 1) c) 1)))
		    (s1 (format nil "~d"
				(combine
				 (map 'list #'(lambda(x)
						(list (format nil "to~d" x)
						      (format nil "vo~d" x)))
				      tmp))))
		    (os1 (string-left-trim "("
					   (string-right-trim ")" s1)))
		    (s2 (if (eql c 1)
			    (format nil "so~d" (select tmp 0))
			  (format nil "~d"
				  (map 'list
				       #'(lambda(x)
					   (format nil "so~d" x))
				       tmp))))
		    (os2 (string-left-trim "("
					   (string-right-trim ")" s2))))
	       (format fh "          (list ~d)~%" os1)
	       (if (<  c 2)
		   (if (eql (* (+ j 1) c) n)
		       (format fh "          ~d)))~%" os2)
		     (format fh "          ~d~%" os2))
		 (format fh "          (list ~d)~%" os2))))
    (when (not (eql (* r c) n))
	  (let* ((tmp (iseq (* r c) (- n 1)))
		 (len (length tmp))
		 (s1 (format nil "~d"
			     (combine (map 'list
					   #'(lambda(x)
					       (list (format nil "to~d" x)
						     (format nil "vo~d" x)))
					   tmp))))
		 (os1 (string-left-trim "("
					(string-right-trim ")" s1)))
		 (s2 (if  (eql (length tmp) 1)
			 (format nil "so~d" (select tmp 0)) ;;<TSL> changed ~d to ~d
		       (format nil "~d"
			       (map 'list
				    #'(lambda(x) (format nil "so~d" x))
				    tmp))))
		 (os2 (string-left-trim "("
					(string-right-trim ")" s2))))

	    (format fh "          (list ~d)~%" os1)
	    (if (< (length tmp) 2)
		(format fh "          ~d)))~%" os2)
	      (format fh "          (list ~d))))~%" os2))))
;;    (close fh)
))
















;;
;;        Cox (proportional hazards) regression objects 
;;
;;               Copyright (c) thomas lumley 1996, 1997
;;
;;                      Version 0.4 (beta)
;;                         22 June 1997
;;
;; You may copy and use this file under the terms of the GNU
;; Public License version 2 (see file COPYING) or any later version. 
;; This is free software; there is NO WARRANTY WHATSOEVER.
;;
;; Basic structure modelled on the regression-model-proto
;;
;; For documentation see the file coxreg.{tex,ps}
;;
;;
;; (require "modelformula") ;; Here it is:
;
; Model formula tools for XLISP-Stat regression methods
;
;
;simplest possible version

(defun lumley-factor (x &key namebase ) "Defines treatment contrast matrix for x and optionally a set of names based on namebase"
  (let* (
	 (xlist (sort-data (remove-duplicates (coerce x 'list) :test #'equalp)))
       	 (p (- (length xlist) 1))
	 (rows (row-list (select (identity-matrix (+ p 1)) (iseq 0 p) (iseq 1 p))))
	 (decoder (mapcar #'cons xlist rows))
	 (xmatrix (apply #'bind-rows (mapcar #'(lambda (xx) (cdr (assoc xx decoder :test #'equalp))) x)))
	 (xnames (if (null  namebase) nil (cdr (mapcar #'(lambda (xx) (write-to-string (list namebase xx) :escape nil)) xlist))))
    )
    (if (null namebase) xmatrix (list xmatrix xnames))
)
)


(defun interaction ( xs &key (is-lumley-factor (repeat t (length xs))) namebases ) "Interactions of any order for categorical and continuous variables"
  (let*  ( (n (length (first xs)))
	  (zlist nil)
	  (znames nil)
	  (p (length xs))
	  (names (if (null namebases) (repeat "." p) namebases))
	  (junk (dotimes (i p)
			(let* (
			       (thisz  (if (elt is-lumley-factor i) (lumley-factor (elt xs i) :namebase (elt names i)) (list (bind-columns (elt xs i)) (list (elt names i)))))
			       )
			  (if (null zlist) (setf zlist (column-list (first thisz)))   (setf zlist (apply #'append (mapcar #'(lambda (xx) (mapcar #'(lambda (yy) (*  xx yy)) (column-list (first thisz)))) zlist))))
			  (if (null znames) (setf znames (second thisz))  (setf znames (apply #'append (mapcar #'(lambda (xx) (mapcar #'(lambda (yy) (write-to-string (list xx yy) :escape nil)) (second thisz))) znames))))
			  )
			)
		)
	  (zmat (apply #'bind-columns zlist))
	  )
    (if (null namebases) zmat (list zmat znames))
))
	  
			  

;
(defun block-test (index beta covmat &key blockname names (block-only nil))
  (let* (
	 (subbeta (select beta index))
	 (subcov  (select covmat index index))
	 (waldchisq (inner-product subbeta (matmult (inverse subcov) subbeta)))
	 (waldp (- 1 (chisq-cdf waldchisq (length index))))
	 (subse (sqrt (diagonal subcov)))
	 (subz (/ subbeta subse))
	 (subp (* 2 (- 1 (normal-cdf (abs subz)))))
	 (nn (if (null names) (repeat "" (length index)) (select names index)))
	 (blockn (if (null blockname) "block" blockname))
	 )
    (format t "~a~20t~13,5g~35t~,4f~%" blockn waldchisq waldp)
    (cond ((null block-only)
	(format t "~5t Variable~25t Estimate~40t Std.Err.~55t p-value~%")
	(dolist (i (iseq 0 (- (length index) 1)))
	(format t "~5t~a~25t~13,5g~40t(~,6g)~55t~,4f~%" (select nn i) (select subbeta i) (select subse i) (select subp i)))
	)
      )
    )
)


(defun design (varlist) (apply #'bind-columns (mapcar #'first varlist)))
(defun names (varlist) (apply #'append (mapcar #'second varlist)))


;;
;; semi-refined version
;;
;; input: a quoted list of lumley-factor, term and interaction statements
;; output: a design matrix, a set of variable names, a set of block names and
;;         a set of block indices.
;;
; a pointless function which doesn't actually get evaluated
(defun term (x) "A metric predictor variable" (x &key namebase) (list (list x)
								      namebase))

(defun col-count (matlist) "adds up columns"  (apply #'sum (mapcar #'(lambda
								       (x)
								       (cond
									((matrixp x) (second (array-dimensions x))) (t 1))) matlist)))
;
; This function does the work. Note that the design matrix is set up backwards
; since cons sticks things on the front of a list.
; 
;


(defun formula (model-formula) 
"Calculates a design matrix, variable names and information for running (block-test) on the fitted model. 
Argument: a quoted list of metric variables, lumley-factor variables and interactions. Metric variables are specified by (term x), lumley-factor variables by (lumley-factor a) and interactions by (interaction (list a b x) :is-lumley-factor (list t t nil)). "
(let* (
       (design-matrix nil)
       (name-list nil)
       (block-names nil)
       (block-indices nil)
       (i 0)
       (junk (dolist (x model-formula) 
		     (setf i (if (null design-matrix) 0 (col-count design-matrix)))
		     (cond
		      ((eql (car x) 'term)  
		       (let* ((namepos (position ':namebase x))
			      )
		       (setf 
			design-matrix (cons (eval (second x)) design-matrix)
			name-list (cons  (if (null (find ':namebase x)) (list (string (second x))) (list (fourth x))) name-list)
			block-names (cons (string (if (null namepos) (second x) (elt x (+ 1 namepos)))) block-names)
			block-indices (cons (list i) block-indices)
			)
		       )
		       )
		       ((eql (car x) 'lumley-factor)
			(let* ((namepos (position ':namebase x))
			       (lumley-factorx (eval (if (null namepos) (append x (list :namebase (string (second x)))) x)))
			       )
			  (setf 
			   design-matrix (cons (first lumley-factorx) design-matrix)
			   name-list (cons (second lumley-factorx) name-list)
			   block-names (cons (string (if (null namepos) (second x) (elt x (+ 1 namepos)))) block-names)
			   block-indices (cons (iseq i (- (col-count design-matrix) 1)) block-indices)
			   )
			  )
			)
		       ((eql (car x) 'interaction)
			(let* ( 
			       (namepos (position ':namebases x))
			       (interx (eval (if (null namepos) (append x (list :namebases (cons 'list (mapcar #'string (cdr (second x)))))) x)))
			       )
			  (setf 
			   design-matrix (cons (first interx) design-matrix)
			   name-list (cons (second interx) name-list)
			   block-names (cons (write-to-string (cons 'interaction (if (null namepos) (cdr (second x)) (cdr (elt x (+ 1 namepos)))) ) :escape nil)  block-names)
			   block-indices (cons (iseq i (- (col-count design-matrix) 1)) block-indices)
			   )
			  )
			)
		       (t (error "I don't understand ~s" x))
		      )
		     )
	     )
       )
  (list (apply #'bind-columns design-matrix) (apply #'append name-list) block-names (- (- (col-count design-matrix) 1) block-indices))
)
)


(defun formula-display (beta cov-mat model-formula &key (block-only t) (intercept t)) 
"Display a model summary for a model with coefficient estimates beta and covariance estimates cov-mat and design matrix from model-formula.  This should really be a method for the relevant object. Little checking is performed: garbage in; garbage out"
(let* ( 
       (xintercept (if intercept 1 0))
       (nblocks (length (fourth model-formula)))
       (varnames (if intercept (cons "Intercept" (second model-formula)) (second model-formula)))
       (block-indices (+ xintercept (fourth model-formula)))
       )
  (format t "~a~20t~a~35t~a~%" "Block" "Wald Chisq" "p-value")
  (if (equal (length beta) (+ xintercept (second (array-dimensions (first model-formula)))))
      (dolist (i (iseq (- nblocks 1) 0))
	      (block-test (elt block-indices i) beta cov-mat :names varnames :blockname (elt (third model-formula) i) :block-only block-only))
    (error "length of beta doesn't match design matrix")
    )
  )
)

;--------- model-formula prototype ------------

(defproto model-formula-proto '(formula design-matrix name-list block-names block-indices) '() *object* "Model formula")

(defmeth model-formula-proto :formula (&optional (new nil set))
"Set or return model formula"
  (when set
	(setf (slot-value 'formula) new)
	(send self :compute))
  (slot-value 'formula))

(defmeth model-formula-proto :isnew (&key formula)
  (send self :formula formula)
)

(defun as-formula (x) (send model-formula-proto :new :formula x))
;

(defmeth model-formula-proto :design-matrix (&optional (new nil set))
"Sets or returns design-matrix"
  (when set
	(setf (slot-value 'design-matrix) new))
  (slot-value 'design-matrix))

(defmeth model-formula-proto :name-list (&optional (new nil set) )
"Sets or returns list of column names"
  (when set
	(setf (slot-value 'name-list) new))
  (slot-value 'name-list))

(defmeth model-formula-proto :block-names (&optional (new nil set))
"Sets or returns list of block names"
  (when set
	(setf (slot-value 'block-names) new))
  (slot-value 'block-names))

(defmeth model-formula-proto :block-indices (&optional (new nil set) )
"Sets or returns list of column names"
  (when set
	(setf (slot-value 'block-indices) new))
  (slot-value 'block-indices))


;

(defmeth model-formula-proto :compute ()
"Internal use"
(let* ( 
       (result (formula (send self :formula)))
       )
  (send self :design-matrix (first result))
  (send self :name-list (second result))
  (send self :block-names (third result))
  (send self :block-indices (fourth result))
))

;
; methods for regression-proto and some descendants
; This will work if :sigma-hat^2*:xtxinv is the covariance matrix
; which is also true for the glim and nonlinear regression prototypes
;

(defun regression-formula (formula-object y &rest args &key (predictor-names nil has-names) (print-summary t) (intercept t) (print t))
"Like regression-model but uses a formula instead of the x argument"
(let* (
       (name-list (if has-names predictor-names (send formula-object :name-list)))
       (result (apply #'regression-model (list (send formula-object ':design-matrix) y  ':predictor-names name-list ':print (null print-summary) args)))
  )
  (send result :add-slot 'formula formula-object)
  (if print-summary (send result :display-with-formula t))
  result
))


(defmeth regression-model-proto :formula (&optional (new nil set))
"Sets or returns the model formula object or nil for none"
  (when (and set (send self :has-slot 'formula))
	(setf (slot-value 'formula) new))
  (slot-value 'formula))

(defmeth regression-model-proto :display-with-formula  (&optional (block-only t))
"Display a summary of the model based on its formula"
(let (
      (formula (if (send self :has-slot 'formula) (send self :formula) nil))
      )
  (cond ((null formula)
	 (format t "No formula present: using default method~%")
	 (send self :display))
	(t 
	 (let* ( 
		(xintercept (if (send self :intercept) 1 0))
		(nblocks (length (send formula :block-indices)))
		(block-indices (+ xintercept (send formula :block-indices)))
		(cov-mat (* (^ (send self :sigma-hat) 2) (send self :xtxinv)))
		(varnames (if (send self :intercept) (cons "Intercept" (send self :predictor-names)) (send self :predictor-names)))
		)
	   (format t "~a~20t~a~35t~a~%" "Block" "Wald Chisq" "p-value")
	   (dolist (i (iseq (- nblocks 1) 0))
		   (block-test (elt block-indices i) (send self :coef-estimates) cov-mat :names varnames :blockname  (elt (send formula :block-names) i) :block-only block-only))
	   )
	 )
	)
  )
)

(defun poissonreg-formula (formula-object y &rest args &key
(predictor-names nil has-names) (print-summary t) (intercept t) (print t))
"Like poissonreg-model but uses a formula instead of the x argument"
(let* (
       (name-list (if has-names predictor-names (send formula-object
:name-list)))
       (result (apply #'poissonreg-model (list (send formula-object
':design-matrix) y  ':predictor-names name-list ':print (null
print-summary) args)))
  )
  (send result :add-slot 'formula formula-object)
  (if print-summary (send result :display-with-formula t))
  result
))


(provide "modelformula")
(provide "coxreg")

;;;
;;; useful functions
;;;

(defun cumprod (s) (accumulate #'* s))

(defun unique (x) (remove-duplicates x :test #'equalp))

(defun argselect (xs x) "returns indices where xs=s" (which (= x xs)))

(defun zero-matrix (k) (* 0 (identity-matrix k)))

(defun one-matrix (k) (outer-product (repeat 1 k) (repeat 1 k)))

(defun firsts (x) (map-elements #'(lambda (xi) (elt xi 0)) x))
(defun seconds (x) (map-elements #'(lambda (xi) (elt xi 1)) x))
(defun thirds (x) (map-elements #'(lambda (xi) (elt xi 2)) x))
(defun nths (x n) (map-elements #'(lambda (xi) (elt xi (- n 1))) x))

(defun logg (x) (if (compound-data-p x)
		    (map-elements #'logg x)
		    (if (> x 0) (log x) 0)))

(defun col2row (x) (row-list (apply #'bind-columns x)))

(defun row2col (x) (column-list (apply #'bind-rows x)))

(defun append-seq (&rest seqs) (apply #'append (mapcar #'(lambda (x) (coerce x 'list)) seqs)))

(defun flatten (listlist) (apply #'append listlist))

(defun list-of-seqs-p (x) 
     (when (listp x) (when (sequencep (car x)) (numberp (car (car x))))))


;  the first four elements of a and b are time status entry stratum

(defun after (a b) 
  (if (= (elt a 3) (elt b 3)) 
      (if (= (elt a 0) (elt b 0))
	  (if (= (elt a 1) (elt b 1)) 
	      (> (elt a 2) (elt b 2))
	      (< (elt a 1) (elt b 1))
	      )
	(> (elt a 0) (elt b 0))
	)
    (> (elt a 3) (elt b 3))
    )
  )

(defun tied (a b)
  (if (and (= (elt a 0) (elt b 0)) (= 1 (elt a 1) (elt b 1)) (= (elt a 2) (elt b 2)) (= (elt a 3) (elt b 3))) 1 0))

(defun scatter-smooth (x y &rest graphargs &key (symmetric nil) (span .6)) 
"Arguments: x y &rest graphargs &key (symmetric nil) (span .6)
Draws a scatterplot with a lowess smoother. If symmetric is t uses the
two-iteration robust version (not good for skewed residuals).
Any extra arguments are passed to plot-points"
  (let* (
	(steps (if symmetric 2 1))
	(smooth (lowess x y :f span :steps steps))
	(plot (apply #'plot-points (append (list x y) graphargs)))
	)
    (send plot :add-lines (first smooth)  (second smooth) )
    plot
  )
)

(defun index-plot (y &rest graphargs)
"Arguments: y &rest graphargs
Plots y against 1:(length y). Other arguments are passed to plot-points"
  (apply #'plot-points (append (list (iseq 1 (length y)) y) graphargs)))


(defun plot-steps (x y &rest graphargs &key (right t))
  "Arguments: x y &rest graphargs &key (right t)
Plots a step function with the given step points. Default is right-continuous,
optionally left continuous"
  (let* (
	 (plot (apply #'plot-points (append (list x y) graphargs)))
	 (xstep (if right 1 0))
	 (ystep (if right 0 1))
	 )
    (dotimes (i (- (length x) 1))
	     (send plot :add-lines (select x (list i (+ i xstep) (+ i 1)))
		   (select y (list i (+ i ystep) (+ i 1))))
	     )
  plot
  )
)

(defun add-steps (x y plot &key (right t))
  "Args: x y plot &key (right t)
Adds a step function to plot, which must inherit from
scatterplot-proto"
  (when (null (kind-of-p plot scatterplot-proto)) (error "Not a plot"))
  (let* (
	 (xstep (if right 1 0))
	 (ystep (if right 0 1))
	 )
    (send plot :add-points x y)
    (dotimes (i (- (length x) 1))
	     (send plot :add-lines (select x (list i (+ i xstep) (+ i 1)))
		   (select y (list i (+ i ystep) (+ i 1))))
	     )
    plot
    )
  )

(defun n+ (i j n) (if (null i) n (+ i j)))

(defun lumley-interp (xnew x y &key (right t))
  "Lumley-Interps the step function with corners at (x y) at the values 
 in xnew. Default is a right-continuous step function like the
 Kaplan-Meier and Nelson-Aalen estimators"
  (let* ((fudge (if right -1 0))
	 (n (- (length y) 1)))
    (map-elements #'(lambda (xnewi) (elt y (n+ (position xnewi x :test #'<)  fudge n))) xnew)
    )
  )

;;;
;;; Tie-handling:  allows various options for handling ties in an
;;; object-oriented framework.  Mathematical details taken from the 
;;; documentation to the S survival4 library by Terry Therneau.
;;;
;;;  Different methods of handling ties can affect the computation of
;;;  \bar Z, d\Lambda and the Schoenfeld residuals.
;;;
;;;  The prototype version is the counting-process  approximation
;;;  Aliases for this are
;;;                      counting-process-ties
;;;                      breslow-ties
;;;                      peto-ties
;;;  to cover all the names by which it is known in the literature.
;;; 
;;; Other methods 
;;;                 efron-ties 
;;;
;;;   (still to be implemented):  
;;;                 exact-ties
;;;


(defproto cox-tie-proto) ;no slots needed.

(defmeth cox-tie-proto  :increment (zlist betahat ns0 ns1 ns2)
  (let* (      
	 (expbtz
	  (map-elements #'(lambda (z) (exp (inner-product z betahat))) zlist))
	 (zbtz (* expbtz zlist))
	 )
    (list
     (- (apply #'+ zlist) (* (length zlist) (/ ns1 ns0)))
     (* (length zlist) (- (/ ns2 ns0) (outer-product (/ ns1 ns0) (/ ns1 ns0))))
     )
    )
  )

(defmeth cox-tie-proto :schoenfeld (zlist betahat ns0 ns1)
  "Args: zlist betahat
Computes the appropriate Schoenfeld residual(s) for the tied failures"
  (let* (
	 (expbtz
	  (map-elements #'(lambda (z) (exp (inner-product z betahat))) zlist))
	 (n (length zlist))
	 )
    (- zlist (repeat (list (/ ns1 ns0)) n))
    )
  )
 
(defmeth cox-tie-proto :dLambda (zlist betahat ns0)
  "Args: zlist betahat ns0 
 Computes the increment of the baseline cumulative hazard for the tied failures.
 equiv-at-risk is the sum of exp(b'z) just before this time, so
 (/ nS0) is the increment if there is only one failure
"
  (/ (length zlist) ns0)
  )

(defmeth cox-tie-proto :loglike (zlist betahat ns0)
  "Args: zlist betahat ns0
Computes the increment in the loglikelihood"
  (let* ((expbtz
	  (map-elements #'(lambda (z) (exp (inner-product z betahat))) zlist))
	 )
    (- (apply #'+ expbtz) (* (length zlist) (log ns0)))
    )
  )

(defmeth cox-tie-proto :prettyprint (&optional (stream t))
  (format stream "Tie-handling prototype~%")
  )

(defproto breslow-ties () () cox-tie-proto)

(defmeth breslow-ties :prettyprint (&optional (stream t))
  (format stream "Breslow's (counting process) approximation~%")
  )

(defproto peto-ties () () cox-tie-proto)

(defmeth peto-ties :prettyprint (&optional (stream t))
  (format stream "Peto's (counting process) approximation~%")
  )

(defproto counting-process-ties () () cox-tie-proto)

(defmeth counting-process-ties :prettyprint (&optional (stream t))
  (format stream "Counting process (Breslow's approximation)~%")
  )

(defproto efron-ties () () cox-tie-proto)

(defmeth efron-ties :prettyprint (&optional (stream t))
  (format stream "Efron's approximation~%")
  )

(defmeth efron-ties :increment (zlist betahat ns0 ns1 ns2)
  (let* (   
	 (expbtz
	  (map-elements #'(lambda (z) (exp (inner-product z betahat))) zlist))
	 (zbtz (* expbtz zlist))
	 (z2btz (* expbtz (map-elements #'outer-product zlist zlist)))
	 (n (length zlist))
	 (w (/ (iseq  n) n))
	 (zsum (apply #'+ zbtz))
	 (z2sum (apply #'+ z2btz))
	 (ebtzsum (apply #'+ expbtz))
	 (zbars
	  (/
	   (mapcar #'(lambda (wi) (- ns1 (* wi zsum))) w)  (- ns0 (* w ebtzsum))))
	 (z2bars
	  (/
	   (mapcar #'(lambda (wi) (- ns2 (* wi  z2sum))) w) (- ns0 (* w ebtzsum))))
	 )
    (list (- (apply #'+ zlist) (apply #'+ zbars))   (- (apply #'+ z2bars) (apply #'+ (mapcar #'outer-product zbars zbars))))
    )
  )

(defmeth efron-ties :schoenfeld (zlist betahat ns0 ns1)
  "Args: zlist betahat
Computes the appropriate Schoenfeld residual(s) for the tied failures"
  (let*  (      
	  (expbtz (map-elements #'(lambda (z) (exp (inner-product z betahat))) zlist))
	  (zbtz (* expbtz zlist))
	  (n (length zlist))
	  (w (/ (iseq  n) n))
	  (zsum (apply #'+ zbtz))
	  (ebtzsum (apply #'+ expbtz))
	  (zbars (/ (mapcar #'(lambda (wi) (- ns1 (* wi zsum))) w) (- ns0 (* w ebtzsum))))
	  )
    (- zlist zbars)
    )
  )

(defmeth efron-ties :dLambda (zlist betahat ns0)
"Args: zlist betahat ns0 
Computes the increment of the baseline cumulative hazard for the tied failures.
equiv-at-risk is the sum of exp(b'z) just before this time, so
(/ nS0) is the increment if there is only one failure"
(let*  (      
	(expbtz (map-elements #'(lambda (z) (exp (inner-product z betahat))) zlist))
	(n (length zlist))
	(w (/ (iseq  n) n))
	(ebtzsum (apply #'+ expbtz))
	)
  (sum (/ (- ns0 (* w ebtzsum))))
  )
)

(defmeth efron-ties :loglike (zlist betahat ns0)
"Args: zlist betahat ns0
Computes the increment in the loglikelihood"
(let*  (      
	(expbtz (map-elements #'(lambda (z) (exp (inner-product z betahat))) zlist))
	(n (length zlist))
	(w (/ (iseq  n) n))
	(ebtzsum (apply #'+ expbtz))
	)
  (- (log ebtzsum) (log (sum (- ns0 (* w ebtzsum)))))
  )
)


;;;;
;;;;
;;;;                The General Cox Regression Prototype
;;;;
;;;;
;;;;  works with a data structure with one record per entry/censoring/event
;;;;  constructed virtually as an index into the supplied x matrix and a
;;;;  type indicator for what happens. This allows left truncation, right 
;;;;  censoring, time-dependent strata and step-function time-dependence in 
;;;;  covariates to be coded using multiple records per case.
;;;;   Arbitrary time-dependent covariates are also possible.
;;;;
;;;;  The Newton-Raphson step is coded separately for models that use the 
;;;;  arbitrary time-dependent covariates. The reason for the separation 
;;;;  is that the first type of calculation can be done by a much faster algorithm. 
;;;;  Many packages (such as  Stata, S-PLUS and SPIDA) allow only this  type
;;;;  of time-dependence. 
;;;;  The fast algorithm could be numerically unstable in certain situations and
;;;;  so the slower general algorithm provides an additional numerical check.
;;;; 
;;;;  The :comments  field describes what model was fitted, as the regression
;;;;  function has no way of distinguishing eg Le, Wei & Amato and Wei, Lin &
;;;;  Weissfeld style multivariate analyses.
;;;;
;;;;  It is a bad idea to construct an object from this prototype directly. Use the
;;;;  supplied functions.

(defproto cox-regression-proto 
  '(times entry status x formula beta id strata weights
         epsilon epsilon-dev count-limit verbose recycle
         eta loglike agnostic-covariance-matrix robust 
	 covariance-matrix tie-method recycle agnostic
	 needs-computing cumulative-hazard base-martingale
	 martingales
	 schoenfeld
	 scores suppress-warnings
	 predictor-names case-labels response-name
	 statusint keyint entryint tieint comments time-dependent zbars
)
  '() 
  *object*
  "Cox regression model")


;;;;
;;;; Slot Accessors
;;;;

(defmeth cox-regression-proto :times (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns failure/censoring times"
  (when set
	(setf (slot-value 'times) new)
	(send self :needs-computing t))
  (slot-value 'times))


(defmeth cox-regression-proto :count-limit (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns iteration limit"
  (when set
	(setf (slot-value 'count-limit) new)
	(send self :needs-computing t))
  (slot-value 'count-limit))

(defmeth cox-regression-proto :entry (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns entry time"
  (when set
	(setf (slot-value 'entry) new)
	(send self :needs-computing t))
  (slot-value 'entry))

(defmeth cox-regression-proto :intercept () nil)

(defmeth cox-regression-proto :status (&optional (new nil set))
  "Message args: (&optional new nil set)
 Sets or returns failure indicator"
  (when set
	(setf (slot-value 'status) new)
	(send self :needs-computing t))
  (slot-value 'status))

(defmeth cox-regression-proto :id (&optional (new nil set))
"Message args: &optional (new nil set)
Sets or returns case id"
(when set
      (setf (slot-value 'id) id))
(slot-value 'id))

(defmeth cox-regression-proto :beta () 
  "Returns coefficient estimates"
  (slot-value 'beta))

(defmeth cox-regression-proto :keyint (&optional (new nil set))
"INTERNAL [ Message args: (&optional new)
  Sets or returns index from munged times into covariates]"
  (when set
	(setf (slot-value 'keyint) new)
	(send self :needs-computing t))
  (slot-value 'keyint))


(defmeth cox-regression-proto :statusint (&optional (new nil set))
  "INTERNAL [ Message args: (&optional new)
 Sets or returns failure status at munged times]"
  (when set
	(setf (slot-value 'statusint) new)
	(send self :needs-computing t))
  (slot-value 'statusint))

(defmeth cox-regression-proto :entryint (&optional (new nil set))
  "INTERNAL [Message args: (&optional new)
 Sets or returns entry status at munged times]"
  (when set
	(setf (slot-value 'entryint) new)
	(send self :needs-computing t))
  (slot-value 'entryint))


(defmeth cox-regression-proto :tieint (&optional (new nil set))
  "INTERNAL [Message args: (&optional new)
 Sets or returns tie status at munged times]"
  (when set
	(setf (slot-value 'tieint) new)
	(send self :needs-computing t))
  (slot-value 'tieint))

(defmeth cox-regression-proto :weights (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns case weights. These have no effect whatsoever."
  (when set
	(setf (slot-value 'weights) new)
	(send self :needs-computing t))
  (slot-value 'weights))



(defmeth cox-regression-proto :strata (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns stratum values"
  (when set
	(setf (slot-value 'strata) new)
	(send self :needs-computing t))
  (slot-value 'strata))

(defmeth cox-regression-proto :agnostic (&optional (new nil set))
  "Message args: (&optional new)
 Use agnostic covariance estimator"
  (when set
	(setf (slot-value 'agnostic) new)
	(send self :needs-computing t))
  (slot-value 'agnostic))


(defmeth cox-regression-proto :tie-method (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns tie-method"
  (when set
	(setf (slot-value 'tie-method) new)
	(send self :needs-computing t))
  (slot-value 'tie-method))


(defmeth cox-regression-proto :num-cases ()
  (length (send self :id)))

(defmeth cox-regression-proto :x (&optional x)
  (if x
      (let ((x (cond
                ((matrixp x) x)
                ((vectorp x) (list x))
                ((and (consp x) (numberp (car x))) (list x))
                (t x))))
        (setf (slot-value 'x) (if (matrixp x) x (apply #'bind-columns x))))
  (slot-value 'x) )
)

(defmeth cox-regression-proto :formula (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns model formula"
  (when set
	(setf (slot-value 'formula) new)
	(send self :needs-computing t))
  (slot-value 'formula))


(defmeth cox-regression-proto :comments (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns model description text"
  (when set
	(setf (slot-value 'comments) new)
	)
  (slot-value 'comments))


(defmeth cox-regression-proto :epsilon (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns convergence criterion for beta"
  (when set
	(setf (slot-value 'epsilon) new)
	(send self :needs-computing t))
  (slot-value 'epsilon))


(defmeth cox-regression-proto :epsilon-dev (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns convergence criterion for loglikelihood"
  (when set
	(setf (slot-value 'epsilon-dev) new)
	(send self :needs-computing t))
  (slot-value 'epsilon-dev))


(defmeth cox-regression-proto :recycle (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns indicator for reusing old values when refitting"
  (when set
	(setf (slot-value 'recycle) new)
	(send self :needs-computing t))
  (slot-value 'recycle))



(defmeth cox-regression-proto :id (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns case identifier (important for agnostic
covariance estimates or to get single residuals per case
with time-dependent covariates)"
  (when set
	(setf (slot-value 'id) new)
	(send self :needs-computing t))
  (slot-value 'id))


(defmeth cox-regression-proto :covariance-matrix (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns covariance matrix"
  (when set
	(setf (slot-value 'covariance-matrix) new)
	(send self :needs-computing t))
  (slot-value 'covariance-matrix))


(defmeth cox-regression-proto :agnostic-covariance-matrix (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns agnostic (model-robust) covariance matrix"
  (when set
	(setf (slot-value 'agnostic-covariance-matrix) new)
	(send self :needs-computing t))
  (slot-value 'agnostic-covariance-matrix))


(defmeth cox-regression-proto :verbose (&optional (new nil set))
  "Message args: (&optional new)
 Sets or returns verboseness control (default is t to 
report iteration information)"
  (when set
	(setf (slot-value 'verbose) new)
	(send self :needs-computing t))
  (slot-value 'verbose))

(defmeth cox-regression-proto :suppress-warnings (&optional (new nil set))
  (when set
	(setf (slot-value 'suppress-warnings) new)
	(send self :needs-computing t))
  (slot-value 'suppress-warnings))



(defmeth cox-regression-proto :loglike (&optional (new nil set))
  "Message args: (&optional new)
Sets or returns loglikelihood"
  (when set
	(setf (slot-value 'loglike) new)
	(send self :needs-computing t))
  (slot-value 'loglike))

(defmeth cox-regression-proto :needs-computing (&optional (new nil set))
  "Message args: (&optional new)
Flag: t if model needs updating"
  (when set
	(setf (slot-value 'needs-computing) new)
	)
  (slot-value 'needs-computing))



(defmeth cox-regression-proto :coef-standard-errors ()
(sqrt (diagonal (send self :covariance-matrix))))

(defmeth cox-regression-proto :agnostic-coef-standard-errors ()
 (sqrt (diagonal (send self :agnostic-covariance-matrix))))

(defmeth cox-regression-proto :predictor-names (&optional (names nil set))
"Message args: (&optional (names nil set))
With no argument returns the predictor names. NAMES sets the names."
  (if set (setf (slot-value 'predictor-names) (mapcar #'string names)))
  (let* ((pfixed (array-dimension (send self :x) 1))
	(ptime (length (first (send self :time-dependent))))
	(p (+ pfixed ptime))
        (p-names (slot-value 'predictor-names)))
    (if (not (and p-names (= (length p-names) p)))
        (setf (slot-value 'predictor-names)
              (mapcar #'(lambda (a) (format nil "Variable ~a" a)) 
                      (iseq 0 (- p 1))))))
  (slot-value 'predictor-names))

(defmeth cox-regression-proto :response-name (&optional (name "Y" set))
"Message args: (&optional name)
With no argument returns the response name. NAME sets the name."
  (if set (setf (slot-value 'response-name) (if name (string name) "Y")))
  (slot-value 'response-name))

(defmeth cox-regression-proto :case-labels (&optional (labels nil set))
"Message args: (&optional labels)
With no argument returns the case-labels. LABELS sets the labels."
  (if set (setf (slot-value 'case-labels) 
                (if labels 
                    (mapcar #'string labels)
                    (mapcar #'(lambda (x) (format nil "~d" x)) 
                            (iseq 0 (- (send self :num-cases) 1))))))
  (slot-value 'case-labels))



(defmeth cox-regression-proto :set-beta (val)
"Args: beta
Set coefficient estimates: only done from within compute-step-beta
 and :initialize-search"
  (setf (slot-value 'beta) val)
  (slot-value 'beta)
)

(defmeth cox-regression-proto :fit-values ()
"Message args: ()
Returns the fitted linear predictor values for the model."
  (matmult (send self :x) (send self :beta)))

(defmeth cox-regression-proto :eta ()
"Message args: ()
Returns linear predictor values for current fit."
    (slot-value 'eta))

(defmeth cox-regression-proto :set-eta (&optional val)
  (if val
      (setf (slot-value 'eta) val)
      (setf (slot-value 'eta)
;;;;	    (+ (send self :offset) (matmult (send self :x) (send self :beta)))))
	    (matmult (send self :x) (send self :beta))))
  (slot-value 'eta))

(defmeth cox-regression-proto :time-dependent (&optional (new nil set))
"Message args: &optional (new nil set)
Sets or returns the information on continuously time-dependent
covariates, consisting of a list containing a list of functions and a covariate matrix"
     (when set (setf (slot-value 'time-dependent) new))
     (slot-value 'time-dependent)
)

(defmeth cox-regression-proto :fit-hazard-ratio (&optional (eta (send self :eta)))
"Message args: (&optional (eta (send self :eta)))
Returns mean values for current or supplied ETA."
(if (null eta) (exp (send self :eta))  (exp eta)))


(defmeth cox-regression-proto :conf-interval (&key (coverage 0.95) (names t))
"Arguments: &key  (coverage 0.95) (names t)
Produces a matrix of Wald confidence intervals for beta with predictor
names if requested  and  transforms them to the hazard ratio scale"
(let* ((beta (send self :beta))
       (se (sqrt (diagonal (send self :covariance-matrix))))
       (zalpha (normal-quant (+ coverage (/ (- 1 coverage) 2))))
       (baseci (apply #'bind-columns (list beta (- beta (* zalpha se)) (+ beta (* zalpha se)))))
       (xform (exp baseci))
       (name-list (if (send self :intercept) (cons "Intercept" (send self :predictor-names)) (send self :predictor-names)))
       )
  (if names (bind-columns name-list xform) xform)
))

;;
;; Graphs and diagnostics
;;
(defmeth cox-regression-proto :cumulative-hazard (&optional (new nil set))
  (when set (setf (slot-value 'cumulative-hazard) new))
  (slot-value 'cumulative-hazard))

(defmeth cox-regression-proto :cumulative-hazard-function (&key (xvec nil))
  "Cumulative hazards evaluated at given covariate vector"
  (let* (
	 (hazards (send self :cumulative-hazard))
	 (beta (send self :beta))
	 (rr (exp (if xvec (inner-product xvec beta) 0)))
	 )
    (map-elements
     #'(lambda(stratum)
	 (list (first stratum) (second stratum) (* rr (third stratum))))
     hazards)
    )
  )



(defmeth cox-regression-proto
  :survival-function (&key (xvec nil))
  "Kaplan-Meier or near offer
Evaluated at specified covariate vector
"
  (let* (
	 (hazards (send self :cumulative-hazard-function))
	 (beta (send self :beta))
	 (rr (exp (if xvec (inner-product xvec beta) 0)))
	 )
    (map-elements
     #'(lambda(stratum)
	 (let* (
		(dcumhaz (difference (third stratum)))
		(survival (^ (append-seq #(1) (cumprod (- 1 dcumhaz))) rr))
		)
	   (list (first stratum) (second stratum) survival)
	   )
	 )
     hazards
     )
    )
  )

(defmeth cox-regression-proto :survival-function-closure
  (&key (stratum 0 has-stratum) (xvec nil))
  "Arguments: &key (stratum 0 has-stratum)  (xvec nil)
Function closure giving survival function for specified stratum and optionally covariate pattern"
  (unless (or has-stratum (= 1 (length (unique (send self :strata))))) 
	  (error "Must specify stratum"))
  (let* (
	 (kms
	  (assoc stratum (coerce (send self :survival-function :xvec xvec) 'list)))
	 (ff (if kms  #'(lambda (ti) (lumley-interp ti (second kms) (third kms)))
	       (error "Stratum ~a not found" stratum)))
	 )
    ff
    )
  )


(defmeth cox-regression-proto :cumulative-hazard-function-closure (&key (stratum 0 has-stratum)  (xvec nil))
  "Arguments: &key (stratum 0 has-stratum)  (xvec nil)
Function closure giving cumulative hazard function for specified stratum and optionally covariate pattern"
  (unless (or has-stratum (= 1 (length (unique (send self :strata))))) 
	  (error "Must specify stratum"))
  (let* (
	 (haz (assoc stratum (coerce (send self :cumulative-hazard-function :xvec xvec) 'list)))
	 (ff (if haz  #'(lambda (ti) (lumley-interp ti (second haz) (third haz)))
	       (error "Stratum ~a not found" stratum)))
	 )
    ff
    )
  )

(defmeth cox-regression-proto :schoenfeld-residuals (&optional (new nil set))
  (when set
	(setf (slot-value 'schoenfeld) new)
	)
  (slot-value 'schoenfeld))


(defmeth cox-regression-proto :zbars (&optional (new nil set))
  "Args: (new nil set)
Sets or returns a list whose elements are the time, stratum number, number at risk 
in the stratum and the weighted mean covariate for each failure"
  (when set
	(setf (slot-value 'zbars) new)
	)
  (slot-value 'zbars))


(defmeth cox-regression-proto :scaled-schoenfeld-residuals ()
  (let* ((ri (send self :schoenfeld-residuals))
	 (n (length ri))
	 (betas (matmult (bind-columns (repeat 1 n)) (bind-rows (send self :beta))))
	 (scaled  (+ betas (matmult  (apply #'bind-rows (mapcar #'second ri)) (* n (send self :covariance-matrix)))))
	 )
    (cons  (mapcar #'first ri) (column-list scaled))
    )
  )

(defmeth cox-regression-proto :plot-cumulative-hazard (&key (one-plot t)(xvec nil))
  "Arguments: (&key (one-plot t) (xvec nil)
Plots cumulative hazard functions, for baseline or for specified
covariate pattern"
  (let* (
	 (hazards (send self :cumulative-hazard-function :xvec xvec))
	 )
    (cond ((and one-plot (> (length hazards) 1)) 
	   (let* ((firstone (car hazards))
		  (rest (cdr hazards))
		  (plot (plot-steps (second firstone) (third firstone)  :variable-labels (list "time" "Hazard")))
		  )
	     (map-elements #'(lambda (stratum)
			       (add-steps (second stratum) (third stratum) plot)) rest)
	     (send plot :adjust-to-data)
	     (list plot)
	     ))
	  (t (map-elements #'(lambda (stratum)
			       (plot-steps (second stratum) (third stratum) :title (format nil "Stratum ~a" (first stratum)) :variable-labels (list "time" "Hazard"))) hazards)
	     )
	  )
    )
  )

(defmeth cox-regression-proto :plot-survival
  (&key (inverted nil) (one-plot t) (xvec nil))
  "Arguments: &key (inverted nil) (one-plot t) (xvec nil)
Plots survival function (or mortality function)
for each stratum at baseline or specified covariate pattern.
"
  (let* ((survivals (send self :survival-function :xvec xvec)))
    (cond
     ((and one-plot (> (length survivals) 1))
      (let* ((firstone (car survivals))
	     (rest (cdr survivals))
	     (plot
	      (plot-steps (second firstone) (if inverted (- 1 (third firstone))
					      (third firstone))
			  :variable-labels
			  (list "time" (if inverted
					   "Proportion failing"
					 "Proportion surviving"))) 
	      )
	     )
	(map-elements
	 #'(lambda (stratum)
	     (add-steps (second stratum) (if inverted (- 1 (third stratum))
					   (third stratum)) plot))
	 rest)
	(send plot :adjust-to-data)
	(if (null inverted) (send plot :range 1 0 1))
	(list plot)
	))
     (t (let (
	      (plots
	       (map-elements
		#'(lambda (stratum)
		    (plot-steps
		     (second stratum)
		     (if inverted (- 1 (third stratum)) (third stratum))
		     :title (format nil "Stratum ~a" (first stratum))
		     :variable-labels
		     (list "time"
			   (if inverted "Proportion failing"
			     "Proportion surviving")))) survivals))
	      )
	  (if (null inverted) (map-elements
			       #'(lambda (plot)
				   (send plot :range 1 0 1)) plots))
	  plots
	  )
	)
     )
    )
  )

(defmeth cox-regression-proto :plot-ph-test (&optional vars)
"Arguments: vars
Plots smoothed scaled Schoenfeld residuals"
(let* (
       (r (send self :scaled-schoenfeld-residuals))
       (vns (send self :predictor-names))
       (varlist (or vars (iseq (length (send self :beta)))))
       )
  (map-elements #'(lambda (v) (scatter-smooth (first r) (elt r (+ 1 v)) :title (format nil "Proportional hazards: ~a" (elt vns v)) :variable-labels (list "Time" "Scaled Schoenfeld Residuals"))) varlist)
 )
)

(defmeth cox-regression-proto :harrell-lee-test ()
  "Harrell & Lee's test for proportional hazards based on the Schoenfeld residuals"
  (second (send self :ph-test :scaled nil :unscaled #'rank))
  ) 

(defmeth cox-regression-proto :ph-test (&key (scaled #'identity) (unscaled nil))
  "Args: &key (scaled #'identity) (unscaled nil)
Tests for proportional hazards based on scaled or raw Schoenfeld residuals"
  (list (cond ((null scaled) nil)
	      ((functionp scaled)
	       (let* ((sr (send self :scaled-schoenfeld-residuals))
		      (srs (cons (funcall scaled (first sr)) (cdr sr)))
		      (cmat (apply #'covariance-matrix srs))
		      (sds (/ (sqrt (diagonal cmat))))
		      (rmat (matmult (diagonal sds) cmat (diagonal sds)))
		      (rhos (cdr (coerce (first (row-list rmat)) 'list)))
		      (n2 (- (length (first srs)) 2))
		      (ts (* rhos (sqrt n2)))
		      (names (send self :predictor-names))
		      (ps (* 2 (t-cdf (- (abs ts)) n2)))
		      )
		 (bind-columns names rhos ts ps)
		 )
	       )
	      )
	(cond ((null unscaled) nil)
	      ((functionp unscaled)
	       (let* ((sr (send self :schoenfeld-residuals))
		      (srs
		       (cons
			(funcall unscaled (firsts sr))
			(column-list (apply #'bind-rows (seconds sr)))))
		      (cmat (apply #'covariance-matrix srs))
		      (sds (/ (sqrt (diagonal cmat))))
		      (rmat (matmult (diagonal sds) cmat (diagonal sds)))
		      (rhos (cdr (coerce (first (row-list rmat)) 'list)))
		      (n2 (- (length (first srs)) 2))
		      (ts (* rhos (sqrt n2)))
		      (names (send self :predictor-names))
		      (ps (* 2 (t-cdf (- (abs ts)) n2))) 
		      )
		 (bind-columns names rhos ts ps)
		 )
	       )
	      )
	)
  )


;;;
;;; COMPUTE methods
;;;


(defmeth cox-regression-proto :initialize-search ()
  (let* ((np (+ (length (column-list (send self :x))) (length (first (send self :time-dependent))))))
  (send self :set-beta (repeat 0 np)))
)

(defmeth cox-regression-proto :compute ()
"Refits the model"
(let* ((epsilon (send self :epsilon))
       (epsilon-dev (send self :epsilon-dev))
       (maxcount (send self :count-limit))
       (low-lim (* 2 (/ machine-epsilon epsilon)))
       (verbose (send self :verbose)))
  (unless (and (send self :beta) (send self :recycle))
	  (send self :initialize-search))
  (do ((count 1 (+ count 1))
       (beta (send self :beta)  (send self :beta))
       (last-beta -1 beta)
       (dev  0 (send self :loglike))
       (last-dev  -1 dev))
	((or (> count maxcount) 
	     (< (max (abs (/ (- beta last-beta)
			     (pmax (abs last-beta) low-lim))))
		epsilon)
	     (< (abs (- dev last-dev)) epsilon-dev)) (when (and  (> count maxcount) (null (send self :suppress-warnings))) (format t "~%~%Failed to converge in ~d iterations~%Treat results with caution ~%" maxcount)))
	(if (car (send self :time-dependent)) (send self :compute-tstep) (send self :compute-step))
	(if verbose 
	    (format t "Iteration ~d: log-likelihood = ~,6g~%" 
		    count (send self :loglike)))
	)
  )
(send self :needs-computing nil)
)
;;
;; this method allows for continuously time-dependent covariates and doesn't 
;; have the potential rounding error accumulation of :compute-step.
;; The problem is that it is  v e r y   s l o w, requiring O(n^2) time
;; 
;; We keep track of which individuals are at risk but have to recompute their 
;; covariates at each failure time.
;; 
;; At each observation time we update the risk set then if it 
;; is a failure we compute the covariates and the score and information increments
;;

(defmeth cox-regression-proto :compute-tstep (&optional (stuff *cox-diagnostics-while-fitting*))
"Does one step of Newton-Raphson algorithm for models with continuously time-dependent covariates"
(let* (
       (keyint (send self :keyint))
       (statusint (send self :statusint))
       (entryint (send self :entryint))
       (tieint (send self :tieint))
       (times (send self :times))
       (strata (send self :strata))
       (stratlist (unique strata))
       (x-list (row-list (send self :x)))
       (oldbeta (send self :beta))
       (timecovs (send self :time-dependent))
       (timefuns (first timecovs))
       (timexs (second timecovs))
       (pfixed (length (first x-list)))
       (ptime (if timecovs (length (first timecovs)) 0))
       (p (+ pfixed ptime))
       (nz (length x-list))
       (nt (length keyint))
       (dcumhaz nil)
       (schoenfelds nil)
       (loglike 0)
       (score (repeat 0 p))
       (varscore (matrix (list p p) (repeat 0 (* p p))))
       (current-ties nil)
       (tie-method (send self :tie-method))
       (last-stratum (elt strata (elt keyint 0)))
       (this-stratum (elt strata (elt keyint 0)))
       (zbars nil)
       (riskset nil)
       (junk (dotimes (i nt)
		      (cond ((equalp this-stratum last-stratum) (setf last-stratum this-stratum this-stratum (elt strata (elt keyint i))))
			    (t (setf riskset nil last-stratum this-stratum this-stratum (elt strata (elt keyint i))))
			    )
		      (cond ((= (elt tieint i) 1)  ;; one of a tied set of failures  
			     (setf current-ties (cons (elt keyint i) current-ties))
			     )
			    (current-ties ;; last of a tied set of failures
			     (unless (= (elt statusint i) 1) (error "Can't happen"))
			     (setf current-ties (cons (elt keyint i) current-ties))
			       ;;; recalculate nS0, nS1, nS2 first	
			     (setf nS0 0)
			     (setf nS1 (repeat 0 p))
			     (setf nS2 (zero-matrix p))
			     (setf riskset (append current-ties riskset))
			     (dolist (this-obs riskset)
				     (let* ((timez (mapcar #'(lambda (f) (funcall f  (elt timexs this-obs) (elt times (elt keyint i)))) timefuns))
					    (z (append-seq (select x-list this-obs) timez))
					    (ebtz (exp (inner-product z oldbeta)))
					    )
				       (setf nS0 (+ nS0 ebtz))
				       (setf nS1 (+ nS1 (* z ebtz)))
				       (setf nS2 (+ nS2 (* ebtz (outer-product z z))))
				       )				       
				     )
			     (let* ((timexlist (select timexs current-ties))
				    (timezlist (mapcar #'(lambda (keyi) (mapcar #'(lambda (f) (funcall f  (elt timexs keyi) (elt times keyi))) timefuns)) current-ties))
				    (zlist (row-list (bind-columns (apply #'bind-rows (select x-list current-ties)) (apply #'bind-rows timezlist))))
				    (temp (send tie-method :increment zlist  oldbeta nS0 nS1 nS2))
				    (thistime (select times (select keyint i)))
				    )
			       (setf score (+ score (first temp)))
			       (setf varscore (+ varscore (second temp)))
			       (setf loglike (+ loglike (send tie-method :loglike zlist oldbeta ns0)))
			       (setf dcumhaz (cons (list thistime  (send tie-method :dLambda  zlist oldbeta nS0) this-stratum) dcumhaz))
			       (setf schoenfelds (append (mapcar #'(lambda (si)  (list thistime si this-stratum)) (send tie-method :schoenfeld zlist oldbeta ns0 ns1)) schoenfelds))
			       (setf zbars (cons (list thistime this-stratum (length riskset) (repeat (/ nS1 nS0)) (length current-ties)) zbars))
			       (setf current-ties nil)
			       )
			     )
			    ((= (elt statusint i) 1)  ;; untied failure
			     (setf riskset (cons (elt keyint i) riskset))
			       ;;; recalculate nS0, nS1, nS2 first	
			     (setf nS0 0)
			     (setf nS1 (repeat 0 p))
			     (setf nS2 (zero-matrix p))
			     (dolist (this-obs  riskset)
				     (let* ((timez (mapcar #'(lambda (f) (funcall f  (elt timexs this-obs) (elt times (elt keyint i)))) timefuns))
					    (z (append-seq (select x-list this-obs) timez))
					    (ebtz (exp (inner-product z oldbeta)))
					    )
				       (setf nS0 (+ nS0 ebtz))
				       (setf nS1 (+ nS1 (* z ebtz)))
				       (setf nS2 (+ nS2 (* ebtz (outer-product z z))))
				       )				       
				     )
			     
			     (let* ((timez (mapcar #'(lambda (f) (funcall f  (elt timexs (elt keyint i)) (elt times (elt keyint i)))) timefuns))
				    (z (append-seq (select x-list (select keyint i)) timez))
				    (ebtz (exp (inner-product z oldbeta)))
				    (thistime (select times (select keyint i)))
				    )
			       (setf current-ties nil)
			       (setf score (+ score (- z (/ nS1 nS0))))
			       (setf varscore (+ varscore (/ nS2 nS0) (/ (outer-product (- nS1) nS1) (* nS0 nS0))))
			       (setf dcumhaz (cons (list thistime (/ ns0) this-stratum) dcumhaz))
			       (setf schoenfelds (cons (list thistime (- z (/ nS1 nS0)) this-stratum) schoenfelds))
			       (setf zbars (cons (list thistime this-stratum (length riskset) (/ nS1 nS0)) zbars))

			       (setf loglike (+ loglike (log ebtz) (- (log nS0))))
			       )
			     )
			    
			    ((= (elt entryint i) 1) ;; censoring
			     (setf riskset (cons (elt keyint i) riskset))
			     )
			    ((= (elt entryint i) -1) ;; entry
			     (if (position (elt keyint i) riskset)
				 (setf riskset (remove (elt keyint i) riskset))
			       (error "Can't happen in entry"))
			     )
			    (t (error "Can't happen in :compute-step"))
			    )
		      ))
       (inv (inverse varscore))
       (cumhaz (map-elements #'(lambda (stratum) (list stratum (append-seq #(0) (firsts (select dcumhaz (argselect (thirds dcumhaz) stratum)))) (append-seq #(0) (cumsum (seconds (select dcumhaz (argselect (thirds dcumhaz) stratum))))))) stratlist))
       )
  (send self :set-beta (+ oldbeta (matmult score inv)))
  (send self :covariance-matrix inv )
  (send self :loglike loglike)
  (send self :cumulative-hazard cumhaz)
  (send self :schoenfeld-residuals schoenfelds)
  (send self :zbars zbars)
  )
)

;;
;; This is not entirely obvious.  The data structure has been setup up so that only a 
;; single addition is required to update each of the risk set statistics for each time
;; point.  This should be less numerically stable than the obvious algorithm, but 
;; a: it should work pretty well in real data; and
;; b: I haven't been able to make it mess up
;;
(defmeth cox-regression-proto :compute-step (&optional (stuff *cox-diagnostics-while-fitting*))
"Does one step of Newton-Raphson algorithm"
(let* (
       (keyint (send self :keyint))
       (statusint (send self :statusint))
       (entryint (send self :entryint))
       (tieint (send self :tieint))
       (times (send self :times))
       (strata (send self :strata))
       (stratlist (unique strata))
       (x-list (row-list (send self :x)))
       (oldbeta (send self :beta))
       (p (length (first x-list)))
       (nz (length x-list))
       (nt (length keyint))
       (dcumhaz nil)
       (schoenfelds nil)
       (nS0 0)
       (nrisk 0)
       (loglike 0)
       (nS1 (repeat 0 p))
       (nS2 (matrix (list p p) (repeat 0 (* p p))))
       (score (copy-seq nS1))
       (varscore (copy-array nS2))
       (current-ties 0)
       (zbars nil)
       (tie-method (send self :tie-method))
       (last-stratum (elt strata (elt keyint 0)))
       (this-stratum (elt strata (elt keyint 0)))
       (junk (dotimes (i nt)
		      ;; work backwards in time, accumulating information as we go
		      (cond ((equalp this-stratum last-stratum) (setf last-stratum this-stratum this-stratum (elt strata (elt keyint i))))
			    (t (setf nrisk 0 nS0 0 nS1 (repeat 0 p) nS2 (matrix (list p p) (repeat 0 (* p p))) last-stratum this-stratum this-stratum (elt strata (elt keyint i))))
			    )
		      (cond ((= (elt tieint i) 1)  ; tied failure
			     (let* (
				    (z (select x-list (select keyint i)))
				    (ebtz (exp (inner-product z oldbeta)))
				    )
			       (setf nS0 (+ nS0 ebtz))
			       (setf nS1 (+ nS1 (* z ebtz)))
			       (setf nS2 (+ nS2 (* ebtz (outer-product z z))))
			       (setf current-ties (+ current-ties 1))
			       (setf nrisk (+ nrisk 1))
			       )
			     )
			    ((> current-ties 0)  ; last of tied set of failures
			     (let* (
				    (z (select x-list (select keyint i)))
				    (ebtz (exp (inner-product z oldbeta)))
				    )
			       (setf nS0 (+ nS0 ebtz))
			       (setf nS1 (+ nS1 (* z ebtz)))
			       (setf nS2 (+ nS2 (* ebtz (outer-product z z))))
			       (setf nrisk (+ 1 nrisk))
			       (setf current-ties (+ current-ties 1)))
			     (let* ((keylist (select keyint (- i (iseq  current-ties))))
				    (zlist (select x-list keylist))
				    (temp (send tie-method :increment zlist  oldbeta nS0 nS1 nS2))
				    (thistime (select times (select keyint i)))
				    )
			       (setf score (+ score (first temp)))
			       (setf varscore (+ varscore (second temp)))
			       (setf loglike (+ loglike (send tie-method :loglike (select x-list keylist) oldbeta ns0)))
			       (setf dcumhaz (cons (list thistime  (send tie-method :dLambda  zlist oldbeta nS0) this-stratum) dcumhaz))
			       (setf schoenfelds (append (mapcar #'(lambda (si)  (list thistime si this-stratum)) (send tie-method :schoenfeld zlist oldbeta ns0 ns1)) schoenfelds))
			       (setf zbars (cons (repeat (list thistime this-stratum nrisk (/ nS1 nS0)) current-ties) zbars))
       (setf current-ties 0)
			       )
			     )
			    ((= (elt statusint i) 1)  ; untied failure
			     (let* ((z (select x-list (select keyint i)))
				    (ebtz (exp (inner-product z oldbeta)))
				    (thistime (select times (select keyint i)))
				    )
			       (setf nS0 (+ nS0 ebtz))
			       (setf nS1 (+ nS1 (* z ebtz)))
			       (setf nS2 (+ nS2 (* ebtz (outer-product z z))))
			       (setf current-ties 0)
			       (setf nrisk (+ nrisk 1))
			       (setf score (+ score (- z (/ nS1 nS0))))
			       (setf varscore (+ varscore (/ nS2 nS0) (/ (outer-product (- nS1) nS1) (* nS0 nS0))))
			       (setf dcumhaz (cons (list thistime (/ ns0) this-stratum) dcumhaz))
			       (setf schoenfelds (cons (list thistime (- z (/ nS1 nS0)) this-stratum) schoenfelds))
			       (setf zbars (cons (list thistime this-stratum nrisk (/ nS1 nS0)) zbars))
			       (setf loglike (+ loglike (log ebtz) (- (log nS0))))
			       )
			    )
			    ((= (elt entryint i) 1) ;censoring
			     (let* ((z (select x-list (select keyint i)))
				    (ebtz (exp (inner-product z oldbeta)))
				    )
			       (setf nS0 (+ nS0 ebtz))
			       (setf nS1 (+ nS1 (* z ebtz)))
			       (setf nS2 (+ nS2 (* ebtz (outer-product z z))))
			       (setf nrisk (+ nrisk 1))
			       )
			     )
			    ((= (elt entryint i) -1) ;left truncation
			     (let* ((z (select x-list (select keyint i)))
				    (ebtz (exp (inner-product z oldbeta)))
				   )
			       (setf nrisk (- nrisk 1))
			       (setf nS0 (- nS0 ebtz))
			       (setf nS1 (- nS1 (* z ebtz)))
			       (setf nS2 (- nS2 (* ebtz (outer-product z z))))
			       )
			     )
			    (t (error "Can't happen in :compute-step")) 
			    )
		      ))
       (inv (inverse varscore))
       (cumhaz (map-elements #'(lambda (stratum) (list stratum (append-seq #(0) (firsts (select dcumhaz (argselect (thirds dcumhaz) stratum)))) (append-seq #(0) (cumsum (seconds (select dcumhaz (argselect (thirds dcumhaz) stratum))))))) stratlist))
       )
  (send self :set-beta (+ oldbeta (matmult score inv)))
  (send self :covariance-matrix inv )
  (send self :loglike loglike)
  (send self :cumulative-hazard cumhaz)
  (send self :schoenfeld-residuals schoenfelds)
  (send self :zbars zbars)
  )
)
    
			    

;;;
;;; DISPLAY method 
;;;

(defmeth cox-regression-proto :display ()
  "Prints the model summary"
  (if (send self :needs-computing) (send self :compute))
  (let ((coefs (coerce (send self :beta) 'list))
	(se-s (if (send self :agnostic) (send self :agnostic-coef-standard-errors) (send self :coef-standard-errors)))
        (probs (send self :coef-probs))
	(x (send self :x))
	(p-names (send self :predictor-names)))
    (format t (send self :comments))
    (if (> (length (unique (send self :strata))) 1) (format t "~% Stratified ") (format t "~%"))
    (format t "Maximum Partial Likelihood Estimates:~2%")
    (format t "~25tCoefficient~40t Std Error ~60t Prob~%")
    (when (send self :intercept)
	  (format t "Constant~25t~13,6g~40t(~,6g) ~60t(~,6f)~%"
		  (car coefs) (car se-s) (car probs))
	  (setf coefs (cdr coefs))
	  (setf probs (cdr probs))
	  (setf se-s (cdr se-s)))
    (dotimes (i (length coefs)) 
	     (format t "~a~25t~13,6g~40t(~,6g) ~60t(~,6f)~%"
		     (select p-names i) (car coefs) (car se-s) (car probs))
	     (setf coefs (cdr coefs) se-s (cdr se-s) probs (cdr probs)))
    (format t "~%")
    (format t "-2log-likelihood:~25t~13,6g~%" (* -2 (send self :loglike)))
    (format t "Number of cases:~25t~9d~%" (send self :num-cases))
    (format t "Number of records: ~25t~9d~%" (length (send self :keyint)))
    (format t "Tie handling: ~25t")
    (send (send self :tie-method) :prettyprint)
 ))
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :n-p
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth cox-regression-proto :n-p()
  (if (send self :intercept)
      (1- (apply #'- (size (send self :x))))
    (apply #'- (size (send self :x)))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :coef-probs
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth cox-regression-proto :coef-probs (&key (prob .05) verbose)
  "
;; Description: : cox-regression-proto :coef-probs (&key (prob .05) verbose)

	coef-probs: estimate/se

and the probability of this estimated t-statistic.

;; Examples:
"
  (let* (
	 (df (send self :n-p))
	 (ts (/ (send self :beta) 
		(if (send self :agnostic)
		    (send self :agnostic-coef-standard-errors)
		  (send self :coef-standard-errors))))
	 (probs (* 2 (- 1 (t-cdf (abs ts) df))))
	 )
    (if verbose
	(let ()
	  (format
	   t "t-statistic for two-sided t-test with ~d df and probability ~,3f:~% ~g~%"
	   df prob (t-quant (- 1 (half prob)) df ))
	  (format t "t-scores for coefficients: ~g:~%" ts)
	  (format t "probabilities for coefficients: ~g.~%" probs)
	  )
      )
    probs
    )
  )






(defmeth cox-regression-proto :display-with-formula (&key (block-only *cox-display-block-only*))
"Arguments: &key (block-only *cox-display-block-only*)
Prints the model summary based on a model formula"
(cond ( (null (send self :formula)) (error "No formula present"))
      (t 
       (if (send self :needs-computing) (send self :compute))
       (let* ((beta (coerce (send self :beta) 'list))
	     (formula (send self :formula))
	     (cov-mat (if (send self :agnostic) (send self :agnostic-covariance-matrix) (send self :covariance-matrix)))
	     (nblocks (length (send formula :block-indices)))
	     (block-indices (send formula :block-indices))
	     (varnames (send self :predictor-names))
	     )
	 (format t (send self :comments))
	 (format t "~%Maximum Partial Likelihood Estimates:~2%")
	 (format t "~a~20t~a~35t~a~%" "Block" "Wald Chisq" "p-value")
	 (dolist (i (iseq 0 (- nblocks 1)))
		 (block-test (elt block-indices i) beta cov-mat :names varnames :blockname  (elt (send formula :block-names) i) :block-only block-only))
	 (format t "~%")
	 (format t "-2log-likelihood:~25t~13,6g~%" (* -2 (send self :loglike)))
	 (format t "Number of cases:~25t~9d~%" (send self :num-cases))
	 (format t "Number of records: ~25t~9d~%" (length (send self :keyint)))
	 (format t "Tie handling: ~25t")
	 (send (send self :tie-method) :prettyprint)
	 )
       )
))


;;;;
;;;; :ISNEW method
;;;;

(defmeth cox-regression-proto :isnew (&key x times status
					   entry
					   id
					   (formula nil)
					   (ties *cox-default-ties*)
					   pweights
					   (verbose t)
					   predictor-names
					   response-name
					   (recycle nil)
					   case-labels
					   init-beta
					   count-limit
					   (print-summary nil)
					   (print t) 
					   statusint 
					   keyint
					   entryint
					   strata
					   tieint
					   comments
					   (suppress-warnings nil)
					   (compute t)
					   (time-dependent nil)
					   )
  (send self :x x)
  (send self :time-dependent time-dependent)
  (send self :times times)
  (send self :status status)
  (send self :entry entry)
  (send self :tie-method ties)
  (if pweights (send self :weights pweights) (send self :weights (repeat 1 (length times))))
  (send self :formula formula)
  (send self :strata strata)
  (send self :recycle recycle)
  (send self :verbose verbose)
  (send self :suppress-warnings suppress-warnings)
  (if predictor-names (send self :predictor-names predictor-names))
  (if response-name (send self :response-name response-name))
  (send self :epsilon *cox-tolerance*) 
  (send self :epsilon-dev  *cox-tolerance*)
  (if (null count-limit) (send self :count-limit *cox-count-limit*) (send self :count-limit count-limit))
  (send self :statusint statusint)
  (send self :keyint keyint)
  (send self :tieint tieint)
  (send self :entryint entryint)
  (send self :comments comments)
  (send self :id id)
  (if print (send self :display))
  (if print-summary (send self :display-with-formula))
  (if (and (null (or print print-summary)) compute) (send self :compute))
)



;;;;
;;;; functions to construct the models and do
;;;;  the necessary data munging
;;;;
;;;; The fitting algorithm uses a data structure sorted 
;;;; backwards in time with a row  for every entry, censoring
;;;; or failure time.  You are not required to understand this.
;;;;

(defun cox-model (&key x times status
		       (id (iseq (length times)) has-id)
		       (entry nil)
		       (strata (* 0 status) has-strata)
		       (agnostic nil)
		       (verbose *cox-verbose*)
		       (print t)
		       (print-summary nil has-print-summary)
		       (init-beta nil)
		       (count-limit *cox-count-limit*)
		       (weights nil)
		       (predictor-names nil has-names)
		       (response-name "Failure time" has-response-name)
		       (ties *cox-default-ties*)
		       (suppress-warnings nil)
		       time-x
		       time-funs
		       )
"Constructs a Cox regression modelling with right censoring and 
optionally left truncation and strata."
 (if (null (and x times status)) (error "You must supply :x :times and :status.~%")
     (let* (
	    (comments (cond ((and agnostic has-id) "LWA/WLW agnostic covariance estimator")
			    ((and agnostic (null has-id)) "LW agnostic covariance estimator")
			    (t "Standard Cox regression")
			    )
		      )
	    (n (length times))
	    (alldata (if entry  
			 (col2row (list (append-seq entry times)  (append-seq (repeat 0 n) status) (append-seq (repeat -1 n) (repeat 1 n)) (append-seq strata strata) (repeat (iseq n) 2)))
		       (col2row (list times  status  (repeat 1 n) strata (iseq n)))
		       ))
	    (rtemp  (sort (copy-list alldata) #'after))
	    (temp (row2col rtemp))
	    (keyint (select temp 4))
	    (statusint (select temp 1))
	    (entryint (select temp 2))
	    (ntimes (length keyint))
	    (timexlist (cond
			( (null time-x) nil)
			((matrixp time-x) (row-list time-x))
			((and (sequencep time-x) (numberp (elt time-x 0))) (row-list (bind-columns time-x)))
			((and (sequencep time-x) (sequencep (elt time-x 0))) (row-list (apply #'bind-columns time-x)))
			(t (error "Wrong sort of :time-x"))
			))
	    ;; tieint =1 if this entry is tied with the previous one, else 0
	    (tieint (append  (mapcar #'tied (select rtemp (iseq 0 (- ntimes 2))) (select rtemp (iseq 1 (- ntimes 1)))) '(0)))
	    )
       (cond ((objectp x)
	      (send cox-regression-proto :new :x (send x :design-matrix)
		    :times times :status status :entry entry
		    :predictor-names (if has-names predictor-names (send x :name-list))
		    :init-beta init-beta
		    :count-limit count-limit
		    :weights weights
		    :formula x
		    :print (if has-print-summary nil print)
		    :print-summary (if has-print-summary print-summary t)
		    :agnostic agnostic
		    :verbose verbose
		    :keyint keyint
		    :strata strata
		    :statusint statusint
		    :entryint entryint
		    :tieint tieint
		    :ties ties
		    :reponse-name response-name
		    :comments comments
		    :suppress-warnings suppress-warnings
		    :time-dependent (list time-funs timexlist)
		    :id id
		))
	     (t (send cox-regression-proto :new :x x
		    :times times :status status :entry entry
		    :predictor-names predictor-names
		    :init-beta init-beta
		    :count-limit count-limit
		    :weights weights
		    :formula x
		    :print print
		    :print-summary nil
		    :agnostic agnostic
		    :verbose verbose
		    :strata strata
		    :keyint keyint
		    :statusint statusint
		    :entryint entryint
		    :tieint tieint
		    :ties ties
		    :reponse-name response-name
		    :comments comments
		    :suppress-warnings suppress-warnings
		    :time-dependent (list time-funs timexlist)
		    :id id
		    )  
		)
	     )
       )
   )
 )


(defun lumleys-kaplan-meier
  (&key times status (entry nil) (strata (repeat 0 (length times))))
  "Args: &key times status strata entry"
  (let* ((fakex (uniform-rand (length times)))
	 (nullcox
	  (cox-model :x fakex :times times :entry entry :status status :strata
		     strata :verbose nil :print nil :print-summary nil
		     :count-limit 1 :suppress-warnings t))
	 )
    (send nullcox :compute)
    (send nullcox :survival-function)
    )
  )

(defun plot-kaplan-meier
  (&key times status (entry nil) (strata  (repeat 0 (length times))) inverted
	(one-plot t))
  "Args: &key times status strata entry inverted (one-plot t)"
  (let* ((fakex (uniform-rand (length times)))
	 (nullcox (cox-model :x fakex :times times :entry entry :status status
			     :strata strata :verbose nil :print nil
			     :print-summary nil :count-limit 1
			     :suppress-warnings t))
	 )
    (send nullcox :compute)
    (send nullcox :plot-survival :inverted inverted :one-plot one-plot)
    )
  )

(defun nelson-aalen (&key times status (entry nil)(strata  (repeat 0 (length times))))
  "Args: &key times status strata entry "
  (let* ((fakex (uniform-rand (length times)))
	 (nullcox (cox-model :x fakex :times times :entry entry :status status :strata strata :verbose nil :print nil :print-summary nil :count-limit 1 :suppress-warnings t))
	 )
    (send nullcox :compute)
    (send nullcox :cumulative-hazard)
    )
  )

(defun plot-nelson-aalen (&key times status (entry nil) (strata  (repeat 0 (length times))) (one-plot t))
"Args: &key times status strata entry (one-plot t)"
(let* ((fakex (uniform-rand (length times)))
       (nullcox (cox-model :x fakex :times times :entry entry :status status :strata strata :verbose nil :print nil :print-summary nil :count-limit 1 :suppress-warnings t))
       )
  (send nullcox :compute)
  (send nullcox :plot-cumulative-hazard :one-plot one-plot)
  )
)

(defun cox-score-test
  (&key groups times status
	(entry nil) (trend nil) (strata (repeat 0 (length times))))
  "
;; Description: 
 (cox-score-test
    &key groups times status
	(entry nil) (trend nil) (strata (repeat 0 (length times))))

Peto's approximate Logrank test for differences between groups.

;; Examples:
;; p. 189, Statistical methods in epidemiology:
 (def groups (combine (repeat 1 300) (repeat 0 150)))
 (def times
   (combine
     (repeat 0 25)(repeat 1 4)(repeat 2 3)(repeat 3 1)(repeat 4 2)
     (repeat 5 1)(repeat 6 264)
     (repeat 0 20)(repeat 1 3)(repeat 2 2)(repeat 3 1)(repeat 4 1)
     (repeat 5 1)(repeat 6 122)))
 (def status (combine (repeat 1 36)(repeat 0 264) (repeat 1 28)(repeat 0 122)))
 (cox-score-test :groups groups :times times :status status)
"
  (let* (
	 (gmodel (lumley-factor groups :namebase "Group "))
	 (gmat (first gmodel))
	 (gnames (second gmodel))
	 (p (second (array-dimensions gmat)))
	 (x (if trend (matmult gmat (iseq 1 p)) gmat))
	 (nullcox
	  (cox-model :x x :times times :entry entry :status status :strata
		     strata :verbose nil :print nil :print-summary nil
		     :count-limit 1 :suppress-warnings t :ties peto-ties))
	 )
    (print p)
    (send nullcox :compute)
    (let* (
	   (beta (send nullcox :beta))
	   (infomat (inverse (send nullcox :covariance-matrix)))
	   (score (matmult beta infomat))
	   (grouptests (/ score (sqrt (diagonal infomat))))
	   (overalltest (inner-product score beta))
	   )
      (format t (if trend "Logrank Trend Test~%" "Logrank test~%"))
      (when (and (> p 1) (null trend))
	    (dotimes
	     (i p)
	     (format t
		     "Group ~a.  Z=~6,4g   p=~6,4g~%"
		     (elt gnames i) (elt grouptests i)
		     (* 2 (- 1 (normal-cdf (abs (elt grouptests i))))))
	     )
	    )
      (format t
	      "Overall test: X^2=~8,6g   p=~6,4g~%"
	      overalltest (- 1 (chisq-cdf overalltest p)))
      (if (< (length (unique times)) (length times))
	  (format
	   t
	   "Note: Tied times present -- results slightly conservative~%")
	)
      (list grouptests overalltest (- 1 (chisq-cdf overalltest p)))
      )
    )
  )
(alias logrank cox-score-test)

(def *cox-count-limit* 20)
(def *cox-default-ties* efron-ties)
(def *cox-verbose* t)
(def *cox-tolerance* (sqrt (sqrt machine-epsilon)))
(def *cox-display-block-only* nil)
(def *cox-diagnostics-while-fitting* nil)

(defun lumley-range (x) (list (min x) (max x)))
(defun survival-slider-plot (cox-object)
  "Arguments: cox-object
Draws a survival plot with sliders

;; Examples:
 (def aml-s #(1 1 0 1 1 0 1 1 0 1 0 1 1 1 1 1 0 1 1 1 1 1 1))
 (def aml-t #(9 13 13 18 23 28 31 34 45 48 161 5 5 8 8 12 16 23 27 30 33 43 45))
 (def aml-x '(1 1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 2 2))
 (def e1 #(1 2 5 2 1 7 3 4 8 8))
 (def e2 #(1 2 5 2 1 7 3 4 8 8))
 (def s1 #(1 1 0 1 1 0))
 (def s2 #(1 1 1 1 1 1 1 0 0 0))
 (def t1 #(4 1 1 2 2 3))
 (def t2 #(2 3 6 7 8 9 9 9 14 17))
 (def x1 #(0 1 1 1 0 0))
 (def x2 #(1 0 0 1 0 1 1 1 0 0))
 (def ovarian-age '(72.33150000000001 74.4932 66.4658 53.3644 50.3397 56.4301
   56.937 59.8548 64.17529999999999 55.1781 56.7562 50.1096 59.6301 57.0521
   39.2712 43.1233 38.8932 44.6 53.9068 44.2055 59.589 74.50409999999999 43.137
   63.2192 64.4247 58.3096)) 
 (def ovarian-dx '(2 2 2 2 2 1 2 2 2 1 1 1 2 2 1 1 2 1 1 2 1 2 2 1 2 1))
 (def ovarian-ecog '(1 1 2 1 1 2 2 2 1 2 2 1 2 1 1 2 2 1 1 1 2 2 1 2 1 1))
 (def ovarian-rx '(1 1 1 2 1 1 2 2 1 2 1 2 2 2 1 1 1 1 2 2 2 1 1 2 2 2))
 (def ovarian-s '(1 1 1 0 1 0 1 1 0 1 1 0 0 0 0 0 0 0 0 0 0 1 1 1 1 0))
 (def ovarian-t '(59 115 156 421 431 448 464 475 477 563 638 744 769 770 803
    855 1040 1106 1129 1206 1227 268 329 353 365 377))
 
 (def
  ovarian-model
  (cox-model
   :status ovarian-s
   :times ovarian-t
   :strata ovarian-rx
   :x (list ovarian-age ovarian-dx ovarian-ecog)
   :predictor-names (list \"Age\" \"Residual disease\" \"performance status\")
   )
  )
 
 (survival-slider-plot ovarian-model)
"
  (let* (
	 (x (column-list (send cox-object :x)))
	 (names (send cox-object :predictor-names))
	 (ranges (mapcar #'lumley-range x))
	 (current-xvec (mapcar #'min x))
	 (current-plot
	  (elt
	   (send cox-object :plot-survival :one-plot t :xvec current-xvec) 0))
	 (actions
	  (mapcar
	   #'(lambda (i) (format nil "(lambda (x) (adjust-to-sliders x ~s))" i))
	   (iseq (length x))))
	 (lspec
	  (mapcar
	   #'(lambda (i) (list (elt names i) (elt ranges i) (elt actions i)))
	   (iseq (length x))))
	 )
    (defun adjust-to-sliders (x i) 
      (setf (select current-xvec i) x)
      (send current-plot :clear-points :draw nil)
      (send current-plot :clear-lines :draw nil)
      (map-elements
       #'(lambda (stratum)
	   (add-steps (second stratum) (third stratum) current-plot))
       (send cox-object :survival-function :xvec current-xvec))
      (send current-plot :range 1 0 1)
      )
    (make-and-run-sliders lspec)
    )
  )

