;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;            Relationship between svd and regression:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def x1 (rand.u 20))
(def x2 (rand.u 20))
(def y (mapcar (lambda(x y z) (+ x (* 2 y) z)) x1 x2 (rand.u 20)))
(def reg (regress (list x1 x2) y))

(def ones (repeat 1 20))
(def xmat (bind-columns ones x1 x2))
(def svd (svd xmat))
(def u (first svd))
(def uut (matmult u (transpose u)))
(def xmat-inv (pseudo-inverse xmat))

(send reg :beta)
(matmult xmat-inv y)

(send reg :se)
(sqrt
 (*
  (diagonal (matmult xmat-inv (transpose xmat-inv)))
  (send reg :s-squared)
  )
 )

(send reg :resss)
(square (norm (matmult (- (id 20) uut) y)))

(send reg :regss)
(square (norm (matmult (- uut (/ (ones 20 20) 20)) y)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 svd-regression
;;
(defun svd-regression(x y)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          svd-regression Doc:
;;
;; Author: Andy Long
;; Created: Thu May 8 13:21:15 CDT 1997
;;
;; Description/Arguments:

       (svd-regression x y)
  
;; Notes: 

;; See Also: ridge-regression

;; Examples:

 (def x1 (rand.u 20))
 (def x2 (rand.u 20))
 (def y (mapcar (lambda(x y z) (+ x (* 2 y) z)) x1 x2 (rand.u 20)))
 (def reg (regress (list x1 x2) y))
 (svd-regression (list x1 x2) y)

 (def x1 (list 7 1 11 11 7 11 3 1 2 21 1 11 10))
 (def x2 (list 26 29 56 31 52 55 71 31 54 47 40 66 68))
 (def x3 (list 6 15 8 8 6 9 17 22 18 4 23 9 8))
 (def x4 (list 60 52 20 47 33 22 6 44 22 26 34 12 12))
 (def x5 
      (/ (list 785 743 1043 876 959 1092 1027 725 931 1159 838 1133 1094) 10))
 (svd-regression (list x1 x2 x3 x4) x5)
"
  (let* (
	 (n (length y))
	 (ones (repeat 1 n))
	 (x
	  (cond
	   ((listp x)
	    (if (listp (first x)) (apply #'bind-columns x) (bind-columns x))
	    )
	   ((matrixp x)
	    x
	    )
	   (t (format
	       t
	       "~%Don't know what to do with an element of type ~s~%."
	       (type-of x)))
	   ))
	 (xmat (bind-columns ones x))
	 (p (second (size xmat)))
	 (n-p (- n p))
	 (svd (svd xmat))
	 (u (first svd))
	 (v (third svd))
	 (lambda (diagonal-to-matrix (second svd)))
	 (uut (matmult u (transpose u)))
	 (xmat-inv (pseudo-inverse xmat))
	 (beta (matmult xmat-inv y))
	 (resss (square (norm (matmult (- (id n) uut) y))))
	 (regss (square (norm (matmult (- uut (/ (ones n n) n)) y))))
	 (ms (/ regss (1- p)))
	 (s-squared (/ resss n-p))
	 (f (/ ms s-squared))
	 (se 
	  (sqrt
	   (*
	    (diagonal (matmult xmat-inv (transpose xmat-inv)))
	    s-squared
	    )
	   )
	  )
	 )
    (values beta se regss resss ms s-squared f)
    )
  )
;;
