#| - Fri Mar 6 17:54:08 1998

	This is a revision of tsvd2.lsp, adding one lambda for each dimension
of the tensor. I tried to get by with one for the whole tensor of tsvd2, and
that doesn't seem stable. I was worried about stability, as it seemed that that
left a lot of redundency which the algorithm could thrash around in.

	This is worse than tsvd1.lsp. So much for revisions! I've got to figure
out what the hell is causing all this pain!

	In all three cases (tsvd, tsvd1, tsvd2) I get things soaring off to
infinity. At least in tsvd I can get the SVD results buried under infinities.
|#
(require "tensor")
;; this here's our function:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				non-linear-system-root
;;
(defun non-linear-system-root(f jack x0 &key (eps 1e-12) (maxits 100) verbose)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			non-linear-system-root Doc:
;;
;; Author: Andy Long
;; Created: Wed Feb 25 20:51:44 1998
;;
;; Description/Arguments: non-linear-system-root
  
;; Arguments: f jack x0 &key (eps 1e-12) (maxits 100)

;; Returns: a vector x which we hope gives the vectoral equation f(x)=0.

;; Notes: For DESS class at the IMSP

;; Examples:
 ;; How about trying to solve linear systems?
 (def A #2a((2 4)(5 2)))
 (def b #(2 1))
 ;; now we set up the two functions, Ax-b:
 (defun f(z)
  (let ((x (first z))
	(y (second z)))
    (list
     (- (+ (* 2 x) (* 4 y)) 2)
     (- (+ (* 5 x) (* 2 y)) 1))))
 (defun jack(z) A)
 ;; and compare solutions:
 (non-linear-system-root #'f #'jack (list 0 0) :eps 1e-12)
 (solve A b)
"
  (do*
   (
    (i 1 (1+ i))
    (x0 x0 x1)
    (x1 
     (+ x0 (solve (funcall jack x0) (- (funcall f x0))))
     (+ x0 (solve (funcall jack x0) (- (funcall f x0))))
     )
    )
   ((or (> i maxits) (< (norm (- x1 x0)) eps)) (print i) x1)
   (if verbose (print x1))
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Now let's create a function that finds the TSVD in general:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				nltsvd
;;
(defun nltsvd(A &key verbose (eps 1e-12) (maxits 100) initial (norm-eps 1e-6))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			nltsvd Doc:
;;
;; Author: Andy Long
;; Created: Fri Feb 27 10:35:52 1998
;;
;; Description/Arguments: nltsvd
  
;; Arguments: A &key verbose (eps 1e-12) (maxits 100) initial (norm-eps 1e-6)

;; Returns: the tsvd of a tensor, obtained using newton's method to solve for a
   root of a system of non-linear equations.

	DOESN'T WORK YET, but it's coming!

;; Notes: Stemming from my DESS class at the IMSP, and looking toward a method
   that generalizes for the TSVD (Tensor SVD).

;; See Also: non-linear-system-root

;; Examples:
 (def A #2a((2 4)(5 6)))
 (def A #2a((2 4 3)(5 6 2)))
 (nltsvd A) ;; :verbose 2
 (mapcar #'mprint (butlast (svd A)))

 ;; (def A #2a((2 4 -1)(5 6 2)(1 2 1)))
 ;; (nltsvd A :verbose 2)

 (def st1 (vectors-to-tensor '(1 1) '(-1 1) '(-4 6)))
 (def st2 (vectors-to-tensor '(-1 1) '(1 1) '(3 2)))
 (def A (+ st1 st2))
 (send (new tensor (size A) A) :norm)
 (send (new tensor (size st1) st1) :norm)
 (send (new tensor (size st2) st2) :norm)

 (def crap (optimize-rank-one-over-tensor A))
 ;; todo: just discovered that optimize-rank-one-over-tensor can also find a
 ;; minimum! I thought it was going to find a maximum only, oops!
 (t-mult (apply #'vectors-to-tensor (first crap)) A (iseq 3) (iseq 3))

 (nltsvd A :maxits 1000)
"
  (let* (
	 ;; If A is a real tensor, then select off the elements. Otherwise,
	 ;; if A is an array, continue; if not, produce an error message.
	 (A (cond
	     ((tensorp A) (send A :elements))
	     ((arrayp A) A)
	     (t (error
		 "nltsvd requires a tensor or an array as an argument."))))

	 ;; size is a list of the array dimensions of A:
	 (size (size A))

	 ;; The subsets, created in two steps, are designed to be used to
	 ;; select the individual vectors of each dimension from a single
	 ;; vector containing them all:
	 (subsets (mapcar #'iseq size))
	 (subsets (+ subsets (combine 0 (butlast (cumsum size)))))

	 (numdims (length size))
	 (numdims-seq   (iseq numdims))
	 (numdims-1-seq (iseq (- numdims 1)))
	 (numdims-2-seq (iseq (- numdims 2)))

	 (but-is (mapcar (lambda(i) (remove i numdims-seq)) numdims-seq))

;;;todo	 (dim (apply #'* (butlast (sort-data size))))
	 ;; The SVD should have at most dim schmidt-tuples with non-zero
	 ;; singular values, but we need to seek them all anyway for the
	 ;; moment:
	 (dim (apply #'* size))

	 ;; We start with just the lagrange multipliers corresponding to the
	 ;; unit size of the singular tensor: later we'll add one for each
	 ;; singular tensor already found:
	 (lagrange-multipliers numdims)
	 (lagrange-seq (iseq lagrange-multipliers))
	 
	 schmidt-tuples ;; contains the vector returned by
			;; non-linear-system-root, which in turn contains the
			;; components of the rank-one tensor and lagrange
			;; multipliers.

	 sts		;; list containing the singular tensors
	 lambdas	;; list containing the singular values

	 order		;; used to order the singular values/tensors, and for
			;; yeoman duty elsewhere
	 norm-decomp
	 )

    (dotimes
     (i dim)
     (print (list 'iteration i))
     (flet (
	    (funcs(z)
		  (if verbose (print "entering funcs"))
		  (let* (
			 ;; uvs is a list containing the vectors of the
			 ;; decomposition, selected from the single vector z
			 ;; (which also contains the singular values):
			 (uvs (mapcar (lambda(slab)(select z slab)) subsets))

			 (uvs-norm-term
			  (* .5 (- 1 (mapcar (lambda(x) (dot x x)) uvs))))

			 ;; uvs-sts is a list of inner products of the trial
			 ;; singular tensor and the already discovered singular
			 ;; tensors (if any):
			 (uvs-sts
			  (mapcar
			   (lambda(st)
			     (inner-product-of-rank-one-tensor-lists uvs st)) 
			   sts))

			 (lambda-norms (select-from-end z lagrange-seq))
			 (lambda-u (select-from-end lambda-norms numdims-seq))
			 
			 ;; We're going to create the approximation to A
			 ;; obtained so far (if any),
			 ;; and turn decomp into a tensor:
			 (decomp (mapcar
				  (lambda(x lam)
				    (* lam
				       (apply #'vectors-to-tensor x)))
				  sts
				  lambda-norms
				  ))
			 (decomp (if decomp (apply #'+ decomp)))

			 ;; from which I make a tensor that I have to use
			 ;; several times:
			 (A-decomposition (if decomp (- A decomp) A))
			 
			 ;; We combine the partials of f into result:
			 (result
			  (combine
			   ;; the partials of f wrt u,
			   (mapcar
			    (lambda(i)
			      (let ((but-i (elt but-is i)))
				#| - Sun Mar 8 13:33:46 1998
				(printall
				 'printall
				 but-i
				 uvs
				 (select uvs but-i)
				 (apply #'vectors-to-tensor (select uvs but-i))
				 (matmult A (first (select uvs but-i)))
				 (matmult (first (select uvs but-i)) A)
				  A-decomposition
				 't-mult
				 (t-mult 
				  (apply #'vectors-to-tensor (select uvs but-i))
				  A-decomposition
				  numdims-1-seq
				  but-i)
				 (elt lambda-u i)
				 (elt uvs i)
				 (* (elt lambda-u i) (elt uvs i))
				 )
|#
				(-
				 (t-mult 
				  (apply #'vectors-to-tensor (select uvs but-i))
				  A-decomposition
				  numdims-1-seq
				  but-i)
				 (* (elt lambda-u i) (elt uvs i)))))
			    numdims-seq)
			   ;; and the partials wrt lambda:
			   (if uvs-sts (combine (- uvs-sts) uvs-norm-term)
			     uvs-norm-term)
			   )
			  )
			 )
		    (if verbose (print result))
		    result
		    )
		  )
	    (jack (z)
		  (if verbose (print "entering jack"))
		  (let* (
			 (uvs (mapcar (lambda(slab)(select z slab)) subsets))
			 
			 ;; uvs-sts is a list of inner products of the trial
			 ;; singular tensor and the already discovered singular
			 ;; tensors (if any):
			 (uvs-sts
			  (mapcar
			   (lambda(st)
			     (inner-product-of-rank-one-tensor-lists uvs st)) 
			   sts))

			 (lambda-norms (select-from-end z lagrange-seq))
			 (lambda-u (select-from-end z numdims-seq))
			 
			 ;; We're going to create the approximation to A
			 ;; obtained so far (if any),
			 ;; and turn decomp into a tensor:
			 (decomp (mapcar
				  (lambda(x lam)
				    (* (apply #'vectors-to-tensor x) lam))
				  sts
				  lambda-norms
				  ))
			 (decomp (if decomp (apply #'+ decomp)))

			 ;; from which I make a tensor that I have to use
			 ;; several times:
			 (A-decomposition (if decomp (- A decomp) A))

			 ;; jack will be a small "gross" matrix containing the
			 ;; minimum necessary to later use combine-matrices: it
			 ;; has one dimension for each dimension of the tensor,
			 ;; plus a dimension for the lagrange-multiplier stuff.
			 (jack (+ numdims 1))
			 (jack (make-array (list jack jack)))
			 )

		    ;; The next two take care of the diagonal:
		    (dotimes
		     (i numdims)
		     (setf
		      (aref jack i i)
		      (* (- (elt lambda-u i)) (id (elt size i))))
		     )
		    (setf
		     (aref jack numdims numdims)
		     (zeros lagrange-multipliers lagrange-multipliers))

		    ;; Now for the a-matrices:
		    (dotimes
		     (i (1- numdims))
		     (let ((keepers (elt but-is i)))
		       (dolist
			(j (iseq (1+ i) (1- numdims)))
			(let ((keepers (remove j keepers)))
			  (if keepers
			      (setf
			       (aref jack i j)
			       (t-mult 
				(apply #'vectors-to-tensor (select uvs keepers))
				A-decomposition
				numdims-2-seq
				keepers)
			       (aref jack j i)
			       (transpose (aref jack i j))
			       )
			    ;; Otherwise, just matrix case!
			    (setf 
			     (aref jack i j) A-decomposition
			     (aref jack j i) (transpose A-decomposition)
			     )
			    )
			  )
			)
		       )
		     )

		    ;; Now for the lagrange multiplier stuff:
		    (dotimes
		     (i numdims)
		     (let* (
			    (ui (elt uvs i))
			    ;; we want all but the ith element:
			    (but-i (elt but-is i))
			    (but-ui (select uvs but-i))
			    (length-ui (length ui))
			    (order (zeros numdims length-ui))

			    ;; result starts out as the ith vector of each
			    ;; singular tensor, weighted by the negative of
			    ;; the inner products of the singular tensors and
			    ;; but-ui:
			    (result
			     (mapcar
			      (lambda(st)
				(* 
				 (elt st i)
				 (- (inner-product-of-rank-one-tensor-lists
				     (select st but-i) but-ui
				     ))))
			      sts
			      )
			     )
			    ;; Finally we bind it into a matrix:
			    (result (if result (apply #'bind-rows result)))
			    )
		       (setf
			(select order i (iseq length-ui))
			(matrix (list 1 length-ui) (- ui))
			result (if result (bind-rows result order) order)
			(aref jack numdims i) result
			(aref jack i numdims) (transpose result)
			)
		       )
		     )
		    (setq jack (apply #'combine-matrices (row-lists jack)))
		    (if verbose (mprint jack))
		    jack
		    )
		  )
	    )
	   (if verbose (print "or not!"))

	   (setq
	    ;; Here is our initial
	    initial
	    (if (and (= i 0) initial) initial
	      (combine 
	       (mapcar
		#'normalize
		(mapcar
		 #'rand.n
		 size
		 )
		)
	       (rand.n lagrange-multipliers)))
	    schmidt-tuples
	    (non-linear-system-root
	     #'funcs #'jack initial 
	     :verbose verbose :maxits maxits :eps eps)
	    crap (print schmidt-tuples)

	    lambda (mean (select-from-end schmidt-tuples numdims-seq))

	    st (mapcar (lambda(i) (select schmidt-tuples i)) subsets)
	    order (if (< lambda 0) (setf (elt st 1) (- (elt st 1))))
	    sts (append sts (list st))
	    ;; and make sure that lambda itself is positive:
	    lambdas (append lambdas (list (abs lambda)))
	    order (pprint (list lambdas sts))
	    lagrange-multipliers (+ 1 lagrange-multipliers)
	    lagrange-seq (iseq lagrange-multipliers)
	    norm-decomp
	    (- A
	       (apply
		#'+ 
		(mapcar 
		 (lambda(lam st)
		   (* lam (apply #'vectors-to-tensor st)))
		 lambdas sts)))
	    norm-decomp
	    (send (new tensor (size norm-decomp) norm-decomp) :norm)
	    )
	   )
     (if (< norm-decomp norm-eps) (return)
       (print norm-decomp)
       )
     )
    ;; order the singular values from largest to smallest:
    (setq
     order (reverse (order lambdas))
     lambdas (select lambdas order)
     sts (select sts order)
     )
    (if verbose (print "This will come!"))
    
    (list lambdas sts)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun inner-product-of-rank-one-tensor-lists(a b)
  "
;; Examples:
 (def a '((1 2) (2 3) (3 4)))
 (def b '((2 3) (3 4) (1 2)))
 (inner-product-of-rank-one-tensor-lists a b)
"
  (apply #'* (mapcar #'sum (* a b))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; problems in tsvd1 suggest this replacement: these are identical up to the
;; 1e-16 place or so!
(defun inner-product-of-rank-one-tensor-lists(a b)
  "
;; Examples:
 (def a '((1 2) (2 3) (3 4)))
 (def b '((2 3) (3 4) (1 2)))
 (inner-product-of-rank-one-tensor-lists a b)
"
  (sum (* 
	(apply #'vectors-to-tensor a)
	(apply #'vectors-to-tensor b)
	)
       )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(try-out nltsvd)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#| - Fri Mar 6 19:07:25 1998


I'm stuck in a two cycle!
(def a #2A((2 4 3) (5 6 2)))

(def mat1
#2a(
    (  1.65788      0.000000             2             4             3     -0.938912             0    )
    ( 0.000000       1.65788             5             6             2      0.607087             0    )
    (        2             5       1.87621      0.000000      0.000000             0     -0.623596    )
    (        4             6      0.000000       1.87621      0.000000             0       6.094010E-2)
    (        3             2      0.000000      0.000000       1.87621             0      0.863287    )
    (-0.938912      0.607087             0             0             0             0             0    )
    (        0             0     -0.623596       6.094010E-2  0.863287             0             0    )
   ))
(def v1
     '(-0.02982252065893176 0.0192828127111897 0.012387178685507871 -0.0012105214289742094 -0.01714844032172369 -0.25011114483489894 -0.137849943672137) )

(def x0 '(0.9389122965991569 -0.6070871800077776 0.6235956296158131 -0.06094009716251455 -0.8632871706182237 -1.6578844757411062 -1.8762134145277747))
;; here I am

(+ x0 (solve mat1 (- v1)))
;; (0.7510630001080396 -0.4856265225273324 0.5480473561627658 -0.05355725352628727 -0.7587003630902122 -1.8166414860922104 -1.7122491700968328)
;; (0.7510630558558521 -0.4856265640987906 0.5480473353132207 -0.053557235294001006 -0.7587003676707775 -1.8166422143453533 -1.7122497036345314) 

(def mat2
#2a(
    (  1.81664      0.000000             2             4             3     -0.751063             0    )
    ( 0.000000       1.81664             5             6             2      0.485627             0    )
    (        2             5       1.71225      0.000000      0.000000             0     -0.548047    )
    (        4             6      0.000000       1.71225      0.000000             0       5.355724E-2)
    (        3             2      0.000000      0.000000       1.71225             0      0.758700    )
    (-0.751063      0.485627             0             0             0             0             0    )
    (        0             0     -0.548047       5.355724E-2  0.758700             0             0    )
   ))
(def v2
     '(-0.029822520658932428 0.01928281271118959 0.012387178685507982 -0.001210521428973363 -0.017148440321724134 0.2000711263700724 0.12114949289996824) )
(def x0 '(0.7510630558558521 -0.4856265640987906 0.5480473353132207
			     -0.053557235294001006 -0.7587003676707775
			     -1.8166422143453533 -1.7122497036345314) )
(+ x0 (solve mat2 (- v2)))
;; (0.9389122580049387 -0.6070871522793757 0.6235956666742921 -0.06094010770743135 -0.8632872271314229 -1.6578852943270999 -1.876213246079339)
|#
#| - Fri Mar 6 22:20:00 1998

(def sts '(((1 1) (2 2)) ((-1 1) (2 -2))))
(def lambda-norms '(3 4))
(setq decomp (mapcar
	 (lambda(x) (apply #'vectors-to-tensor x))
	 sts))
(apply
 #'+
 (* (select lambda-norms (length sts)) decomp)))
|#
#| - Sat Mar 7 08:18:26 1998
OUTPUT 
#2A((2 4) (5 6)) 
#2A((4.09842503678051 5.52230580169939) (7.256866052811806 9.778056972118819)) 
(8.9539043383412) 
(((-0.6274163263702638 -1.1109331508946907) (-0.7295393639872552 -0.9829969869302896))) 
5.0289140493744275 

(mapcar #'norm '((-0.6274163263702638 -1.1109331508946907) (-0.7295393639872552 -0.9829969869302896)))
(mapcar #'normalize '((-0.6274163263702638 -1.1109331508946907) (-0.7295393639872552 -0.9829969869302896)))

OUTPUT 
#2A((2 4) (5 6)) 
#2A((-0.8502895603049073 0.6310494470252488) (0.48021391012397907 -0.3563947349052506)) 
(0.8475847241756916) 
(((-1.238561133420983 0.6994961629240704) (0.809965048118927 -0.6011222759714248))) 
8.955557775047327 
(mapcar #'norm '((-1.238561133420983 0.6994961629240704) (0.809965048118927 -0.6011222759714248)))
(mapcar #'normalize '((-1.238561133420983 0.6994961629240704) (0.809965048118927 -0.6011222759714248)))
(mprint (setq B (* 0.8475847241756916
	   (outer-product '(-1.238561133420983 0.6994961629240704) '(0.809965048118927 -0.6011222759714248)))))
(svd (- A B))
(#2A((-0.49175878861240074 -0.8707314705593591) (-0.8707314705593591 0.49175878861240074)) (8.955557775047328 0.8933000267487773) #2A((-0.5959623134688998 0.8030124039669605) (-0.8030124039669605 -0.5959623134688998)) T)
|#
#| - Sat Mar 7 09:16:32 1998
comparison of solutions with tsvd and tsvd1:

(def jack
#2a(
    (-3      0      2      4     -1      0    )
    ( 0     -3      5      6     -1      0    )
    ( 2      5     -4      0      0     -2    )
    ( 4      6      0     -4      0     -2    )
    (-1     -1      0      0      0      0    )
    ( 0      0     -2     -2      0      0    )
   )
     )
(def func '(9 19 -1 2 -1.0 -7.0))
(solve jack func)
;; (0.6808510638297876 0.31914893617021256 1.914893617021276 1.5851063829787235 -0.8723404255319158 -1.8510638297872344)

(def jack
     #2a(
	 (-6      0      2      4     -2      0    )
	 ( 0     -6      5      6     -2      0    )
	 ( 2      5     -8      0      0     -4    )
	 ( 4      6      0     -8      0     -4    )
	 (-2     -2      0      0      0      0    )
	 ( 0      0     -4     -4      0      0    )
	 )
     )
(def func '(6 16 -9 -6 -1.0 -7.0))
(solve jack func)
;; (0.7094240837696336 -0.2094240837696336 0.9869109947643979 0.763089005235602 -2.6151832460732987 0.36910994764397886)
|#
