#| - Mon Mar 9 09:57:39 1998
        Pithed tsvd2 a little,
1) removing the lambda-singular-tensor terms from the partials wrt ui,
2) using pseudo-solve instead of solve in non-linear-system-root, and
3) adding a "rejector", which pitches "solutions" if the system norm is not
small.
|#
(require "tensor")
(require "nonnewt1")
#| - Fri Mar 6 08:11:18 1998

	This is a revision of tsvd1.lsp, based on a candlelight session with
the algorithm last night.
|#
#| - Sun Mar 8 20:58:52 1998

	It's working, finally, but producing somewhat strange results. They
	are, however, the same strange results that tsvd.lsp and tsvd1-new.lsp
	are producing!

|#
;; this here's our function:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				non-linear-system-root
;;
(defun non-linear-system-root(f jack x0 &key (eps 1e-12) (maxits 100) verbose)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			non-linear-system-root Doc:
;;
;; Author: Andy Long
;; Created: Wed Feb 25 20:51:44 1998
;;
;; Description/Arguments: non-linear-system-root
  
;; Arguments: f jack x0 &key (eps 1e-12) (maxits 100)

;; Returns: a vector x which we hope gives the vectoral equation f(x)=0.

;; Notes: For DESS class at the IMSP

;; Examples:
 ;; How about trying to solve linear systems?
 (def A #2a((2 4)(5 2)))
 (def b #(2 1))
 ;; now we set up the two functions, Ax-b:
 (defun f(z)
  (let ((x (first z))
	(y (second z)))
    (list
     (- (+ (* 2 x) (* 4 y)) 2)
     (- (+ (* 5 x) (* 2 y)) 1))))
 (defun jack(z) A)
 ;; and compare solutions:
 (non-linear-system-root #'f #'jack (list 0 0) :eps 1e-12)
 (solve A b)
"
  (do*
   (
    (i 1 (1+ i))
    (x0 x0 x1)
    (x1 
     (+ x0 (pseudo-solve (funcall jack x0) (- (funcall f x0))))
     (+ x0 (pseudo-solve (funcall jack x0) (- (funcall f x0))))
     )
    )
   ((or (> i maxits) (< (norm (- x1 x0)) eps)) (print i) x1)
   (if verbose (print x1))
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Now let's create a function that finds the TSVD in general:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				nltsvd
;;
(defun nltsvd(A &key verbose (eps 1e-12) (maxits 100) initial (norm-eps 1e-6))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			nltsvd Doc:
;;
;; Author: Andy Long
;; Created: Fri Feb 27 10:35:52 1998
;;
;; Description/Arguments: nltsvd
  
;; Arguments: A &key verbose (eps 1e-12) (maxits 100) initial (norm-eps 1e-6)

;; Returns: the tsvd of a tensor, obtained using newton's method to solve for a
   root of a system of non-linear equations.

	DOESN'T WORK YET, but it's coming!

;; Notes: Stemming from my DESS class at the IMSP, and looking toward a method
   that generalizes for the TSVD (Tensor SVD).

;; See Also: non-linear-system-root

;; Examples:
 (def A #2a((2 4)(5 6)))
 (def A #2a((2 4 3)(5 6 2)))
 (nltsvd A)

 ;; (def A #2a((2 4 -1)(5 6 2)(1 2 1)))
 ;; (nltsvd A :verbose 2)

 (def a #3A(((-0.9562550022010874 -0.7837553833206803) (0.3178325831753665 -0.09749946752966593)) ((-0.06698119877233262 -0.18842949479223647) (0.02949082327057139 0.3176719983598651))))
 (nltsvd A :maxits 500)

 (def st1 (vectors-to-tensor '(1 1) '(-1 1) '(-4 6)))
 (def st2 (vectors-to-tensor '(-1 1) '(1 1) '(3 2)))
 (def A (+ st1 st2))
 (send (new tensor (size A) A) :norm)
 (send (new tensor (size st1) st1) :norm)
 (send (new tensor (size st2) st2) :norm)

 (def crap (optimize-rank-one-over-tensor A))
 ;; todo: just discovered that optimize-rank-one-over-tensor can also find a
 ;; minimum! I thought it was going to find a maximum only, oops!
 (t-mult (apply #'vectors-to-tensor (first crap)) A (iseq 3) (iseq 3))

 (nltsvd A)
"
  (let* (
	 ;; If A is a real tensor, then select off the elements. Otherwise,
	 ;; if A is an array, continue; if not, produce an error message.
	 (A (cond
	     ((tensorp A) (send A :elements))
	     ((arrayp A) A)
	     (t (error
		 "nltsvd requires a tensor or an array as an argument."))))

	 ;; size is a list of the array dimensions of A:
	 (size (size A))

	 ;; The subsets, created in two steps, are designed to be used to
	 ;; select the individual vectors of each dimension from a single
	 ;; vector containing them all:
	 (subsets (mapcar #'iseq size))
	 (subsets (+ subsets (combine 0 (butlast (cumsum size)))))

	 (numdims (length size))
	 (numdims-seq   (iseq numdims))
	 (numdims-1-seq (iseq (- numdims 1)))
	 (numdims-2-seq (iseq (- numdims 2)))

	 (but-is (mapcar (lambda(i) (remove i numdims-seq)) numdims-seq))

	 ;; The SVD should have at most dim schmidt-tuples with non-zero
	 ;; singular values, but we need to seek them all anyway for the
	 ;; moment:
;;;	 (dim (apply #'* (butlast (sort-data size))))
	 (dim (apply #'* size))

	 ;; We start with just the lagrange multipliers corresponding to the
	 ;; unit size of the singular tensor: later we'll add one for each
	 ;; singular tensor already found:
	 (lagrange-multipliers 1)
	 (lagrange-seq (iseq 1))
	 
	 schmidt-tuples ;; contains the vector returned by
			;; non-linear-system-root, which in turn contains the
			;; components of the rank-one tensor and lagrange
			;; multipliers.

	 sts		;; list containing the singular tensors
	 lambdas	;; list containing the singular values

	 order		;; used to order the singular values/tensors, and for
			;; yeoman duty elsewhere
	 fval		;; the function value at a supposed zero.
	 )

    (dotimes
     (i dim)
     (print (list 'iteration i))
     (flet (
	    (funcs(z)
		  (if verbose (print "entering funcs"))
		  (let* (
			 ;; uvs is a list containing the vectors of the
			 ;; decomposition, selected from the single vector z
			 ;; (which also contains the singular values):
			 (uvs (mapcar (lambda(u)(select z u)) subsets))
			 
			 ;; We'll need this norm term (the partial derivative
			 ;; of f wrt the new lagrange multiplier):
			 (uvs-norm-term
			  (* .5
			     (- 1 (inner-product-of-rank-one-tensor-lists
				   uvs uvs))))

			 ;; uvs-sts is a list of inner products of the trial
			 ;; singular tensor and the already discovered singular
			 ;; tensors (if any):
			 (uvs-sts
			  (mapcar
			   (lambda(st)
			     (inner-product-of-rank-one-tensor-lists uvs st))
			   sts))

			 (lambda-norms (select-from-end z lagrange-seq))

			 ;; Our decomp contains at least uvs:
			 (decomp (list uvs))
			 ;; if there are already some sts, however, we need to
			 ;; tack our decomp onto those:
			 (decomp (if sts (smoosh (list sts decomp)) decomp))
			 ;; Now we turn them into tensor, weight them by their
			 ;; singular values and add them up:
			 (decomp (mapcar
				  (lambda(x lam)
				    (* (apply #'vectors-to-tensor x) lam))
				  decomp
				  lambda-norms
				  ))
			 (decomp (if decomp (apply #'+ decomp)))
;;todo			 (decomp (final decomp))

			 ;; This is a tensor that I have to use several times:
			 (A-decomposition (- A decomp))
			 
			 ;; We combine the partials of f into result:
			 (result
			  (combine
			   ;; the partials of f wrt u,
			   (mapcar
			    (lambda(i)
			      (let ((but-i (elt but-is i)))
				(t-mult 
				 (apply #'vectors-to-tensor (select uvs but-i))
				 A-decomposition
				 numdims-1-seq
				 but-i)
				))
			    numdims-seq)
			   ;; and the partials wrt lambda:
			   (if uvs-sts (combine (- uvs-sts) uvs-norm-term)
			     uvs-norm-term)
			   )
			  )
			 )
		    (if verbose (print result))
		    result
		    )
		  )
	    (jack (z)
		  (if verbose (print "entering jack"))
		  (let* (
			 (uvs (mapcar (lambda(slab)(select z slab)) subsets))
			 
			 (lambda-norms (select-from-end z lagrange-seq))
			 (lambda-u (final lambda-norms))
			 
			 (all-rank-ones
			  (if sts (smoosh (list sts (list uvs))) (list uvs)))

			 ;; Our decomp contains at least (* 2 uvs) (NB:
			 ;; this is twice the initial value of decomp used in
			 ;; func):
			 (decomp (list (* 2 uvs)))
			 ;; if there are already some sts, however, we need to
			 ;; tack our decomp onto those:
			 (decomp (if sts (smoosh (list sts decomp)) decomp))
			 ;; Now we weight them by their singular values and add
			 ;; them up:
			 (decomp (mapcar
				  (lambda(x lam)
				    (* (apply #'vectors-to-tensor x) lam))
				  decomp
				  lambda-norms
				  ))
			 (decomp (if decomp (apply #'+ decomp)))
;;todo			 (decomp (final decomp))

			 ;; This is a tensor that I have to use several times:
			 (A-decomposition (- A decomp))

			 ;; jack will be a small "gross" matrix containing the
			 ;; minimum necessary to later use combine-matrices: it
			 ;; has one dimension for each dimension of the tensor,
			 ;; plus a dimension for the lagrange-multiplier stuff.
			 (jack (+ numdims 1))
			 (jack (make-array (list jack jack)))
			 )

		    ;; The next two take care of the diagonal:
		    (dotimes
		     (i numdims)
		     (let ((short-u (select uvs (elt but-is i))))
		       (setf
			(aref jack i i)
			(* (- lambda-u)
			   (inner-product-of-rank-one-tensor-lists
			    short-u short-u)
			   (id (elt size i))))
		       )
		     )
		    (setf
		     (aref jack numdims numdims)
		     (zeros lagrange-multipliers lagrange-multipliers))

		    ;; Now for the a-matrices:
		    (dotimes
		     (i (1- numdims))
		     (let ((keepers (elt but-is i)))
		       (dolist
			(j (iseq (1+ i) (1- numdims)))
			(let ((keepers (remove j keepers)))
			  (if keepers
			      (setf
			       (aref jack i j)
			       (t-mult 
				(apply #'vectors-to-tensor (select uvs keepers))
				A-decomposition
				numdims-2-seq
				keepers)
			       (aref jack j i)
			       (transpose (aref jack i j))
			       )
			    ;; Otherwise, just matrix case!
			    (let ()
			      (setf (aref jack i j) A-decomposition)
			      (setf (aref jack j i) (transpose A-decomposition))
			      )
			    )
			  )
			)
		       )
		     )

		    ;; Now for the lagrange multiplier stuff:
		    (dotimes
		     (i numdims)
		     (let* (
			    ;; we want all but the ith element:
			    (but-i (elt but-is i))

			    ;; coefs are the inner products of the singular
			    ;; tensors generated so far and the new guess for a
			    ;; singular tensor.
			    (coefs
			     (mapcar
			      (lambda(st)
				(inner-product-of-rank-one-tensor-lists
				 (select uvs but-i)
				 (select st but-i)
				 )
				)
			      all-rank-ones
			      )
			     )
			    ;; result starts out as the ith vector of each
			    ;; singular tensor:
			    (result
			     (mapcar (lambda(st) (elt st i)) all-rank-ones))
			    ;; Then we weight that vector by the negative of
			    ;; the inner products coefs:
			    (result (* (- coefs) result))
			    ;; Finally we bind it into a matrix:
			    (result (apply #'bind-rows result))
			    )
		       (setf
			(aref jack numdims i) result
			(aref jack i numdims) (transpose result)
			)
		       )
		     )
		    (setq jack (apply #'combine-matrices (row-lists jack)))
		    (if verbose (print "leaving jack"))
		    jack
		    )
		  )
	    )
	   
	   (if verbose (print "or not!"))
	   (setq
	    initial
	    (if (and (= 0 i) initial) initial
	      (combine 
	       (mapcar #'normalize (mapcar #'rand.n size))
	       (if lambdas (combine lambdas (rand.u 1))
		 (rand.u 1))
	       ))
	    order (printall 'initial initial)
	    schmidt-tuples
	    (non-linear-system-root
	     #'funcs #'jack initial 
	     :verbose verbose :maxits maxits :eps eps
	     )
	    fval (funcall #'funcs schmidt-tuples)
	    )
	   
	   (if (< (norm fval) norm-eps)
	       (setq
		order (printall
		       'schmidt-tuples
		       schmidt-tuples
		       'function-value
		       fval
		       'ratios-of-lambdas
		       (/ 
			(select-from-end schmidt-tuples lagrange-seq)
			(if lambdas (combine lambdas 1) 1)))
		lambda (final schmidt-tuples)
		;; Here I'm normalizing the st:
		st (mapcar (lambda(i) (normalize (select schmidt-tuples i))) subsets)
		order (if (< lambda 0) (setf (elt st 1) (- (elt st 1))))
		sts (append sts (list st))
		;; and make sure that lambda itself is positive:
		lambdas (append lambdas (list (abs lambda)))
		order (printall 'singular-stuff lambdas sts)
		lagrange-multipliers (+ 1 lagrange-multipliers)
		lagrange-seq (iseq lagrange-multipliers)
		
		norm-decomp
		(- A
		   (apply
		    #'+ 
		    (mapcar 
		     (lambda(lam st)
		       (* lam (apply #'vectors-to-tensor st)))
		     lambdas sts)))
		norm-decomp
		(send (new tensor (size norm-decomp) norm-decomp) :norm)
		)
	     )
	   )
     (if (< norm-decomp norm-eps) (return)
       (print norm-decomp)
       )
     )
    ;; order the singular values from largest to smallest:
    (setq
     order (reverse (order lambdas))
     lambdas (select lambdas order)
     sts (select sts order)
     )
    (if verbose (print "This will come!"))
    
    (list lambdas sts)
    )
  )

(defun inner-product-of-rank-one-tensor-lists(a b)
  (apply #'* (mapcar #'sum (* a b))))

(try-out nltsvd)

#| - Sun Mar 8 17:48:30 1998
;; I'm trying to discover the relationship between the singular values and the
;; lambdas in the schmidt-uples
(nltsvd a :norm-eps 1e-3)
(nltsvd st1 :norm-eps 1e-3)
|#

#| - Mon Mar 9 09:33:14 1998

(defun vect-test(crap)
  (pprint
   (mapcar
    (lambda(crap)
      (mapcar
       (lambda(i)
	 (let ((but-i (remove i (iseq (length crap)))))
	   (/ 
	    (t-mult
	     (apply #'vectors-to-tensor (select crap but-i))
	     A (iseq 2) but-i)
	    (elt crap i) 
	    )
	   )
	 )
       (iseq (length crap))
       )
      )
    crap)))
;; (0.08401299703960788) 
(def crap
     '(
       ((0.17915061040951058 0.9838216600532333) (-0.6000188746120693 -0.7999858436930405) (0.8756923329446266 -0.4828694834238309))
       ))
(vect-test crap)

;; I've identified these as eigentensors of my random tensor
 (def a #3A(((-0.9562550022010874 -0.7837553833206803)
	     (0.3178325831753665 -0.09749946752966593))
	    ((-0.06698119877233262 -0.18842949479223647)
	     (0.02949082327057139 0.3176719983598651))))
;; using the command
 (nltsvd A :maxits 500)

(def ets
     '(
       ((-0.9868068532938147 -0.16190192800692543)
	(0.9844555820252675 -0.1756337297311993)
	(0.7858094253764382 0.6184687114070944)) 
       ((-0.21004815200103133 0.9776910421196215)
	(-0.12300692823026352 0.9924058119576662)
	(-0.1618767236012316 0.9868109881614262))
       ((-0.7260321003248611 0.6876608097731547)
	(0.01920490852183674 0.9998155687368886)
	(-0.5538726528932066 0.8326013958534069))
       ((0.17915061040951058 0.9838216600532333)
	(-0.6000188746120693 -0.7999858436930405)
	(0.8756923329446266 -0.4828694834238309))
       ))
(vect-test ets)
(mapcar
 (lambda (x)
   (apply #'inner-product-of-rank-one-tensor-lists x))
 (all-pairs ets))

Once a single et is identified, however, I don't manage to find another: only
the "almost ets" that are related to it.

Now I've removed part of the pithing:
(1.2669366023088657 0.33677121113856084 0.2357838876055511 0.08282478695990372) 
(def crap
     '(((-0.9868068532938146 -0.16190192800692543) (0.9844555820252675 -0.17563372973119926) (0.7858094253764383 0.6184687114070945)) ((-0.16190192800692546 0.9868068532938147) (0.2054760576776626 -0.9786621427853669) (0.11909920820205662 -0.9928823588953746)) ((0.9868068532938147 0.16190192800692546) (0.17563372973119942 0.9844555820252675) (0.6184687114070949 -0.7858094253764378)) ((0.16190192800692543 -0.9868068532938147) (-0.9786621427853106 -0.20547605767793076) (0.9928823588953786 0.11909920820202209))))
(vect-test crap)
;; only the first is an et: the rest have only two et directions.
|#
