(defun hulp ()
  "
	Hulp is a point and click, interactive help window system. To activate
it, just type (hulp).

	It includes an index of all functions and variables defined in the
system, and search and apropos facilities.

;; Notes: New documentation is added to the manual.lsp file, in the etc
;; subdirectory.

;; Examples:
 (hulp)
"
  (let* (
	 (e1 (mapcar #'first (second %manual%)))
	 (k1 (send text-item-proto :new "Parts"))
	 (k2 (send text-item-proto :new "Chapters"))
	 (k3 (send text-item-proto :new "Sections"))
	 (l1
	  (send list-item-proto :new e1
		:action
		#'(lambda (x)
		    (if x
			(let* (
			       (k1 (send self :selection))
			       (d1 (send self :slot-value 'dialog))
			       (e1 (send d1 :slot-value 'items))
			       (l2 (second (second (first e1))))
			       (l3 (second (third (first e1))))
			       (v1 (second (elt (second %manual%) k1)))
			       (rr (length v1))
			       )
			  (send l2 :clean)
			  (send l3 :clean)
			  (dotimes
			   (i rr)
			   (send l2 :set-text i (first (elt v1 i))))
			  )))))
	 (l2
	  (send list-item-proto :new (repeat "" 40) ;; was 40
		:action
		#'(lambda (x)
		    (if x
			(let* (
			       (d1 (send self :slot-value 'dialog))
			       (e1 (send d1 :slot-value 'items))
			       (l1 (second (first (first e1))))
			       (l3 (second (third (first e1))))
			       (k1 (send l1 :selection))
			       (k2 (send self :selection))
			       (v1 (second (elt (second %manual%) k1)))
			       (v2 (if (< k2 (length v1))
					  (second (elt v1 k2))))
			       (rr (length v2))
			       )
			  (if (= 1 rr)
			      (find-and-show-string 
			       (elt (send self :slot-value 'list-data) k2)))
			  (send l3 :clean)
			  (dotimes (i rr) (send l3 :set-text i (elt v2 i)))
			  )))))
	 (l3
	  (send list-item-proto :new (repeat "" 110) ;; was 60
		:action
		#'(lambda (x)
		    (if x
			(let* (
			       (k3 (send self :selection))
			       (t3 (elt
				    (send self :slot-value 'list-data)
				    k3))
			       )
			  (find-and-show-string t3)
			  )))))
	 (sr (send button-item-proto :new "Search"
		   :action
		   #'(lambda () 
		       (find-and-show-string 
			(get-string-dialog 
			 "Give Me a String to Search For")))
		   ))
	 (ap (send button-item-proto :new "Apropos"
		   :action
		   #'(lambda () 
		       (hulp-index 
			(find-and-show-substring 
			 (get-string-dialog 
			  "Give Me a Substring to Search For"))))
		   ))
	 (in (send button-item-proto :new "Index"
		   :action #'hulp-index))
	 (ok (send button-item-proto :new "Enough !"
		   :action
		   #'(lambda () 
		       (let ((d1 (send self :slot-value 'dialog)))
			 (send d1 :close)))))
	 (cdiag (send dialog-proto :new
		      (list
		       (list
			(list k1 l1 in ap sr ok)
			(list k2 l2)
			(list k3 l3)))
		      :default-button ok :title "All the Help You Need"))
	 )
    )
  )

(defun hulp-index
  (&optional
   (hh
    (mapcar #'string-downcase
	    (mapcar #'symbol-name
		    (combine
		     (package-obarray ::xlisp))))))
  (if hh
      (let* (
	     (hh (select hh (order hh)))
	     (ll
	      (send list-item-proto :new hh :columns 2
		    :action
		    #'(lambda(x)
			(if x
			    (let* (
				   (kk (send self :selection))
				   (tt
				    (elt
				     (send self :slot-value 'list-data) 
				     kk))
				   )
			      (find-and-show-string tt)
			      )))))
	     (ok
	      (send button-item-proto :new "Enough !"
		    :action #'(lambda () 
				(send (send self :slot-value 'dialog) :close))))
	     (dd (send dialog-proto :new 
		       (list (list ll ok))
		       :default-button ok :title "Index"))
	     )
	)))

(defmeth list-item-proto :clean ()
  "
List-item-proto method :clean sets all list elements to the empty string
\"\".
" 
  (let* (
	 (x (send self :slot-value 'list-data))
	 (n (length x))
	 )
    (send self :selection nil)
    (dotimes (i n) (send self :set-text i ""))
    ))

(defun find-and-show-string (x)
  "
;;; ael
Fixed things up a bit.
;;; lea
;; Examples:
 (find-and-show-string \"help\")
"
  (if (> (length x) 0) (x11-display-help x))
  )

(defun find-and-show-substring (x)
  "
;; Description: function find-and-show-substring (string)

	This function searches the list of functions, variables, etc for those
 whose names contain the substring given.

;; Examples
 (find-and-show-substring \"plot\")
 (find-and-show-substring \"slithy-toves\")
"
  (let* (
	 (show (apropos-list x))
	 ;; let's only consider those with function doc:
	 (has-doc (mapcar #'(lambda(s) (if (documentation s 'function) t nil))
			  show))
	 (show (if (and has-doc (which has-doc)) (select show (which has-doc))))
	 (show (mapcar #'string-downcase (mapcar #'symbol-name show)))
	 ;; remove the variable we already couldn't find:
	 (show (remove x show :test #'equal))
	 )
    (cond
     ((not show)
      (message-dialog (format nil "No Such Substring: ~a" x))
      )
     ((= (length show) 1)
      (find-and-show-string (first show))
      )
     (t
      (let* (
	     (ll
	      (send list-item-proto :new (select show (order show))
		    :action
		    #'(lambda(x)
			(if x
			    (let* (
				   (kk (send self :selection))
				   (tt
				    (elt
				     (send self :slot-value 'list-data) 
				     kk))
				   )
			      (find-and-show-string tt)
			      )))))
	     (try-out
	      (send button-item-proto :new "Try out"
		    :action
		    #'(lambda()
			(let* (
			       (kk (send ll :selection))
			       (tt
				(if kk
				    (elt
				     (send ll :slot-value 'list-data) kk)))
			       )
			  (if tt
			      (let ()
				(helplass (read-from-string tt))
				(format t "~%> ")
				(force-output)
				)
			    (message-dialog "Please make a selection.")
			    )))))
	     (help
	      (send button-item-proto :new "Help"
		    :action
		    #'(lambda()
			(let* (
			       (kk (send ll :selection))
			       (tt
				(if kk
				    (elt
				     (send ll :slot-value 'list-data) kk)))
			       )
			  (if tt (find-and-show-string tt)
			    (message-dialog "Please make a selection.")
			    )
			  (force-output)
			  ))))
	     (help-string (format nil "There are several commands
that contain the string
   \"~a\".
Please make a selection:" x))
	     (dd (send dialog-proto :new 
		       (list help-string (list ll (list help try-out)))
		       :default-button try-out :title x))
	     ))
      )
     )
    )
  )

(defun x11-display-help (name)
  "
;; Description:  (x11-display-help name)

Displays help in an xterm window. Help documentation is found either in the
actual definition of a function/macro/method, or in the file xlisp.hlp.

Be careful about modifying xlisp.hlp - help may fail to load if it is
improperly modified. On the other hand, I like to modify it, so you might too!

;; Note: This is really a helper created for hulp.

;; Examples:
 (x11-display-help \"x11-display-help\")
 (pause 120) ;; just added this to permit both to be displayed. The second will
	     ;; fail to find any documentation, of course!
 (x11-display-help \"dksldkfjsk\")
 (pause 120)
 (x11-display-help \"string-\")
"
  (let (
	(fname (strcat *andy-default-path* "Help" separator "help.txt"))
	check
	)
    (with-open-file
     (stream fname :direction :output)
     (setq check (helpless (first (string-to-list name)) stream))
     )
    (if check
	(let ()
	  #+unix
	  (system
	   (format
	    nil
	    "xterm -geom 81x~d+0+0 -title \"~a\" -e less -m \"~a\" &"
	    (min 40 (+ 2 (size (read-file fname)))) name fname
	    ))
	  #+msdos
	  (system (strcat "notepad " fname))
	  #+macintosh
	  (let ((w (send display-window-proto :new :title name)))
	    (send w :location 30 18)
	    (send w :size 590 300)
	    (with-open-file
	     (instream fname) 
	     (do
	      ((filebit (read-line instream nil nil)
			(read-line instream nil nil)))
	      ((not filebit))
	      (send w :paste-string filebit)
	      (send w :paste-string (string #\Newline))
	      )
	     )
	    )
	  )
      )
    )
  )

(if (find :macintosh *features*)
    ;;ael    (require "machulp"))
    (require "machulp.fsl"))
