;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Alias Aids
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This is a list of aliases, so that we can keep track of what's been done:
(def *alias-list* nil)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; alias Doc:    Creation Date:5/7/96        Author: Andy Long
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro alias (this that)
  "
;; Documentation: MACRO alias

      Alias gives a new name to an old function (which is good for
shortening, or renaming, an otherwise difficult-to-type name).

;; Examples:
 (func f(x y) x * y)
 (alias hap f)
 (hap 2 3)

 (alias gle2 get-lambda-expression)
 (gle2 #'f)

 (alias cap2 close-all-plots)
 (plot f)
 (cap2)
" 
  (let* (
	 (bing (symbol-name `,that))
	 (bong (symbol-name `,this))
	 (docf (documentation `,that 'function))
	 (bang
	  (strcat 
	   "       "
	   Bong
	   " is an alias for "
	   bing
	   ", defined using the alias function.
What follows is the documentation for "
	   bing
	   ", if any:
" 
	   (if docf docf "")
	   )
	  )
	 )
    (setq *alias-list* (append *alias-list* (list (list bong bing))))
    `(defun ,this (&rest x) ,bang (apply #',that x))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for MACRO-ALIAS:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro macro-alias (this that)
  "
;; Description: MACRO (macro-alias this that)

       Macro-alias allows one to rename ('alias') any useful macros.
You may not like the name 'help', and prefer the name 'hap'!

;; Examples:
 (macro-alias hap help)
 (hap help)
 (macro-alias gle2 gle)
 (gle2 hap)
 (hap gle2)
" 
  (let* (
	 (x ',@x)
	 (bing (symbol-name `,that))
	 (bong (symbol-name `,this))
	 (docf (documentation `,that 'function))
	 (bang
	  (strcat
	   "      "
	   bong
	   " is an alias for macro "
	   bing
	   ",
defined using the macro-alias macro. What follows is the help information
available concerning "
	   bing
	   ":
"
	   (if docf docf "")
	   ))
	 )
    (setq *alias-list* (append *alias-list* (list (list bong bing))))
    `(defmacro
       ,this				;; the name
       (&rest x)			;; the arguments
       ,bang				;; the documentation
       (append (list (quote ,that)) x)	;; the definition
       )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; aliases Doc:    Creation Date:5/7/96        Author: Andy Long
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun aliases()
  "
;; Description/Arguments: FUNCTION aliases

       (aliases)
  
Conjures up the list of everything that's been aliased so far, by just printing
*alias-list*.

;; See Also: alias, *alias-list*

;; Example:
 (aliases)
"
  (dolist
   (i *alias-list*)
   (print i)
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for FUNCTION-TO-MACRO:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro function-to-macro (this that)
  "
      (function-to-macro this that)

      Creates
this
      a new macro to function
that.

Doesn't work! Or rather, it works, but it doesn't do what I want it to do: I
want to be able to invoke a macro rather than a function, as this saves me
quoting everything, and calling function #'f and other horrible stuff.

	In order to do that in general I'd have to know which arguments are
functions, which symbols, etc., so this just won't work I'm afraid:

;; Documentation:
Function-To-Macro creates a macro to an old function  (usually good for
shortening, or renaming an otherwise difficult-to-type name).

;; Examples:
;; this'n doesn't work.
;; (function-to-macro gle2 get-lambda-expression)
;; (get-lambda-expression #'gle2)
;; (gle2 gle2)

;; this'n works, but what's the use?
 (func f(x) x^2)
 (function-to-macro hap f)
 (hap 2)
 (def stuff '(1 2 3))
 (hap stuff)
" 
  (let* (
	 (bing (symbol-name `,that))
	 (bong (symbol-name `,this))
	 (bang
	  (strcat 
	   "       "
	   Bong
	   " is a macro alias for function "
	   bing
	   ", defined using the
function-to-macro function. For more information, type

                      (help "
	   bing
	   ")"
	   ))
	 (printall bong bang bing)
	 (string
	  (format
	   nil
	   "(defmacro ~s (&rest x) ~s `(funcall (function ~s) ,@x))"
	   this bang that
	   ))
	 )
    (print string)
    (eval-string string)
    nil
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; gle Doc:    Creation Date: long, long ago!        Author: Andy Long
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro gle (x &optional method)
  "
;; Description/Arguments:

       (gle f &optional method), where f is a function, or an object and method
  is the method corresponding to the object.
  
;; Notes: a macro for the function get-lambda-expression.

;; See Also: (help get-lambda-expression)

;; Examples:
 (func f(x) x ^ 2)
 (gle f)

 ;; Notice that self is an implicit argument to a method:
 (defmeth scatterplot-proto :gle-test () (plot sin))
 (gle scatterplot-proto :gle-test)
 ;; Notice that it is the first argument:
 (defmeth scatterplot-proto :gle-test (x) (sin x))
 (gle scatterplot-proto :gle-test)
"
  (if method
      `(get-lambda-expression (send ,x :get-method ,method))
    `(get-lambda-expression #',x)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
