;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			animate2 - trying to smooth it out
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun animate2 (f parameters xmin xmax &key
		   ymin ymax
		   (speed 0)
		   loops
		   (clear t)
		   (variable-labels '("x" "y"))
		   )
  "
;; DESCRIPTION:

 (animate2 f parameters xmin xmax
		&key
		   ymin ymax
		   (speed 0)
		   loops
		   (clear t)
		   (variable-labels '(\"x\" \"y\"))
		   )

This is my own animation program, for arbitrary functions. Too cool!

 Options:
	speed:
	   can be used to slow down an animation (values in sixtieths of
	   seconds) 
	loops: a positive integer
	    loops will loop the animation back and forth twice loops times
	variable-labels: a list of two labels (defaults to \"x\" and \"y\").


The title defaults to the lambda expression of the function f.

					;	       Andy Long, 1995

;; Examples:

 (defun f (parameter x) (/ (+ (* 3 x) 5) (+ x parameter)))
 (def parameters (rseq -5 5 25))
 (animate2 #'f parameters -10 10)
 (animate2 #'f parameters -10 10 :ymin -10 :ymax 10 :loops 1)

;; with more bells and whistles:
 (animate2 #'f parameters -10 10 :ymin -10 :ymax 10 :loops 1)
 (animate2 #'f parameters -10 10 :ymin -10 :ymax 10 :loops 3
	 :variable-labels '(\"crumcake\" \"suds\"))
"
  (andy-print "in animate")
  (let* (
	 (params parameters)
	 (iterations (length parameters))
	 polyy
	 parameter
	 (num-pnts (if *points* *points* 100))
	 (polyx (rseq xmin xmax num-pnts))
	 )
    (if loops
	(setq params (append parameters (reverse parameters))
	      params (append (repeat params loops))
	      iterations (length params))
      )
    (dotimes
     (j iterations)
     (setq
      parameter (elt params j)
      polyy (mapcar f 
		    (if (listp parameter)
			(repeat (list parameter) (length polyx))
		      (repeat parameter (length polyx)))
		    polyx)
      )
     (if (> j 0)
	 (let ()
	   (if clear
	       (let ()
		 (send my-plot :start-buffering)
		 (send my-plot :clear-lines :draw t)
		 )
	     )
	   (send my-plot :add-lines polyx polyy
		 :color 'black :variable-labels variable-labels)
	   (if *x-axis* (send my-plot :abline 0 0))
	   (if *y-axis* 
	       (let()
		 (setq my-plot.range (send my-plot :range 1))
		 (send my-plot :add-lines '(0 0) my-plot.range)
		 )
	     )
	   (if clear (send my-plot :buffer-to-screen))
	   )
       (let()
	 (def my-plot (plot-lines polyx polyy
				  :color 'black
				  :variable-labels variable-labels)
	      )
	 (send my-plot :add-slot 'plot-item
	       (list (list 'lines polyx polyy 'black)))
	 (if *title* (send my-plot :title *title*)
	   (send my-plot :title "Animation"))
	 (if *x-axis* (send my-plot :abline 0 0))
	 (send my-plot :range 0 xmin xmax)
	 (if *range* (send my-plot :range 1 (first *range*) (second *range*)))
	 (if *y-axis* 
	     (let()
	       (setq my-plot.range (send my-plot :range 1))
	       (send my-plot :add-lines '(0 0) my-plot.range)
	       )
	   )
	 )
       )
     (pause speed)
     )
    (andy-print "leaving animate2...")
    my-plot
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; animate Doc:    Modification Date:1/19/96        Author: Andy Long
;;
;;
(defmacro animate (f parameters &rest xrange &key loops)
  "
;; Description/Arguments:

       (animate f parameters &rest xrange &key loops)
  
       Animate is a macro which takes a function, defined on two variables, and
 plots slices of it as the FIRST variable ranges over a given parameter list
 (parameters).

       If the loops key is used, then the animation will be looped for the
number of times specified.

;; Notes: Animate makes use of the function animate2.

;; See Also: animate2

;; Examples:
 (domain -10 10)
 (range -10 10)
 (title \"Animation Example\")

 (func f (x y) sin x + y)
 (def parameters (grid -pi pi 25))
 (animate f parameters)

 (func f (y x) sin x + y)
 (def parameters (grid -5 5 25))
 (animate f parameters :loops 3)
 (animate f parameters -2pi 2pi)
 (animate f parameters -2pi 2pi :loops 3)

 (func g (p x) (3 * x + 5) / (x + p))
 (animate g parameters -10 10 :loops 3)
"
  (let (
	(loops (first (catch-keys '(:loops) xrange)))
	(xrange (remove-keys '(:loops) xrange))
	xmin xmax
	)
    (cond
     (xrange
      (setq xmin (first xrange))
      (setq xmax (second xrange))
      )
     (*domain*
      (setq xmin (first *domain*))
      (setq xmax (second *domain*))
      )
     (t
      (setq xmin -1)
      (setq xmax 1)
      )
     )
    (if loops
	`(animate2 #',f ,parameters ,xmin ,xmax :loops ,loops)
      `(animate2 #',f ,parameters ,xmin ,xmax)
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; slice Doc:    Modification Date:1/19/96        Author: Andy Long
;;
;;
(defmacro slice (f parameters &rest xrange &key loops clear)
  "
;; Description/Arguments:

       (slice f parameters &rest xrange &key loops clear)
  
       Slice is a macro which takes a function, defined on two variables, and
 plots slices of it as the FIRST variable ranges over a given parameter list
 (parameters).

       If the loops key is used, then the animation will be looped for the
 number of times specified.

;; Notes: Slice makes use of the function slice2.

;; See Also: slice2

;; Examples:
 (domain -10 10)
 (range -10 10)
 (title \"Animation Example\")

 (func f (x y) sin x + y)
 (def parameters (grid -pi pi 25))
 (slice f parameters)

 (func f (y x) sin x + y)
 (def parameters (grid -5 5 25))
 (slice f parameters)
 (slice f parameters -2pi 2pi)

 (func g (p x) (3 * x + 5) / (x + p))
 (slice g parameters -10 10)
 (slice g parameters -10 10 :loops 10)
"
  (printall loops xrange)
  (let (
	(loops (first (catch-keys '(:loops) xrange)))
	(xrange (remove-keys '(:loops :clear) xrange))
	xmin xmax
	)
    (printall loops xrange)
    (if loops (setq clear t))
    (cond
     (xrange
      (setq xmin (first xrange))
      (setq xmax (second xrange))
      )
     (*domain*
      (setq xmin (first *domain*))
      (setq xmax (second *domain*))
      )
     (t
      (setq xmin -1)
      (setq xmax 1)
      )
     )
    `(animate2 #',f ,parameters ,xmin ,xmax :clear ,clear :loops ,loops)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
