#|
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                            ;;
;;			backparse.lsp: Function Parser                        ;;
;;                                                                            ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	This is a parser from prefix to infix, so that I can actually have a
prayer of identifying a function.

	The strategy is that everything in prefix is either unary or binary. So
I just have to check the number of elements of the list, leaving unaries alone
and switching the order of binaries.
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				 back-parse, the master:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun back-parse (e)
  "
Description:

	This is a parser from infix to prefix, so that one actually has a
prayer of identifying a function given as reverse polish notation:

;; Examples:
 (back-parse '(+ 1 2))
 (back-parse '(+ (/ 1 3) 2))
 (back-parse '(+ (/ 1 x) 2))
 (back-parse '(- (^ 3 (- X))))

 ;; Here's some mindless fun!
 (parser (back-parse '(- (^ 3 (- X)))))
 (back-parse (parser (back-parse '(- (^ 3 (- X))))))

 (func f(r TT) if (x < 0) 0 (2 * pi * r))
 (what f)
 (gle f)

 (func f(r TT) (2 * (pi ^ r)))
 (what f)
 (gle f)
"
  (let* (
	 (e (if (listp e) e (list e)))
	 (len-e (length e))
	 (new-e (copy-list e))  ;; this caused a lot of agony! Use copy-list...
	 e-member
	 j
	 )
    (if (eq (first e) 'if)
	(setq new-e
	      (list 'if
		    (if (listp (second e)) (back-parse (second e)) (second e))
		    (if (listp ( third e)) (back-parse ( third e)) ( third e))
		    (if (listp (fourth e)) (back-parse (fourth e)) (fourth e))
		    )
	      )
      (dotimes (j len-e)
	       (setq e-member (nth j new-e))
	       (if (listp e-member)
		   (setl new-e j (back-parse e-member))
		 )
	       )
      );; Now each list represents a coherent lisp whole.

    (if (= (length new-e) 1)
	(setq new-e (single-list new-e))
      (if (= (length new-e) 3)
	  (setq new-e (back-parse-whole new-e))
	)
      )
    new-e
    )
  )

(defun back-parse-whole (e)
  (list (second e) (first e) (third e))
  )
#|
(defun test-back-parse ()
  (printall
   (back-parse '(- 3))
   (back-parse '(+ 2 (- 3)))
   (back-parse '(- (^ 3 X)))
   (back-parse '(- (^ 3 (- X))))
   (back-parse '(- (^ 3 (+ Y (- X)))))
   (back-parse '(- (/ (^ 3 (+ Y (- X))) 6)))
   (back-parse '(- (* (^ 3 (+ Y (- X))) (^ X 3))))
   (back-parse '(+ (- (^ 3 (+ Y (- X)))) (^ X 3)))
   (back-parse '(- (/ (^ 3 (+ Y (- X))) (^ (TAN X) 3))))
   (back-parse '(- (/ (^ 3 (+ Y (- X))) (^ (TAN X) 3))))
   (back-parse '(+ X 3))
   (back-parse '(SIN X))
   (back-parse '(+ (^ X 3) (TAN X)))
   (back-parse '(+ (^ X 3) (^ (TAN X) 2)))
   (back-parse '(+ (^ Y 3) (LN X)))
   (back-parse '(LN (+ (^ Y 3) (LN X))))
   (back-parse '(+ (COSH X) (- X)))
   (back-parse '(/ (+ X (- 4)) (^ (+ (* 2 X) 4) 2)))
   (back-parse '(+ X (- (EXP (SQRT 3)))))
   (back-parse '(+ (+ (^ X 3) (- (* 2 X))) 3))
   (back-parse '(IF (< X 0) 0 (/ (SIN X) 3)))
   (back-parse '(IF (< X 0) 0 (SIN X)))
   )
  nil
  )
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; what Doc:    Creation Date://        Author: Andy Long
;;
;;
(defmacro what (f)
  "
;; Description/Arguments:

       (what f)
  
Gives the lambda expression of f in infix notation, using back-parse.

;; Notes:

;; See Also: back-parse, get-lambda-expression

;; Examples:
 (func f (x) x^2)
 (what f)

 (func f(L C) 1/(2*pi*sqrt(L*C)))
 (gle f)
 (what f)
"
  (let ((bing (eval `(third (gle ,f)))))
    (setq bing (back-parse bing))
    `',bing
    )
  )
