#|
	I asked my dad for help with Bezier curves after reading a paper which
used them. He sent me the following info, which I used (along with some other
references) to write the following functions. Thanks Dad!

To: Andrew E Long  <long@sol.acs.uwosh.edu>
From: long@bgnet.bgsu.edu (Cliff Long)
X-Sender: long@bgnet.bgsu.edu
Subject: Re: variety
Status: R

Andy,

Here is a Maple programs that you may be able to use.

Bezier Curves
> with(linalg):with(plots):
> wt:=[(1-t)^3,3*(1-t)^2*t,3*(1-t)*t^2,t^3];

> v1:=[0,0,0];v2:=[2,0,0];v3:=[2,2,0];v4:=[0,2,3];
> r:=wt[1]*v1+wt[2]*v2+wt[3]*v3+wt[4]*v4;
> rv:=evalm(r);
> rvc:=convert(rv,list);
> tubeplot(rv,t=0..1,radius=.1,style=patch,axes=boxed);
> plot3d(rv,t=0..1,s=0..1,axes=boxed,scaling=constrained);
> v:=diff(r,t);
> vel0:=subs(t=0,v);
> vel1:=subs(t=1,v);
> a:=diff(v,t);
> a0:=subs(t=0,a);
> a1:=subs(t=1,a);

Dad

Cliff Long                                                        419-372-7462
Department of Mathematics & Statistics                      long@math.bgsu.edu
Bowling Green State University                                FAX 419-372-6092
Bowling Green OH 43403
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Bezier
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun bezier(vertices &optional (pnts-per-point 20))
  "
;; Description:

	Function bezier (vertices &optional (pnts-per-point 100)) plots a space
curve (created using Bezier polynomials) joining a list of vertices which the
user supplies. The user also specifies the control points - that is, between
every pair of verticies is a pair of control points, which guide the curve. The
curve does not pass through these control points, in general.

;; Examples:
 (bezier '((0 0 0)(2 0 0)(2 2 0)(0 2 3)))
 (bezier '((0 0 0)(2 2 0)(2 -1 0)(1 0 0)))
 (bezier '(
    (0 0 0) ;; points on the curve (the curve passes through these)
	(2 0 0)(2 2 0) ;; control points
    (0 2 3)
    	(1 1 1)(1 2 0)
    (2 2 0)
	(3 3 0)(3 4 0)
    (4 4 4)
    ))
"
  (let (
	(my-plot (send spin-proto :new 3))
	(newpoint (first vertices))
	)
    (send my-plot :add-points
	  (list (list (first newpoint))
		(list (second newpoint))
		(list (third newpoint))
		)
	  )
    (do
     ((z (copy-list vertices) (rest (rest (rest z)))))
     ((<= (length z) 3))
     (let ((v (list (first z) (second z) (third z) (fourth z))))
       (dolist
	(i (rseq 0 1 pnts-per-point))
	(let* (
	       (wi (list
		    (^ (- 1 i) 3)
		    (* 3 (^ (- 1 i) 2) i)
		    (* 3 (^ i 2) (- 1 i))
		    (^ i 3)
		    )
		   )
	       (point (apply #'+  (* wi v)))
	       )
 	  (send my-plot :add-lines
		(list
		 (list (first newpoint) (first point))
		 (list (second newpoint) (second point))
		 (list (third newpoint) (third point))
		 )
		)
	  (setq newpoint point)
	  )
	)
       )
     )
    
    (dolist
     (newpoint vertices)
     (send my-plot :add-points
	   (list (list (first newpoint))
		 (list (second newpoint))
		 (list (third newpoint))
		 )
	   )
     )
    
    (send my-plot :mouse-mode 'hand-rotate)
    (send my-plot :depth-cuing nil)
    (send my-plot :adjust-to-data)
    (if *real-axes* (send my-plot :real-axes))
    
    my-plot
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Bezier2
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun bezier2(vertices &optional (pnts-per-point 20))
  "
;; Description:

	Function bezier2 (vertices &optional (pnts-per-point 100)) plots a space
curve (created using Bezier2 polynomials) joining a list of vertices which the
user supplies.

	This is the algorithm as given in Burden and Faires, \"Numerical
Analysis\". It seems to give the same results as bezier1.

;; Examples:
 (bezier2 '((0 0 0)(2 0 0)(2 2 0)(0 2 3)))
 (bezier2 '((0 0 0)(2 2 0)(2 -1 0)(1 0 0)))
 (bezier2 '(
    (0 0 0) ;; points on the curve (the curve passes through these)
	(2 0 0)(2 2 0) ;; control points
    (0 2 3)
    	(1 1 1)(1 2 0)
    (2 2 0)
	(3 3 0)(3 4 0)
    (4 4 4)
    ))
"
  (let (
	(my-plot (send spin-proto :new 3))
	(newpoint (first vertices))
	)
    (do
     ((z (copy-list vertices) (rest (rest (rest z)))))
     ((<= (length z) 3))
     (let* (
	    (v1 (first z))
	    (v+ (+ v1 (/ (- (second z) v1) 3)))
	    (v2 (fourth z))
	    (v- (+ v2 (/ (- (third z) v2) 3)))

	    (p1 v1)
	    (p2 (* 3 (- v+ v1)))
	    (p3 (* 3 (+ v- v1 (- (* 2 v+)))))
	    (p4 (- v2 v1 (* 3 (- v- v+))))

	    (v (list p1 p2 p3 p4))
	    )
       (print v+)
       (print v-)
       (dolist
	(i (rseq 0 1 pnts-per-point))
	(let* (
	       (wi (list 1 i (^ i 2) (^ i 3)))
	       (point (apply #'+  (* wi v)))
	       )
 	  (send my-plot :add-lines
		(list
		 (list (first newpoint) (first point))
		 (list (second newpoint) (second point))
		 (list (third newpoint) (third point))
		 )
		)
	  (setq newpoint point)
	  )
	)
       )
     )

    (dolist
     (newpoint vertices)
     (send my-plot :add-points
	   (list
	    (list (first newpoint))
	    (list (second newpoint))
	    (list (third newpoint))
	    )
	   )
     )

    (send my-plot :mouse-mode 'hand-rotate)
    (send my-plot :depth-cuing nil)
    (send my-plot :adjust-to-data)
    (if *real-axes* (send my-plot :real-axes))

    my-plot
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Bezier-interp
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun bezier-interp(vertices slopes &optional (pnts-per-point 20))
  "
;; Description:

	Function bezier-interp (vertices &optional (pnts-per-point 100)) plots
a space curve (created using Bezier-Interp polynomials) joining a list of
vertices which the user supplies.

;; Examples:
 (bezier-interp
   '((0 0 0)(2 0 0)(2 2 0)(0 2 3)) ;; points
   '((0 0 0)(2 0 0)(2 2 0)(0 2 3)) ;; slopes
   )
 (bezier-interp
   '(
    (-1 0) ;; all are points on the curve (the curve passes through them).
    (0 1)
    (1 .5)
    (0 0)
    (1 -1)
    )
   '(
    (0 1.5) ;; slopes
    (1.5 0)
    (0 -1.5)
    (-3 -1.5)
    (1.5 -1.5)
    )
   50
   )
"
  (let* (
	 (p1 (first vertices))
	 (verts (list p1))
	 (ms slopes)
	 )
    (dolist
     (p2 (rest vertices))
     (setq
      verts (append verts (list (+ p1 (first ms)) (- p2 (second ms)) p2))
      ms (rest ms)
      p1 p2
      )
     )
    (bezier3 verts pnts-per-point)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Bezier-interp-with-cusps
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun bezier-interp-with-cusps
  (vertices slopes &optional (pnts-per-point 20) guide)
  "
;; Description: Function bezier-interp-with-cusps

             (vertices slopes &optional (pnts-per-point 20) guide)

plots a space curve (created using Bezier polynomials) joining a list of
vertices which the user supplies. The slopes at the vertices must also be
provided, and that means right and left for each vertex. (It is this which
allows for cusps in the curve). That is, there must be slope information for
each pair of adjacent points.

        You can use either the slopes, or guidepoints (in which case you should
set the number of points per point and guide to true. See the first example
below.

;; See also: bezier3, bezier-interp

;; Examples:
 (bezier-interp-with-cusps ;; the letter n? p. 154 B-F, NA, using guidepoints
   '(;; points
    (3 6) (2 2) (6 6) (5 2) (6.5 3)
    )
   '(;; guide-points
    (3.3 6.5) (2.5 2.5)
    (2.8 3) (5 5.8)
    (5.8 5) (4.5 2.5)
    (5.5 2.2) (6.4 2.8)
    )
   50 t
   )
 (bezier-interp-with-cusps ;; Burden and Faires, Numerical Analysis, p. 152
   '((0 0)(1 0)) '((2 2)(-1 1)))
 (bezier-interp-with-cusps ;; Burden and Faires, Numerical Analysis, p. 152
   '((0 0)(1 0)) '((1 1)(.5 -.5)))
"
  (let* (
	 (p1 (first vertices))
	 (verts (list p1))
	 (ms slopes)
	 )
    (dolist
     (p2 (rest vertices))
     (setq
      verts (if guide
		(append verts (list (first ms) (second ms) p2))
	      (append verts (list (+ p1 (first ms)) (- p2 (second ms)) p2))
	      )
      ms (rest (rest ms))
      p1 p2
      )
     )
    (bezier3 verts pnts-per-point)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Bezier3
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun bezier3(vertices &optional (pnts-per-point 20))
  "
;; Description:

	Function bezier3 (vertices &optional (pnts-per-point 100)) plots a space
curve (created using Bezier3 polynomials) joining a list of vertices which the
user supplies. The user also specifies the control points - that is, between
every pair of verticies is a pair of control points, which guide the curve. The
curve does not pass through these control points, in general.

;; Examples:
 (bezier3 '((0 0 0)(2 2 0)(2 -1 0)(1 0 0))) ;; These are 3-d points, but they
					    ;; all lie in a single plane.
 (bezier3 '((0 0)(2 2)(2 -1)(1 0)))
 (bezier3 '(
    (0 0 0) ;; points on the curve (the curve passes through these)
	(2 0 0)(2 2 0) ;; control points
    (0 2 3)
    	(1 1 1)(1 2 0)
    (2 2 0)
	(3 3 0)(3 4 0)
    (4 4 4)
    ))
"
  (let ((newpoint (list (first vertices))))
    (do
     ((z (copy-list vertices) (rest (rest (rest z)))))
     ((<= (length z) 3))
     (let* (
	    (v1 (first z))
	    (v+ (+ v1 (/ (- (second z) v1) 3)))
	    (v2 (fourth z))
	    (v- (+ v2 (/ (- (third z) v2) 3)))
	    (v (list v1 v+ v- v2))
	    )
       (dolist
	(i (rseq 0 1 pnts-per-point))
	(let* (
	       (wi (list
		    (^ (- 1 i) 3)
		    (* 3 (^ (- 1 i) 2) i)
		    (* 3 (^ i 2) (- 1 i))
		    (^ i 3)
		    )
		   )
	       (point (apply #'+ (* wi v)))
	       )
	  (setq newpoint (append newpoint (list point)))
	  )
	)
       )
     )
    (let ((my-plot
	   (if (eq (length (first newpoint)) 3)
	       (eval `(plot '(0) '(0) '(0)))
	     (eval `(plot '(0) '(0)))
	     )
	   )
	  )
      (if (eq (length (first newpoint)) 3)
	  (let ()
	    (send my-plot :add-points 
		  (list (xs newpoint) (ys newpoint) (zs newpoint)))
	    (send my-plot :add-lines 
		  (list (xs newpoint) (ys newpoint) (zs newpoint)))
	    (send my-plot :adjust-to-data)
	    )
	(let ()
	  (send my-plot :add-plot-item
		(list 'points (xs newpoint) (ys newpoint))
		(list 'lines (xs newpoint) (ys newpoint)))
	  (send my-plot :update-plot)
	  )
	)	      
      my-plot
      )
    )
  )
