;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Modulus
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun modulus(z)
  "
      See abs. The modulus of a complex number is its length if thought of as a
vector.

;; Example:
 (setq z (complex (sqrt 3) 1))
 (modulus z)
 (argument z)
"
  (abs z)
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Argument
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun argument(z)
  "
;; Description: (argument z)
Finds the argument of a complex number.

;; Examples:
 (argument #c(-1 -1))
 (argument #c(1 1))
 (argument #c(0 1))
 (argument #c(1 0))

 (setq z (complex (sqrt 3) 1))
 (modulus z)
 (argument z)
"
  (cond
   ((= 0 (imagpart z)) (if (> (realpart z) 0) 0 pi))
   ((= 0 (abs z)) 0)
   (t (* (signum (imagpart z)) (acos (/ (realpart z) (abs z)))))
   )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                plot-complex
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun plot-complex (complexes)
  "
;; Description: (plot-complex complexes)

	Plots lists of complex numbers.

;; Examples:
 (data complexes #c(1 2) #c(0 1) #c(-1 1))
 (plot-complex complexes)
"
  (lines off)
  (let (
	(my-plot (plot (realpart `,complexes) (imagpart `,complexes)))
	(maxx (* 1.1 (max (realpart `,complexes))))
	(maxy (* 1.1 (max (imagpart `,complexes))))
	)
    (send my-plot :variable-label 0 "real part")
    (send my-plot :variable-label 1 "imaginary part")
    (send my-plot :range 0 (- maxx) maxx)
    (send my-plot :range 1 (- maxy) maxy)
    (send my-plot :plot-item)
    )
  )
