;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                list-to-string
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun list-to-string(input-list)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; list-to-string Doc:    Creation Date:5/12/96   Author: Andy Long
;;
;;			Modified: 1/27/98
;;
;; Description/Arguments:

       (list-to-string list)
  
Takes a list, returns a string. But then you knew that!

;; Notes: created as a helper for func.

;; See Also: func

;; Examples:
 (list-to-string '(3/2))
 (list-to-string '(3 / 2))
 (list-to-string '(3/2 + 5/4))
 (list-to-string '(3-2 + 5/4))
"
  (let ((result ""))
    (dolist
     (next input-list)
     (setq
      result
      (strcat result " " 
	      (cond
	       ((consp next) (list-to-string next))
	       ((fixnump next) (num-to-string next))
	       (t (write-to-string next))
	       ))
      )
     )
    (strcat "(" (subseq result 1) ")")
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                no-space
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun no-space (line &key dash)
  "
;; Description: takes a string and puts spaces between arithmetic operations,

;; Returns: a list, of the string as a lisp expression.

	If you put a pair of -- together, it means to replace them with a
	single -
		: woe to him who uses double -- in function names!

;; Examples:
 (no-space \"3/2\")
 (no-space \"6*37/2\")
 (no-space \"6-37/2*name-sake\" :dash t)
 (no-space \"6-37/2*name--sake-3\")
"
  (let ((chars (coerce line 'list))(sp #\space))
    (setq
     chars (grep-replace '(#\- #\-) '(#\& #\\ #\& #\\ #\&) chars)
     chars (grep-replace '(#\+) (list sp #\+ sp) chars)
     chars (grep-replace '(#\*) (list sp #\* sp) chars)
     chars (grep-replace '(#\/) (list sp #\/ sp) chars)
     chars (grep-replace '(#\^) (list sp #\^ sp) chars)
     chars (if dash chars (grep-replace '(#\-) (list sp #\- sp) chars))
     chars (grep-replace '(#\& #\\ #\& #\\ #\&) '(#\-) chars)
     )
    (read
     (make-string-input-stream
      (strcat "(" (coerce chars 'string) ")")))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; list-to-string-with-negation   Creation Date:5/12/96   Author: Andy Long
;;
;;
(defun list-to-string-with-negation(input-list)
  "
;; Description/Arguments:

       (list-to-string-with-negation list)
  
;; Notes: same as list-to-string, only it lets us use negation of numbers,
;; special symbols (e.g. pi). A helper for calc, func.

;; See Also: list-to-string

;; Examples:
 (list-to-string-with-negation '(3/2))
 (list-to-string-with-negation '(3 / 2))
 (list-to-string-with-negation '(3/2 + 5/4))
 (list-to-string-with-negation '(3 + -2))
 (list-to-string-with-negation '(3 + -pi))
 (list-to-string-with-negation '(3 + func-pi))
"
  (let ((result ""))
    (dolist
     (temp input-list)
     (setq
      result
      (strcat
       result " "
       (cond
	((consp temp) (list-to-string-with-negation temp))
	((fixnump temp)
	 (if (realp temp)
	     (if (< temp 0) 
		 (strcat "(- " (num-to-string (abs temp)) ")")
	       (num-to-string temp)
	       )
	   (num-to-string temp)
	   )
	 )
	(t
	 (let ((temp (symbol-name temp)))
	   (if (> (length temp) 1)
	       (if (equalp #\- (read-char (make-string-input-stream temp)))
		   (setq temp
			 (strcat "(- " (string-left-trim "-" temp) ")"))
		 )
	     )
	   temp))
	))
      )
     )
    (strcat "(" (subseq result 1) ")")
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for FUNC:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Modified 5/12/96 to remove necessity of spaces, hooray! ael.
;;
;;	Modified 1/27/98 to add option of replacing - by --, instead of using
;;the :dash option. ael.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro func (thistle var &rest that)
  "
;; Documentation: (func <name> <arguments> <body> &key dash)

	Func is a natural method for defining mathematical functions. It
allows the user to input functions in a natural way (rather than the usual
'Reverse Polish Notation' of lisp). 

;; Notes: For the moment func has the following 'flaw': if you try to
define a function f in terms of another function which has a \"-\" in it,
func will deduce that there is a subtraction in progress: e.g., 

			(func f(x) (normal-rand x))

See the last examples for ways to handle this case.

;; Examples:
 (func f(x) x+2*x^2*sin(1/x))
 (new-plot)
 (plot f)

 (func f(x) sin(1/x))
 (plot f)

 (func f(x y) sin(x)*y)
 (spin-domain -pi pi -pi pi)
 (plot f)

 ;; works fine for functions of several variables:
 (func f(x y) (abs(x)-y))
 (plot f)
 ;;
 ;; 'Dash' functions cause special headaches, but there are several
 ;; work-arounds. Sorry that it's not easier, but imagine what you are
 ;; expecting the program to do! It would probably be better to refuse the use
 ;; of dash functions, except that lisp is full of them.
 ;;
 ;; Straight-forward use of the normal-rand function will cause an error: (or
 ;; rather, create the function with definition ((+ NORMAL (- RAND)) X)):
 ;; (func f(x) (normal-rand x))
 ;; One work-around is to alias any necessary \"-\" functions with other
 ;; functions: e.g.,
 (alias rand.normal normal-rand)
 (func f(x) (rand.normal x))
 (f 1)
 ;; Alternatively, you can use the dash option:
 (func f(x) (normal-rand x) :dash t)
 (f 1)
 ;; But this can cause trouble if you have both a subtraction and a
 ;; dash-function in the same definition: the following won't work,
 ;; (func f(x) 3-(normal-rand x) :dash t)
 ;; but if you add spaces to indicate subtraction, it will:
 (func f(x) 3 - (normal-rand x) :dash t)
 (f 1)
 ;; One last method: replace any true dashes by double dashes. Woe to 
 ;; anyone who uses double dashes in the names of their functions!
 (func f(x) 3-(normal--rand x))
 (f 1)
"
  (let* (
	 (dash (first (catch-keys '(:dash) that)))
	 (that (remove-keys '(:dash) that))
	 (a (eval
	     `(no-space 
	       ,(eval
		 `(list-to-string ',that)
		 )
	       :dash ,dash
	       )
	     )
	    )
	 (a (recursive-grep-replace '(+ -) '(-) a))
	 (a (recursive-grep-replace '(/ -) '(* -1 /) a))
	 (a (recursive-grep-replace '(* -) '(* -1 *) a))
	 (a (recursive-grep-replace '(- -) '(+) a))
	 )
    `(function 
      ,(eval
	`(defun ,thistle ,var 
	   ,(eval
	     `(parser ',a)
	     )
	   )
	)
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for calc:
;;
;;          Modified 5/12/96 to remove necessity of spaces, hooray!
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro calc (&rest e)
  "
;; Description:

calc serves as an infix evaluator:

          (calc expressions)

will evaluate your standard infix expressions.

;; Examples:

 (calc 3*cos(2))
 (calc (3*cos(2))^2)
 (calc ln(abs(3*cos(2))))
 (calc 2*(tan(1)/5))
 (calc '(3*cos(2))) ;; calc just ignores the quote
 (calc 2 + -2)
 (setq a '(1 2))
 (setq b '(3 4))
 (calc a * b)
 (calc '(1 2) * '(3 4))
 (calc (1 2) * (3 4))
"
  (eval
   `(parser
     ',(eval
	`(no-space 
	  ,(eval
	    `(list-to-string-with-negation ',e)
	    )
	  )
	)
     )
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for define:
;;
(defmacro define (newvar &rest e)
  "
;; Description:

define serves as an infix definer:

          (define variable expressions)

will assign values to variables using standard infix expressions.

;; See also: calc, parser

;; Examples:

 (define total 3*(cos 2))
 total
 (define total (3*cos 2)^2)
 total
 (define total ln(abs(3*cos 2)))
 total
 (define total 2*(tan 1/5))
 total
 (define total '(3*cos 2)) ; is okay, but why?
 total
"
  `(def ,newvar (eval (calc ',e)))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                calc_space
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro calc_space (&rest e)
  "
;; Description: macro calc_space

          (calc_space expressions)

calc_space serves as an infix evaluator that works even if functions have
hyphens in them. Otherwise, one would use calc.

calc_space is also clumsier, because you have to leave spaces between
operators, etc.

;; See also: calc

;; Examples:
 (calc_space 2 + -2)
 (setq a '(1 2))
 (setq b '(3 4))
 (calc_space a * b)
 (calc_space '(1 2) * '(3 4))
 (calc_space (1 2) * (3 4))
"
  (let ((ctmp 
	 (strcat "(" 
		 (eval `(list-to-string ',e))
		 ")")
	 ))
    (eval
     `(parser
       ',(eval
	  `(read (make-string-input-stream ,ctmp))
	  )
       )
     )
    )
  )
