;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 jacobian
;;
(defun jacobian(functions variables
			  &key 
			  (print nil)
			  (function nil)
			  (matrix-function nil)
			  )
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          jacobian Doc:
;;
;; Author: Andy Long
;; Created: Tue Nov 4 20:24:14 1997
;;
;; Description/Arguments:

	(jacobian functions variables
			  &key 
			  (print nil)
			  (function nil)
			  (matrix-function nil)
			  )
  
	Creates the Jacobian matrix for a list or vector of functions, with
given variables. The list/vector of variables must have the same length as the
list/vector of functions.

	If the function key is true, then a matrix of functions will be
returned rather than the formulas; if the matrix-function key is true, then a
matrix valued function will be returned.

	If print is true and function is nil, then the matrix will be printed
using mprint.

;; Notes:

;; See Also:

;; Examples:
 (mprint (jacobian #((func f(x y) x*y^2) (func f(x y) x^2+y^3)) '(x y)))
 (jacobian (list (func f(x y) x) (func f(x y) x^2*y^3)) '(x y) :print t)

 ;; Here we return a matrix of functions, which we may want to plot, for
 ;; example:
 (plot 
  (aref
   (jacobian (list (func f(x y) x) (func f(x y) x^2*y^3)) '(x y) :function t)
   1 1))

 ;; Let's create a matrix valued function which returns the jacobian for a
 ;; given set of arguments:
 (defun Jack(x y) 
   (funcall 
    (jacobian (list (func f(x y) x) (func f(x y) x^2*y^3)) '(x y)
              :matrix-function t)
    x y))
 (Jack 1 2)
"
  (let ((l (length functions)))
    (if (= l (length variables))
	(let* (
	       (function (if matrix-function t function))
	       (funcs (coerce functions 'list))
	       (vars (coerce variables 'list))
	       (contents
		(smoosh
		 (mapcar
		  #'(lambda(x)
		      (mapcar
		       #'(lambda(y) 
			   (if function 
			       (back-parse 
				(eval `(lambda-deriv ,y ',x)))
			     (eval `(diff ,y ,x))))
		       funcs))
		  vars)
		 )
		)
	       (result
		(if matrix-function 
		    (eval `(lambda ,variables 
			     (matrix
			      (list ,l ,l)
			      (mapcar
			       (lambda(function) (funcall function ,@variables))
			       (list ,@contents)))
			     ))
		  (matrix (list l l) contents))
		)
	       )
	  (if (and print (not function)) (mprint result))
	  result
	  )
      (format t "Function vector and variable list must be of the same length.~%")
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
