;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Necessities:
;;
;;	Some variables and special functions which are of general interest.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

					;
					;sapaclisp likes these:
					;
(def single-float-epsilon 1e-6)
#+msdos (def MOST-POSITIVE-FIXNUM 1e303) ;; otherwise dog chokes
#-msdos (def MOST-POSITIVE-FIXNUM 1e308)
					;
					;  Variables:
					;
(defconstant -pi (- pi))
(defconstant egor 42)

(defconstant nan #.not-a-number)
(defconstant 2pi (* 2 pi))
(defconstant -2pi (- 2pi))
(defconstant 3pi (* 3 pi))
(defconstant -3pi (- 3pi))
(defconstant 4pi (* 4 pi))
(defconstant -4pi (- 4pi))
(defconstant 5pi (* 5 pi))
(defconstant -5pi (- 5pi))

(defconstant pi/2 (/ pi 2))
(defconstant -pi/2 (- pi/2))
(defconstant pi/3 (/ pi 3))
(defconstant -pi/3 (- pi/3))
(defconstant pi/4 (/ pi 4))
(defconstant -pi/4 (- pi/4))
(defconstant pi/5 (/ pi 5))
(defconstant -pi/5 (- pi/5))

(defconstant 3pi/2 (/ 3pi 2))
(defconstant -3pi/2 (- 3pi/2))
(defconstant 5pi/2 (/ 5pi 2))
(defconstant -5pi/2 (- 5pi/2))

#|
These had to be removed to comply with 3-50, wherein they were already defined.

(defconstant 1/2 .5)
(defconstant 1/3 (/ 1 3))
(defconstant 1/4 .25)
(defconstant 1/5 .2)
(defconstant 1/6 (/ 1 6))
(defconstant 1/7 (/ 1 7))
(defconstant 1/8 .125)
(defconstant 1/9 (/ 1 9))

(defconstant -1/2 -.5)
(defconstant -1/3 (- 1/3))
(defconstant -1/4 -.25)
(defconstant -1/5 (- .2))
(defconstant -1/6 (- 1/6))
(defconstant -1/7 (- 1/7))
(defconstant -1/8 -.125)
(defconstant -1/9 (- 1/9))

(defconstant 2/3 (/ 2 3))
(defconstant 2/5 (/ 2 5))
(defconstant 3/2 1.5)
(defconstant 3/4 .75)
(defconstant 3/5 .6)
|#
					;
					;  Functions:
					;
(defun ln(x) 
  "
ln is an alias to the log function of base e.

;; Examples:
 (ln 3)
 (ln -3)
"
  (log x))
(defun ceil(x)
  "
ceil is an alias to the ceiling function, which returns the next integer larger
than or equal to a number.

;; Examples:
 (ceil 2.5)
 (ceil -2.5)
"
  (ceiling x))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                sinc
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun sinc(x)
  "
;; Description:

	The sinc function is a standard function in image processing. It is
  equal to sin(x)/x, except at zero where it is defined to be zero, making it
  continuous and differentiable everywhere.

;; Notes: Vectorized.

;; Examples:
 (sinc 3)
 (sinc (list 0 3))
 (plot sinc -2pi 2pi)
 "
  (if (listp x) (mapcar #'sinc x)
    (if (= x 0) 1 (/ (sin x) x)))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                half
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun half(&rest x)
  "A self-explanatory numerical function!
;; Examples:
 (half 3)
 (half 2 3)
 (half (list 2 3))
"
  (if (= 1 (length x)) (/ (first x) 2)
    (mapcar #'(lambda(x) (/ x 2)) x))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                square
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun square(&rest x)
  "A self-explanatory numerical function!
;; Examples:
 (square 3)
 (square 2 3)
 (square (list 2 3))
"
  (if (= 1 (length x)) (^ (first x) 2)
    (mapcar #'(lambda(x) (^ x 2)) x))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                cube
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun cube(&rest x)
  "A self-explanatory numerical function!
;; Examples:
 (cube 3)
 (cube 2 3)
 (cube (list 2 3))
"
  (if (= 1 (length x)) (^ (first x) 3)
    (mapcar #'(lambda(x) (^ x 3)) x))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; digits Doc:    Creation Date:5/7/96        Author: Andy Long
;;
;;
(defun digits (n &optional decimals)
  "
;; Description/Arguments:

       (digits decimals &optional n)
  
Sets the number of digits that xlispstat produces for numerical output. If used
with the additional n, digits will set instead the following C format string:

                   %<decimals>.<n>g

;; Thanks to:
;; ---|  Steven D. Majewski   (804-982-0831)  <sdm7g@Virginia.EDU>  |---

;; See Also: *float-format*

;; Examples:
 (digits 6)
 pi
 (digits 10)
 pi
"
  (setf *float-format* 
	(if decimals
	    (format nil "%~d.~dg" n decimals)
	  (format nil "%.~dg" n)
	  ))
  (format t "Current format: ~s" *float-format*)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(defun print-with (number digits)
  "
	Usage: print-with (number digits)

If you want to print pi with only 4 digits, then

;; Examples:
 (print-with pi 4)
 (print-with (* 100000 pi) 4) ;; scientific notation
"
  (let ((fmt *float-format*))
    (setq *float-format* (format nil "%.~dg" digits))
    (princ number)(terpri)
    (setq *float-format* fmt)
    )
  (values)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;;;  The functions  evaluate-polynomial
;;;                 multiply-2-polynomials
;;;                 multiply-polynomials
;;;                 zeros-of-polynomial
;;;  evaluate, multiply and find the roots of polynomials whose coefficients
;;;  are represented by sequences of numbers.
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
(defun evaluate-polynomial
       (polynomial
        z
        &key
        (degree-of-polynomial (1- (length polynomial)))
        (number-of-derivatives 0)
        (complex-values (make-array (1+ number-of-derivatives)))
        (bounds-to-be-computed nil)
        (bounds (if bounds-to-be-computed
                  (make-array (1+ number-of-derivatives)))))
  "given
   [1] polynomial (required)
       ==> sequence of length n+1 with real or
           complex-valued numbers giving the n+1
           coefficients of a polynomial of nth order;
           (elt polynomial i) = coefficient of z^(n-i)
   [2] z (required)
       ==> complex point at which the polynomial
           to be evaluated
   [3] degree-of-polynomial (keyword; (1- (length polynomial)))
       ==> degree of polynomial
   [4] number-of-derivatives (keyword; 0)
       ==> number of derivatives to be evaluated
   [5] complex-values (keyword; array of length (1+ number-of-derivatives))
       <== sequence with values of polynomial
       and derivatives
   [6] bounds-to-be-computed (keyword; nil)
       ==> if t, error bounds are computed
       for values in complex-values
   [7] bounds (keyword; array of length (1+ number-of-derivatives))
       <== sequence with error bounds
returns
   [1] the sequence complex-values containing the value
       of the polynomial and its derivatives
   [2] bounds, a sequence containing optional error bounds
---       
Note: this is a Lisp version of cmlib routine cpevl;
d1 was specified in cpevl as (expt 2 (- 1 (i1mach 11))),
where (i1mach 11) ==> THE NUMBER OF BASE-2 DIGITS (SINGLE PRECISION).
I have taken this to be equivalent to machine epsilon.
If this is in fact NOT the case, then the optional error bounds
might not be computed correctly.

;; Examples:

 (evaluate-polynomial #(1 2 3) 1) ;==> #(6.0)
 (evaluate-polynomial #(1 2 3) 2) ;==> #(11.0)
"
  (let ((d1 single-float-epsilon)
        ci cim1)
    (dotimes (j (1+ degree-of-polynomial))
      (dotimes (i (min (1+ number-of-derivatives)
                       (1+ (- degree-of-polynomial j))))
        (setf ci (if (plusp j) (elt complex-values i) 0.0))
        (setf cim1 (if (plusp i)
                     (elt complex-values (1- i)) (elt polynomial j)))
        (setf (elt complex-values i) (+ cim1 (* z ci)))
        (if bounds-to-be-computed
          (let* ((bi (if (plusp j) (elt bounds i) 0.0))
                 (bim1 (if (plusp i) (elt bounds (1- i)) 0.0))
                 (tf (+ bi (* (complex-of-absolutes ci)
                             (+ (* 3.0 d1) (* 4.0 d1 d1)))))
                 (r (realpart (* (complex-of-absolutes z)
                                 (complex (realpart tf) (- (imagpart tf))))))
                 (s (imagpart (* (complex-of-absolutes z) tf))))
            (setf (elt bounds i)
                  (if (plusp j)
                    (* (1+ (* 8 d1))
                       (+ bim1
                          (* d1 (complex-of-absolutes cim1))
                          (complex r s)))
                    0.0)))))))
  (values complex-values bounds))
        
;-------------------------------------------------------------------------------
(defun multiply-2-polynomials
       (polynomial-1
        polynomial-2
        &key               
        (degree-of-1
         (1- (length polynomial-1)))
        (degree-of-2
         (1- (length polynomial-2)))
        (product-polynomial
         (make-array (+ degree-of-1 degree-of-2 1))))
  "given
   [1] polynomial-1 (required)
       ==> sequence of length n+1 with real or
           complex-valued numbers giving the n+1
           coefficients of a polynomial of nth order;
           (elt polynomial-1 i) = coefficient of z^(n-i)
   [2] polynomial-2 (required)
       ==> another sequence representing another polynomial
   [3] degree-of-1 (keyword; (1- (length polynomial-1)))
       ==> degree of first polynomial
   [4] degree-of-2 (keyword; (1- (length polynomial-2)))
       ==> degree of second polynomial
   [5] product-polynomial (keyword; array of length (+ degree-of-1 degree-of-2 1))
       <== a sequence with product of two polynomials
returns
   [1] product-polynomial, an sequence with coefficients
       of polynomial given by the product of
       polynomial-1 and polynomial-2
---       
Note: this routine works for a polynomial p represented either by
        p(0) + p(1)*z + ... + p(degp)*z^degp
                  or by
        p(0)*z**degp + p(1)*z**(degp-1) + ... + p(degp)

;; Examples:

 (multiply-2-polynomials #(1 2 3) #(4 3 2 1))
        ;==>#(4.0 11.0 20.0 14.0 8.0 3.0)
 (multiply-2-polynomials #(4 3 2 1) #(1 2 3))
        ;==>#(4.0 11.0 20.0 14.0 8.0 3.0)
 (multiply-2-polynomials #(4 3 2 1) #(2)) ;==>#(8.0 6.0 4.0 2.0)
"
  (let (k)
    (dotimes (i (+ degree-of-1 degree-of-2 1) product-polynomial)
      (setf (elt product-polynomial i) 0.0)
      (dotimes (j (1+ degree-of-1))
        (setf k (- i j))
        (if (and (>= k 0) (<= k degree-of-2))
          (incf (elt product-polynomial i)
                (* (elt polynomial-1 j) (elt polynomial-2 k))))))))

;-------------------------------------------------------------------------------
(defun multiply-polynomials (&rest polys)
  "given
   [1] sequences representing any number of polynomials (required)
returns
   [1] a sequence representing their product
---
Note: this function was written by Andrew G. Bruce

;; Examples:
 (multiply-2-polynomials #(1 2) #(4 3)) ;==> #(4.0 11.0 6.0)
 (multiply-2-polynomials #(4.0 11.0 6.0) #(1 2 3))
         ;==> #(4.0 19.0 40.0 45.0 18.0)
 (multiply-polynomials #(1 2) #(4 3) #(1 2 3))
         ;==> #(4.0 19.0 40.0 45.0 18.0)
 (multiply-polynomials #(1) #(4) #(2)) ;==> #(8.0)
"
  (let ((n (length polys)))
    (cond ((= n 0) polys)
          ((= n 1) (first polys))
          ((> n 1) (reduce #'multiply-2-polynomials polys)))))

;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                simul-keep-value
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun simul-keep-value (a b value &key (test #'=))
  "
;; Description: function (simul-keep-value a b value &key (test #'=))

	simultaneously keeps elements value from list a, returning it with
the associated list b.

	To turn this into simul-eliminate-value, we simply use the

			:test #'/=

key.

;; See also: order, sort-data, simul-eliminate-value, simul-eliminate-zero

;; Examples:
 (setq a '(0 3 2 4))
 (setq b '(8 9 7 3))
 (simul-keep-value a b 0)
 (setq a '(0 3 2 4))
 (setq b '(8 0 0 0))
 (simul-keep-value a b 2)
 (setq a '(0 3 0 4))
 (setq b '(8 0 8 0))
 (simul-keep-value a b 4)
 ;; the following takes every value of a not equal to 4:
 (simul-keep-value a b 4 :test #'/=)
"
  (let* (
	 (keepers
	  (which
	   (mapcar
	    (lambda(x) (setf x (if (funcall test x value) t nil)))
	    (copy-list a)
	    )))
	 )
    (list (select a keepers) (select b keepers))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                simul-eliminate-zero
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun simul-eliminate-zero (a b)
  "
;; Description: function (simul-eliminate-zero a b)

	simultaneously eliminates zeros from list a, returning it with
the associated list b.

			(simul-eliminate-zero a b)

;; See also: order, sort-data, simul-eliminate-value 
;; Examples:
 (setq a '(0 3 2 4))
 (setq b '(8 9 7 3))
 (simul-eliminate-zero a b)
 (setq a '(0 3 2 4))
 (setq b '(8 0 0 0))
 (simul-eliminate-zero a b)
 (setq a '(0 3 0 4))
 (setq b '(8 0 8 0))
 (simul-eliminate-zero a b)
"
  (simul-keep-value a b 0 :test (lambda(x v) (/= x 0)))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                simul-eliminate-value
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun simul-eliminate-value (a b value &key (test #'=))
  "
;; Description:

       function (simul-eliminate-value a b value &key (test #'=)

	simultaneously eliminates elements value from list a, returning it with
the associated list b. It is really simply a call to simul-keep-value with the
default test of #'=.

;; See also: order, sort-data

;; Examples:
 (setq a '(0 3 2 4))
 (setq b '(8 9 7 3))
 (simul-eliminate-value a b 0)

 (setq a '(0 3 2 4))
 (setq b '(8 0 0 0))
 (simul-eliminate-value a b 2)

 (setq a '(0 3 0 4))
 (setq b '(8 0 8 0))
 (simul-eliminate-value a b 4)

 (setq a '(a b c a))
 (setq b '(8 0 8 0))
 (simul-eliminate-value a b 'a :test #'equalp)
"
  (simul-keep-value a b value :test (lambda(x v) (not (funcall test x v))))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                simul-sort
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun simul-sort (a b)
  "
;; Description: function (simul-sort a b)

	simultaneously sorts list a, returning it with associated list b.

;; See also: order, sort-data

;; Examples:
 (setq a '(0 3 2 4))
 (setq b '(8 9 7 3))
 (simul-sort a b)
 (setq a '(0 3 2 4))
 (setq b '(a b c d))
 (simul-sort a b)
"
  (list (sort-data a) (select b (order a)))
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                sort-with-index
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun sort-with-index (a)
  "
;; Description: function (sort-with-index a)

	Sorts list a, from lowest to highest, returning it with associated list
index (i.e., sort-with-index returns a list of two lists: the sorted list, and
a list of indices which tell the order in which the list was sorted).

	This is useful if you have several other lists which should be sorted
in the same order as the original. Have a look at the following examples.

;; See also: order, sort-data, simul-sort

;; Examples:
 (setq a '(4 3 2 1))
 (setq b '(8 9 7 3))
 (setq index (second (sort-with-index a)))
 (select b index)
 (setq b '(a b c d))
 (select b index)
"
  (simul-sort a (iseq (length a)))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          remove-if-equals
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun remove-if-equals (xx item &key (test #'equalp))
  "
 		remove-if-equals (list item &key (test #'equalp))

;; Examples:
 (remove-if-equals (iseq 10) 5) ;; (0 1 2 3 4 6 7 8 9)
 (remove-if-equals (list \"john\" \"pete\") \"pete\") ;; (\"john\")

;; Note: same as
 (remove 5 (iseq 10))
 (remove \"pete\" (list \"john\" \"pete\") :test #'equalp) ;; (\"john\")
"
  (remove-if #'(lambda(x) (funcall test x item)) xx)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          remove-if-does-not-equal
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun remove-if-does-not-equal (xx item &key (test #'equalp))
  "
 		remove-if-does-not-equal (list item &key (test #'equalp))

	The default is to use equalp to test for equality.

;; Examples:
 (remove-if-does-not-equal (iseq 10) 5) ;; (5)
 (remove-if-does-not-equal (list \"john\" \"pete\") \"pete\") ;; (\"pete\")

;; Note: Same as 
 (remove 5 (iseq 10) :test-not #'equalp) ;; (5)
 (remove \"pete\" (list \"john\" \"pete\") :test-not #'equalp) ;; (\"pete\")
"
  (remove-if #'(lambda(x) (not (funcall test x item))) xx)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          simul-remove-if, simul-keep-if
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun simul-remove-if (a b &optional (test #'(lambda(x) (= x 0))))
  "
;; Description:
		simul-remove-if (a b &optional (test #'(lambda(x) (= x 0))))

	Note that the default is to remove zero elements. 

;; See Also: simul-keep-if (which simul-remove-if simply calls).

;; Examples:
 (simul-remove-if '(0 1 2) '(a b c)) ;; ((1 2) (B C))
 (simul-remove-if '(0 1 2) '(a b c) #'(lambda(x) (< x 2))) ;; ((2) (C))
"
  (simul-keep-if a b (lambda(x) (not (funcall test x))))
  )

(defun simul-keep-if (a b &optional (test #'(lambda(x) (= x 0))))
  "
;; Description:
		simul-keep-if (a b &optional (test #'(lambda(x) (= x 0))))

	Note that the default is to keep zero elements.

;; See also: simul-remove-if

;; Examples:
 (simul-keep-if '(0 1 2) '(a b c)) ;; ((0) (A))
 (simul-keep-if '(0 1 2) '(a b c) #'(lambda(x) (< x 2))) ;; ((0 1) (A B))
"
  (let (
	(keepers
	 (which
	  (mapcar
	   (lambda(x) (setf x (if (funcall test x) t nil)))
	   (copy-list a)
	   ))))
    (list (select a keepers) (select b keepers))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                restrict
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun restrict
  (x a b
     &key
     y
     (left #'>=)
     (right #'<=)
     )
  "
;; Description: (restrict x a b
     &key
      y
      (left #'>=)
      (right #'<=)
      )

	Restricts a list or vector to those value in the interval [a,b]
 (unless right or left is set to other functions, in which case
 either (a,b], [a,b), or (a,b) will be chosen).

	If the y option is selected, then corresponding elements of the y
 vector will be returned as well.

;; Examples:
 (restrict '(0 2 3 -1 5 10 200) 2 10)
 ;; (2 3 5 10)
 (restrict '(0 2 3 -1 5 10 200) 2 10 :left #'>)
 ;; (3 5 10)
 (restrict '(0 2 3 -1 5 10 200) 2 10 :y '(a b c d e f g))
 ;; ((2 3 5 10) (B C E F))
 (restrict '(0 2 3 -1 5 10 200) 2 10 :y '(a b c d e f g) :left #'>)
 ;; ((3 5 10) (C E F))
 (restrict '(0 2 3 -1 5 10 200) 2 10 :y '(a b c d e f g)
    :left #'> :right #'<)
 ;; ((3 5) (C E))
"
  (if y (simul-keep-if
	 x y
	 #'(lambda(x) (and (funcall left x a)
			   (funcall right x b))))
    (select-if x #'(lambda(x) (and (funcall left x a)
				   (funcall right x b))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            quadratic
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun quadratic(a b c &optional x)
  "
;; Documentation: (quadratic a b c &optional x)

			ax^2 + bx + c

	Returns the roots of a quadratic, and the peak or pit value of x
 (where the parabola hits its maximum or minimum), as values. See
 catch-multiple-values (aliased to cmv) if you want to 'catch' all of them.

	If x is provided, the quadratic is evaluated at x (may be a vector).

;; Examples:
 (quadratic 1 0 -1) ;; x^2 - 1, +1, -1, peaking at 0
 (quadratic 1 -2 1) ;; x^2 - 2x + 1 = (x-1)^2
 (quadratic 1 -2 1 0)
 (quadratic 1 -2 1 (rseq 0 1 10))

 ;; Use cmv (catch-multiple-values) to turn the results of quadratic into a
 ;; list:
 (def results (cmv (quadratic 1 -2 1 (rseq 0 1 10))))
 (new-plot)
 (plot (final results))
"
  (let* (
	 (dis (sqrt (- (^ b 2) (* 4 a c))))
	 (r1 (/ (if (> b 0) (- (- b) dis) (+ (- b) dis)) (* 2 a)))
	 (r2 (- (- r1) (/ b a)))
	 (peak (if (= a 0) nil (/ (- b) (* 2 a))))
	 )
    (values (list r1 r2) peak (if x (* a (- x r1) (- x r2))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 truncate-to
;;
(defun truncate-to(number place &key round)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; truncate-to Doc:    Creation Date:1/28/97        Author: Andy Long
;;
;; Description/Arguments:

       (truncate-to number place &key round)
  
	Truncates number to the place decimal place. So if you give .1 for
the place, the tenths place is the last you'll see.

;; Notes:

;; See Also: round, truncate, round-to

;; Examples:
 (truncate-to 4.37 .1)
 ;; 4.3
 (truncate-to 4.37 .1 :round t)
 ;; 4.4
 (truncate-to (list 5.432 9.780 4.37) .1)
 ;; (5.4 9.7 4.3)
 (truncate-to (list 5.432 9.780 4.37) .01)
 ;; (5.43 9.77 4.37)
 ;; Notice that 9.78 is truncated to 9.77, as 9.78 is stored internally as
 ;; 9.779999999....
 (truncate-to 9.78 .01)
"
  (round-to number place :truncate (if round nil t))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 round-to
;;
(defun round-to(number place &key truncate)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; round-to Doc:    Creation Date:1/28/97        Author: Andy Long
;;
;; Description/Arguments:

       (round-to number place &key truncate)
  
	Rounds number to the place decimal place. So if you type 1e-1 for
the place, the tenths place is the last you'll see.

;; Notes:

;; See Also: round, truncate, truncate-to

;; Examples:
 (round-to 4.37 .1)
 ;; 4.4
 (round-to 4.37 .1 :truncate t)
 ;; 4.3
 (round-to (list 5.432 9.780 4.37) .1)
 ;; (5.4 9.8 4.4)
 (round-to (list 5.432 9.780 4.37) .01)
 ;; (5.43 9.78 4.37)
"
  (let ((f (if truncate #'truncate #'round)))
    (if (listp number)
	(* place (mapcar f (/ number place)))
      (first (* place (mapcar f (list (/ number place)))))
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 not-a-numberp
;;
(defun not-a-numberp(x)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; not-a-numberp Doc:    Creation Date:6?/97        Author: Andy Long
;;
;; Description/Arguments:

       (not-a-numberp x)
  
	Decides if a number is #.not-a-number or not. Vectorized. Not-a-numberp
proceeds in the following way:

	1) if x is not a number, nil
	2) if x is a number, then
		a) if complex, nil
		b) if real, but it fails to be either >, <, or equalp 0,
		   then t

Not-a-number is a number which fails comparison with 0.

;; Notes: (= 0 nan) ;; t!

;; See Also:

;; Examples:
 (= 0 nan)			;; t (!)
 (not-a-numberp \"hi\")		;; nil
 (not-a-numberp 3)		;; nil
 (not-a-numberp 0.0)		;; nil
 (not-a-numberp #.not-a-number) ;; t
 (not-a-numberp nan)		;; t
 (not-a-numberp #c(1 2))	;; nil
"
  (if (listp x) (mapcar #'not-a-numberp x)
    (if (numberp x)
	(if (complexp x) nil
	  (if (or (> x 0) (< x 0) (equalp x 0)) nil t)
	  )
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 prune
;;
(defun prune(x xmin xmax
	       &key (value #.not-a-number) (mintest #'<) (maxtest #'>))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; prune Doc:    Creation Date://97        Author: Andy Long
;;
;; Description/Arguments:

	(prune x xmin xmax
	       &key (value #.not-a-number) (mintest #'<) (maxtest #'>))
  
	Prune destructively modifies a list by pruning the limits.

;; Notes:

;; See Also:

;; Examples:
 (def stuff (iseq 10))
 (prune stuff 2 8)
 stuff
"
  (let ((badstuff (union (where x xmin mintest) (where x xmax maxtest))))
    (setf (select x badstuff) (repeat value (length badstuff)))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 nilly
;;
(defun nilly (&rest x)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; nilly Doc:    Creation Date:long, long ago.        Author: Andy Long
;;
;; Description/Arguments:

       (nilly &rest x)
  
	Nilly just returns nil. This function is used by other functions as an
alias: you can 'turn off' a function by aliasing it to nilly. It takes any
argument(s) and returns nil.

;; Examples:
 (nilly 'phone-book)
 (nilly 1 2 'phone-book)
"
  nil)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 all-pairs
;;
(defun all-pairs (x &key (apply #'list))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          all-pairs Doc:
;;
;; Author: Andy Long
;; Created: Wed May 7 10:19:18 CDT 1997
;;
;; Description/Arguments:

	(all-pairs x &key (apply #'list))
  
	Creates pairs of all variables in the list, and performs an operation
on them. The default is to list them.

;; Notes:

;; See Also:

;; Examples:
 (all-pairs (iseq 4))
 (all-pairs (iseq 4) :apply #'+)
 (all-pairs (iseq 4) :apply #'*)

 (def x1 (list -3 -1 -1 -3 -3 -1 1 -1 1 -1 1))
 (def x2 (list -3 -1 -1 -1 1 -3 -3 1 -1 -1 1 ))
 (def x3 (list -3 -1 -1 -1 1 -1 1 -3 -3 1 -1))
 (def x4 (list -3 -1 -1 1 -1 -1 1 1 -1 -3 -3))
 (all-pairs (list x1 x2 x3 x4) :apply #'*)
"
  (mapcon
   (lambda(x) (mapcar (lambda(y) (funcall apply (first x) y)) (rest x)))
   x
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 all-subsets
;;
(defun all-subsets(x)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          all-subsets Doc:
;;
;; Author: Andy Long
;; Created: Wed May 7 14:15:15 CDT 1997
;;
;; Description/Arguments:

       (all-subsets x)
  
;; Notes:

;; See Also:

;; Examples:
 (all-subsets (iseq 1))
 (all-subsets (iseq 2))
 (all-subsets (iseq 3))

"
  (if (= 1 (length x)) (list x nil)
    (all-sets-of-two-sets (list (first x)) (rest x))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 all-sets-of-two-sets
;;
(defun all-sets-of-two-sets(x y)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          all-sets-of-two-sets Doc:
;;
;; Author: Andy Long
;; Created: Wed May 7 14:20:58 CDT 1997
;;
;; Description/Arguments:

       (all-sets-of-two-sets x y)
  
	This is a helper function for function all-subsets. Just turns out to
  be handy!

;; Notes:

;; See Also:

;; Examples:
 (all-sets-of-two-sets '(a) '(b))
 (all-sets-of-two-sets '(a) '(b c))
 (all-sets-of-two-sets '(a b) '(c d e))
"
  (let* (
	 (single-x (if (= 1 (length x)) t nil))
	 (single-y (if (= 1 (length y)) t nil))
	 (z
	  (cond
	   ((and single-x single-y)
	    (list nil x y (list (first x) (first y))))
	   (single-x
	    (append 
	     (mapcar (lambda(y) (union y x)) (all-subsets y))
	     (all-subsets y)
	     ))
	   (single-y
	    (append 
	     (mapcar (lambda(x) (union y (reverse x))) (all-subsets x))
	     (all-subsets x)
	     ))
	   (t
	    (let ((tmp (union y (reverse x))))
	      (all-sets-of-two-sets (list (first tmp)) (rest tmp)))
	    )
	   )
	  )
	 )
    (select z (order (mapcar #'size z)))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 distance
;;
(defun distance(a b)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          distance Doc:
;;
;; Author: Andy Long
;; Created: Fri Jun 27 16:11:47 CDT 1997
;;
;; Description/Arguments:

       (distance a b)
  
;; Notes:

;; See Also:

;; Examples:
 (distance (vector 1 2 3) (vector 3 2 1))
"
  (norm (- a b))
  )
;;
#|
This is already defined! Don't know why I didn't find it first....
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 mod
;;
(defun mod(integer base)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          mod Doc:
;;
;; Author: Andy Long
;; Created: Mon Nov 10 20:48:11 1997
;;
;; Description/Arguments:

       (mod integer base)
  
;; Notes:

;; See Also:

;; Examples:
 (mod 10 3) ;; 1
 (mod 12 3) ;; 0
"
  (- integer (* (floor (/ integer base)) base))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
|#
