#+macintosh
(progn
  ;;**** check over exports
  (export '(find-menu set-menu-bar
		      *apple-menu* *file-menu* *edit-menu* *command-menu*
		      *standard-menu-bar*))
  
  (mapcar #'(lambda (x) (send *apple-menu* :delete-items x))
	  (send *apple-menu* :items))
  (mapcar #'(lambda (x) (send *file-menu* :delete-items x))
	  (send *file-menu* :items)) 
  (mapcar #'(lambda (x) (send *edit-menu* :delete-items x))
	  (send *edit-menu* :items))
  (mapcar #'(lambda (x) (send *command-menu* :delete-items x)) 
	  (send *command-menu* :items))
  (if (boundp '*andy-menu*)
      (mapcar #'(lambda (x) (send *andy-menu* :delete-items x)) 
	      (send *andy-menu* :items))
    )
;;;;
;;;; Editing Methods
;;;;
  
  (defmeth edit-window-proto :edit-selection ()
    (send (send edit-window-proto :new)
	  :paste-stream (send self :selection-stream)))
  
  #|
  (defmeth edit-window-proto :eval-selection ()
    (let ((s (send self :selection-stream)))
      (do ((expr (read s nil '*eof*) (read s nil '*eof*)))
	  ((eq expr '*eof*))
	  (eval expr))))
|#
  
  (defmeth edit-window-proto :eval-selection ()
    "
      This is a handy improvement on the eval-selection that comes with
xlisp-stat. It actually prints out the result (the other version will only show
output if it comes in the form of a plot, essentially:
      This is only useful for macs, for which the edit-window-proto is defined.
"
    (let ((s (send self :selection-stream)))
      (do ((expr (read s nil '*eof*) (read s nil '*eof*)))
	  ((eq expr '*eof*))
	  (print (eval expr))
	  (terpri)
	  )
      )
    )
  
  (let ((last-string ""))
    (defmeth edit-window-proto :find ()
      "Method args: ()
Opens dialog to get string to find and finds it. Beeps if not found."
      (let ((s (get-string-dialog "String to find:" :initial last-string)))
	(when s
	      (if (stringp s) (setq last-string s))
	      (unless (and (stringp s) (send self :find-string s))
		      (sysbeep)))))
    (defmeth edit-window-proto :find-again ()
      (unless (and (stringp last-string) 
		   (< 0 (length last-string))
		   (send self :find-string last-string))
	      (sysbeep))))
  
;;;;
;;;; General Menu Methods and Functions
;;;;
  (defmeth menu-proto :find-item (str)
    "Method args: (str)
Finds and returns menu item with tile STR."
    (dolist (item (send self :items))
	    (if (string-equal str (send item :title))
		(return item))))
  
  (defun find-menu (title)
    "Args: (title)
Finds and returns menu in the menu bar with title TITLE."
    (dolist (i *hardware-objects*)
	    (let ((object (nth 2 i)))
	      (if (and (kind-of-p object menu-proto) 
		       (send object :installed-p) 
		       (string-equal (string title) (send object :title)))
		  (return object)))))
  
  (defun set-menu-bar (menus)
    "Args (menus)
Makes the list MENUS the current menu bar."
    (dolist (i *hardware-objects*)
	    (let ((object (nth 2 i)))
	      (if (kind-of-p object menu-proto) (send object :remove))))
    (dolist (i menus) (send i :install)))
  
;;;;
;;;; Apple Menu
;;;;
  (defvar *apple-menu* (send apple-menu-proto :new (string #\apple)))
  (send *apple-menu* :append-items 
	(send menu-item-proto :new "About XLISP-STAT"
	      :action 'about-xlisp-stat))
  
;;;;
;;;; File Menu
;;;;
  (defvar *file-menu* (send menu-proto :new "File"))
  
  (defproto file-edit-item-proto '(message) '() menu-item-proto)
  
  (defmeth file-edit-item-proto :isnew (title message &rest args)
    (setf (slot-value 'message) message)
    (apply #'call-next-method title args))
  
  (defmeth file-edit-item-proto :do-action ()
    (send (front-window) (slot-value 'message)))
  
  (defmeth file-edit-item-proto :update ()
    (send self :enabled (kind-of-p (front-window) edit-window-proto)))
  
  (send *file-menu* :append-items 
	(send menu-item-proto :new "Load" :key #\L :action
	      #'(lambda ()
		  (let ((f (open-file-dialog t)))
		    (when f (load f) (format t "; finished loading ~s~%" f)))))
	(send menu-item-proto :new "Load-p" :key #\M
	      :action #'(lambda() (load (open-file-dialog) :print t)))
	(send menu-item-proto :new "Compile" :key #\K
	      :action #'(lambda() (compile-file (open-file-dialog))))

	(send dash-item-proto :new)

	(send menu-item-proto :new "New Edit" :key #\N
	      :action #'(lambda () (send edit-window-proto :new)))
	(send menu-item-proto :new "Open Edit" :key #\O
	      :action #'(lambda ()
			  (send edit-window-proto :new :bind-to-file t)))
	(send dash-item-proto :new)
	(send file-edit-item-proto :new "Save Edit" :save :key #\S)
	(send file-edit-item-proto :new "Save Edit As" :save-as)
	(send file-edit-item-proto :new "Save Edit Copy" :save-copy)
	(send file-edit-item-proto :new "Revert Edit" :revert)
	(send dash-item-proto :new)
	(send menu-item-proto :new "Quit" :key #\Q :action 'exit))

;;;;
;;;; Edit Menu
;;;;
  (defproto edit-menu-item-proto '(item message) '() menu-item-proto)

  (defmeth edit-menu-item-proto :isnew (title item message &rest args)
    (setf (slot-value 'item) item)
    (setf (slot-value 'message) message)
    (apply #'call-next-method title args))
  
  (defmeth edit-menu-item-proto :do-action ()
    (unless (system-edit (slot-value 'item))
	    (let ((window (front-window)))
	      (if window (send window (slot-value 'message))))))
          
  (defvar *edit-menu* (send menu-proto :new "Edit"))
  (send *edit-menu* :append-items
	(send edit-menu-item-proto :new "Undo" 0 :undo :enabled nil)
	(send dash-item-proto :new)
	(send edit-menu-item-proto :new "Cut" 2 :cut-to-clip :key #\X)
	(send edit-menu-item-proto :new "Copy" 3 :copy-to-clip :key #\C)
	(send edit-menu-item-proto :new "Paste" 4 :paste-from-clip :key #\V)
	(send dash-item-proto :new)
	(send menu-item-proto :new "Copy-Paste" :key #\/ :action
	      #'(lambda () 
		  (let ((window (front-window)))
		    (when  window
			   (send window :copy-to-clip)
			   (send window :paste-from-clip)))))
	(send dash-item-proto :new)
	(send menu-item-proto :new "Find ..." :key #\F :action
	      #'(lambda () 
		  (let ((window (front-window))) 
		    (if window (send window :find)))))
	(send menu-item-proto :new "Find Again" :key #\A :action
	      #'(lambda () 
		  (let ((window (front-window))) 
		    (if window (send window :find-again)))))
	(send dash-item-proto :new)
	(send menu-item-proto :new "Edit Selection" :action
	      #'(lambda () (send (front-window) :edit-selection)))
	(send menu-item-proto :new "Eval Selection" :key #\E :action
	      #'(lambda () (send (front-window) :eval-selection))))

;;;;
;;;; Command Menu
;;;;
  (defvar *command-menu* (send menu-proto :new "Command"))
  (send *command-menu* :append-items
	(send menu-item-proto :new "Show XLISP-STAT"
	      :action #'(lambda () (send *listener* :show-window)))
	(send dash-item-proto :new)
	(send menu-item-proto :new "Clean Up" :key #\, :action #'clean-up)
	(send menu-item-proto :new "Abort" :key #\. :action #'top-level)
	(send dash-item-proto :new)
	(send menu-item-proto :new "Help" :key #\H :action #'help-select)
	(send menu-item-proto :new "Apropos" :key #\A :action #'apropos-select)
	(send menu-item-proto :new "Complete (with ctrl-c)" :key #\c-c
	      :action #'complete-select)
	(send dash-item-proto :new)
	(let ((item (send menu-item-proto :new "Capture Session to File"
			  :key #\D)))
	  (send item :action 
		#'(lambda () 
		    (cond
		     ((send item :mark) (dribble) (send item :mark nil))
		     (t (let ((f (set-file-dialog "Dribble file:")))
			  (when f
				(dribble f)
				(send item :mark t)))))))
	  item))

  (defvar *andy-menu* (send menu-proto :new "Calculus"))
  (send *andy-menu* :append-items
	(send menu-item-proto :new "Tutorials" :key #\T
	      :action #'tutorials)
	(send menu-item-proto :new "Overview"
	      :action #'(lambda ()
			  (load (concatenate
				 'string *andy-default-path* "tutorial"
				 separator "student.lsp") :print t)))
	(send dash-item-proto :new)
	(send menu-item-proto :new "Show Plots"
	      :action #'(lambda () (dolist (i (agw)) (send i :show-window))))
	(send menu-item-proto :new "Hide Plots"
	      :action #'(lambda () (dolist (i (agw)) (send i :hide-window))))
	(send menu-item-proto :new "Close Plots"
	      :action #'close-all-plots)
	(send dash-item-proto :new)
	(send menu-item-proto :new "Previous Command" :key #\P
	      :action #'(lambda ()
			  (print -)
			  (print (eval -))
			  (terpri)
			  )
	      )
	(send menu-item-proto :new "Add Command to History" :key #\1
	      :action #'(lambda () (history -2)))
	(send menu-item-proto :new "History" :key #\2
	      :action #'history)
	(send dash-item-proto :new)
	(send menu-item-proto :new "Abort" :key #\. :action #'top-level)
	)

  (defconstant *standard-menu-bar*
    (list *apple-menu* *file-menu* *edit-menu*
	  *command-menu* *andy-menu*))
  )


#+macintosh
(defmeth edit-window-proto :eval-selection ()
  "
      This is a handy improvement on the eval-selection that comes with
xlisp-stat. It actually prints out the result (the other version will only show
output if it comes in the form of a plot, essentially:
      This is only useful for macs, for which the edit-window-proto is defined.
"
  (let ((s (send self :selection-stream)))
    (do ((expr (read s nil '*eof*) (read s nil '*eof*)))
	((eq expr '*eof*))
	(print (eval expr))
	(terpri)
	)
    )
  )

#+macintosh (set-menu-bar *standard-menu-bar*)

;; Modelled after mouse-mode-item-proto, found in graphics.lsp
;; 								AEL
;; OVERLAY-MODE-ITEM-PROTO. Rotates among overlay on, and off

(defproto overlay-mode-item-proto '(graph) () menu-item-proto
  "Menu item for changing the overlay mode")

(defmeth overlay-mode-item-proto :isnew (v)
  (setf (slot-value 'graph) v)
  (call-next-method "Overlay Mode ..."))

(defmeth overlay-mode-item-proto :do-action ()
  (let ((graph (slot-value 'graph)))
    (send graph :toggle-overlay-mode)))

(defmeth graph-proto :overlay-modes ()
  "Method args: ()
Returns a list of the current overlay modes."
  '("on" "off"))

(defmeth graph-proto :choose-overlay-mode ()
  "Method args: ()
Presents a dialog to set the overlay mode."
  (let* ((modes (send self :overlay-modes))
         (m (choose-item-dialog "New Mode:" modes
                                :initial (position (send self :overlay-modes)
                                                   modes))))
    (if m
	(list
	 (if (equalp (nth m modes) "on")
	     (let()
	       (send self :add-overlay
		     (send scatterplot-control-overlay-proto :new))
	       (send self :margin 0 0 0 20)
	       )
	   (send self :delete-overlay (first (send self :slot-value 'overlays)))
	   )
	 (send self :redraw)
	 )))
  )

(defmeth graph-proto :toggle-overlay-mode ()
  "Method args: ()
Presents a dialog to set the overlay mode."
  (let* ((modes (send self :overlay-modes))
         (m (first (send self :slot-value 'overlays))))
    (if m
	(send self :delete-overlay (first (send self :slot-value 'overlays)))
      (let()
	(send self :add-overlay
	      (send scatterplot-control-overlay-proto :new))
	(send self :margin 0 0 0 20)
	)
      )
    (send self :redraw)
    )
  )

(defmeth graph-proto :make-menu-item (item-template)
  (if (kind-of-p item-template menu-item-proto)
      item-template
    (case item-template
	  (dash (send dash-item-proto :new))
	  (link (send link-item-proto :new self))
	  (erase-selection
	   (send graph-item-proto :new "Remove Selection" self 
		 :erase-selection :any-points-selected-p))
	  (focus-on-selection
	   (send graph-item-proto :new "Focus on Selection" self 
		 :focus-on-selection :any-points-selected-p))
	  (showing-labels 
	   (send graph-item-proto :new "Labels Showing " self
		 :showing-labels :showing-labels :toggle t))
	  (show-all
	   (send graph-item-proto :new "Show All" self 
		 :show-all-points :all-points-showing-p :negate t))
	  (selection
	   (send graph-item-proto :new "Selection ..." self 
		 :selection-dialog))
	  (mouse (send mouse-mode-item-proto :new self))
	  (resize-brush 
	   (send graph-item-proto :new "Resize Paint Brush " self :resize-brush))
	  (redraw 
	   (send graph-item-proto :new "Redraw Plot" self :redraw))
	  (rescale 
	   (send graph-item-proto :new "Adjust Plot to Data "
		 self :adjust-to-data))
	  (make-labels
	   (send graph-item-proto :new "Make Labels in Plot "
		 self :make-labels))
	  (add-plot-item
	   (send graph-item-proto :new "Add Item to Plot " self :add-to-plot))
	  (options 
	   (if (not (small-machine-p))
	       (send graph-item-proto :new "More Options ..." self :set-options)))
	  (andy-options 
	   (if (not (small-machine-p))
	       (send graph-item-proto :new "Andy's Options ..."
		     self :set-andy-options)))
	  (overlays (send overlay-mode-item-proto :new self))
	  (slicer
	   (if (not (small-machine-p))
	       (send graph-item-proto :new
		     "Slicer ..." self :make-slicer-dialog)))
	  (symbol
	   (send graph-item-proto :new "Selection Symbol" self
		 :set-selection-symbol :any-points-selected-p))
	  (color
	   (if (screen-has-color)
	       (send graph-item-proto :new "Selection Color" self
		     :set-selection-color :any-points-selected-p)))
	  (postscript-image
	   (send graph-item-proto :new "Save to Postscript File "
		 self :postscript-image))
	  (save-image
	   (send graph-item-proto :new "Save to File" self :ask-save-image)))))

(defmeth graph-proto :new-menu
  (&optional title &key (items (send self :menu-template)))
  (unless title (setq title (slot-value 'menu-title)))
  (if (slot-value 'menu) (send (slot-value 'menu) :dispose))
  (flet ((make-item (item) (send self :make-menu-item item)))
	(let ((menu (send menu-proto :new title)))
	  (send self :menu menu)
	  (apply #'send menu :append-items
		 (remove nil (mapcar #'make-item items)))
	  menu)))


(send graph-proto :menu-template
      '(link showing-labels mouse resize-brush dash 
	     erase-selection focus-on-selection
	     show-all symbol
	     color
	     selection dash 
	     overlays
	     slicer 
	     rescale 
	     make-labels
	     add-plot-item
	     andy-options
	     options
	     postscript-image
	     #+unix save-image
	     ))
(send scatterplot-proto :menu-template (send graph-proto :menu-template))
#|
(send scatterplot-proto :menu-template
      '(link showing-labels mouse resize-brush dash 
	     erase-selection focus-on-selection
	     show-all symbol
	     color
	     selection dash 
	     overlays
	     slicer 
	     rescale 
	     make-labels
	     add-plot-item
	     andy-options
	     options
	     postscript-image
	     #+unix save-image
	     ))
|#

(defmeth graph-proto :postscript-image ()
  "Puts up a dialog asking for a postscript file name, then creates a file if
the name is filled in."
  (let ((name
	 (get-string-dialog "Postscript Filename: "
			    :location (send self :location))
	 ))
    (if name (postscript name :plot self))))

(defmeth graph-proto :set-andy-options ()
  "Method args: ()
Opens dialog to set plot options. Items are obtained using the
:make-options-dialog-items message."
  (let* (
	 (x-label (send text-item-proto :new "Mouse-Modes:"))
	 (y-label (send text-item-proto :new "Basic Color:"))
	 (toggle-label (send text-item-proto :new "Toggles:"))
	 
	 (prompt (send text-item-proto :new "Title:"))
	 (name (send edit-text-item-proto
		     :new (if (send self :title) (send self :title) "")
		     :text-length 35))
	 
	 (promptx (send text-item-proto :new "X-label:"))
	 (namex (send edit-text-item-proto
		      :new (if (send self :variable-label 0)
			       (send self :variable-label 0) "")
		      :text-length 10))
	 
	 (prompty (send text-item-proto :new "Y-label:"))
	 (namey (send edit-text-item-proto
		      :new (if (send self :variable-label 1)
			       (send self :variable-label 1) "")
		      :text-length 10))
	 
	 (lines (send toggle-item-proto :new "Lines" :value *lines*))
	 (overlay (send toggle-item-proto :new "Toggle overlay" :value nil))
	 (use-color (send toggle-item-proto
			  :new "Use color" :value (if *use-color* t nil)))
	 (gnuplot (send toggle-item-proto :new "Gnuplot file" :value nil))
	 
	 (x-items (list "selecting" "brushing" "zoom-in" "hand-rotate"))
	 (x-item-symbols '(selecting brushing zoom-in hand-rotate))
	 (x-item (send choice-item-proto :new x-items :value 2))
	 
	 (colors (list "Black" "Blue " "Green" "Red  "))
	 (colorsyms '(Black Blue Green Red))
	 (y-item (send choice-item-proto :new colors :value 0))
	 cancel ok items
	 )
    
    (setf cancel (send modal-button-proto :new "Cancel"))
    (setf ok
	  (send modal-button-proto
		:new "OK"
		:action #'(lambda()
			    (list
			     (send name :text)
			     (send y-item :value)
			     (send x-item :value)
			     (send lines :value)
			     (send overlay :value)
			     (send gnuplot :value)
			     (send use-color :value)
			     (send namex :text)
			     (send namey :text)
			     )
			    )))
		
    (setf items 
	  (send modal-dialog-proto :new
		(list
		 (list
		  (list y-label y-item)
		  (list x-label x-item)
		  (list toggle-label lines overlay use-color gnuplot)
		  )
		 (list promptx namex prompty namey)
		 (list prompt name)
		 (list ok cancel)
		 )
		:default-button ok
		))
    (setf items (send items :modal-dialog))
    
    (if items
	(let()
	  (if (not (equalp (nth 0 items) ""))
	      (send self :title (nth 0 items))
	    )
	  (if (nth 1 items) (draw-color (nth (nth 1 items) colorsyms)))
	  (if (nth 2 items) (list (send self :mouse-mode
					(nth (nth 2 items) x-item-symbols))))
	  (if (nth 3 items) (lines on) (lines off))
	  (if (nth 4 items) (send self :toggle-overlay-mode))
	  (if (nth 5 items) (send self :to-gnuplot))
	  (use-color (nth 6 items))
	  (send self :variable-label 0 (nth 7 items))
	  (send self :variable-label 1 (nth 8 items))
	  
	  (send self :plot-item)
	  )
      )
    )
  )

(defun find-and-complete-substring (x)
  "
;; Examples:
 (find-and-complete-substring \"hel\")
 (find-and-complete-substring \"help\")
"
  (let* (
         (part (do-all-symbols
                 (s)
                 (let (zoo)
                   (if (string-search x s)
                       (append zoo (list s))))))
         (part (mapcar #'string (apropos-list x)))
         (parts (mapcar
                 #'(lambda (a) (string-search x a)) part))
         (parts (mapcar
                 #'(lambda (a) (string= (string-upcase x) a :end2
                                        (length x))) part))
         (any (mapcar #'(lambda (a) (eq t a)) parts))
         (show (if any (which any)))
         )
    (case (length show)
      (0 (message-dialog "No Such Substring"))
      (1 (send (front-window) :cut-to-clip)
         (send (front-window) :paste-string (elt part (first show)))
         )
      #|
      (t (setq choice (CHOOSE-ITEM-DIALOG "which?" (select part show)))
         (if choice
	     (send (front-window) :paste-string (nth choice (select part show))
		   )
	   )
	 )
|#
      (t (scroll-dialog
	  (select part show)
	  :title "which?"
	  :action
	  #+unix #'print
	  #-unix (lambda(x) (send (front-window) :paste-string x))
	  ))
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 scroll-dialog
;;
(defun scroll-dialog(string-of-items &key action (title "Scroll-dialog"))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          scroll-dialog Doc:
;;
;; Author: Andy Long
;; Created: Mon May 19 14:50:33 CDT 1997
;;
;; Description/Arguments:

       (scroll-dialog string-of-items &key action (title \"Scroll-dialog\"))
  
;; Notes:

;; See Also:

;; Examples:
 (scroll-dialog (mapcar #'symbol-name *variables*) :action #'print)
 (scroll-dialog (list \"(plot sin)\" \"(mprint (id 4))\"))
"
  (if string-of-items
      (let* (
	     (f (if action action #'eval-string))
	     (action
	      #'(lambda (x)
		  (if x
		      (let* (
			     (kk (send self :selection))
			     (tt
			      (elt
			       (send self :slot-value 'list-data) 
			       kk))
			     )
			(funcall f tt)
			))))
	     (ll (send list-item-proto :new string-of-items :action action))
	     (dd (send dialog-proto :new 
		       (list (list ll))
		       :title title))
	     )
	)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Fake menu bar for UNIX systems with graphics
;;; This is a complete hack but at least provides enough functionality
;;; to do the examples in the book.
;;;
#+unix
(progn
  (export 'find-menu)
  (defun make-fake-menu-bar ()
    (cond
     ((and (boundp '*fake-menu-bar*) *fake-menu-bar*)
      (send *fake-menu-bar* :show-window))
     (t (let* ((ascent (send graph-window-proto :text-ascent))
	       (descent (send graph-window-proto :text-descent))
	       (gap (floor (/ ascent 2)))
	       (width 400)
	       )
	  (setf *fake-menu-bar*
		(send graph-window-proto :new 
		      :title "Menu Bar"
		      :menu-button nil
		      :size (list width (+ ascent descent (* 2 gap)))))
	  (send *fake-menu-bar* :menu (send menu-proto :new "Close"))
	  (send (send *fake-menu-bar* :menu) :append-items
		(send menu-item-proto :new "Remove the menus on this bar "
		      :action #'remove-menus-on-menu-bar)
		(send menu-item-proto :new "Start a new menu bar "
		      :action (lambda() (setf *fake-menu-bar* nil)))
		)
	  )
	(send *fake-menu-bar* :add-slot 'menus)
	
	(defmeth *fake-menu-bar* :menus (&optional (menus nil set))
	  (if set (setf (slot-value 'menus) menus))
	  (slot-value 'menus))

	(defmeth *fake-menu-bar* :install-menu (menu)
	  (unless (member menu (send self :menus))
		  (send self :menus (append (send self :menus) (list menu)))
		  (send self :show-window)
		  (send self :redraw)))

	(defmeth *fake-menu-bar* :remove-menu (menu)
	  (send self :menus (remove menu (send self :menus)))
	  (send self :redraw))

	(defmeth *fake-menu-bar* :redraw ()
	  (let* ((ascent (send self :text-ascent))
		 (gap (floor (/ ascent 2)))
		 (menus (send self :menus))
		 (left gap)
		 (bottom (+ gap ascent)))
	    (apply #'send self :erase-rect (send self :view-rect))
	    (dolist (m menus)
	      (let ((title (send m :title)))
		(send self :draw-string title left bottom)
		(setf left (+ left gap (send self :text-width title)))))))

	(defmeth *fake-menu-bar* :do-click (x y m1 m2)
	  (let* ((loc (+ (list x y) (send self :location)))
		 (gap (floor (/ (send self :text-ascent) 2)))
		 (menus (send self :menus))
		 (x (- x gap)))
	    (dolist (m menus)
	      (let ((w (send self :text-width (send m :title))))
		(when (< 0 x w)
		      (apply #'send m :popup loc)
		      (return))
		(setf x (- x gap w))))))
	(defun find-menu (name)
	  (dolist (m (send *fake-menu-bar* :menus))
	    (if (string-equal (string name) (send m :title))
		(return m))))))
    )
  (defmeth menu-proto :install ()
    (make-fake-menu-bar)
    (send *fake-menu-bar* :install-menu self))
  (defmeth menu-proto :remove ()
    (send *fake-menu-bar* :remove-menu self))
  )
