;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                     *object* method :apropos
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
 I have an :apropos method in my
 <http://galen.med.virginia.edu/~sdm7g/XES/Sources/genutils.lsp>
 which makes it easier to find method names:

 Also, look at some other code for examples -- there's a lot of stuff 
 on http://www.stat.ucla.edu, and I have quite a lot of graphics code
 on http://galen.med.virginia.edu/~sdm7g/XES/


 ---|  Steven D. Majewski   (804-982-0831)  <sdm7g@Virginia.EDU>  |---
 ---|  Department of Molecular Physiology and Biological Physics  |---
 ---|  University of Virginia             Health Sciences Center  |---
 ---|  P.O. Box 10011            Charlottesville, VA  22906-0011  |---
 All power corrupts and obsolete power corrupts obsoletely. - Ted Nelson
|#
(defmeth *object* :apropos (str &key help)
  "
Author: Steven Majewski

	:apropos makes it easier to find method names of an object.

;; Examples:
 (send scatterplot-proto :apropos \"draw\")
 (send scatterplot-proto :apropos \"draw\" :help t)
 (send histogram-proto :apropos \"plot\")
"
  (dolist 
   (name (send self :doc-topics))
   (when (string-search str (symbol-name name)) 
	 (if help 
	     (progn (send self :help name) (terpri))
	   (format t "~s~%" name))))
  (format t "~a : ~a SLOTS:~%" (slot-value 'proto-name) self)
  (dolist 
   (name (send self :slot-names))
   (when (string-search str (symbol-name name))
	 (format t "~s : ~a~%" name (slot-value name)))))

(defmeth *object* :dispose (&rest e)
  "
 I need to define this so that calls to call-next-method have someplace to end
 up. This method doesn't do anything!
"
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                      *object* method  :save-self
;;
(defmeth *object* :save-self (&optional name)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          :save-self Doc:
;;
;; Author: Andy Long
;; Created: Sat Nov 1 08:57:02 1997
;;
;; Description/Arguments:

       *object* method :save-self (&optional name)
  
	The name is a symbol of the proto necessary to save the object. This
method asks for the values of :own-slots, and supplies the appropriate save
command to re-create the object.

	I created this method as a helper for the :save method I create in
macro uber-proto. I couldn't think of any slicker way to solve the :save
problem there.

;; See Also: uber-proto

;; Examples:
 
 (uber-proto longs (hio hoi) (thisn))
 (uber-proto andy (hio hoi) (thisn) longs)
 (def andyroo (send andy :new 1 2))
 (send andyroo :save-self)
 (send andy :save-self 'longs)
"
  (let (
	(vals 
	 (reverse
	  (mapcar (lambda(x) (slot-value x)) (send self :own-slots)))
	 )
	(name (if name name (send self :slot-value 'proto-name)))
	)
    `(apply #'send ,name :new ',vals)))

