;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Plotit
;;			the workhorse of plot
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun plotit(x)
  "
	Plotit is the workhorse of plot. It takes a single argument (which may
be a list of info), and goes through a check-list to figure out what to do with
the stuff.

See: plot
"
  (let (
	(p.range *range*)
	(p.domain (if *domain* *domain* '(-1 1)))
	(npoints (if *points* *points* 100))
	nvars
	dimen
	plotit.x
	plotit.y
	funcs
	(tplot (if *lines* 'lines 'points))
	)
    
    ;; alias just equate functions to other functions (defined in aliases.lsp
    ;; in this directory).
    (if *lines*
	(let()
	  (eval `(alias plot.type plot-lines))
	  (eval `(alias mat.type matrix-plot))
	  )
      (let()
	(eval `(alias plot.type plot-points))
	(eval `(alias mat.type matrix-point-plot))
	)
      )
    
    ;; okay, this is where the various options come into play:
    (cond 
     
     ;;  x is a function:
     ((functionp x)
      ;; Decide whether it's one- or two-d:
      ;; nvars is the number of independent variables (from the lambda exp)
      (setq nvars (length (second (get-lambda-expression x))))
      (setq dimen (1+ nvars))
      (cond
       ;; added the nvars=0 case for closures, which have 0 args:
       ((or (= nvars 1) (= nvars 0))
	(if (not *domain*) (setq *domain* '(-1 1)))
	(setq plotit.x (rseq (first *domain*) (second *domain*) npoints))
	(setq plotit.y (mapcar x plotit.x))
	(def my-plot (plot.type plotit.x plotit.y))
	(if *lines* (send my-plot :add-slot 'lines-to-kill (iseq npoints)))
	(send my-plot :add-slot 'plot-item (list x))
	)
       ((= nvars 2)
	(if (not *spin-domain*) (setq *spin-domain* '(-1 1 -1 1)))
	(if (not *spin-points*) (setq *spin-points* 6))
	(def my-plot (spin-function x
				    (first *spin-domain*)
				    (second *spin-domain*)
				    (third *spin-domain*)
				    (fourth *spin-domain*)
				    :num-points *spin-points*
				    :spline *spin-spline*
				    )
	     )
	(send my-plot :add-slot 'plot-item (list x))
	)))
     
     ;;  x is a list:
     ((listp x) ; x is a list of some sort:
      (let ((x_len (length x)))   
	(setq dimen 2)
	
	;; Given that it's a list, these are the options:
	(cond
	 
	 ;; x is a list containing (at least one, maybe some) functions:
	 ((functionp (first x))
	  
	  ;; Decide whether it's one- or two-d function:
	  ;; nvars is the number of independent variables (from the lambda exp)
	  (setq nvars (length (second (get-lambda-expression (first x)))))

	  #| - Sun Mar 29 12:02:37 1998
I'm going to establish the rule that if it's a function, but doesn't have a
lambda-expression, then it's a subroutine or a closure with a single argument.
|#
	  (if (= nvars 0) (setq nvars 1))
	  ;;ael

	  (setq dimen (1+ nvars))
	  
	  (cond
	   
	   ((functionp (second x)) ;; it's a list of a bunch of functions!
	    
	    (cond
	     ((= nvars 1)
	      ;;  might have some domain info at the tail end:
	      (if (and (numberp (nth (- x_len 2) x))
		       (numberp (nth (1- x_len) x)))
		  (let()
		    (setq p.domain (list (nth (- x_len 2) x)
					 (nth (1- x_len) x)))
		    (setq funcs (select x (iseq 1 (- x_len 3))))
		    )
		(setq funcs (rest x))
		)
	      (setq plotit.x
		    (rseq (first p.domain) (second p.domain) npoints))
	      (setq plotit.y (mapcar (first x) plotit.x))
	      (def my-plot (plot.type plotit.x plotit.y))
	      (send my-plot
		    :add-slot 'plot-item (append (list (first x)) funcs))
	      (dolist
	       (i funcs)
	       (setq plotit.y (mapcar i plotit.x))
	       (if *lines*
		   (send my-plot :add-lines plotit.x plotit.y)
		 (send my-plot :add-points plotit.x plotit.y))
	       )
	      (if *lines*
		  (send my-plot
			:add-slot
			'lines-to-kill (iseq (* (1+ (length funcs)) npoints)))
		)
	      (send my-plot :adjust-to-data)
	      )
	     ((= nvars 2)
	      
	      (setq funcs (rest x))
	      
	      (if (not *spin-domain*) (setq *spin-domain* '(-1 1 -1 1)))
	      (if (not *spin-points*) (setq *spin-points* 6))
	      
	      (def my-plot (spin-andy x 
				      (first *spin-domain*)
				      (second *spin-domain*)
				      (third *spin-domain*)
				      (fourth *spin-domain*)
				      :num-points *spin-points*
				      :spline *spin-spline*
				      )
		   )
	      (send my-plot :add-slot 'plot-item nil)
	      )
	     (t (error "Haven't a clue how to plot this: sorry!"))
	     ))
	   
	   ;; x is a list containing some additional function information:
	   ((= (length x) 3)
	    (setq p.domain (list (second x) (third x)))
	    (setq plotit.x (rseq (first p.domain) (second p.domain) npoints))
	    (def my-plot (plot.type plotit.x (mapcar (first x) plotit.x)))
	    (if *lines* (send my-plot :add-slot 'lines-to-kill (iseq npoints)))
	    (send my-plot :add-slot 'plot-item (list (first x)))
	    )
	   
	   ;; not sure what's going on: just plot the first function:
	   (t
	    (format t "~%What's that? I'll cogitate on it....~%")
	    (setq plotit.x 
		  (rseq (first p.domain) (second p.domain) npoints))
	    (def my-plot (plot.type plotit.x (mapcar (first x) plotit.x)))
	    (if *lines* (send my-plot :add-slot 'lines-to-kill (iseq npoints)))
	    (send my-plot :add-slot 'plot-item (list x))
	    )
	   )
	  )
	 
	 ;; x is really a straight list of numbers:
	 ((and (not (listp (first x)))
	       (or (typep (first x) 'float)
		   (typep (first x) 'complex)
		   (typep (first x) 'fixnum)))
	  (if (typep (first x) 'complex)
	      (let (
		    (reals (realpart x))
		    (imags (imagpart x))
		    )
		(setq dimen 2)
		(def my-plot (plot.type reals imags))
		(send my-plot :add-slot 'plot-item
		      (list (list tplot reals imags)))
		(setq p.domain (get-nice-range (min reals) (max reals) *x-tics*))
		)
	    (let()
	      (setq dimen 1)
	      (def my-plot (plot.type (iseq 1 x_len) x))
	      (send my-plot :add-slot 'plot-item
		    (list tplot (list (iseq 1 x_len) x)))
	      (setq p.domain (get-nice-range 1 x_len *x-tics*))
	      ))
	  )
	 
	 ;; x is a pair of lists:
	 ((= x_len 2 )
	  (setq dimen 2)
	  (let* (
		 (xx (first x))
		 (yy (second x))
		 (gooduns (intersection (which xx) (which yy)))
		 (xx
		  (if (= (length gooduns) (length xx)) xx (select xx gooduns)))
		 (yy
		  (if (= (length gooduns) (length yy)) yy (select yy gooduns)))
		 )
	    (def my-plot (plot.type xx yy))
	    (send my-plot :add-slot 'plot-item (list (list tplot xx yy)))
	    (setq p.domain (get-nice-range (min xx) (max xx) *x-tics*))
	    ))
	 
	 ;; three points/lists:
	 ((= x_len 3)
	  (setq dimen 3)
	  (def my-plot (spin-plot (list (first x) (second x) (third x))))
	  (if *lines*
	      (send my-plot :add-lines (list (first x) (second x) (third x)))
	    )
	  (send my-plot :add-slot 'plot-item
		;;  nil
		(list (list (first x) (second x) (third x)))
		)
	  )
	 
	 ;; ? what's going on here?
	 (t
	  (format t "~%What's that derned thing you passed me? I'll try!~%")
	  (cond
	   ((= (length (first x)) 2)
	    (let (
		  (xx (mapcar #'first x))
		  (yy (mapcar #'second x))
		  )
	      (setq dimen 2)
	      (setq p.domain (get-nice-range (min xx) (max xx) *x-tics*))
	      (def my-plot (plot.type xx yy))
	      (send my-plot :add-slot 'plot-item (list (list tplot xx yy)))
	      )
	    )
	   ((= (length (first x)) 3)
	    (format
	     t "~%Let's try a three-d plot, since there are three lists....~%")
	    (setq dimen 3)
	    (def my-plot
		 (spin-plot
		  (list
		   (mapcar #'first x) (mapcar #'second x) (mapcar #'third x)
		   )))
	    (send my-plot :add-slot 'plot-item nil)
	    )
	   (t
	    (format
	     t "~%I'll paste the columns together and see what happens....~%")
	    (setq dimen 3)
	    (def my-plot (mat.type (apply #'bind-columns x)))
	    (send my-plot :add-slot 'plot-item nil)
	    )
	   )
	  )
	 )
	)
      )
     
     ;;  x is a matrix, rather than a list:
     ((matrixp x)
      (setq dimen 3)
      (def my-plot (mat.type x))
      (send my-plot :add-slot 'plot-item nil)
      )
     
     ;;  x is a vector of numbers, possibly complex:
     ((and (vectorp x) (not (typep x 'string)))
      (cond
       ((realp (aref x 0))
	(setq dimen 1)
	(def my-plot (plot.type (iseq 1 (length x)) x))
	(send my-plot :add-slot 'plot-item
	      (list (list tplot (iseq 1 (length x)) x)))
	(setq p.domain (get-nice-range 1 (length x) *x-tics*))
	)
       ((complexp (aref x 0))
	(setq dimen 2)
	(def my-plot (plot.type (realpart x) (imagpart x)))
	(send my-plot :add-slot
	      'plot-item (list (list tplot (realpart x) (imagpart x))))
	(setq
	 p.domain
	 (get-nice-range (min (realpart x)) (max (realpart x)) *x-tics*))
	)
       ))
     
     ;;  x may as well be a comic book:
     (t
      (error
       "Sorry, I can't figure out how to handle that thing you passed me.
It may as well be a comic book, for all I can tell. Try typing

                          (help plot)

for some examples of the use of the plot function. Or better yet, fix me!")
      )
     )
    
    ;; Here we do a little last minute fenagling with the plot:
    (if (< dimen 3)
	(let()
	  (if *variable-labels*
	      (let()
		(send my-plot
		      :variable-label '(0 1) (select *variable-labels* '(0 1)))
		(send my-plot :y-axis t t *y-tics*)
		(send my-plot
		      :x-axis t t
		      (if (third p.domain) (third p.domain) *x-tics*))
		))
	  
	  (if p.range (send my-plot :range 1 (first p.range) (second p.range)))
	  (send my-plot :range 0 (first p.domain) (second p.domain))
	  
	  (if *x-axis* (send my-plot :add-lines (send my-plot :range 0) '(0 0)))
	  (if *y-axis* (send my-plot :add-lines '(0 0) (send my-plot :range 1)))
	  
	  ;; This is useful! use those mouse-buttons:
	  (if *andy-mode*
	      (send my-plot :mouse-mode *mouse-mode*)
	    (send my-plot :mouse-mode 'selecting))
	  
	  (if (/= 0 (send my-plot :num-points))
	      (let()
		(send my-plot :selection
		      (iseq 0 (- (send my-plot :num-points) 1)))
		(send my-plot :point-symbol
		      (send my-plot :selection) *plot-symbol*)
		)
	    )
	  (send my-plot :showing-labels t)
	  )
      
      (if my-plot
	  (let()
	    (send my-plot :mouse-mode 'hand-rotate)
	    (if *spin-axes* (send my-plot :showing-axes t)
	      (send my-plot :showing-axes nil))
	    (if *spin-depth* (send my-plot :depth-cuing t)
	      (send my-plot :depth-cuing nil))

	    (if (/= 0 (send my-plot :num-points))
		(let()
		  (send my-plot :selection
			(iseq 0 (- (send my-plot :num-points) 1)))
		  (send my-plot :point-symbol
			(send my-plot :selection) *spin-symbol*)
		  )
	      )
	    (if *real-axes* (send my-plot :real-axes))
	    )
	)
      )
    (if *title* (send my-plot :title *title*))
    (send my-plot :add-slot 'andy-options (list ':axes t))
    (send my-plot :plot-item)
    my-plot
    )
  )

(defun andy-mode (&optional (on/off t))
  (if on/off
      (defmeth scatterplot-proto :isnew (&rest args)
	(apply #'call-next-method args)
	(send self :add-overlay (send scatterplot-control-overlay-proto :new))
	(send self :margin 0 0 0 20)
	)
    (let()
     (defmeth scatterplot-proto :isnew (&rest args)
       (apply #'call-next-method args)
       (send self :mouse-mode 'selecting)
       )
     (setq *andy-mode* nil)
     )
    )
  )

(defun verbose (&optional (on-off nil on-off-set))
  "Makes xlisp-stat a little more talkative!"
  (if on-off-set (setq *verbose* on-off)
    (setq *verbose* (not *verbose*))
    )
  )
