;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		The essential "get-screen-point-type" routine
;;
;; Hooray! I really had to work to find this in the documentation, the rats.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :center-on (&key (motion-on nil))
  (setq lps nil)
  (send self
	:while-button-down
	#'(lambda (x y) (setq lps (append lps (list (list x y)))))
	motion-on)
  (if lps (let* (
		 (a (first (first lps)))
		 (b (second (first lps)))
		 (c (first (first (last lps))))
		 (d (second (first (last lps))))
		 )
	    (list
	     (send self :canvas-to-real a b)
	     (send self :canvas-to-real c d)
	     (length lps)
	     )
	    )
    nil)
  )

(send graph-proto :add-mouse-mode 'center-on
      :title "Center on..." :click :center-on)
(send scatterplot-proto :add-mouse-mode 'center-on
      :title "Center on..." :click :center-on :motion-on nil)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				With-do-key
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
(defmeth graph-proto :with-do-key ()
;  (setq lps (send self :do-key))
;  (print lps)
  (setq lps (send self :do-click (x y m1 m1)))
  (print lps)
  )

(send graph-proto :add-mouse-mode 'with-do-key
      :title "Do-key on..." :click :with-do-key)
(send scatterplot-proto :add-mouse-mode 'with-do-key
      :title "Do-key on..." :click :with-do-key)

(send graph-proto :help :add-slot)
ADD-SLOT
Method args: (slot &optional value)
Installs slot SLOT in object, if it does not already exist, and
sets its value to VALUE.
|#



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				plot-item
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	The idea of plot-item is to make the plot recall what was plotted in
it, so that if the range changes, it can be updated.

	I want to add this feature to the zoom, so that functions can be
continually updated to adjust to new range and domain values.

	Also so that data can be scoped.

(plot sinc)
(send my-plot :plot-item)

;; Now choose the mouse mode of Zoom!
|#

(send scatterplot-proto :add-slot 'plot-item nil)
(send graph-proto :add-slot 'plot-item nil)

(defmeth graph-proto :plot-item (&optional new)
  "
	I created the :plot-item method so that I could control exactly what
gets plotted in a plot. It is a list of lists of things, each of which is
plotted whenever the :plot-item method is invoked without an argument. With an
argument, the 'plot-item slot is reset to that item.

	Now I think that only the :add-lines option of old won't work with
this. The problem is that those lines will be cleared, but they haven't been
recorded. I think that I'll introduce a slot for those lines that I want to
kill only (problem is that :clear-lines won't take as an argument those lines
that you want removed).
"
  (when new (setf (slot-value 'plot-item) new))
  (let (
	(line-colors *line-colors*)
	(stuff-to-plot (slot-value 'plot-item))
	(andy-options
	 (if (send self :has-slot 'andy-options)
	     (slot-value 'andy-options)
	   nil
	   )
	 )
	(lines-to-kill
	 (if (send self :has-slot 'lines-to-kill)
	     (slot-value 'lines-to-kill)
	   (let ()
	     (send self :add-slot 'lines-to-kill nil)
	     nil
	     )
	   )
	 )
	line-color self.domain self.range xmin xmax
	x-coords-of-text
	y-coords-of-text
	texts
	)
    (if stuff-to-plot
	(let(
	     (count 0)
	     (axes (final (catch-keys '(:axes) andy-options)))
	     (vlabels
	      (not
	       (eval `(and ,@
			   (mapcar (lambda(x) (equalp "" x))
				   (send self :variable-label '(0 1)))
			   )
		     )
	       )
	      )	     
	     (stuff-to-plot
	      (if (listp stuff-to-plot)
		  stuff-to-plot
		(list stuff-to-plot)
		)
	      )
	     )
	  (if lines-to-kill
	      (send self
		    :linestart-masked
		    lines-to-kill
		    (repeat t (length lines-to-kill))))
	  
	  (if axes
	      (if vlabels
		  (let() 
		    (send self :y-axis t t *y-tics*)
		    (send self :x-axis t t *x-tics*)
		    )
		(let()
		  (send self :y-axis t nil *y-tics*)
		  (send self :x-axis t nil *x-tics*)
		  )
		)
	    )
	  (setq self.domain (send self :range 0))
	  (setq xmin (first self.domain))
	  (setq xmax (second self.domain))
	  
;;; ael: only the lines are going to be cleared: the points and strings can
;;; stay right where they are! Saves time, and it was unnecessary. The point of
;;; clearing is primarily so that I could clear out the functions. They need to
;;; be updated as the domain changes.
;;;	  (send self :clear-lines)
;;; Now I'm keeping the lines, and only masking them.
	  
	  (if axes
	      (let ()
		(if *y-axis*
		    (let ()
		      (setq self.range (send self :range 1))
		      (if (and
			   (> 0 (first self.domain))
			   (< 0 (second self.domain)))
			  (let ()
			    (send self :add-lines '(0 0) self.range)
			    (setq
			     lines-to-kill
			     (combine
			      lines-to-kill
			      (select-from-end (iseq (send self :num-lines))
					       (iseq 2))
			      )
			     )
			    )
			)
		      )
		  )
		(if *x-axis*
		    (let ()
		      (setq self.range (send self :range 1))
		      (if (and
			   (> 0 (first self.range))
			   (< 0 (second self.range)))
			  (let ()
			    (send self :add-lines self.domain '(0 0))
			    (setq
			     lines-to-kill
			     (combine
			      lines-to-kill
			      (select-from-end (iseq (send self :num-lines))
					       (iseq 2))
			      )
			     )
			    )
			)
		      )
		  )
		)
	    )
	  
	  (dolist
	   (i stuff-to-plot)
	   (if *use-color*
	       (let()
		 (setq line-color (first line-colors))
		 (setq line-colors (circulate line-colors))
		 (if *verbose*
		     (format t
			     "Item ~d has color ~a.~%"
			     (incf count) (symbol-name line-color))
		   )
		 )
	     (setq line-color 'black)
	     )
	   (cond
	    ((functionp i)
	     (let* (
		    (x (rseq xmin xmax *points*))
		    (y (mapcar i x))
		    )
	       (apply
		#'send
		self
		:add-lines (list x y)
		(list
		 :color line-color
		 :width *line-width*
		 :type *line-type*
		 )
		))
	     (setq
	      lines-to-kill
	      (combine
	       lines-to-kill
	       (select-from-end (iseq (send self :num-lines))
				(iseq *points*)))
	      )
	     )
	    ((not (listp i))
	     ;; if it's not a list or function, do nothing.
	     )
	    ((stringp (first i))
	     (send self :text (first i) (second i) (third i))
	     )
	    ((equal (first i) 'function)
	     (let* (
		    (x (rseq xmin xmax *points*))
		    (y (mapcar (second i) x))
		    )
	       (if (fourth i)
		   (let ()
		     (apply
		      #'send
		      self
		      :add-lines (list x y)
		      (list
		       :color (if (third i) (third i) line-color)
		       :width (if (fifth i) (fifth i) *line-width*)
		       :type (if (sixth i) (sixth i) *line-type*)
		       )
		      )
		     (setq
		      lines-to-kill
		      (combine
		       lines-to-kill
		       (select-from-end (iseq (send self :num-lines))
					(iseq *points*)))
		      )
		     )
		 (send
		  self
		  :add-points x y
		  :color (if (third i) (third i) line-color)
		  )
		 )
	       )
	     )
	    ((equal (first i) 'path)
	     (let (
		   (patchcolor (if (third i) (third i) 'white))
		   (linecolor (if (fourth i) (fourth i) line-color))
		   (color (if (third i) (third i) 'white))
		   (tmp (second i))
		   )
	       (send self :draw-color patchcolor)
	       (send self :paint-poly tmp)
	       (send self :draw-color linecolor)
	       (send self :frame-poly tmp)
	       )
;;ael caused overflow:
;;	     (send self :redraw)
	     )
	    ((equal (first i) 'patch)
	     (let (
		   (patchcolor (if (third i) (third i) 'white))
		   (linecolor (if (fourth i) (fourth i) line-color))
		   (tmp
		    (mapcar
		     (lambda(x)
		       (send self :scaled-to-canvas (first x) (second x)))
		     (second i)
		     )
		    )
		   )
	       (send self :draw-color patchcolor)
	       (send self :paint-poly tmp)
	       (send self :draw-color linecolor)
	       (send self :frame-poly tmp)
;;ael	       (send self :redraw)
	       )
	     )
	    (t nil)
	    )
	   )
	  )
      )
    (setf (slot-value 'lines-to-kill) (rest lines-to-kill))
    )
  (slot-value 'plot-item)
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             remove-plot-item
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :remove-plot-item (&rest e)
  "
;; Description:

	graph-proto :remove-plot-item (&rest e &key draw)

	Removes an item that you've added using add-plot-item.

;; Examples:

 (plot {cos x} {sin x} {x} -pi pi)
 (make-title \"Hello World\")

 ;; You can remove them sequentially:
 (send my-plot :remove-plot-item {x})
 (send my-plot :remove-plot-item {sin x})
 (send my-plot :remove-plot-item \"Hello World\")

 (plot {cos x} {sin x} {x} -pi pi)
 (make-title \"Hello World\")
 ;; or you can remove them all at once:
 (send my-plot :remove-plot-item {x} {sin x} \"Hello World\")

 (def x (rand.u 10))
 (def y (rand.u 10))
 (plot sin)
 (send my-plot :add (list 'points x y))
 (send my-plot :add (list 'lines x y))
 (send my-plot :remove-plot-item (list x y))
"
  (let (
	(new-plot-item (slot-value 'plot-item))
	keepers
	)
    (dolist
     (item e)
     (cond
      ((stringp item)
       (setq
	keepers
	(mapcar
	 (lambda(x) (if (listp x)
			(or (equalp item (first x)) (equalp item (second x)))))
	 new-plot-item
	 ))
       )
      ((and (listp item) (= (length item) 2))
       (setq
	keepers
	(mapcar
	 (lambda(x)
	   (if (listp x)
	       (and 
		(equalp (first item) (second x))
		(equalp (second item) (third x)))
	     ))
	 new-plot-item
	 ))
       )
      ((functionp item)
       (setq
	keepers
	(mapcar
	 (lambda(x)
	   (if (functionp x)
	       (equalp
		(get-lambda-expression x)
		(get-lambda-expression item))
	     (if (functionp (first x))
		 (equalp
		  (get-lambda-expression (first x))
		  (get-lambda-expression item))
	       (if (functionp (second x))
		   (equalp
		    (get-lambda-expression (second x))
		    (get-lambda-expression item))
		 ))))
	 new-plot-item
	 )
	)
       )
      )
     (if keepers
	 (setq
	  keepers
	  (second
	   (simul-keep-if keepers (iseq (length new-plot-item)) #'null)
	   )
	  new-plot-item
	  (select new-plot-item keepers)
	  )
       )
     )
    (send self :clear)
    (send self :replot-item (if new-plot-item new-plot-item '(nil)))
    nil
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        :replot-item
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :replot-item (&optional new)
  "
	I created the :replot-item method so that I could redraw exactly what
gets plotted in a plot.
"
  (when new (setf (slot-value 'plot-item) new))
  (let (
	(line-colors *line-colors*)
	(stuff-to-plot (slot-value 'plot-item))
	(andy-options
	 (if (send self :has-slot 'andy-options)
	     (slot-value 'andy-options)
	   nil
	   )
	 )
	(lines-to-kill
	 (if (send self :has-slot 'lines-to-kill)
	     (slot-value 'lines-to-kill)
	   (let ()
	     (send self :add-slot 'lines-to-kill nil)
	     nil
	     )
	   )
	 )
	line-color self.domain self.range xmin xmax
	x-coords-of-text
	y-coords-of-text
	texts
	)
    (if stuff-to-plot
	(let(
	     (count 0)
	     (axes (final (catch-keys '(:axes) andy-options)))
	     (vlabels
	      (not
	       (eval `(and ,@
			   (mapcar (lambda(x) (equalp "" x))
				   (send self :variable-label '(0 1)))
			   )
		     )
	       )
	      )	     
	     (stuff-to-plot
	      (if (listp stuff-to-plot)
		  stuff-to-plot
		(list stuff-to-plot)
		)
	      )
	     )
	  (if axes
	      (if vlabels
		  (let() 
		    (send self :y-axis t t *y-tics*)
		    (send self :x-axis t t *x-tics*)
		    )
		(let()
		  (send self :y-axis t nil *y-tics*)
		  (send self :x-axis t nil *x-tics*)
		  )
		)
	    )
	  (setq self.domain (send self :range 0))
	  (setq xmin (first self.domain))
	  (setq xmax (second self.domain))

	  (send self :clear)

	  (if axes
	      (let ()
		(if *y-axis*
		    (let ()
		      (setq self.range (send self :range 1))
		      (if (and
			   (> 0 (first self.domain))
			   (< 0 (second self.domain)))
			  (send self :add-lines '(0 0) self.range)
			)
		      ))
		(if *x-axis*
		    (let ()
		      (setq self.range (send self :range 1))
		      (if (and
			   (> 0 (first self.range))
			   (< 0 (second self.range)))
			  (send self :add-lines self.domain '(0 0))
			)
		      ))
		)
	    )

	  (dolist
	   (i stuff-to-plot)
	   (if *use-color*
	       (let()
		 (setq line-color (first line-colors))
		 (setq line-colors (circulate line-colors))
		 (if *verbose*
		     (format t
			     "Item ~d has color ~a.~%"
			     (incf count) (symbol-name line-color))
		   )
		 )
	     (setq line-color 'black)
	     )
	   (cond
	    ((functionp i)
	     (let* (
		    (x (rseq xmin xmax *points*))
		    (y (mapcar i x))
		    )
	       (apply
		#'send
		self
		:add-lines (list x y)
		(list
		 :color line-color
		 :width *line-width*
		 :type *line-type*
		 )
		))
	     (setq
	      lines-to-kill
	      (combine
	       lines-to-kill
	       (select-from-end (iseq (send self :num-lines))
				(iseq *points*)))
	      )
	     )
	    ((listp i)
	     (cond
	      ((stringp (first i))
	       (setq texts (combine texts (first i)))
	       (setq x-coords-of-text (combine  x-coords-of-text (second i)))
	       (setq y-coords-of-text (combine  y-coords-of-text (third i)))
	       )
	      ((equal (first i) 'points)
	       (send self :add-points
		     (second i)
		     (third i)
		     :color (if (fourth i) (fourth i) line-color)
		     :symbol (if (fifth i) (fifth i) *plot-symbol*)
		     )
;;
;; The following redraw appears to be necessary in order to get the plot
;; symbols to work - unfortunately, it obliterates any text in the field. To
;; take care of that, I add all text last:
;;
	       (send self :redraw)
	       )
	      ((equal (first i) 'lines)
	       (send self :add-lines
		     (second i)
		     (third i)
		     :color (if (fourth i) (fourth i) line-color)
		     :width (if (fifth i) (fifth i) *line-width*)
		     :type (if (sixth i) (sixth i) *line-type*)
		     )
	       (send self :redraw)
	       )
	      ((equal (first i) 'function)
	       (let* (
		      (x (rseq xmin xmax *points*))
		      (y (mapcar (second i) x))
		      )
		 (if (fourth i)
		     (let ()
		       (apply
			#'send
			self
			:add-lines (list x y)
			(list
			 :color (if (third i) (third i) line-color)
			 :width (if (fifth i) (fifth i) *line-width*)
			 :type (if (sixth i) (sixth i) *line-type*)
			 )
			)
		       (setq
			lines-to-kill
			(combine
			 lines-to-kill
			 (select-from-end (iseq (send self :num-lines))
					  (iseq *points*)))
			)
		       )
		   (send
		    self
		    :add-points x y
		    :color (if (third i) (third i) line-color)
		    )
		   )
		 )
	       )
	      ((equal (first i) 'path)
	       (let (
		     (patchcolor (if (third i) (third i) 'white))
		     (linecolor (if (fourth i) (fourth i) line-color))
		     (color (if (third i) (third i) 'white))
		     (tmp (second i))
		     )
		 (send self :draw-color patchcolor)
		 (send self :paint-poly tmp)
		 (send self :draw-color linecolor)
		 (send self :frame-poly tmp)
		 )
	       (send self :redraw)
	       )
	      ((equal (first i) 'patch)
	       (let (
		     (patchcolor (if (third i) (third i) 'white))
		     (linecolor (if (fourth i) (fourth i) line-color))
		     (color (if (third i) (third i) 'white))
		     (tmp
		      (mapcar
		       (lambda(x)
			 (send self :scaled-to-canvas (first x) (second x)))
		       (second i)
		       )
		      )
		     )
		 (send self :draw-color patchcolor)
		 (send self :paint-poly tmp)
		 (send self :draw-color linecolor)
		 (send self :frame-poly tmp)
		 (send self :redraw)
		 )
	       )
	      (t ; this should be a pair of list (or vectors), which will be
		 ; plotted:
	       (if *lines*
		   (let ()
		     (send self :add-lines (first i) (second i)
			   :color (if (third i) (third i) line-color)
			   :width (if (fourth i) (fourth i) *line-width*)
			   :type (if (fifth i) (fifth i) *line-type*)
			   )
		     (send self :redraw)
		     )
		 ;; otherwise, points:
		 (let()
		   (if (and *point-labels*
			    (=
			     (length *point-labels*)
			     (length (first i))
			     )
			    )
		       (let ()
			 (send self :add-points (first i) (second i)
			       :color line-color
			       :symbol *plot-symbol*
			       :point-labels *point-labels*
			       )
			 (send self :redraw)
			 (setq texts (combine texts *point-labels*))
			 (setq x-coords-of-text
			       (combine x-coords-of-text (first i)))
			 (setq y-coords-of-text
			       (combine  y-coords-of-text (second i)))
			 )
		     (let ()
		       (send self :add-points (first i) (second i)
			     :color line-color
			     :symbol *plot-symbol*
			     )
		       (send self :redraw)
		       )
		     )
		   )
		 )
	       )
	      )
	     )
	    ((vectorp i)
	     (setq tmp (iseq (length i)))
	     (if *lines*
		 (let ()
		   (send self :add-lines tmp i :color line-color
			 :width *line-width*)
		   (send self :redraw)
		   )
	       ;; otherwise, points:
	       (let()
		 (if (and *point-labels* (= (length *point-labels*)
					    (length (first i))
					    ))
		     (let ()
		       (send self :add-points tmp i
			     :point-labels *point-labels*
			     :color line-color)
		       (send self :redraw)
		       (setq texts (combine texts *point-labels*))
		       (setq x-coords-of-text
			     (combine x-coords-of-text (first i)))
		       (setq y-coords-of-text
			     (combine  y-coords-of-text (second i)))
		       )
		   (let ()
		     (send self :add-points tmp i 
			   :symbol *plot-symbol*
			   :color line-color)
		     (send self :redraw)
		     )
		   )
		 )
	       )
	     )
	    (t
	     (print "can't handle this plot-item yet!")
	     )
	    )
	   )
	  )
      )
    ;; I do this last, so that the redraw commands after add-points don't erase
    ;; the labels:
    (send self :text texts x-coords-of-text y-coords-of-text)
    )
  (slot-value 'plot-item)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             add-plot-item
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :add-plot-item (&rest e)
  "
;; Description:

	graph-proto :add-plot-item (&rest e &key draw)

	I'm continually adding things that can be added to a plot. For the
moment the list includes lines, points, strings, functions, and patches (or
paths, which are lists of coordinates which hook back on themselves at the
end).

	Default is to update the plot, calling method plot-item. These are the
sorts of plot-items you can pass:

	(list string x y)
	(list 'string string x y)
	(list 'points x y color symbol point-labels)
	(list 'lines x y color width type)
	(list 'function f color lines-on width type)
	(list 'path path patch-color line-color)

;; Examples:
 (new-plot)
 (lines off)
 (plot (* 2 (rand.n 10)) (* 2 (rand.n 10)))
 (send my-plot :add-plot-item (list 'lines (rand.n 10) (rand.n 10)))
 (send my-plot :add-plot-item (func f(x) x^2))

 (plot (* 2 (rand.n 10)) (* 2 (rand.n 10)))
 (send my-plot :add-plot-item
   (list 'lines (rand.n 10) (rand.n 10) 'red)
   (list 'function (func f(x) x) 'blue 'points)
   (list 'function (func f(x) x^2) 'blue 'lines)
   )
"
  (let (
	(draw (if (grep '(:draw) e)
		       (first (catch-keys '(:draw) e))
		     t))
	(e (remove-keys '(:draw) e))
	)
    (dolist
     (i e)
     (cond
      ((not (listp i))
       ;; if i's not a list, do nothing.
       )
      ((stringp (first i))
       )
      ((functionp (first i))
       )
      ((equal (first i) 'string)
       )
      ((equal (first i) 'points)
       (send self :add-points
	     (second i)
	     (third i)
	     :color (if (fourth i) (fourth i) *draw-color*)
	     :symbol (if (fifth i) (fifth i) *plot-symbol*)
	     :point-labels (if (sixth i) (sixth i)
			     (if (= (length (second i)) (length *point-labels*))
				 *point-labels*
			       nil))
	     )
       )
      ((equal (first i) 'lines)
       (send self :add-lines
	     (second i)
	     (third i)
	     :color (if (fourth i) (fourth i) *draw-color*)
	     :width (if (fifth i) (fifth i) *line-width*)
	     :type (if (sixth i) (sixth i) *line-type*)
	     )
       )
      ((equal (first i) 'function)
       )
      ((equal (first i) 'path)
       )
      ((equal (first i) 'patch)
       )
      (t
       (print "Sorry, can't add this type of plot-item yet!")
       )
      )
     )
    (setf (slot-value 'plot-item) (append (slot-value 'plot-item) e))
    (if draw (send self :plot-item))
    )
  nil
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             add
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :add (&rest e)
  "
;; Description:

	graph-proto :add (&rest e &key draw)

	I'm continually adding things that can be added to a plot. For the
moment the list includes lines, points, strings, functions, and patches (or
paths, which are lists of coordinates which hook back on themselves at the
end).

	Default is to update the plot, calling method plot-item. These are the
sorts of plot-items you can pass:

	(list string x y)
	(list 'string string x y)
	(list 'points x y color symbol point-labels)
	(list 'lines x y color width type)
	(list 'function f color lines-off width type)
	(list 'path path patch-color line-color)

;; Examples:
 (new-plot)
 (lines off)
 (plot (* 2 (rand.n 10)) (* 2 (rand.n 10)))
 (send my-plot :add (list 'lines (rand.n 10) (rand.n 10)))
 (send my-plot :add (list (func f(x) x^2)))

 (plot (* 2 (rand.n 10)) (* 2 (rand.n 10)))
 (send my-plot :add
   (list \"howdy\" 0 1)
   (list 'lines (rand.n 10) (rand.n 10) :color 'red)
   (list 'points (rand.n 10) (rand.n 10) :color 'red :symbol 'wedge1)
   (list 'function (func f(x) x) :color 'blue :lines-off t :type 'dashed)
   (list 'function (func f(x) x^2) :color 'blue :width 2)
   )
"
  (let (
	(draw (if (grep '(:draw) e)
		       (first (catch-keys '(:draw) e))
		     t))
	(e (remove-keys '(:draw) e))
	)
    (dolist
     (i e)
     (let* (
	    (keys
	     (catch-keys
	      '(:color :symbol :point-labels :width :type :num-points :lines-off)
	      i)
	     )
	    (color (if (elt keys 0) (elt keys 0) *draw-color*))
	    (symbol (if (elt keys 1) (elt keys 1) *plot-symbol*))
	    (point-labels (elt keys 2))
	    (width (if (elt keys 3) (elt keys 3) *line-width*))
	    (type (if (elt keys 4) (elt keys 4) *line-type*))
	    (num-points (if (elt keys 5) (elt keys 5) *points*))
	    (lines-off (elt keys 6))
	    )
       (cond
	((not (listp i))
	 ;; if i's not a list, do nothing.
	 )
	((stringp (first i))
	 (setf (slot-value 'plot-item) (append (slot-value 'plot-item) (list i)))
	 )
	((functionp (first i))
	 (setf (slot-value 'plot-item)
	       (append (slot-value 'plot-item)
		       (list (list 
			'function
			(first i)
			color
			(not lines-off)
			width
			type
			))
		       ))
	 )
	((equal (first i) 'string)
	 (setf (slot-value 'plot-item)
	       (append (slot-value 'plot-item) (list (rest i))))
	 )
	((equal (first i) 'points)
	 (setq point-labels (if point-labels point-labels
			      (if (= (length (second i))
				     (length *point-labels*))
				  *point-labels*
				nil)))
	 (send self :add-points
	       (second i)
	       (third i)
	       :color (if color color *draw-color*)
	       :symbol (if symbol symbol *plot-symbol*)
	       :point-labels point-labels
	       )
	 (setf (slot-value 'plot-item)
	       (append (slot-value 'plot-item)
		       (list (list 
			'points
			(second i)
			(third i)
			color
			symbol
			point-labels
			))
		       ))
	 )
	((equal (first i) 'lines)
	 (send self :add-lines
	       (second i)
	       (third i)
	       :color (if color color *draw-color*)
	       :width (if width width *line-width*)
	       :type (if type type *line-type*)
	       )
	 (setf (slot-value 'plot-item)
	       (append (slot-value 'plot-item)
		       (list (list 
			'lines
			(second i)
			(third i)
			color
			width
			type
			))
		       ))
	 )
	((equal (first i) 'function)
	 (setf (slot-value 'plot-item)
	       (append (slot-value 'plot-item)
		       (list (list 
			'function
			(second i)
			color
			(not lines-off)
			width
			type
			))
		       ))
	 )
	((equal (first i) 'path)
	 (setf (slot-value 'plot-item) (append (slot-value 'plot-item) (list i)))
	 )
	(t
	 (print "Sorry, can't add this type of plot-item yet!")
	 )
	)
       )
     )
    (if draw (send self :plot-item))
    )
  nil
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Reworked methods:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                new add-function:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :add-function
  (f xmin xmax &rest args
     &key (num-points 50)
          (line-color nil)
	  (lines-on *lines*)
     )
  "Message args: (f xmin xmax &key (num-points 50) (line-color nil)
Adds plot of function F of one real variable over the range between xmin
and xmax to the plot. The function is evaluated at NUM-POINTS points."
  (unless (= 2 (send self :num-variables)) (error "only works for 2D plots"))
  (let* (
	 (x (rseq xmin xmax num-points))
         (y (mapcar f x))
	 )
    (send self
	  :add
	  (list
	   'function f
	   :color (if line-color line-color *draw-color*)
	   :lines-off (not lines-on)
	   )
	  :draw nil)
    (if *use-color*
	(let (
	      (line-color (if line-color line-color *draw-color*))
	      )
	  (send self :draw-color line-color)
	  (if lines-on (apply #'send self :add-lines (list x y) args)
	    (apply #'send self :add-points (list x y) args))
	  (send self :draw-color *draw-color*)
	  )
      (if lines-on (apply #'send self :add-lines (list x y) args)
	(apply #'send self :add-points (list x y) args))
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  		method spin-proto :plot-item (&optional new)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth spin-proto :plot-item (&optional new)
  "
	Not so sophisticated for the spin-proto, yet! But I can add in a
function, points, or lines.

;; Examples:
 (plot {x*y :v '(x y)})
 (send my-plot :add-plot-item (list 'function {x+y :v '(x y)}))
 (send my-plot
   :add-plot-item
   (list 'lines (list 1 0) (list 1 0) (list 1 2)
         :color 'yellow :type 'dashed :width 3))
 (send my-plot
   :add-plot-item
   (list 'points (list 1 0) (list 1 0) (list 1 2)
   :color 'yellow :symbol 'wedge1 :point-labels (list \"hi\" \"lo\")))
"
  (andy-print "In spin-proto plot-item.")
  (when new (setf (slot-value 'plot-item) new))
  (let (
	(stuff-to-plot (slot-value 'plot-item))
	(line-color 'white)
	self.domain self.range xmin xmax
	x-coords-of-text
	y-coords-of-text
	z-coords-of-text
	texts
	)
    (if stuff-to-plot
	(dolist
	 (i (if (listp stuff-to-plot) stuff-to-plot (list stuff-to-plot)))
	 (cond
	  ((functionp i)
	   nil
	   )
	  ((listp i)
	   (let* (
		  (keys
		   (catch-keys
		    '(:color :symbol :point-labels :width :type :num-points
			     :xmin :xmax :ymin :ymax) i)
		   )
		  (color (elt keys 0))
		  (symbol (elt keys 1))
		  (point-labels (elt keys 2))
		  (width (elt keys 3))
		  (type (elt keys 4))
		  (num-points (elt keys 5))
		  (xmin (elt keys 6))
		  (xmax (elt keys 7))
		  (ymin (elt keys 8))
		  (ymax (elt keys 9))
		  )
	     (cond
	      ((stringp (first i))
	       (setq texts (combine texts (first i)))
	       (setq x-coords-of-text (combine x-coords-of-text (second i)))
	       (setq y-coords-of-text (combine y-coords-of-text (third i)))
	       (setq z-coords-of-text (combine z-coords-of-text (fourth i)))
	       )
	      ((equal (first i) 'points)
	       (send self :add-points
		     (list
		      (second i)
		      (third i)
		      (fourth i)
		      )
		     :color (if color color line-color)
		     :symbol (if symbol symbol *plot-symbol*)
		     :point-labels (if point-labels point-labels nil)
		     )
;;;
;;; The following redraw appears to be necessary in order to get the plot
;;; symbols to work - unfortunately, it obliterates any text in the field. To
;;; take care of that, I add all text last:
;;;
;;	       (send self :redraw)
	       )
	      ((equal (first i) 'lines)
	       (send self :add-lines
		     (list
		      (second i)
		      (third i)
		      (fourth i)
		      )
		     :color (if color color line-color)
		     :width (if width width *line-width*)
		     :type (if type type *line-type*)
		     )
	       )
	      ((equal (first i) 'function)
	       (send self :add-function
		     (second i)
		     (if xmin xmin (first *spin-domain*))
		     (if xmax xmax (second *spin-domain*))
		     (if ymin ymin (third *spin-domain*))
		     (if ymax ymax (fourth *spin-domain*))
		     :color (if color color line-color)
		     :width (if width width *line-width*)
		     :type (if type type *line-type*)
		     :num-points (if num-points num-points *spin-points*)
		     )
	       )
	      (t ; this should be a trio of list (or vectors), which will be
					; plotted:
	       (if *lines*
		   (send self 
			 :add-lines
			 (list
			  (first i)
			  (second i)
			  (third i)
			  )
			 :color (if color color line-color)
			 :width (if width width *line-width*)
			 :type (if type type *line-type*)
			 )
		 ;; otherwise, points:
		 (let ()
		   (send self :add-points
			 (list
			  (first i)
			  (second i)
			  (third i)
			  )
			 :color (if color color line-color)
			 :symbol (if symbol symbol *plot-symbol*)
			 :point-labels (if point-labels point-labels nil)
			 )
		   )
		 )
	       )
	      )
;;	     (send self :redraw)
	     )
	   )
	  )
	 )
      )
    (send self :add-points
	  (mapcar
	   #'rest
	   (list x-coords-of-text y-coords-of-text z-coords-of-text)
	   )
	  :color *draw-color*
	  :point-labels (rest texts))
    )
  (andy-print "plot-item farewell: ")
  (slot-value 'plot-item)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                           :add-plot-item
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth spin-proto :add-plot-item (&rest e)
  "
;; Description:

	spin-proto :add-plot-item (&rest e)

	I'm continually adding things that can be added to a plot. For the
moment the list includes lines, points, functions.

;; Examples:
 (plot (rand.n 10) (rand.n 10) (rand.n 10))
 (send my-plot :add-plot-item
   (list \"hi\" .4 .5 .2)
   (list 'points (rand.u 10) (rand.u 10) (rand.u 10) :color 'yellow
         :point-labels (mapcar #'num-to-string (iseq 3 12)))
   :draw t)

 (plot (rand.n 10) (rand.n 10) (rand.n 10))
 (send my-plot :add-plot-item
   (list \"hi\" .4 .5 .2)
   (list 'points (rand.u 10) (rand.u 10) (rand.u 10) :color 'yellow
         :point-labels (mapcar #'num-to-string (iseq 3 12)))
   )
"
  (let (
	(draw (first (catch-keys '(:draw) e)))
	(e (remove-keys '(:draw) e))
	)
    (if draw
	(let ()
	  (send self :plot-item (append (slot-value 'plot-item) e))
	  (send self :redraw)
	  )
      (setf (slot-value 'plot-item) (append (slot-value 'plot-item) e))
      )
    )
  nil
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Zoom!
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :zoom (&key (factor .9) (draw t))
  (setq mouse-info (send self :center-on))
  (if mouse-info
      (if (> 100 (third mouse-info)) ; held down, so zoom out
	  (list 
	   (if (not *zoom-out-factor*) (setq *zoom-out-factor* 1.1))
	   (send self :zoomer (first mouse-info) :factor *zoom-out-factor*)
	   )
	(list 
	 (if (not *zoom-in-factor*) (setq *zoom-in-factor* .9))
	 (send self :zoomer (first mouse-info) :factor *zoom-in-factor*)
	 )
	) 
    )
    mouse-info
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        update-plot
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :update-plot (&rest args)
  (send self :plot-item)
  (send self :adjust-to-data)
  (send self :plot-item)
  (values)
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         zoomer
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :zoomer (coords &key (factor .9) (draw t))

  ;; First thing is to get the current domain and range:
  (setq self.domain (send self :range 0))
  (setq self.range (send self :range 1))

  ;; then get the new spread:
  (setq xdiff (* .5 factor (- (second self.domain) (first self.domain))))
  (setq ydiff (* .5 factor (- (second self.range) (first self.range))))

  ;; obtain the new domain and range,
  (setq self.domain (list (- (first coords) xdiff) (+ (first coords) xdiff)))
  (setq self.range (list (- (second coords) ydiff) (+ (second coords) ydiff)))
  
  ;; and now actually set the new domain and range:
  (send self :range 0 (first self.domain) (second self.domain))
  (send self :range 1 (first self.range) (second self.range))

;;ael experimental  (if draw (send self :redraw))
  (if draw (list
	    (send self :redraw)
	    (send self :plot-item)
	    )
    )
  )

(send graph-proto :add-mouse-mode 'zoom
      :title "Zoom!" :click :zoom)

(send scatterplot-proto :add-mouse-mode 'zoom
      :title "Zoom!" :click :zoom)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Zoom-box
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :zoom-box (&key (draw t))
  (setq mouse-info (send self :center-on))
  (if mouse-info
      (if (> (third mouse-info) 100)
	  (send self :zoomer-box (first mouse-info) (second mouse-info))
	(send self :zoom-in :factor 2)
	)
    nil
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Zoomer-box
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :zoomer-box (c1 c2 &key (draw t))

  ;; then get the new spread:
  (setq xmin (min (first c1) (first c2)))
  (setq xmax (max (first c1) (first c2)))
  (setq ymin (min (second c1) (second c2)))
  (setq ymax (max (second c1) (second c2)))

  ;; obtain the new domain and range,
  (setq self.domain (list xmin xmax))
  (setq self.range (list ymin ymax))
  
  ;; and now actually set the new domain and range:
  (send self :range 0 (first self.domain) (second self.domain))
  (send self :range 1 (first self.range) (second self.range))

  (if draw (list
	    (send self :redraw)
	    (send self :plot-item)
	    )
    )
  )

(send graph-proto :add-mouse-mode 'zoom-box
      :title "Zoom-Box!" :click :zoom-box)

(send scatterplot-proto :add-mouse-mode 'zoom-box
      :title "Zoom-Box!" :click :zoom-box)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         zoom-in
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :zoom-in (&key (factor .9) (draw t))
  ;; First thing is to get the current domain and range:
  (setq self.domain (send self :range 0))
  (setq self.range (send self :range 1))

  (setq coords (list (mean self.domain) (mean self.range)))

  ;; then get the new spread:
  (setq xdiff (* .5 factor (- (second self.domain) (first self.domain))))
  (setq ydiff (* .5 factor (- (second self.range) (first self.range))))

  ;; obtain the new domain and range,
  (setq self.domain (list (- (first coords) xdiff) (+ (first coords) xdiff)))
  (setq self.range (list (- (second coords) ydiff) (+ (second coords) ydiff)))
  
  ;; and now actually set the new domain and range:
  (send self :range 0 (first self.domain) (second self.domain))
  (send self :range 1 (first self.range) (second self.range))

  (if draw (list
	    (send self :redraw)
	    (send self :plot-item)
	    )
    )
  )



;; DO-KEY
;; Method args: (char shift option)
;; Message received when user hits a key.


#|
;; This one should be useful!

(defun active-graph-windows ()
"Args: ()
Return list of all currently visible graph windows."
  (remove-if-not #'(lambda (x) (kind-of-p x graph-window-proto))
		 (mapcar #'third *hardware-objects*)))


(dribble ":temp.options")
(send my-plot :help :ABLINE)
(send my-plot :help :ACTIVATE)
(send my-plot :help :ADD-FUNCTION)
(send my-plot :help :ADD-LINES)
(send my-plot :help :ADD-METHOD)
(send my-plot :help :ADD-MOUSE-MODE)
(send my-plot :help :ADD-POINTS)
(send my-plot :help :ADD-SLOT)
(send my-plot :help :ADJUST-POINTS-IN-RECT)
(send my-plot :help :ADJUST-SCREEN)
(send my-plot :help :ADJUST-SCREEN-POINT)
(send my-plot :help :ADJUST-TO-DATA)
(send my-plot :help :ALL-POINTS-SHOWING-P)
(send my-plot :help :ALL-POINTS-UNMASKED-P)
(send my-plot :help :ALLOCATE)
(send my-plot :help :ANY-POINTS-SELECTED-P)
(send my-plot :help :APPLY-TRANSFORMATION)
(send my-plot :help :BACK-COLOR)
(send my-plot :help :BRUSH)
(send my-plot :help :BUFFER-TO-SCREEN)
(send my-plot :help :CANVAS-HEIGHT)
(send my-plot :help :CANVAS-TO-REAL)
(send my-plot :help :CANVAS-TO-SCALED)
(send my-plot :help :CANVAS-WIDTH)
(send my-plot :help :CENTER)
(send my-plot :help :CHOOSE-MOUSE-MODE)
(send my-plot :help :CLEAR)
(send my-plot :help :CLEAR-LINES)
(send my-plot :help :CLEAR-MASKS)
(send my-plot :help :CLEAR-POINTS)
(send my-plot :help :CLICK-RANGE)
(send my-plot :help :CLIP-RECT)
(send my-plot :help :CLOSE)
(send my-plot :help :CONTENT-ORIGIN)
(send my-plot :help :CONTENT-RECT)
(send my-plot :help :CONTENT-VARIABLES)
(send my-plot :help :COPY-TO-CLIP)
(send my-plot :help :CURRENT-VARIABLES)
(send my-plot :help :CURSOR)
(send my-plot :help :CUT-TO-CLIP)
(send my-plot :help :DELETE-DOCUMENTATION)
(send my-plot :help :DELETE-METHOD)
(send my-plot :help :DELETE-MOUSE-MODE)
(send my-plot :help :DELETE-SLOT)
(send my-plot :help :DISPOSE)
(send my-plot :help :DO-CLICK)
(send my-plot :help :DO-IDLE)
(send my-plot :help :DO-KEY)
(send my-plot :help :DO-MOTION)
(send my-plot :help :DOC-TOPICS)
(send my-plot :help :DOCUMENTATION)
(send my-plot :help :DRAG-GREY-RECT)
(send my-plot :help :DRAW-BITMAP)
(send my-plot :help :DRAW-BRUSH)
(send my-plot :help :DRAW-COLOR)
(send my-plot :help :DRAW-DATA-LINES)
(send my-plot :help :DRAW-DATA-POINTS)
(send my-plot :help :DRAW-LINE)
(send my-plot :help :DRAW-MODE)
(send my-plot :help :DRAW-POINT)
(send my-plot :help :DRAW-STRING)
(send my-plot :help :DRAW-STRING-UP)
(send my-plot :help :DRAW-SYMBOL)
(send my-plot :help :DRAW-TEXT)
(send my-plot :help :DRAW-TEXT-UP)
(send my-plot :help :ERASE-ARC)
(send my-plot :help :ERASE-BRUSH)
(send my-plot :help :ERASE-OVAL)
(send my-plot :help :ERASE-POLY)
(send my-plot :help :ERASE-RECT)
(send my-plot :help :ERASE-SELECTION)
(send my-plot :help :ERASE-WINDOW)
(send my-plot :help :FIND)
(send my-plot :help :FIXED-ASPECT)
(send my-plot :help :FOCUS-ON-SELECTION)
(send my-plot :help :FRAME-ARC)
(send my-plot :help :FRAME-LOCATION)
(send my-plot :help :FRAME-OVAL)
(send my-plot :help :FRAME-POLY)
(send my-plot :help :FRAME-RECT)
(send my-plot :help :FRAME-SIZE)
(send my-plot :help :GET-METHOD)
(send my-plot :help :H-SCROLL-INCS)
(send my-plot :help :HAS-H-SCROLL)
(send my-plot :help :HAS-METHOD)
(send my-plot :help :HAS-SLOT)
(send my-plot :help :HAS-V-SCROLL)
(send my-plot :help :HELP)
(send my-plot :help :HIDE-WINDOW)
(send my-plot :help :IDLE-ON)
(send my-plot :help :INTERNAL-DOC)
(send my-plot :help :ISNEW)
(send my-plot :help :LINE-TYPE)
(send my-plot :help :LINE-WIDTH)
(send my-plot :help :LINESTART-COLOR)
(send my-plot :help :LINESTART-COORDINATE)
(send my-plot :help :LINESTART-MASKED)
(send my-plot :help :LINESTART-NEXT)
(send my-plot :help :LINESTART-SCREEN-COORDINATE)
(send my-plot :help :LINESTART-TRANSFORMED-COORDINATE)
(send my-plot :help :LINESTART-TYPE)
(send my-plot :help :LINESTART-WIDTH)
(send my-plot :help :LINKED)
(send my-plot :help :LOCATION)
(send my-plot :help :MARGIN)
(send my-plot :help :MASK-SELECTION)
(send my-plot :help :MENU)
(send my-plot :help :METHOD-SELECTORS)
(send my-plot :help :MOUSE-MODE)
(send my-plot :help :MOUSE-MODE-TITLE)
(send my-plot :help :MOUSE-MODES)
(send my-plot :help :MOVE-BRUSH)
(send my-plot :help :NEW)
(send my-plot :help :NUM-LINES)
(send my-plot :help :NUM-POINTS)
(send my-plot :help :NUM-VARIABLES)
(send my-plot :help :OWN-METHODS)
(send my-plot :help :OWN-SLOTS)
(send my-plot :help :PAINT-ARC)
(send my-plot :help :PAINT-OVAL)
(send my-plot :help :PAINT-POLY)
(send my-plot :help :PAINT-RECT)
(send my-plot :help :PARENTS)
(send my-plot :help :PASTE-FROM-CLIP)
(send my-plot :help :PASTE-STREAM)
(send my-plot :help :PASTE-STRING)
(send my-plot :help :POINT-COLOR)
(send my-plot :help :POINT-COORDINATE)
(send my-plot :help :POINT-HILITED)
(send my-plot :help :POINT-LABEL)
(send my-plot :help :POINT-MASKED)
(send my-plot :help :POINT-SCREEN-COORDINATE)
(send my-plot :help :POINT-SCREEN-STATE)
(send my-plot :help :POINT-SELECTED)
(send my-plot :help :POINT-SHOWING)
(send my-plot :help :POINT-STATE)
(send my-plot :help :POINT-SYMBOL)
(send my-plot :help :POINT-TRANSFORMED-COORDINATE)
(send my-plot :help :POINTS-HILITED)
(send my-plot :help :POINTS-IN-RECT)
(send my-plot :help :POINTS-SELECTED)
(send my-plot :help :POINTS-SHOWING)
(send my-plot :help :PRECEDENCE-LIST)
(send my-plot :help :PRINT)
(send my-plot :help :PROTO)
(send my-plot :help :RANGE)
(send my-plot :help :REAL-TO-CANVAS)
(send my-plot :help :REDRAW)
(send my-plot :help :REDRAW-CONTENT)
(send my-plot :help :REMOVE)
(send my-plot :help :REPARENT)
(send my-plot :help :REPLACE-SYMBOL)
(send my-plot :help :RESIZE)
(send my-plot :help :RESIZE-BRUSH)
(send my-plot :help :RETYPE)
(send my-plot :help :REVERSE-COLORS)
(send my-plot :help :ROTATE-2)
(send my-plot :help :SCALE)
(send my-plot :help :SCALE-TO-RANGE)
(send my-plot :help :SCALE-TYPE)
(send my-plot :help :SCALED-RANGE)
(send my-plot :help :SCALED-TO-CANVAS)
(send my-plot :help :SCREEN-RANGE)
(send my-plot :help :SCROLL)
(send my-plot :help :SELECTION)
(send my-plot :help :SELECTION-DIALOG)
(send my-plot :help :SELECTION-STREAM)
(send my-plot :help :SET-MODE-CURSOR)
(send my-plot :help :SET-OPTIONS)
(send my-plot :help :SET-SELECTION-COLOR)
(send my-plot :help :SET-SELECTION-SYMBOL)
(send my-plot :help :SHIFT)
(send my-plot :help :SHOW)
(send my-plot :help :SHOW-ALL-POINTS)
(send my-plot :help :SHOW-WINDOW)
(send my-plot :help :SHOWING-LABELS)
(send my-plot :help :SIZE)
(send my-plot :help :SLICE-VARIABLE)
(send my-plot :help :SLOT-NAMES)
(send my-plot :help :SLOT-VALUE)
(send my-plot :help :START-BUFFERING)
(send my-plot :help :TEXT-ASCENT)
(send my-plot :help :TEXT-DESCENT)
(send my-plot :help :TEXT-WIDTH)
(send my-plot :help :TITLE)
(send my-plot :help :TRANSFORMATION)
(send my-plot :help :UNDO)
(send my-plot :help :UNMASK-ALL-POINTS)
(send my-plot :help :UNSELECT-ALL-POINTS)
(send my-plot :help :UNSHOW-ALL-POINTS)
(send my-plot :help :UPDATE)
(send my-plot :help :USE-COLOR)
(send my-plot :help :V-SCROLL-INCS)
(send my-plot :help :VARIABLE-LABEL)
(send my-plot :help :VIEW-RECT)
(send my-plot :help :VISIBLE-RANGE)
(send my-plot :help :WHILE-BUTTON-DOWN)
(send my-plot :help :X-AXIS)
(send my-plot :help :Y-AXIS)
(dribble)


> (send my-plot :help :ABLINE)
ABLINE
Message args: (a b)
Adds the graph of the line A + B x to the plot.
NIL
> (send my-plot :help :ACTIVATE)
ACTIVATE
Method args: (active)
Installs menu if ACTIVE is true; removes it otherwise. (Macintosh only.)
NIL
> (send my-plot :help :ADD-FUNCTION)
ADD-FUNCTION
Message args: (f xmin xmax &key (num-points 50)
Adds plot of function F of one real variable over the range between xmin
and xmax to the plot. The function is evaluated at NUM-POINTS points.
NIL
> (send my-plot :help :ADD-LINES)
ADD-LINES
Method args: (lines &key type (draw t))
or:          (x y  &key point-labels (draw t))
Adds lines to plot. LINES is a list of sequences, the 
coordinates of the line  starts. TYPE is normal or dashed. If
DRAW is true the new lines are added to the screen. For a 2D
plot LINES can be replaced by two sequences X and Y.
NIL
> (send my-plot :help :ADD-METHOD)
ADD-METHOD
Method args: (selector method)
Installs METHOD for SELECTOR in object.
NIL
> (send my-plot :help :ADD-MOUSE-MODE)
ADD-MOUSE-MODE
Method args: (mode &key (title (string mode)) (cursor 'arrow) click motion)
Adds MODE to the mode list. TITLE is the string used in the menu, CLICK
and MOTION are the selectors for the mouse actions.
NIL
> (send my-plot :help :ADD-POINTS)
ADD-POINTS
Method args: (points &key point-labels (draw t))
or:          (x y  &key point-labels (draw t))
Adds points to plot. POINTS is a list of sequences, 
POINT-LABELS a list of strings. If DRAW is true the new points
are added to the screen. For a 2D plot POINTS can be replaced
by two sequences X and Y.
NIL
> (send my-plot :help :ADD-SLOT)
ADD-SLOT
Method args: (slot &optional value)
Installs slot SLOT in object, if it does not already exist, and
sets its value to VALUE.
NIL
> (send my-plot :help :ADJUST-POINTS-IN-RECT)
ADJUST-POINTS-IN-RECT
Method args: (left top width height state)
Adjusts points in rectangle to have specified state. STATE should be HILITED or SELECTED.
If it is HILITED, highlighted points outside the rectangle are unhighlighted.
NIL
> (send my-plot :help :ADJUST-SCREEN)
ADJUST-SCREEN
Method args: ()
Checks all points and adjusts their screen states to match internal state.
NIL
> (send my-plot :help :ADJUST-SCREEN-POINT)
ADJUST-SCREEN-POINT
Method args: (i)
Adjusts the screen highlight state of point I to match internal state. Internal state
should not be INVISIBLE.
NIL
> (send my-plot :help :ADJUST-TO-DATA)
ADJUST-TO-DATA
Method args: (&key (draw t))
Sets ranges to the actual range of variables in the original coordinate
system. If DRAW is true sends :RESIZE and :REDRAW messages.
NIL
> (send my-plot :help :ALL-POINTS-SHOWING-P)
ALL-POINTS-SHOWING-P
Method ars: ()
NIL
> (send my-plot :help :ALL-POINTS-UNMASKED-P)
ALL-POINTS-UNMASKED-P
Method ars: ()
NIL
> (send my-plot :help :ALLOCATE)
ALLOCATE
Method args: ()
Allocates a new graph.
NIL
> (send my-plot :help :ANY-POINTS-SELECTED-P)
ANY-POINTS-SELECTED-P
Method ars: ()
NIL
> (send my-plot :help :APPLY-TRANSFORMATION)
APPLY-TRANSFORMATION
Method args: (a &key draw basis)
Applies matrix A to current transformation. If draw is true the :REDRAW-CONTENT
message is sent.
NIL
> (send my-plot :help :BACK-COLOR)
BACK-COLOR
Method args: (&optional color)
Sets or retrieves current background color. Choices are in the list *COLORS*.
NIL
> (send my-plot :help :BRUSH)
BRUSH
Method args: (x y width height)
Sets or retrieves current brush. Brush is specified in terms of the lower lefthand
corner and its width and height.
NIL
> (send my-plot :help :BUFFER-TO-SCREEN)
BUFFER-TO-SCREEN
Method args: (&optional left top width height)
Reduces buffering level. If level reaches zero copies content of rectangle
(LEFT TOP WIDTH HEIGHT to window. Default rectangle is the window's view rectangle.
NIL
> (send my-plot :help :CANVAS-HEIGHT)
CANVAS-HEIGHT
Method args: ()
Returns current canvas height.
NIL
> (send my-plot :help :CANVAS-TO-REAL)
CANVAS-TO-REAL
Method args: (x y)
Returns list of real coordinates, in the original coordinate system, of
canvas point (X, Y), based on current content variables.
NIL
> (send my-plot :help :CANVAS-TO-SCALED)
CANVAS-TO-SCALED
Method args: (x y)
Returns list of scaled coordinates, in the scaled coordinate system, of
canvas point (X, Y), based on current content variables.
NIL
> (send my-plot :help :CANVAS-WIDTH)
CANVAS-WIDTH
Method args: ()
Returns current canvas width.
NIL
> (send my-plot :help :CENTER)
CENTER
Method args: (var &optional shift &key (draw t))
Sets or retrieves current center for variable VAR. Sends :RESIZE and :REDRAW
messages if DRAW is true. Vectorized.
NIL
> (send my-plot :help :CHOOSE-MOUSE-MODE)
CHOOSE-MOUSE-MODE
Method args: ()
Presents a dialog to set the mouse mode.
NIL
> (send my-plot :help :CLEAR)
CLEAR
Message args: (&key (draw t))
Clears the plot data. If DRAW is nil the plot is redrawn; otherwise its
current screen image remains unchanged.
NIL
> (send my-plot :help :CLEAR-LINES)
CLEAR-LINES
Method args: (&key (draw t))
Removes all lines from the plot. If DRAW is true the :REDRAW-CONTENT
message is sent.
NIL
> (send my-plot :help :CLEAR-MASKS)
CLEAR-MASKS
Method args: ()
Unmasks all points, lines and strings.
NIL
> (send my-plot :help :CLEAR-POINTS)
CLEAR-POINTS
Method args: (&key (draw t))
Removes all points from the plot. If DRAW is true the :REDRAW-CONTENT
message is sent.
NIL
> (send my-plot :help :CLICK-RANGE)
CLICK-RANGE
Method args: (&optional width height)
Sets or retrieves the size of the current mouse click range.
NIL
> (send my-plot :help :CLIP-RECT)
CLIP-RECT
Method args: (&optional left top width height)
Sets or retrieves current clip rectangle. If NIL is supplied clipping is turned off.
A result of NIL means clipping is disabled.
NIL
> (send my-plot :help :CLOSE)
CLOSE
Method args: ()
Closes and deallocates the graph window.
NIL
> (send my-plot :help :CONTENT-ORIGIN)
CONTENT-ORIGIN
Method args: (&optional x y)
Sets or retrieves the current content origin.
NIL
> (send my-plot :help :CONTENT-RECT)
CONTENT-RECT
Method args: (&optional left top width height)
Sets or retrieves current content rectangle.
NIL
> (send my-plot :help :CONTENT-VARIABLES)
CONTENT-VARIABLES
Method args: (&optional xvar yvar)
Sets or retrieves the indices of the current content variables.
NIL
> (send my-plot :help :COPY-TO-CLIP)
COPY-TO-CLIP
Method args: ()
Sends the plot the :REDRAW message, surrounded by open and close picture commands.
Then puts the picture on the clip board.
NIL
> (send my-plot :help :CURRENT-VARIABLES)
CURRENT-VARIABLES
Method args: (&optional x y &key (draw t))
Sets or retrieves current variables.
NIL
> (send my-plot :help :CURSOR)
CURSOR
Method args: (&optional cursor)
Sets or retrieves the current window cursor. Choices are in the list *cursors*.
NIL
> (send my-plot :help :CUT-TO-CLIP)
CUT-TO-CLIP
Method args: ()
Does nothing.
NIL
> (send my-plot :help :DELETE-DOCUMENTATION)
DELETE-DOCUMENTATION
Method args: (topic)
Deletes object documentation for TOPIC.
NIL
> (send my-plot :help :DELETE-METHOD)
DELETE-METHOD
Method args: (selector)
Deletes method for SELECTOR in object if it exists.
NIL
> (send my-plot :help :DELETE-MOUSE-MODE)
DELETE-MOUSE-MODE
Method args: (mode)
Deletes MODE to the mode list.
NIL
> (send my-plot :help :DELETE-SLOT)
DELETE-SLOT
Method args: (slot)
Deletes slot SLOT from object if it exists.
NIL
> (send my-plot :help :DISPOSE)
DISPOSE
Method args: ()
Closes and deallocates the graph window.
NIL
> (send my-plot :help :DO-CLICK)
DO-CLICK
Method args: (x y extend option)
Sends appropriate action message for mouse mode to plot.
NIL
> (send my-plot :help :DO-IDLE)
DO-IDLE
Method args: ()
Message received from system in idle state when idling is on.
NIL
> (send my-plot :help :DO-KEY)
DO-KEY
Method args: (char shift option)
Message received when user hits a key.
NIL
> (send my-plot :help :DO-MOTION)
DO-MOTION
Method args: (x y)
Sends appropriate action message for mouse mode to plot.
NIL
> (send my-plot :help :DOC-TOPICS)
DOC-TOPICS
Method args: ()
Returns all topics with documentation for this object.
NIL
> (send my-plot :help :DOCUMENTATION)
DOCUMENTATION
Method args: (topic &optional val)
Retrieves or sets object documentation for topic.
NIL
> (send my-plot :help :DRAG-GREY-RECT)
DRAG-GREY-RECT
Method args: (x y width height)
Drags grey rectangle starting at (LIST (- X WIDTH) (- Y HEIGHT) WIDTH HEIGHT)
while mouse button is down. Returns the final rectangle. Should be called when
the mouse is down.
NIL
> (send my-plot :help :DRAW-BITMAP)
DRAW-BITMAP
Method args: (image left top)
Draws IMAGE, a matrix of 0's and 1's, with top left corner at (LEFT TOP)
NIL
> (send my-plot :help :DRAW-BRUSH)
DRAW-BRUSH
Method args: ()
Draws brush at its current position.
NIL
> (send my-plot :help :DRAW-COLOR)
DRAW-COLOR
Method args: (&optional color)
Sets or retrieves current drawing color. Choices are in the list *COLORS*.
NIL
> (send my-plot :help :DRAW-DATA-LINES)
DRAW-DATA-LINES
Method args: (var1 var2 m n)
Draws lines with indices between m (inclusive) and n (exclusive) using VAR1 and VAR2 
coordinates.
NIL
> (send my-plot :help :DRAW-DATA-POINTS)
DRAW-DATA-POINTS
Method args: (var1 var2 m n)
Draws points with indices between m (inclusive) and n (exclusive) using VAR1 and VAR2 
coordinates.
NIL
> (send my-plot :help :DRAW-LINE)
DRAW-LINE
Method args: (x1 y1 x2 y2)
Draws line from (x1 y1) to (x2 y2).
NIL
> (send my-plot :help :DRAW-MODE)
DRAW-MODE
Method args: (&optional mode)
Sets or returns current drawing mode. Choices are NORMAL and XOR.
NIL
> (send my-plot :help :DRAW-POINT)
DRAW-POINT
Method args: (x y)
Draws a single pixel at (X Y)
NIL
> (send my-plot :help :DRAW-STRING)
DRAW-STRING
Method args: (string x y)
Draws STRING horizontally starting at (x y).
NIL
> (send my-plot :help :DRAW-STRING-UP)
DRAW-STRING-UP
Method args: (string x y)
Draws STRING vertically starting at (x y).
NIL
> (send my-plot :help :DRAW-SYMBOL)
DRAW-SYMBOL
Method args: (symbol hilited x y)
Draws SYMBOL at (x y). Choices are in list *PLOT-SYMBOLS*
NIL
> (send my-plot :help :DRAW-TEXT)
DRAW-TEXT
Method args: (string x y h v)
Draws STRING horizontally positioned relative to (x y). H controls horizontal
justification. 0 = left justified, 1 = centered, 2 = right justified. V controls
vertical positioning. 0 = above (x y), 1 = below (x y).
NIL
> (send my-plot :help :DRAW-TEXT-UP)
DRAW-TEXT-UP
Method args: (string x y h v)
Same as :DRAW-TEXT but rotated by 90 degrees.
NIL
> (send my-plot :help :ERASE-ARC)
ERASE-ARC
Method args: (left top width height start delta)
Fills the arc in (LEFT TOP WIDTH HEIGHT) from START by DELTA with the
background color. Angles are in degrees.
NIL
> (send my-plot :help :ERASE-BRUSH)
ERASE-BRUSH
Method args: ()
Removes brush from the screen.
NIL
> (send my-plot :help :ERASE-OVAL)
ERASE-OVAL
Method args: (left top width height)
Fills the oval in (LEFT TOP WIDTH HEIGHT) with the background color.
NIL
> (send my-plot :help :ERASE-POLY)
ERASE-POLY
Method args: (poly &optional (from-origin t))
Erases content of polygon. POLY is a list of lists of two numbers. If
FROM-ORIGIN is true coordinates are relative to the origin; otherwise
they are relative to the previous point.
NIL
> (send my-plot :help :ERASE-RECT)
ERASE-RECT
Method args: (left top width height)
Frames the rectangle (LEFT TOP WIDTH HEIGHT).
NIL
> (send my-plot :help :ERASE-SELECTION)
ERASE-SELECTION
Method args: ()
Sets selected points states to invisible and sends :ADJUST-SCREEN message.
NIL
> (send my-plot :help :ERASE-WINDOW)
ERASE-WINDOW
Method args: ()
Erases the entire window canvas.
NIL
> (send my-plot :help :FIND)
FIND
Method args: ()
Does nothing.
NIL
> (send my-plot :help :FIXED-ASPECT)
FIXED-ASPECT
Method args: (&optional fixed)
Sets or retrieves current size adjustment option (true or NIL).
NIL
> (send my-plot :help :FOCUS-ON-SELECTION)
FOCUS-ON-SELECTION
Message args: ()
removes all unselected points invisible
NIL
> (send my-plot :help :FRAME-ARC)
FRAME-ARC
Method args: (left top width height start delta)
Frames the arc in (LEFT TOP WIDTH HEIGHT) from START by DELTA. Angles are in
degrees.
NIL
> (send my-plot :help :FRAME-LOCATION)
FRAME-LOCATION
Method args: (&optional left top)
Moves window frame to (LEFT TOP) if supplied. Returns list of 
current left, top. Adjusts for the menu bar.
NIL
> (send my-plot :help :FRAME-OVAL)
FRAME-OVAL
Method args: (left top width height)
Frames the oval in (LEFT TOP WIDTH HEIGHT).
NIL
> (send my-plot :help :FRAME-POLY)
FRAME-POLY
Method args: (poly &optional (from-origin t))
Outlines polygon. POLY is a list of lists of two numbers. If FROM-ORIGIN is 
true coordinates are relative to the origin; otherwise they are relative to
the previous point.
NIL
> (send my-plot :help :FRAME-RECT)
FRAME-RECT
Method args: (left top width height)
Fills the rectangle (LEFT TOP WIDTH HEIGHT) with the background color.
NIL
> (send my-plot :help :FRAME-SIZE)
FRAME-SIZE
Method args: (&optional width height)
Sets window frame width and size to WIDTH and SIZE if supplied. 
Returns list of current WIDTH and HEIGHT. Adjusts for the menu bar.
NIL
> (send my-plot :help :GET-METHOD)
GET-METHOD
Method args: (selector)
Returns method for SELECTOR symbol from object's precedence list.
NIL
> (send my-plot :help :H-SCROLL-INCS)
H-SCROLL-INCS
Method args: (inc page-inc)
Sets or retrieves the increments scrolled by the buttons and page areas
of the horizontal scroll bar.
NIL
> (send my-plot :help :HAS-H-SCROLL)
HAS-H-SCROLL
Method args: (&optional x)
Determines or sets whether window has a horizontal scrollbar. If X is 
supplied and is NIL the scroll bar is removed. If X is a number a scroll
bar is added and the canvas width is set to the number (between 0 and 3200).
If X is T a scroll bar is added and the canvas width is set to the maximum
of the screen's width and height.
NIL
> (send my-plot :help :HAS-METHOD)
HAS-METHOD
Method args: (selector &optional own)
Returns T if method for SELECTOR exists, NIL if not. If OWN is not NIL
only checks the object; otherwise check the entire precedence list.
NIL
> (send my-plot :help :HAS-SLOT)
HAS-SLOT
Method args: (slot &optional own)
Returns T if slot SLOT exists, NIL if not. If OWN is not NIL
only checks the object; otherwise check the entire precedence list.
NIL
> (send my-plot :help :HAS-V-SCROLL)
HAS-V-SCROLL
Method args: (&optional x)
Determines or sets whether window has a vertical scrollbar. If X is 
supplied and is NIL the scroll bar is removed. If X is a number a scroll
bar is added and the canvas height is set to the number (between 0 and 3200).
If X is T a scroll bar is added and the canvas height is set to the maximum
of the screen's width and height.
NIL
> (send my-plot :help :HELP)
HELP
Method args: (&optional topic)
Prints help message for TOPIC, or general help if TOPIC is NIL.
NIL
> (send my-plot :help :HIDE-WINDOW)
HIDE-WINDOW
Method args: ()
Hides the window without deallocating it. Returns NIL.
NIL
> (send my-plot :help :IDLE-ON)
IDLE-ON

Method args: (&optional on)
Sets or returns idling state. On means :do-idle method is sent each pass through
the event loop.
NIL
> (send my-plot :help :INTERNAL-DOC)
INTERNAL-DOC
Method args (topic &optional value)
Retrieves or installs documentation for topic.
NIL
> (send my-plot :help :ISNEW)
ISNEW
 Method args: (vars &key (title "Graph") location size
                          (go-away t) menu (black-on-white t)
                          has-h-scroll has-v-scroll menu-title
                          variable-labels scale)
Initializes and send :allocate message to basic plot window.
NIL
> (send my-plot :help :LINE-TYPE)
LINE-TYPE
Method args: (&optional type)
Sets or returns current line type. Choices are SOLID and DASHED.
NIL
> (send my-plot :help :LINE-WIDTH)
LINE-WIDTH
Method args: (&optional width)
Sets or retrieves current line width.
NIL
> (send my-plot :help :LINESTART-COLOR)
LINESTART-COLOR
Method args: (line &optional color)
Sets or retrieves color of line start LINE. Vectorized.
NIL
> (send my-plot :help :LINESTART-COORDINATE)
LINESTART-COORDINATE
Method args: (var line &optional value)
Sets or retrieves coordinate for variable VAR and line start LINE in the original
coordinate system. Vectorized.
NIL
> (send my-plot :help :LINESTART-MASKED)
LINESTART-MASKED
Method args: (line &optional masked)
Sets or retrieves masked state (true or NIL) of line start LINE. Vectorized.
NIL
> (send my-plot :help :LINESTART-NEXT)
LINESTART-NEXT
Method args: (line &optional next)
Sets or returns index of line start to which line is to be drawn. Negative values
mean no line. Vectorized.
NIL
> (send my-plot :help :LINESTART-SCREEN-COORDINATE)
LINESTART-SCREEN-COORDINATE
Method args: (var line)
Returns rounded coordinate for variable VAR and line start LINE in the screen
coordinate system. Vectorized.
NIL
> (send my-plot :help :LINESTART-TRANSFORMED-COORDINATE)
LINESTART-TRANSFORMED-COORDINATE
Method args: (var line)
Returns coordinate for variable VAR and line start LINE in the transformed
coordinate system. Vectorized.
NIL
> (send my-plot :help :LINESTART-TYPE)
LINESTART-TYPE
Method args: (line &optional type)
Sets or retrieves line type. Vectorized.
NIL
> (send my-plot :help :LINESTART-WIDTH)
LINESTART-WIDTH
Method args: (line &optional width)
Sets or retrieves the line width for the line start. Vectorized.
NIL
> (send my-plot :help :LINKED)
LINKED
Method ars: (&optional on)
Sets or retrieves plot's linking state.
NIL
> (send my-plot :help :LOCATION)
LOCATION
Method args: (&optional left top)
Moves window content to (LEFT TOP) if supplied. Returns list of 
current left, top. Adjusts for the menu bar.
NIL
> (send my-plot :help :MARGIN)
MARGIN
Method args: (&optional left top right bottom &key (draw t))
Sets or retrieves current list of margin sizes.
NIL
> (send my-plot :help :MASK-SELECTION)
MASK-SELECTION
Method args: ()
Masks selected points and sends :ADJUST-SCREEN message 
NIL
> (send my-plot :help :MENU)
MENU
Method args: (&optional menu)
Sets or retrieves window's menu.
NIL
> (send my-plot :help :METHOD-SELECTORS)
METHOD-SELECTORS
Method args: ()
Returns list of method selectors available to object.
NIL
> (send my-plot :help :MOUSE-MODE)
MOUSE-MODE
Method args (&optional mode)
Sets or retrieves current mouse mode.
NIL
> (send my-plot :help :MOUSE-MODE-TITLE)
MOUSE-MODE-TITLE
Method args: (mode)
Returns title for menu item for mouse mode MODE.
NIL
> (send my-plot :help :MOUSE-MODES)
MOUSE-MODES
Method args: ()
Returns a list of the current mouse modes.
NIL
> (send my-plot :help :MOVE-BRUSH)
MOVE-BRUSH
Method args: (x y)
Moves the brush's lower left hand corner to (X, Y).
NIL
> (send my-plot :help :NEW)
NEW
Method args: (&rest args)
Creates new object using self as prototype.
NIL
> (send my-plot :help :NUM-LINES)
NUM-LINES
Method args: ()
Returns the number of line starts in the plot.
NIL
> (send my-plot :help :NUM-POINTS)
NUM-POINTS
Method args: ()
Returns the number of points in the plot.
NIL
> (send my-plot :help :NUM-VARIABLES)
NUM-VARIABLES
Method args: ()
Returns the number of variables in the plot.
NIL
> (send my-plot :help :OWN-METHODS)
OWN-METHODS
Method args ()
Returns copy of selectors for methods owned by object.
NIL
> (send my-plot :help :OWN-SLOTS)
OWN-SLOTS
Method args: ()
Returns list of names of slots owned by object.
NIL
> (send my-plot :help :PAINT-ARC)
PAINT-ARC
Method args: (left top width height start delta)
Fills the oval in (LEFT TOP WIDTH HEIGHT) from START by DELTA with the
drawing color. Angles are in degrees.
NIL
> (send my-plot :help :PAINT-OVAL)
PAINT-OVAL
Method args: (left top width height)
Fills the oval in (LEFT TOP WIDTH HEIGHT) with the drawing color.
NIL
> (send my-plot :help :PAINT-POLY)
PAINT-POLY
Method args: (poly &optional (from-origin t))
Paints content of polygon in current draw color. POLY is a list of lists of two
numbers. If FROM-ORIGIN is true coordinates are relative to the origin; 
otherwise they are relative to the previous point.
NIL
> (send my-plot :help :PAINT-RECT)
PAINT-RECT
Method args: (left top width height)
Fills the rectangle (LEFT TOP WIDTH HEIGHT) with the drawing colorod args: ()
Does nothing.
NIL
> (send my-plot :help :POINT-COLOR)
POINT-COLOR
Method args: (point &optional color)
Sets or retrieves color of point POINT. Vectorized.
NIL
> (send my-plot :help :POINT-COORDINATE)
POINT-COORDINATE
Method args: (var point &optional value)
Sets or retrieves coordinate for variable VAR and point POINT in the original
coordinate system. Vectorized.
NIL
> (send my-plot :help :POINT-HILITED)
POINT-HILITED
Method args: (point &optional hilited)
Sets or returns highlighting status (true or NIL) of POINT. Sends 
:ADJUST-SCREEN message if states are set. Vectorized.
NIL
> (send my-plot :help :POINT-LABEL)
POINT-LABEL
Method args: (point &optional label)
Sets or retrieves label of point POINT. Vectorized.
NIL
> (send my-plot :help :POINT-MASKED)
POINT-MASKED
Method args: (point &optional masked)
Sets or rue or NIL) of POINT. Sends 
:ADJUST-SCREEN message if states are set. Vectorized.
NIL
> (send my-plot :help :POINT-LABEL)
POINT-LABEL
Method args: (point &optional label)
Sets or retrieves label of point POINT. Vectorized.
NIL
> (send my-plot :help :POINT-MASKED)
POINT-MASKED
Method args: (point &optional masked)
Sets or retrieves masked state (true or NIL) of point POINT. Vectorized.
NIL
> (send my-plot :help :POINT-SCREEN-COORDINATE)
POINT-SCREEN-COORDINATE
Method args: (var point)
Returns rounded coordinate for variable VAR and point POINT in the screen
coordinate system. Vectorized.
NIL
> (send my-plot :help :POINT-SCREEN-STATE)
POINT-SCREEN-STATE
Method args: (point &optional state)
Sets or retrieves internal screen state of point POINT. Does no
drawing. Vectorized.
NIL
> (send my-plot :help :POINT-SELECTED)
POINT-SELECTED
Method args: (point &optional selected)
Sets or returns selection status (true or NIL) of POINT. Sends 
:ADJUST-SCREEN message if states are set. Vectorized.
NIL
> (send my-plot :help :POINT-SHOWING)
POINT-SHOWING
Method args: (point &optional selected)
Sets or returns visibility status (true or NIL) of POINT. Sends 
:ADJUST-SCREEN message if states are set. Vectorized.
NIL
> (send my-plot :help :POINT-STATE)
POINT-STATE
Method args: (point &optional state)
Sets or retrieves state (invisible, normal, hilited or selected) of point
POINT. Vectorized.
NIL
> (send my-plot :help :POINT-SYMBOL)
POINT-SYMBOL
Method args: (point &optional symbol)
Sets or retrieves symbol of point POINT. Vectorized.
NIL
> (send my-plot :help :POINT-TRANSFORMED-COORDINATE)
POINT-TRANSFORMED-COORDINATE
Method args: (var point)
Returns coordinate for variable VAR and point POINT in the transformed
coordinate system. Vectorized.
NIL
> (send my-plot :help :POINTS-HILITED)
POINTS-HILITED
Method args: (&optional list)
Sets or Return indices of currently highlighted points.
NIL
> (send my-plot :help :POINTS-IN-RECT)
POINTS-IN-RECT

Method args: (left top width height)
Returns list of indices of points in screen rectangle.
NIL
> (send my-plot :help :POINTS-SELECTED)
POINTS-SELECTED
Method args: (&optional list)
Sets or Return indices of current selection.
NIL
> (send my-plot :help :POINTS-SHOWING)
POINTS-SHOWING
Method args: (&optional list)
Sets or Return indices of currently visible points.
NIL
> (send my-plot :help :PRECEDENCE-LIST)
PRECEDENCE-LIST
Method args: ()
Returns copy of the precedence list.
NIL
> (send my-plot :help :PRINT)
PRINT
Method args: (&optional (stream *standard-output*))
Default object printing method.
NIL
> (send my-plot :help :PROTO)
Sorry, no help available on PROTO
NIL
> (send my-plot :help :RANGE)
RANGE
Method args: (index &optional low high)
Sets or retrieves variable's original coordinate range. Vectorized.
NIL
> (send my-plot :help :REAL-TO-CANVAS)
REAL-TO-CANVAS
Method args: (x y)
Returns list of canvas coordinates of point (X, Y), in the original coordinate system,
based on current content variables.
NIL
> (send my-plot :help :REDRAW)
REDRAW
Method args: ()
Redraws entire plot.
NIL
> (send my-plot :help :REDRAW-CONTENT)
REDRAW-CONTENT
Method args: ()
Redraws plot's content.
NIL
> (send my-plot :help :REMOVE)
REMOVE
Method args: ()
Closes and deallocates the graph window.
NIL
> (send my-plot :help :REPARENT)
REPARENT
Method args: (&rest parents)
Changes precedence list to correspond to PARENTS. Does not change descendants.
NIL
> (send my-plot :help :REPLACE-SYMBOL)
REPLACE-SYMBOL
Method args: (oldsym oldh newsym newh x y)
Replaces OLDSYM at (x y) by NEWSYM. Hilighting states are set by OLDH and NEWH.
Available symbols are in the list *PLOT-SYMBOLS*
NIL
> (send my-plot :help :RESIZE)
RESIZE

Method args: ()
Adjusts internal layout after resizing.
NIL
> (send my-plot :help :RESIZE-BRUSH)
RESIZE-BRUSH
Method args: ()
Opens the brush resizing dialog and resets the brush if the OK button is clicked.
NIL
> (send my-plot :help :RETYPE)
RETYPE
Method args: (proto &rest args)
Changes object to inherit directly from prototype PROTO. PROTO
must be a prototype and SELF must not be one.
NIL
> (send my-plot :help :REVERSE-COLORS)
REVERSE-COLORS
Method args: ()
Reverses drawing and background colors and sends self the :REDRAW message.
NIL
> (send my-plot :help :ROTATE-2)
ROTATE-2
Method args: (var1 var2 angle &key (draw t))
Rotates int the plane of variables with indices VAR1 and VAR2 by ANGLE, in
radians. sends the :REDRAW-CONTENT message if DRWA is true.
NIL
> (send my-plot :help :SCALE)
SCALE
Method args: (var &optional scale &key (draw t))
Sets or retrieves current scale for variable VAR. Sends :RESIZE and :REDRAW
messages if DRAW is true.
NIL
> (send my-plot :help :SCALE-TO-RANGE)
SCALE-TO-RANGE
Method args: (var low high &key (draw t))
Scales and shifts data to map visible range into specified range. Sends
:RESIZE and :REDRAW messages if DRAW is true.
NIL
> (send my-plot :help :SCALE-TYPE)
SCALE-TYPE
Method args: (&optional new)
Sets or returns scale type.
NIL
> (send my-plot :help :SCALED-RANGE)
SCALED-RANGE
Method args: (index &optional low high)
Sets or retrieves variable's transformed coordinate range. Vectorized.
NIL
> (send my-plot :help :SCALED-TO-CANVAS)
SCALED-TO-CANVAS
Method args: (x y)
Returns list of canvas coordinates of point (X, Y), in the scaled coordinate system,
based on current content variables.
NIL
> (send my-plot :help :SCREEN-RANGE)
SCREEN-RANGE
Method args: (index &optional low high)
Sets or retrieves variable's screen coordinate range. Vectorized.
NIL
> (send my-plot :help :SCROLL)
SCROLL
Method args: (x y)
Sets or returns current position of left top corner of view rectangle.
X or Y are ignored if no horizontal or vertical scroll bar is present.
NIL
> (send my-plot :help :SELECTION)
SELECTION
Method args: (&optional list)
Sets or Return indices of current selection.
NIL
> (send my-plot :help :SELECTION-DIALOG)
SELECTION-DIALOG
Method args: ()
Open dialog to save or set selection.
NIL
> (send my-plot :help :SELECTION-STREAM)
SELECTION-STREAM
Method args: ()
Does nothing.
NIL
> (send my-plot :help :SET-MODE-CURSOR)
SET-MODE-CURSOR
Method args: ()
Installs appropriate cursor for current mode.
NIL
> (send my-plot :help :SET-OPTIONS)
SET-OPTIONS
Method args: ()
Opens dialog to set plot options. Items are obtained using the
:make-options-dialog-items message.
NIL
> (send my-plot :help :SET-SELECTION-COLOR)
SET-SELECTION-COLOR
Method args: ()
Open dialog to set color used to plot selected points.
NIL
> (send my-plot :help :SET-SELECTION-SYMBOL)
SET-SELECTION-SYMBOL
Method args: ()
Open dialog to set symbol used to plot selected points.
NIL
> (send my-plot :help :SHIFT)
SHIFT
Method args: (var &optional shift &key (draw t))
Sets or retrieves current shift for variable VAR. Sends :RESIZE and :REDRAW
messages if DRAW is true.
NIL
> (send my-plot :help :SHOW)
SHOW
Method Args: ()
Prints object's internal data.
NIL
> (send my-plot :help :SHOW-ALL-POINTS)
SHOW-ALL-POINTS
Method args: ()
Sets all point states to normal and sends :ADJUST-SCREEN message 
NIL
> (send my-plot :help :SHOW-WINDOW)
SHOW-WINDOW
Method args: ()
Makes window visible and moves it to the front. Returns NIL.
NIL
> (send my-plot :help :SHOWING-LABELS)
SHOWING-LABELS
Method args: (&optional showing)
Sets or retrieves current labeling state (true or NIL).
NIL
> (send my-plot :help :SIZE)
SIZE
Method args: (&optional width height)
Sets window content width and size to WIDTH and SIZE if supplied.
Returns list of current WIDTH HEIGHT. Adjusts for the menu bar.
NIL
> (send my-plot :help :SLICE-VARIABLE)
SLICE-VARIABLE
Method args: (var low high)
Masks points lines and strings with variable VAR original coordinates outside
of (LOW, HIGH).
NIL
> (send my-plot :help :SLOT-NAMES)
SLOT-NAMES
Method args: ()
Returns list of slots available to the object.
NIL
> (send my-plot :help :SLOT-VALUE)
SLOT-VALUE
Method args: (sym &optional val)
Sets and retrieves value of slot named SYM. Sugnals an error if slot
does not exist.
NIL
> (send my-plot :help :START-BUFFERING)
START-BUFFERING
Method args: ()
Starts sending drawing result so buffer or increases buffering level if already
buffering.
NIL
> (send my-plot :help :TEXT-ASCENT)
TEXT-ASCENT
Method args: ()
Returns ascent in pixels for the window's text font.
NIL
> (send my-plot :help :TEXT-DESCENT)
TEXT-DESCENT
Method args: ()
Returns descent in pixels for the window's text font.
NIL
> (send my-plot :help :TEXT-WIDTH)
TEXT-WIDTH
Method args: (string)
Returns widh in pixels of STRING in the window's font.
NIL
> (send my-plot :help :TITLE)
TITLE
Method args: (&optional string)
Sets or retrieves window title.
NIL
> (send my-plot :help :TRANSFORMATION)
TRANSFORMATION
Method args: (&optional a &key (draw t))
Sets or retrieves transformation. A should be a matrix or NIL. If draw is true
the :REDRAW-CONTENT message is sent.
NIL
> (send my-plot :help :UNDO)
UNDO
Method args: ()
Does nothing.
NIL
> (send my-plot :help :UNMASK-ALL-POINTS)
UNMASK-ALL-POINTS
Method args: ()
Unmasks all points and sends :ADJUST-SCREEN message 
NIL
> (send my-plot :help :UNSELECT-ALL-POINTS)
UNSELECT-ALL-POINTS
Method args: ()
Unselects all points.
NIL
> (send my-plot :help :UNSHOW-ALL-POINTS)
UNSHOW-ALL-POINTS
Method args: ()
Makes all points invisible.
NIL
> (send my-plot :help :UPDATE)
UPDATE
Method args: (resized)
Sends self the :RESIZED method if RESIZED is true, redraws the frame and
controls, and sends self the :REDRAW message.
NIL
> (send my-plot :help :USE-COLOR)
USE-COLOR
Method args: (&optional use)
Sets or retrieves current color use state. Has no effect on B/W systems.
NIL
> (send my-plot :help :V-SCROLL-INCS)
V-SCROLL-INCS
Method args: (inc page-inc)
Sets or retrieves the increments scrolled by the buttons and page areas
of the vertical scroll bar.
NIL
> (send my-plot :help :VARIABLE-LABEL)
VARIABLE-LABEL
Method args: (var &optional label)
Sets or returns label for variable with index VAR. Vectorized.
NIL
> (send my-plot :help :VIEW-RECT)
VIEW-RECT
Method args: ()
Returns the current view rectangle as list of the form
(LEFT TOP WIDTH HEIGHT)
NIL
> (send my-plot :help :VISIBLE-RANGE)
VISIBLE-RANGE
Method args: (var)
Returns list of min and max of variable VAR over visible, unmasked points,
lines and strings. Vectorized.
NIL
> (send my-plot :help :WHILE-BUTTON-DOWN)
WHILE-BUTTON-DOWN
Method args: (fcn &optional (motion-only t))
Calls fcn repeatedly while mouse button is down. FCN should take two arguments,
the current x and y coordinates of the mouse. Returns NIL. Should be called
when button is already down.
NIL
> (send my-plot :help :X-AXIS)
X-AXIS
Method args: (&optional showing labeled ticks)
Sets or retrieves current acis label state. SHOWING and LABELED should be
true or NIL; TICKS should be a number. All three should be supplied for setting
a new state. A list of the three properties is returned.
NIL
> (send my-plot :help :Y-AXIS)
Y-AXIS
Method args: (&optional showing labeled ticks)
Sets or retrieves current acis label state. SHOWING and LABELED should be
true or NIL; TICKS should be a number. All three should be supplied for setting
a new state. A list of the three properties is returned.
NIL
|#



#|

	This from the ViSta package: I liked it lots!

;; set up local environment with function to project (x, y) point onto
;; "globe" over the plot
(flet ((calcsphere (x y)
		   (let* ((norm-2 (+ (* x x) (* y y)))
			  (rad-2 (^ 1.7 2))
			  (z (if (< norm-2 rad-2) (sqrt (- rad-2 norm-2)) 0)))
		     (if (< norm-2 rad-2) 
			 (list x y z)
		       (let ((r (sqrt (max norm-2 rad-2))))
			 (list (/ x r) (/ y r) (/ z r)))))))
      
      ;; define the :DO-HAND-ROTATE method in the local environment
      (defmeth spin-proto :do-hand-rotate (x y m1 m2)
	(let* ((oldp (apply #'calcsphere 
			    (send self :canvas-to-scaled x y)))
	       (p oldp)
	       (vars (send self :content-variables))
	       (trans (identity-matrix (send self :num-variables))))
	  (send self :idle-on nil)
	  (send self :while-button-down 
		#'(lambda (x y) 
		    (setf oldp p)
		    (setf p (apply #'calcsphere 
				   (send self :canvas-to-scaled x y)))
		    (setf (select trans vars vars) (make-rotation oldp p))
		    (when m1 
			  (send self :slot-value 'rotation-type trans)
			  (send self :idle-on t))
		    (send self
			  :apply-transformation
			  trans)))))
      )

|#
;; add a new "mouse mode", with menu title, cursor and mouse method name
(send spin-proto :add-mouse-mode 'hand-rotate
      :title "Hand Rotate" :cursor 'hand :click :do-hand-rotate)

;; set up local environment with function to project (x, y) point onto
;; "globe" over the plot
(flet ((calcsphere (x y)
		   (let* ((norm-2 (+ (* x x) (* y y)))
			  (rad-2 (^ 1.7 2))
			  (z (if (< norm-2 rad-2) (sqrt (- rad-2 norm-2)) 0)))
		     (if (< norm-2 rad-2) 
			 (list x y z)
		       (let ((r (sqrt (max norm-2 rad-2))))
			 (list (/ x r) (/ y r) (/ z r)))))))
      
      ;; define the :DO-HAND-ROTATE method in the local environment
      (defmeth spin-proto :do-hand-rotate (x y m1 m2)
	(let* ((oldp (apply #'calcsphere 
			    (send self :canvas-to-scaled x y)))
	       (p oldp)
	       (vars (send self :content-variables))
	       (trans (identity-matrix (send self :num-variables))))
	  (send self :idle-on nil)
	  (send self :while-button-down 
		#'(lambda (x y) 
		    (setf oldp p)
		    (setf p (apply #'calcsphere 
				   (send self :canvas-to-scaled x y)))
		    (setf (select trans vars vars) (make-rotation oldp p))
		    (when m1 
			  (send self :slot-value 'rotation-type trans)
			  (send self :idle-on t))
		    ;; (print (solve (send self :transformation) '(0 0 1)))
		    (send self
			  :apply-transformation
			  trans)
		    ;; ael
		    ))
	  (if (send self :linked)
	      (dolist (i (agw))
		      (if (and
			   (neq i self)
			   (send i :linked)
			   )
			  (let ()
			    (send i :transformation (send self :transformation))
			    (send i :show-window)
			    )
			)
		      )
	    )
	  (send self :show-window)
	  ;; ael
	  )
	)
      )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                           true-axes
;;
;; Here's a method to plot the true axes in a 3-d plot:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :true-axes ()
  "
This is a method to plot the true axes in a 2-d plot.

;; Examples:
 (send my-plot :true-axes)
"
  (let (
	(xs (send self :range 0))
        (ys (send self :range 1))
	(zs (send self :range 2))
	)
    (send self :add-lines
	  (list
	   (list (half (abs (- (second xs) (first xs)))) 0 0 0 0)
	   (list 0 0 (half (abs (- (second ys) (first ys)))) 0 0)
	   (list 0 0 0 0 (half (abs (- (second zs) (first zs)))))
	   )
	  )
    (send self :add-points
	  (list
	   (list 0 (half (abs (- (second xs) (first xs)))) 0 0)
	   (list 0 0 (half (abs (- (second ys) (first ys)))) 0)
	   (list 0 0 0 (half (abs (- (second zs) (first zs)))))
	   )
	  :point-labels *true-axes-labels*
	  )
    (send self :showing-labels t)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                           true-axes
;;
;; Here's a method to plot the real axes in a 3-d plot:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :real-axes ()
  "
This is a method to plot the real axes in a 2-d plot.
;; Examples:
 (send my-plot :real-axes)
"
  (send self :true-axes)
  (send self
	:selection
	(iseq
	 (- (send self :num-points) 4) (- (send self :num-points) 1))
	)	   
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Udina thefts
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth scatterplot-proto :do-show-coordinates (x y m1 m2)
  (let* ((xy (cond (m1 (list x y))
                   (m2 (send self :canvas-to-scaled x y))
                   (t (send self :canvas-to-real x y))
                   ))
	 (s (format nil "~s" xy))
	 (str-size (send self :text-width s))
	 (left (> (+ x str-size) (send self :canvas-width)))
	 (horz (if left 2 0))
	 (vert 0)
	 (mode (send self :draw-mode)))
    (send self :draw-mode 'xor)
    (send self :draw-text s x y horz vert)
    (send self :while-button-down #'(lambda (x y) nil))
    (send self :draw-text s x y horz vert)
    (send self :draw-mode mode)))

(send scatterplot-proto :add-mouse-mode 'show-coordinates 
      :title "Show Coordinates" :cursor 'finger 
      :click :do-show-coordinates)
(send scatterplot-proto :add-mouse-mode 'zoom-in 
      :title "Zoom in" :cursor 'cross 
      :click :do-zoom-in)

(defmeth scatterplot-proto :draw-string-while-button (s x y h v)
  (let* ((oldx x)
         (oldy y)
         (origin (first (transpose (send self :scaled-range '(0 1)))))
         (origin (apply #'send self :scaled-to-canvas origin))
         (mode (send self :draw-mode)))
    (send self :draw-mode 'xor)
    (send self :draw-line x y (first origin) y)
    (send self :draw-line x y x (second origin))
    (send self :draw-text s x y h v)
    (send self :while-button-down
          #'(lambda (nx ny)
              (send self :draw-text s oldx oldy h v)
              (send self :draw-text s nx ny h v)
              (setq oldx nx oldy ny)))
    ;redraw things for erasing
    (send self :draw-text s oldx oldy h v)
    (send self :draw-line x y (first origin) y)
    (send self :draw-line x y x (second origin))
    (send self :draw-mode mode)))


(defmeth scatterplot-proto :frame-rect (l2 t2 w2 h2)
  (when (< w2 0) (setq w2 (- w2)) (setq l2 (- l2 w2)))
  (when (< h2 0) (setq h2 (- h2)) (setq t2 (- t2 h2)))
  (call-next-method l2 t2 w2 h2))

(defmeth scatterplot-proto :do-zoom-in (x y m1 m2)
  (if (or m1 m2)
      (send self :adjust-to-data)
    (let* ((oldx x)
	   (oldy y)
	   (old-draw-mode (send self :draw-mode))
	   (rect (progn (send self 
			      :while-button-down
			      #'(lambda (nx ny)
				  (send self :draw-mode 'xor)
				  (send self :frame-rect x y
					(- oldx x) (- oldy y))
				  (send self :frame-rect x y
					(- nx x) (- ny y))
				  (setf oldx nx)
				  (setf oldy ny)))
			(list x y (- oldx x) (- oldy y))))
	   (l (nth 0 rect))
	   (to (nth 1 rect))
	   (w (nth 2 rect))
	   (h (nth 3 rect))
	   (lb (send self :canvas-to-scaled l (+ to h)))
	   (tr (send self :canvas-to-scaled (+ l w) to))
	   (xmin (nth 0 lb))
	   (xmax (nth 0 tr))
	   (ymin (nth 1 lb))
	   (ymax (nth 1 tr))
	   )
      (if (= 0 (* w h))
	  (let* (
		 (current-domain (send self :scaled-range 0))
		 (current-range (send self :scaled-range 1))
		 (xmin (nth 0 current-domain))
		 (xmax (nth 1 current-domain))
		 (ymin (nth 0 current-range))
		 (ymax (nth 1 current-range))
		 (xcenter (mean (list xmax xmin)))
		 (ycenter (mean (list ymax ymin)))
		 (xdiff (/ (* *zoom-out-factor* (- xmax xmin)) 2))
		 (ydiff (/ (* *zoom-out-factor* (- ymax ymin)) 2))
		 (xmin (- xcenter xdiff))
		 (xmax (+ xcenter xdiff))
		 (ymin (- ycenter ydiff))
		 (ymax (+ ycenter ydiff))
		 )
	    ;; it would be better to do this using the :scale option (see
	    ;; vista.lsp)
	    (send self :set-ranges
		  (min xmin xmax) (max xmin xmax)
		  (min ymin ymax) (max ymin ymax)
		  )
	    (send self :draw-mode old-draw-mode)
	    )
	(list
	 (send self :frame-rect l to w h)
	 (send self :draw-mode old-draw-mode)
	 (send self :set-ranges
	       (min xmin xmax) (max xmin xmax)
	       (min ymin ymax) (max ymin ymax)
	       )
	 )
	)
      )
    )
  (send self :redraw)
  (send self :plot-item)
  )

(defmeth scatterplot-proto :set-ranges (xmin xmax ymin ymax &key (recalc nil))
  (send self :scaled-range 0 xmin xmax :draw nil)
  (send self :scaled-range 1 ymin ymax :draw nil)
  (when recalc ()) ;this is to be done...................
  (unless (featurep :macintosh)
	  (send self :redraw))
  (send self :update nil))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		Use do-key to set the domain and range on the fly
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth scatterplot-proto :change-domain ()
  "
Changes the domain, using the get-value-dialog.
"
  (let* (
         (location (send self :location))
	 (domain (get-values-dialog "Domain (e.g. -1 4)" :location location))
	 )
    (if (= 2 (length domain))
	(let()
	 (send self :range 0 (first domain) (second domain))
	 (send self :plot-item)
	 )
      )
    )
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                change-range
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth scatterplot-proto :change-range ()
  "
Changes the range, using the get-value-dialog.
"
  (let* (
         (location (send self :location))
	 (range (get-values-dialog "Range (e.g. -1 4)" :location location))
	 )
    (if (= (length range) 2)
	(let()
	 (send self :range 1 (first range) (second range))
	 (send self :plot-item)
	 )
      )
    )
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        change-domain-and-range
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
(defmeth scatterplot-proto :change-domain-and-range ()
  "
Changes the domain and range, using the get-value-dialog.
"
  (let* (
	 (location (send self :location))
	 (self.domain (send self :range 0))
	 (self.range  (send self :range 1))
	 
	 (xmin-prompt (send text-item-proto :new "xmin:"))
	 (xmin (send edit-text-item-proto
		     :new (num-to-string (first self.domain))
		     :text-length 8))
	 (xmax-prompt (send text-item-proto :new "xmax:"))
	 (xmax (send edit-text-item-proto
		     :new (num-to-string (second self.domain))
		     :text-length 8))
	 (ymin-prompt (send text-item-proto :new "ymin:"))
	 (ymin (send edit-text-item-proto
		     :new (num-to-string (first self.range))
		     :text-length 8))
	 (ymax-prompt (send text-item-proto :new "ymax:"))
	 (ymax (send edit-text-item-proto
		     :new (num-to-string (second self.range))
		     :text-length 8))
	 cancel ok items
	 )
    (defun collect-values ()
      (list
       (send xmin :text)
       (send xmax :text)
       (send ymin :text)
       (send ymax :text)
       )
      )
    (setf cancel (send modal-button-proto :new "Cancel"))
    (setf ok (send modal-button-proto :new "OK"
		   :action #'collect-values))

    (setf items 
	  (send modal-dialog-proto :new
		(list
		 (list xmin-prompt xmin)
		 (list xmax-prompt xmax)
		 (list ymin-prompt ymin)
		 (list ymax-prompt ymax)
		 (list ok cancel)
		 )
		:default-button ok
		:location location
		)
	  )
    (setf items (send items :modal-dialog))
    (if items
	(list
	 (send self :range 0
	       (eval (read (make-string-input-stream (first items))))
	       (eval (read (make-string-input-stream (second items))))
	       )
	 (send self :range 1
	       (eval (read (make-string-input-stream (third items))))
	       (eval (read (make-string-input-stream (fourth items))))
	       )
	 )
      )
    )
  )

|#
(defmeth scatterplot-proto :change-domain-and-range ()
  "
Changes the domain and range, using the :change-domain and :change-range
methods.
"
  (let* (
         (location (send self :location))
	 (domain (get-values-dialog
		  "Domain and Range (e.g. -1 4 -2 3):"
		  :location location))
	 )
    (if (= 4 (length domain))
	(let()
	 (send self :range 0 (first domain) (second domain))
	 (send self :range 1 (third domain) (fourth domain))
	 (send self :plot-item)
	 )
      )
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                 scatterplot do-key, and some methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth scatterplot-proto :do-key (c m1 m2)
"
  The following keys are currently defined: typing them while a scatterplot is
active will give the following results:

                        h - help (this message)

d - change domain       r - change range        b - change both

p - replot the item     q - remove the plot     a - adjust-to-data
P - print a postscript file                     u - update the plot
v - shrink the plot
V - grow the plot

n - move right          N - move down           z - zoom in
m - move left           M - move up             Z - zoom out

l - make the window large (of full screen-size)
s - make the window small (250 by 250)
C-m - choose-mouse-mode ('C-m' denotes control-m)
C-z - mouse-mode zoom-in
C-s - mouse-mode selecting
C-b - mouse-mode brushing
"
  (case c
	(#\a (send self :adjust-to-data))
	(#\b (send self :change-domain-and-range))
	(#\d (send self :change-domain))
	(#\h (princ
"
  The following keys are currently defined: typing them while a scatterplot is
active will give the following results:

                        h - help (this)         i - info (send :help command)

d - change domain       r - change range        b - change both

			P - print the plot to a postscript file

p - replot the item     q - remove the plot     a - adjust-to-data
u - update the plot     v - shrink the plot     V - grow the plot

n - move right          N - move down           z - zoom in
m - move left           M - move up             Z - zoom out

l - make the window large (of full screen-size)
s - make the window small (250 by 250)
C-m - choose-mouse-mode ('C-m' denotes control-m)
C-z - mouse-mode zoom-in
C-s - mouse-mode selecting
C-b - mouse-mode brushing
")
	 )
	(#\i (send self :help))
	(#\l
 	 (let (
	       (screen-size (screen-size))
	       )
	   (send self :location 0 20)
	   (send self :size (first screen-size) (- (second screen-size) 20))
	   (send self :plot-item)
	   )
	 )
	(#\m (send self :shift 0 0 0))
	(#\n (send self :shift 1 0 0))
	(#\M (send self :shift 3 0 0))
	(#\N (send self :shift 2 0 0))
	(#\p (send self :plot-item))
	(#\P
	 (let ((name
		(get-string-dialog "Filename: "
				   :location (send self :location))
		))
	   (if name (postscript name :plot self)))
	 )
	(#\q (send self :remove))
	(#\r (send self :change-range))
	(#\s
	 (send self :location 0 20)
	 (send self :size 250 250)
	 (send self :plot-item)
	 )
	(#\u (send self :update-plot))
	(#\v (let((size (send self :size)))
	       (send self :size
		     (floor (* .9 (first size)))
		     (floor (* .9 (second size))))))
	(#\V (let((size (send self :size)))
	       (send self :size
		     (floor (* 1.1 (first size)))
		     (floor (* 1.1 (second size))))))

	(#\z (send self :shift 5 0 0)) ;; zoom in
	(#\Z (send self :shift 4 0 0)) ;; zoom out

	;; These are for keypad use:
	(#\C-m (send self :choose-mouse-mode))
	(#\C-z (send self :mouse-mode 'zoom-in))
	(#\C-s (send self :mouse-mode 'selecting))
	(#\C-b (send self :mouse-mode 'brushing))
	(#\4 (send self :shift 0 0 0))
	(#\6 (send self :shift 1 0 0))
	(#\2 (send self :shift 2 0 0))
	(#\8 (send self :shift 3 0 0))
	(#\5 (send self :shift 5 0 0)) ;; zoom in
	(#\- (send self :shift 4 0 0)) ;; zoom out
	(#\+ (send self :plot-item))
	(#\= (send self :remove))
	(#\C-q (send self :remove)) ;; control characters work too!

	(#\7
	 (send self :location 0 20)
	 (send self :size 250 250)
	 )
	(#\9
	 (let (
	       (screen-size (screen-size))
	       )
	   (send self :location 0 20)
	   (send self :size (first screen-size) (- (second screen-size) 20))
	   )
	 )
	)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                       spin do-key, and some methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth spin-proto :change-range ()
  "
Changes the range, using the get-value-dialog.
"
  (let* (
	 (location (send self :location))
	 (zmin (first (get-value-dialog "zmin" :location location)))
	 (zmax (if zmin (first (get-value-dialog "zmax" :location location))))
	 )
    (if (and zmin zmax)
	(list
	 (send self :range 2 zmin zmax)
	 (send self :plot-item)
	 )
      )
    )
  )

(defmeth spin-proto :change-domain-and-range ()
  "
Changes the domain and range, using the get-value-dialog.
"
  (let* (
	 (location (send self :location))
	 (xmin (first (get-value-dialog "xmin" :location location)))
	 (xmax (if xmin (first (get-value-dialog "xmax" :location location))))
	 (ymin (first (get-value-dialog "ymin" :location location)))
	 (ymax (if ymin (first (get-value-dialog "ymax" :location location))))
	 )
    (if (and xmin xmax) (send self :range 0 xmin xmax))
    (if (and ymin ymax) (send self :range 1 ymin ymax))
    (if (or (and xmin xmax) (and ymin ymax)) (send self :plot-item))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                text
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :text (text x y &key (h 0) (v 0) draw)
  (if (listp text)
      (mapcar (lambda(x y z) (send self :text x y z :draw draw)) text x y)
    (let* (
	   (coords (send self :real-to-canvas x y))
	   (x (first coords))
	   (y (second coords))
	   )
      (send self :draw-text text x y h v)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                hidden
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth spin-proto :hidden (&key patch)
  "
;; Description: spin-proto :hidden (&key patch)

	Hidden-line plot of a function of two-variables.

;; Examples:
 (plot (lambda(x y) (* x y)))
 (send my-plot :hidden)
 (send my-plot :hidden :patch t)
"
  (hid :plot self :patch patch)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                do-key
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth spin-proto :do-key (c m1 m2)
  "
  The following keys are currently defined: typing them while a spin plot
is active will give the following results:

                        h - help (this message)

d - change domain       r - change range		v - shrinks plot
p - replot the item     q - remove the plot		V - grows plot

			b - hidden line removal

l - make the window large (of full screen-size)
s - make the window small (300 by 300)
t - report the current transformation

m,n - roll		M,N - pitch		<,> (or ,,.) - yaw
1 - hand-rotate mouse-mode
2 - selecting mouse-mode
3 - brushing mouse-mode
"
  (case c
	(#\h (princ
"
  The following keys are currently defined: typing them while a spin plot is
active will give the following results:

                        h - help (this message)

d - change domain       r - change range		v - shrinks plot
p - replot the item     q - remove the plot		V - grows plot

			b - hidden line removal

l - make the window large (of full screen-size)
s - make the window small (300 by 300)
t - report the current transformation

m,n - roll		M,N - pitch		<,> (or ,,.) - yaw

1 - hand-rotate mouse-mode
2 - selecting mouse-mode
3 - brushing mouse-mode
")
	 )
	(#\b (send self :hidden))
	(#\d (send self :change-domain-and-range))
	(#\i (send self :help))
	(#\l
	 (let (
	       (screen-size (screen-size))
	       )
	   (send self :location 0 20)
	   (send self :size (first screen-size) (- (second screen-size) 20))
	   )
	 )
	(#\m
	 (send self :rotation-type 'rolling)
	 (send self :rotate)
	 )
	(#\M
	 (send self :rotation-type 'pitching)
	 (send self :rotate)
	 )
	(#\n
	 (send self :rotation-type 'rolling)
	 (send self :angle (- (send self :angle)))
	 (send self :rotate)
	 (send self :angle (- (send self :angle)))
	 )
	(#\N
	 (send self :rotation-type 'pitching)
	 (send self :angle (- (send self :angle)))
	 (send self :rotate)
	 (send self :angle (- (send self :angle)))
	 )
	(#\p (send self :plot-item))
	(#\P 
	 (let (
	       (name
		(get-string-dialog "Filename: "
				   :location (send self :location))
		)
	       )
	   (if name (postscript-3d name :plot self)))
	 )
	(#\q (send self :remove))
	(#\Q (send self :remove))
	(#\r (send self :change-range))
	(#\s
	 (send self :location 0 20)
	 (send self :size 300 300)
	 )
	(#\t (mprint (send self :transformation)))
	(#\v (let((size (send self :size)))
	       (send self :size
		     (floor (* .9 (first size)))
		     (floor (* .9 (second size))))))
	(#\V (let((size (send self :size)))
	       (send self :size
		     (floor (* 1.1 (first size)))
		     (floor (* 1.1 (second size))))))
	(#\1 (send self :mouse-mode 'hand-rotate))
	(#\2 (send self :mouse-mode 'selecting))
	(#\3 (send self :mouse-mode 'brushing))
	(#\4 (send self :choose-mouse-mode))
	(#\.
	 (send self :rotation-type 'yawing)
	 (send self :rotate)
	 )
	(#\>
	 (send self :rotation-type 'yawing)
	 (send self :rotate)
	 )
	(#\,
	 (send self :rotation-type 'yawing)
	 (send self :angle (- (send self :angle)))
	 (send self :rotate)
	 (send self :angle (- (send self :angle)))
	 )
	(#\<
	 (send self :rotation-type 'yawing)
	 (send self :angle (- (send self :angle)))
	 (send self :rotate)
	 (send self :angle (- (send self :angle)))
	 )
	)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                       histogram do-key, and some methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth histogram-proto :change-range ()
  "
Changes the range, using the get-value-dialog.
"
  (let* (
	 (location (send self :location))
	 (ymin (first (get-value-dialog "ymin" :location location)))
	 (ymax (if ymin (first (get-value-dialog "ymax" :location location))))
	 )
    (if (and ymin ymax)
	(list
	 (send self :range 1 ymin ymax)
	 (send self :plot-item)
	 )
      )
    )
  )

(defmeth histogram-proto :change-domain ()
  "
Changes the domain and range, using the get-value-dialog.
"
  (let* (
         (location (send self :location))
	 (xmin (first (get-value-dialog "xmin" :location location)))
	 (xmax (if xmin (first (get-value-dialog "xmax" :location location))))
	 )
    (if (and xmin xmax) (send self :range 0 xmin xmax))
    (if (and xmin xmax) (send self :plot-item))
    )
  )

(defmeth histogram-proto :change-bins ()
  "
Changes the number of bins, using the get-value-dialog.
"
  (let* (
         (location (send self :location))
	 (numbins
	  (first (get-value-dialog "Number of Bins:" :location location)))
	 )
    (if numbins (send self :num-bins numbins))
    )
  )

(defmeth histogram-proto :do-key (c m1 m2)
  "
  The following keys are currently defined: typing them while a histogram plot
is active will give the following results:

                        h - help (this message)

d - change domain       r - change range       i - info (send :help command)
p - replot the item     q - remove the plot

l - make the window large (of full screen-size)
s - make the window small (250 by 250)
"
  (case c
	(#\h (princ
"
  The following keys are currently defined: typing them while a histogram plot
is active will give the following results:

                        h - help (this message)

d - change domain       r - change range       b - change number of bins
p - replot the item     q - remove the plot    n - bin counts

l - make the window large (of full screen-size)
s - make the window small (250 by 250)
")
	 )
	(#\b (send self :change-bins))
	(#\n (print (print (send self :bin-counts))))
	(#\i (send self :help))
	(#\d (send self :change-domain))
	(#\r (send self :change-range))
	(#\l
	 (let (
	       (screen-size (screen-size))
	       )
	   (send self :location 0 20)
	   (send self :size (first screen-size) (- (second screen-size) 20))
	   )
	 )
	(#\s
	 (send self :location 0 20)
	 (send self :size 250 250)
	 )

	(#\p (send self :plot-item))
	(#\q (send self :remove))
	)
  )

#|
Doesn't work! Rats! That would have been too easy. Instead I have the rather
kludgy readtable file.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                       listener-proto do-key, and some methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth window-proto :do-key (c m1 m2)
  (case c
	(#\c-t (tutorials))
	(#\c-p (history))
	(#\c-l
	 (let (
	       (screen-size (screen-size))
	       )
	   (send self :location 0 20)
	   (send self :size (first screen-size) (- (second screen-size) 20))
	   )
	 )
	(#\c-s
	 (send self :location 0 20)
	 (send self :size 250 250)
	 )
	)
  )

|#


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                           andy-add-points
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :andy-add-points (&rest e)
  "
 (plot (rand.u 100) (rand.u 100))
 (send my-plot :andy-add-points (rand.n 10) (rand.n 10))
"
  (let (
	(x (first e))
	(y (second e))
	(z (if (= 3 (length e)) (third e) nil))
	)
    (if z (send self :add-points x y z)
      (send self :add-points x y))
    (setf (slot-value 'plot-item)
	  (if z (append (slot-value 'plot-item) (list (list x y z)))
	    (append (slot-value 'plot-item) (list (list x y)))
	    )
	  )
    )
  nil
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                   line methods for spin-proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	I stole these from gnuplot, to get at a three-d plot.
But I changed the whiles to do loops, as "while don't compile":
|#
(defmeth spin-proto :get-line (start)
  ":get-line 0 "
  (let (xcoords ycoords zcoords)
    (do
     ((next start (send self :linestart-next next)))
     ((not next))
     (setq xcoords (cons (send self :linestart-coordinate 0 next) xcoords))
     (setq ycoords (cons (send self :linestart-coordinate 1 next) ycoords))
     (setq zcoords (cons (send self :linestart-coordinate 2 next) zcoords))
     )
    (list xcoords ycoords zcoords)
    )
  )

(defmeth spin-proto :getlines ()
  "This method produces the three-d lines in a spin-plot. Returns the lines of
self in a list of lists format."
  (let (
	(last (send self :num-lines))
	resul
	)
    (do
     ((first 0 (+ first (length (first aline)))))
     ((>= first last))
     (setq aline (send self :get-line first))
     (setq resul (cons aline resul))
     )
    resul
    )
  )

#|
(defmeth spin-proto :3d-2d ()
  ":3d-2d () takes the three-d lines and give the 2-d canvas coordinates"
  (let (
	(first 0)
	(last (send self :num-lines))
	aline 
	resul
	(mat (matprod (send self :transformation)
		      (diagonal-to-matrix (/ (send self :scale (iseq 3))))))
	)
    (WHILE (< first last)
	   (setf aline
		 (select
		  (row-list
		   (matprod
		    mat
		    (apply #'bind-rows (send self :get-line first))
		    )
		   )
		  (iseq 2)
		  )
		 )
	   (setq resul (cons aline resul))
	   (setq first (+  first (length (first aline)))))
    resul))

|#
(defmeth spin-proto :3d-2d ()
  ":3d-2d () takes the three-d lines and give the 2-d canvas coordinates"
  (let (
	(last (send self :num-lines))
	aline 
	resul
	(mat (matprod (send self :transformation)
		      (diagonal-to-matrix (/ (send self :scale (iseq 3))))))
	)
    (do
     ((first 0 (+ first (length (first aline)))))
     ((>= first last))
     (setf aline
	   (select
	    (row-list
	     (matprod
	      mat
	      (apply #'bind-rows (send self :get-line first))
	      )
	     )
	    (iseq 2)
	    )
	   )
     (setq resul (cons aline resul))
     )
    resul)
  )

(defmeth spin-proto :3d-2d-points ()
  "this method takes the three-d points and give the 2-d canvas coordinates,
plus any label.
"
  (let* (
	 (last (send self :num-points))
	 (x (send self :point-coordinate 0 (iseq last)))
	 (y (send self :point-coordinate 1 (iseq last)))
	 (z (send self :point-coordinate 2 (iseq last)))
	 (names (send self :point-label (iseq last)))
	 resul
	 (mat (matprod (send self :transformation)
		       (diagonal-to-matrix (/ (send self :scale (iseq 3))))))
	 )
    (setf resul (row-list (matprod mat (bind-rows x y z))))
    (setq resul (weave
		 (coerce (first resul) 'list)
		 (coerce (second resul) 'list)
		 names
		 ))
    )
  )

#|
(defmeth spin-proto :do-hand-job (x y m1 m2)
  (let* ((oldp (apply #'calcsphere 
		      (send self :canvas-to-scaled x y)))
	 (p oldp)
	 (vars (send self :content-variables))
	 (trans (identity-matrix (send self :num-variables))))
    (send self :idle-on nil)
    (send self :while-button-down 
	  #'(lambda (x y) 
	      (setf oldp p)
	      (setf p (apply #'calcsphere 
			     (send self :canvas-to-scaled x y)))
	      (printall oldp p (list x y) vars (send self :canvas-to-scaled x y))
	      (setf (select trans vars vars) (make-rotation oldp p))
	      (mprint (make-rotation oldp p))
	      (when m1 
		    (send self :slot-value 'rotation-type trans)
		    (send self :idle-on t))
	      ;; (print (solve (send self :transformation) '(0 0 1)))
	      (send self
		    :apply-transformation
		    trans)
	      ;; ael
	      ))
    (if (send self :linked)
	(dolist (i (agw))
		(if (and
		     (neq i self)
		     (send i :linked)
		     )
		    (let ()
		      (send i :transformation (send self :transformation))
		      (send i :show-window)
		      )
		  )
		)
      )
    ))
(send spin-proto :add-mouse-mode 'hand-job
      :title "Hand Job" :cursor 'hand :click :do-hand-job)

(defun calcsphere (x y)
                   (let* ((norm-2 (+ (* x x) (* y y)))
                          (rad-2 (^ 1.7 2))
                          (z (if (< norm-2 rad-2) (sqrt (- rad-2 norm-2)) 0)))
                     (if (< norm-2 rad-2)
                         (list x y z)
                       (let ((r (sqrt (max norm-2 rad-2))))
                         (list (/ x r) (/ y r) (/ z r))))))
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             andy-options
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :andy-options (&rest e)
  "
"
  (let (
	(options (slot-value 'andy-options))
	)
    (if e (setf (slot-value 'andy-options) (combine options e))
      options
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             add-to-plot
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :add-to-plot (&rest e)
  "
 (plot cos -pi pi)
 (message-dialog \"In the next dialog box, type in

                      (func f(x) sin x)

 and see what happens!\")
 (send my-plot :add-to-plot)
"
  (let (
	(bing (if (not e)
		  (get-string-dialog
		   "What do you want to add? Enter a valid lisp expression:"
		   :initial ""
		   :location (send self :location)
		   )
		)
	      )
	bong
	)
    (if e (send self :add-plot-item e)
      (list
       (setq bong (eval (read (make-string-input-stream bing))))
       (setf (slot-value 'plot-item)
	     (append (slot-value 'plot-item) (list bong)))
       (send self :plot-item)
       )
      )
    )
  nil
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             points+-sd
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :points+-sd (&rest e)
  "
;; Description:

	method graph-proto :points+-sd (x y sd
		&key
		(color 'black)
		(symbol 'diamond)
		(type 'solid)
		(line-size 2)
		(point-size 3)
		)

	This method provides the means to put points and their standard
deviations on a plot. Options are for color of lines and points, symbol of
points, type of line, and size of lines.

;; Examples:

 (plot sin)
 (send my-plot :points+-sd (rand.u 10) (rand.u 10) (rand.u 10))

 (plot)
 (send my-plot :points+-sd (rand.u 10) (rand.u 10) (rand.u 10)
    :color 'blue :symbol 'wedge1 :type 'dashed :line-size 2)
 (send my-plot :range '(0 1) '(0 -1) '(1 2))

 (plot)
 (send my-plot :points+-sd (rand.u 10) (rand.u 10) (rand.u 10)
    :type 'dashed :color 'blue :symbol 'wedge1 :point-size 1 :line-size 1)
 (send my-plot :range '(0 1) '(0 -1) '(1 2))
"
  (let* (
	 (args (catch-keys '(:color :symbol :type :line-size :point-size) e))
	 (e (remove-keys '(:color :symbol :type :line-size :point-size) e))

	 (color (if (first args) (first args) 'black))
	 (symbol (if (second args) (second args) 'diamond))
	 (type (if (third args) (third args) 'solid))
	 (line-size (if (fourth args) (fourth args) 2))
	 (point-size (if (fifth args) (fifth args) 3))
	 (x (first e))
	 (y (second e))
	 (z (third e))
	 )
    (setf (slot-value 'plot-item)
	  (append (slot-value 'plot-item)
		  (mapcar
		   (lambda(x y z)
		     (list 'lines (list x x) (list (- y z) (+ y z))
			   color line-size type)
		     )
		   x y z)
		  (mapcar
		   (lambda(x y)
		     (list 'points (list x) (list y) color symbol point-size)
		     )
		   x y)
		  )
	  )
    (send self :plot-item)
    nil
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                           :points-to-histogram
;;
(defmeth graph-proto :points-to-histogram
  (x y
     &key
     (fill-color 'blue)
     (line-color 'black)
     (squeeze 2)
     (basement 0)
     )
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; points-to-histogram Doc:    Creation Date:3/13/97        Author: Andy Long
;;
;; Description/Arguments:

        graph-proto method :points-to-histogram
  (x y
     &key
     (fill-color 'blue)
     (line-color 'black)
     (squeeze 2)
     (basement 0)
     )
  
	This method adds a histogram to a plot.

;; Notes:

;; See Also:

;; Examples:

 ;; Histograms are ordinarily drawn to the x-axis:
 (plot)
 (send my-plot :points-to-histogram (rseq -.5 .5 10) (rand.u 10)
   :fill-color 'red :line-color 'black)
 (send my-plot :points-to-histogram (rseq -.5 .5 10) (- (rand.u 10))
   :fill-color 'yellow :line-color 'black)

 ;; Histograms will lay on top of each other:
 (plot)
 (send my-plot :points-to-histogram (rseq -.5 .5 10) (rand.u 10)
   :fill-color 'red :line-color 'black)
 (send my-plot :points-to-histogram (rseq -.5 .5 10) (rand.u 10)
   :fill-color 'yellow :line-color 'black)

 ;; Appropriate shifts will put histograms next to each other:
 (plot)
 (send my-plot :points-to-histogram (- (rseq -.5 .5 3) .125) (rand.u 3)
   :fill-color 'red :line-color 'black :squeeze 4)
 (send my-plot :points-to-histogram (+ (rseq -.5 .5 3) .125) (rand.u 3)
   :fill-color 'yellow :line-color 'black :squeeze 4)

 ;; Adjust bottom of histograms using basement:
 (plot)
 (send my-plot :points-to-histogram (- (rseq -.5 .5 3) .125) (rand.u 3)
   :fill-color 'red :line-color 'black :squeeze 4)
 (send my-plot :points-to-histogram (+ (rseq -.5 .5 3) .125) (rand.u 3)
   :fill-color 'yellow :line-color 'black :squeeze 4 :basement -.5)

 ;; Stack histograms using basement:
 (plot)
 (def y (* .5 (rand.u 10)))
 (send my-plot :points-to-histogram (rseq -.5 .5 10) y
   :fill-color 'red :line-color 'black)
 (send my-plot :points-to-histogram (rseq -.5 .5 10) (+ y (* .5 (rand.u 10)))
   :fill-color 'yellow :line-color 'black :basement y)
"
  (let (
	(xdiff (/ (- (second x) (first x)) squeeze))
	(b (if (numberp basement) (repeat basement (length x)) basement))
	)
    (mapcar
     (lambda(x y b)
       (let (
	     (x1 (- x xdiff))
	     (x2 (+ x xdiff))
	     )
	 (send self :add-plot-item
	       (list
		'patch
		(list (list x1 b)(list x2 b)(list x2 y)(list x1 y)(list x1 b))
		fill-color
		line-color
		)
	       :draw nil
	       )
	 )
       )
     x y b
     )
    )
  (send self :plot-item)
  nil
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
