#|
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                  Stuff for Power calculations in Families
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	I borrowed this from the UCLA stuff. Someone did this to allow power
computations off of a web-server, and I just took the code to use it for my
own evil purposes. Start-power is the function to have a look at, although
test-start-power serves as a decent interface to all the families.

						ael, early 1997.

Now I've adapted it to use load, so that all this stuff is only loaded as
necessary.

	I create a macro to load a file which will define the function - a
curious trick! We'll see how it works in practice....

|#
(defmacro start-power (&rest args)
  "
           start-power (family model type num-sides &rest args)

  family:
   	 			 (0 'normal-family-proto)
                                 (1 'exponential-family-proto)
                                 (2 'binomial-family-proto)
                                 (3 'poisson-family-proto)
                                 (4 'correlation-family-proto)))
  model:
          family (0 (case model (0 :normal-1-sample)
                                (1 :normal-2-sample-equal-var)
                                (2 :normal-2-sample-unequal-var)
                                (3 :lognormal-equal-var)))
                 (1 (case model (0 :exponential-1-sample)
                                (1 :exponential-2-sample)))
                 (2 (case model (0 :binomial-1-sample)
                                (1 :binomial-1-sample-arcsine)
                                (2 :binomial-2-sample-arcsin)
                                (3 :binomial-2-sample-median)
                                (4 :fishers-exact-test)
                                (5 :proportion-responders-cons-exp)
                                (6 :case-control)
                                (7 :binomial-k-sample)))
                 (3 (case model (0 :poisson-1-sample)
                                (1 :poisson-2-sample)))
                 (4 (case model (0 :correlation-coeff-outcome)))))
       )

  type:
        0 sample size
	1 power

  num-sides:
        1 (sided)
        2 (sided)

;; Examples:

 ;; NORMAL:
 ;; power calculations: normal, 
 ;; mu (h0), mu (ha), sigma, alpha, sample size
 (start-power 0 0 1 1 2 5 5 .01 25)
 (start-power 0 0 1 1 2 5 2 .01 25)
 (start-power 0 0 1 1 2 5 2 .01 10)
 (start-power 0 0 1 1 2 3 2 .01 10)

 ;; Almost reproduces table 7.9.1 of p. 256 in Daniel's Biostatistics, 6th ed.:
 (mapcar
   {(start-power 0 0 1 2 17.5 x 3.6 .05 100) :dash t}
   (rseq 16 19 7))
 ;; Example 7.9.2, page 257, ibid:
 (start-power 0 0 1 1 65 55 15 .01 20)

 ;; sample size computations:
 ;; one sample: mu (h0), mu (ha), sigma, alpha, power
 (start-power 0 0 0 1 2 3 2 .01 .05)

 ;; two sample: mu1, mu2, sigma, alpha, power
 (start-power 0 1 0 1 2 3 2 .01 .05)
 (start-power 0 1 0 1 2 3 2 .01 .95)

 ;; two-sided, unequal variance: Mu-1 Mu-2 Sigma-1 Sigma-2 Alpha power
 (start-power 0 2 0 2 2 3 2 4 .01 .95)

 ;; EXPONENTIAL:
 ;; two sample, one-sided: Mu-1 Mu-2 Alpha power
 (start-power 1 1 0 1 2 3 .01 .05)
 ;; two sample, one-sided: Mu-1 Mu-2 Alpha sample-1 sample-2
 (start-power 1 1 1 1 2 3 .01 25 33)
 ;; one sample, one-sided: Mu-o Mu-a Alpha sample-size
 (start-power 1 0 1 1 2 3 .01 25)
"
  (load-calc power "examples")
  `(start-power ,@args)
  )

(defmacro test-start-power ()
  "
	This is the interface for the power/sample-size calculator.

;; Examples:
 (test-start-power)
"
  (load-calc power "examples")
  (test-start-power)
  )
