;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                scatterplot-overlay-proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto scatterplot-overlay-proto '(graph))

(defmeth scatterplot-overlay-proto :graph () (slot-value 'graph))
(defmeth scatterplot-overlay-proto :resize () nil)
(defmeth scatterplot-overlay-proto :redraw () nil)
(defmeth scatterplot-overlay-proto :do-click (x y m1 m2) nil)

(defmeth scatterplot-proto :isnew (&rest args)
  (apply #'call-next-method args)
  (send self :add-overlay (send scatterplot-control-overlay-proto :new))
  (send self :margin 0 0 0 20)
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                scatterplot-control-overlay-proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto scatterplot-control-overlay-proto 
          '(top lefts gap side ascent box-top text-base)
          ()
          scatterplot-overlay-proto)

(defmeth scatterplot-control-overlay-proto :isnew ()
  (setf (slot-value 'gap) 5)
  (setf (slot-value 'side) 10)
  (setf (slot-value 'ascent) (send graph-proto :text-ascent))
  (let ((w1 (send graph-proto :text-width "Left-Right"))
	(w2 (send graph-proto :text-width "Down-Up"))
	(w3 (send graph-proto :text-width "Zoom"))
	(gap (slot-value 'gap))
        (side (slot-value 'side)))
    (setf (slot-value 'lefts)
          (list (* 2 gap)
                (+ (* 3 gap) side)
                (+ (* 6 gap) (* 2 side) w1)
                (+ (* 7 gap) (* 3 side) w1)
                (+ (* 11 gap) (* 4 side) w1 w2)
                (+ (* 12 gap) (* 5 side) w1 w2))))
  )

(defmeth scatterplot-control-overlay-proto :resize ()
  (let* ((graph (send self :graph))
         (height (send graph :canvas-height))
         (bottom-margin (fourth (send graph :margin)))
         (top (+ (- height bottom-margin) 1))
         (gap (slot-value 'gap))
         (side (slot-value 'side))
         (ascent (send graph :text-ascent))
         (text-base (+ top gap (max side ascent)))
         (box-top (- text-base side)))
    (setf (slot-value 'top) top)
    (setf (slot-value 'text-base) text-base)
    (setf (slot-value 'box-top) box-top)))

(defmeth scatterplot-control-overlay-proto :redraw ()
  (let ((graph (slot-value 'graph))
        (top (slot-value 'top))
        (lefts (slot-value 'lefts))
        (gap (slot-value 'gap))
        (side (slot-value 'side))
        (text-base (slot-value 'text-base))
        (box-top (slot-value 'box-top)))
    (send graph :draw-line 0 top (send graph :canvas-width) top)
    (mapcar #'(lambda (x) (send graph :frame-rect x box-top side side))
            lefts)
    (mapcar #'(lambda (s x y) (send graph :draw-string s x y))
            '("Left-Right" "Down-Up" "Zoom")
            (+ (select lefts '(1 3 5)) gap side) 
            (repeat text-base 3))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :do-click
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth scatterplot-control-overlay-proto :do-click (x y m1 m2)
  (let ((graph (slot-value 'graph))
        (top (slot-value 'top))
        (lefts (slot-value 'lefts))
        (gap (slot-value 'gap))
        (side (slot-value 'side))
        (text-base (slot-value 'text-base))
        (box-top (slot-value 'box-top)))
    (when (< top y)
          (if (< box-top y text-base)
              (let ((i (car (which (< lefts x (+ lefts side))))))
                (when i (send graph :while-button-down
			      #'(lambda (x y)(send graph :shift i x y)) nil)
		      )
		)
	    )
	  t)
    (if (not (< top y)) 
	(send graph :do-zoom-in x y m1 m2)
      (let()
       (send graph :redraw-content)
       (send graph :plot-item)
       )
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :do-click
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth scatterplot-control-overlay-proto :do-click (x y m1 m2)
  (let* ((graph (slot-value 'graph))
	 (mouse-mode (send graph :mouse-mode))
	 (top (slot-value 'top))
	 (lefts (slot-value 'lefts))
	 (gap (slot-value 'gap))
	 (side (slot-value 'side))
	 (text-base (slot-value 'text-base))
	 (box-top (slot-value 'box-top))
	 )
    (if (equalp mouse-mode 'zoom-in)
	(let ()
	  (when (< top y)
		(if (< box-top y text-base)
		    (let ((i (car (which (< lefts x (+ lefts side))))))
		      (when
		       i
		       (send graph :while-button-down
			     #'(lambda (x y)(send graph :shift i x y)) nil)
		       )
		      )
		  )
		t)
	  (if (not (< top y)) 
	      (send graph :do-zoom-in x y m1 m2)
	    (let()
	      (send graph :redraw-content)
	      (send graph :plot-item)
	      )
	    )
	  )
      nil
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :shift
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth scatterplot-proto :shift (i x y)
  (let* (
         (sizes (send self :range '(0 1)))
	 (xrange (first sizes))
	 (yrange (second sizes))
	 (xmin (nth 0 xrange))
	 (xmax (nth 1 xrange))
	 (ymin (nth 0 yrange))
	 (ymax (nth 1 yrange))
	 (xdiff (- xmax xmin))
	 (ydiff (- ymax ymin))
	 (xhalfdiff (/ (- xmax xmin) 2))
	 (yhalfdiff (/ (- ymax ymin) 2))
	 (xcenter (mean (list xmax xmin)))
	 (ycenter (mean (list ymax ymin)))
	 shift stretch
	 )
    (if (oddp i)
	(list
	 (setf shift *shift-factor*)
	 (setf stretch *zoom-in-factor*)
	 )
      (list
       (setf shift (- *shift-factor*))
       (setf stretch *zoom-out-factor*)
       )
      )
    (if (< i 2)
	(send self :range 0
	      (- xmin (* shift xdiff))
	      (- xmax (* shift xdiff))
	      )
      (if (< i 4)
	   (send self :range 1
		 (- ymin (* shift ydiff))
		 (- ymax (* shift ydiff))
		 )
	(list
	 (send self :start-buffering)
	 (send self :range 0
	       (- xcenter (* stretch xhalfdiff))
	       (+ xcenter (* stretch xhalfdiff))
	       )
	 (send self :range 1
	       (- ycenter (* stretch yhalfdiff))
	       (+ ycenter (* stretch yhalfdiff))
	       )
	 (send self :buffer-to-screen)
	 )
	)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                slots
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth scatterplot-proto :slots ()
  (let ((slot-vals nil))
    (dolist
     (i (send self :slot-names))
     (setq slot-vals (adjoin (list (slot-value i)) slot-vals))
     )
    slot-vals
    )
  )
