(defun density-plot (data)
  "
;; Description: (density-plot data-set)

	Makes a kernel fit to the density of a data set, and provides a slider to
allow one to change the width of the kernel. One can change the kernel type
from the plot menu.

	This is stolen, with slight modification, from Luke's book.

;; Example:
 (density-plot (rand.n 100))
"
  (let* (
	 (ndata `,data)
	 (w (plot-lines (kernel-dens ndata :width 1)))
	 (data-width (- (max ndata) (min ndata)))
	 (data-bit (/ data-width (length ndata)))
	 )

    (if *title* (send w :title *title*))

    (send w :add-slot 'kernel-width (/ data-width 2))
    (send w :add-slot 'kernel-type 'b)
    
    (defmeth w :kernel-width (&optional width)
      (when width
	    (setf (slot-value 'kernel-width) width)
	    (send self :set-lines))
      (slot-value 'kernel-width))
    
    (defmeth w :kernel-type (&optional type)
      (when type
	    (setf (slot-value 'kernel-type) type)
	    (send self :set-lines))
      (slot-value 'kernel-type))
    
    (defmeth w :set-lines ()
      (let ((width (send self :kernel-width))
	    (type (send self :kernel-type)))
	(send self :clear-lines :draw nil)
	(send self :add-lines (kernel-dens data :width width :type type))))

    (defmeth w :choose-kernel ()
      (let* (
	     (types '("Bisquare" "Gaussian" "Triangle" "Uniform"))
	     (i (choose-item-dialog "Kernel Type" types))
	     )
	(if i (send w :kernel-type (select '(b g t u) i)))))

    (setf kernel-item (send menu-item-proto :new "Kernel Type"
			    :action #'(lambda () (send w :choose-kernel))))
    (send (send w :menu) :append-items kernel-item)
    
    (interval-slider-dialog (list data-bit data-width)
			    :points (length ndata)
			    :action #'(lambda (s) (send w :kernel-width s)))
    w
    )
  )
