;;
;;following code modifies Luke's spin-control-overlay-proto to add zoom boxes
;;Forrest W. Young, September 25, 1991
;;

(defmeth spin-control-overlay-proto :isnew ()
  (setf (slot-value 'gap) 2)
  (setf (slot-value 'side) 10)
  (setf (slot-value 'ascent) (send graph-proto :text-ascent))
  (let ((w1 (send graph-proto :text-width "Pitch"))
        (w2 (send graph-proto :text-width "Roll"))
        (w3 (send graph-proto :text-width "Yaw"))
        (w4 (send graph-proto :text-width "Zoom"))
        (gap (slot-value 'gap))
        (side (slot-value 'side)))
    (setf (slot-value 'lefts)
          (list (* 2 gap)
                (+ (* 3 gap) side)
                (+ (* 6 gap) (* 2 side) w1)
                (+ (* 7 gap) (* 3 side) w1)
                (+ (* 11 gap) (* 4 side) w1 w2)
                (+ (* 12 gap) (* 5 side) w1 w2)
                (+ (* 17 gap) (* 6 side) w1 w2 w3)
                (+ (* 18 gap) (* 7 side) w1 w2 w3)
                ))))

(defmeth spin-control-overlay-proto :redraw ()
  (let ((graph (slot-value 'graph))
        (top (slot-value 'top))
        (lefts (slot-value 'lefts))
        (gap (slot-value 'gap))
        (side (slot-value 'side))
        (text-base (slot-value 'text-base))
        (box-top (slot-value 'box-top)))
    (send graph :draw-line 0 top (send graph :canvas-width) top)
    (mapcar #'(lambda (x) (send graph :frame-rect x box-top side side))
            lefts)
    (mapcar #'(lambda (s x y) (send graph :draw-string s x y))
            '("Pitch" "Roll" "Yaw" "Zoom")
            (+ (select lefts '(1 3 5 7)) gap side) 
            (repeat text-base 4))))

(defmeth spin-control-overlay-proto :do-click (x y m1 m2)
  (let ((graph (slot-value 'graph))
        (top (slot-value 'top))
        (lefts (slot-value 'lefts))
        (gap (slot-value 'gap))
        (side (slot-value 'side))
        (text-base (slot-value 'text-base))
        (box-top (slot-value 'box-top))
        (big-small t)
        (nv t)
        )
    (when (< top y)
          (send graph :idle-on nil)
          (setf nv (send graph :num-variables))
          (if (< box-top y text-base)
              (let ((i (car (which (< lefts x (+ lefts side)))))
                    (angle (abs (send graph :angle))))
                (when i
                      (when (< i 6)
                            (send graph :rotation-type 
                                  (select '(pitching rolling yawing)
                                          (floor (/ i 2))))
                            (send graph :angle (if (oddp i) angle (- angle)))
                            (send graph :while-button-down
                                  #'(lambda (x y) (send graph :rotate))
                                  nil)
                            (send graph :idle-on m1))
                      (when (> i 5)
                            (if (= i 6) (setf big-small .9)
                                (setf big-small 1.111))
                            (send graph :while-button-down
                                  #'(lambda (x y)
                                      (send graph :scale (iseq nv) 
                                            (/ (send graph :scale (iseq nv))
                                               big-small)))
                                  nil)))))
          t)))


(defmeth spin-proto :set-variables-with-labels (v labs)
  "Method Args: (variables labels)
Sets the variables indicated in the VARIABLES list to have the labels in the
LABELS list.  All other variables are unlabeled."
  (let ((n (send self :num-variables)))
    (send self :variable-label (iseq n) (repeat "" n))            
    (send self :variable-label v labs)))


(defmeth scatmat-proto :do-variable-focus (x y m1 m2)
"
Method to focus on variable subplots. Assumes that three other plots exist.
These plots MUST be named scatterplot, spin-plot and histogram.  A click on a
subplot sends it to scatterplot and sends its horizontal variable to histogram.
A click on a diagonal cell sends that variable to histogram.  Shift-clicks send
first three selected variables to spin-plot as well.
"
  (let ((cur-var (send self :current-variables))
        (nvar (send self :num-variables))
        (scale-type (send scatterplot :scale-type)))
    (when (not m1)
          (when (/= (select cur-var 0) (select cur-var 1))
                (send scatterplot :current-variables 
                      (select cur-var 0) (select cur-var 1) :draw nil)
                (if scale-type 
                    (send scatterplot :redraw)
                    (send scatterplot :adjust-to-data)))
          (send histogram :current-variables 
                (select cur-var 0) nvar :draw nil) 
          (send histogram   :adjust-to-data)
          (setf spin-var ()))
    (when m1
          (when (= (select cur-var 0) (select cur-var 1))
                (setf cur-var (list (select cur-var 0))))
          (when (< (length spin-var) 3)
                (setf spin-var (adjoin (select cur-var 0) spin-var))
                (when (< (length spin-var) 3)
                      (if (equal (length cur-var) 2)
                          (setf spin-var 
                                (adjoin (select cur-var 1) spin-var)))))
          (when (= (length spin-var) 3)
                (setf spin-var (reverse spin-var))
                (send spin-plot 
                      :current-variables (select spin-var 0)
                      (select spin-var 1) (select spin-var 2)
                      :draw nil)
                (send scatterplot :current-variables 
                      (select spin-var 0) (select spin-var 1) :draw nil)
                (send histogram :current-variables 
                      (select spin-var 0) nvar :draw nil) 
                (let ((cur-var (send spin-plot :current-variables)))
                  (send spin-plot :set-variables-with-labels cur-var
                        (select (send spin-plot :variable-labels) cur-var))
                  (send spin-plot :redraw))
                (if scale-type 
                    (send scatterplot :redraw)
                    (send scatterplot :adjust-to-data))
                (send histogram   :adjust-to-data)
                (setf spin-var ())))))



(def graph-side 200)
(def location11 '(22 42))
(def graph-size (list graph-side graph-side))
(def graph-side (+ 22 200))
(def location12 (+ location11 (list graph-side 0)))
(def location21 (+ location11 (list 0 graph-side)))
(def location22 (+ location11 (list graph-side graph-side)))

(defun spreadplot (vars point-labels var-labels 
                        &key
			(scale-type 'variable)
                        (show-histogram t) 
                        (show-pca-scree nil)
                        (data-object nil))
  "
Args: (variables point-labels variables-labels &key scale-type show-histogram)
When more than two variables, displays a scatterplot-matrix, spin-plot,
scatterplot and histogram (when show-histogram not nil) of VARIABLES, using
POINT-LABLES and VARIABLE-LABELS.  When only two variables, displays only a
scatterplot.  Scale-type can be nil, fixed or variable.  These work as
described for other plots.
" 

  (let ((variables vars)
	(variable-labels var-labels))
    (setf spin-var nil)
    (when (> (length variables) 9) 
	  (setf variables (select variables (iseq 9)))
	  (setf variable-labels (select variable-labels (iseq 9)))
	  )
    (when (> (length variables) 2)
	  (setf scatmat (scatterplot-matrix variables
					    :show nil
					    :size graph-size
					    :location location11
					    :scale-type scale-type
					    :point-labels point-labels
					    :variable-labels variable-labels))
	  (send scatmat   :show-window)
	  (setf spin-plot (spin-plot variables
				     :show nil
				     :go-away nil
				     :size graph-size
				     :location location12   
				     :scale-type scale-type
				     :title "Spinplot"
				     :point-labels point-labels
				     :variable-labels variable-labels)) 
	  (send spin-plot :show-window) 
	  (send spin-plot :mouse-mode 'hand-rotate)
	  (send spin-plot :linked t)
	  (send spin-plot :set-variables-with-labels '(0 1 2)
		(select (send spin-plot :variable-labels) '(0 1 2))) 
	  (defmeth spin-plot :close ()
	    (send scatmat :close))
	  (setf scatterplot (plot-points variables
					 :show nil
					 :go-away nil
					 :size graph-size
					 :location location21
					 :scale-type scale-type
					 :title "Scatterplot"
					 :point-labels point-labels
					 :variable-labels variable-labels))
	  (send scatterplot :linked t)
	  (when scale-type
		(send scatterplot :x-axis t t 0)
		(send scatterplot :y-axis t t 0)
		)
	  (send scatterplot :show-window)
	  (defmeth scatterplot :close ()
	    (send scatmat :close))
	  (setf histogram (histogram variables
				     :size graph-size
				     :location location22
				     :show nil
				     :go-away nil
				     :variable-labels variable-labels))
	  (send histogram :linked t) 
	  (when show-histogram (send histogram   :show-window))
	  (defmeth histogram :close ()
	    (send scatmat :close))
	  
	  #|
        (setf list-obs (send current-object :obs-namelist))
        (send list-obs  :linked t)
        (defmeth list-obs :close ()
          (send scatmat :close))
|#
	  
	  (send scatmat :add-mouse-mode 'focus-on-variables
		:title "Focus On Variables"
		:click :do-variable-focus
		:cursor 'finger)
	  (send scatmat :mouse-mode 'focus-on-variables)
	  (defmeth scatmat :close ()
	    (send spin-plot   :remove)
	    #|
          (send list-obs    :remove)
|#
	    
	    (send scatterplot :remove)
	    (send histogram   :remove)
	    (if show-pca-scree (send pca-scree-plot :remove))
	    #|
          (send *vista* :vis-window-status nil)
|#
	    
	    (call-next-method)))
    (when (= (length variables) 2)
	  (setf scatterplot (plot-points variables
					 :show nil
					 :scale-type scale-type
					 :title "Scatterplot"
					 :point-labels point-labels
					 :variable-labels variable-labels))
	  (send scatterplot :size 498 438) 
	  (send scatterplot :location 1 19) 
	  (send scatterplot :linked t) 
	  (when scale-type
		(send scatterplot :x-axis t t 0)
		(send scatterplot :y-axis t t 0)
		) 
	  (send scatterplot :show-window))
    #|
  (send *vista* :vis-window-status t)
|#
    ))

(defmeth graph-proto :add-rays (rays &key ray-labels)
  "
Method args: (rays &key ray-labels)
Adds rays to spin-plot.  Rays are lines drawn from the origin of the spin-plot
to locations specified by RAYS, a list of sequences.  There must be one
sequence for each dimension of the spin-plot.  The ray-ends are labled when
RAY-LABELS, a list of strings, is used.  Forrest W. Young
"
  (let* ((ndim   (send self :num-variables))
         (nrays  (length (select rays 0)))
         (center (send self :center (iseq ndim)))
	 ;;   (center (repeat 0 ndim))
         (rays-matrix (transpose (matrix (list ndim nrays) (combine rays))))
         (line-matrix ()))
    (dotimes (i nrays)
             (setf line-matrix 
		   (matrix (list 2 ndim) 
			   (combine center (+ center (row rays-matrix i)))))
             (send self :add-lines (column-list line-matrix)))
    (when ray-labels 
          (send self :add-points 
                (column-list 
                 (+ rays-matrix 
                    (matrix
		     (list nrays ndim)
		     (combine 
		      (make-list nrays :initial-element center)))))
                :point-labels ray-labels))
    )
  )
