;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;   Needed for compiling the necessary files of the calculus pantheon
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Added the following line to hopefully take care of the lack of the compiler
;; for ms-dog:

;; Peter Norvig likes to use question marks in his files!
;; Thus I remove the special definitions of ? and !:
(setf (aref *readtable* (char-int #\?))
      (aref *readtable* (char-int #\a)))
(setf (aref *readtable* (char-int #\!))
      (aref *readtable* (char-int #\a)))

(flet (
       (compile-if-needed (file &optional load)
			  (let (
				(lspfile (merge-pathnames file ".lsp"))
				(fslfile (merge-pathnames file ".fsl"))
				)
			    
			    ;; unless both .lsp and .fsl exist, and the date on
			    ;; .lsp is older than .fsl,
			    (unless (and (probe-file lspfile)
					 (probe-file fslfile)
					 (< (file-write-date lspfile)
					    (file-write-date fslfile))
					 )
				    (compile-file file :load load))))
       )
      (let (
	    (tutorialsfiles '("tutorial" "book"))
	    (contribfiles '("gnuplot" "barchart"
			    "clmath" "lisp3"
			    "f2cl_mac"
			    "eigen"
			    ))
	    (loopfiles '("auxmacs"  "loop"
			 "norvig" "macsyma" "macrules" "patmatch"
			 ;; "auxfns"
			 ))
	    (hulpfiles '("manual" "helper" "hulp" "machulp"))
	    (etcfiles '("backcomp" "utility" "utility2" "readfile" "help"
			"helpful" "geoeas" "uber-pro" "inspect" "readtab"
			;; "sheet" ;; this didn't work well anyway! It's now an
			;; example...
			"dialogs" "tstamp" "bitmap" "postscri"
			))
	    (mathfiles '("plotmeth" "vista" "matrix" "math" "roots"
			 "deriv" "integrat" "parser" "func" "backpars"
			 "euler" "ucla"
			 "taylor" "rk-4" "orbitals" "geostats" "diffeq" 
			 "scatover" "statplot" "newfuncs" "steepest" 
			 "plot" "complex" "fourier" "specfunc" "data"
			 "aliases" "menus" "spline" "bezier" "regress"
			 "splus" "nonlin" "muller" "stats" "histo"
			 "power"
			 "probfuns" "hidden" "linalg" "epidemio"
			 "plotit" "animate" "labels" "obmeth" "jacobian"
			 "multivar" "pde" "inverse"
			 ;; "compile-probs"
			 ))
	    )
	(dolist (f etcfiles)
		(let (
		      (fn (make-pathname :name f
					 :directory '(:relative "etc")))
		      )
		  (compile-if-needed fn))
		)
	(dolist (f tutorialsfiles)
		(let (
		      (fn (make-pathname :name f
					 :directory '(:relative "tutorial")))
		      )
		  (compile-if-needed fn))
		)
	(dolist (f contribfiles)
		(let (
		      (fn (make-pathname :name f
					 :directory '(:relative "contrib")))
		      )
		  (compile-if-needed fn))
		)
	(dolist (f loopfiles)
		(let (
		      (fn (make-pathname :name f
					 :directory '(:relative "loop")))
		      )
		  (compile-if-needed fn)
		  )
		)
	(dolist (f hulpfiles)
		(let (
		      (fn (make-pathname :name f
					 :directory '(:relative "hulp")))
		      )
		  (compile-if-needed fn))
		)
	(dolist (f mathfiles)
		(let (
		      (fn (make-pathname :name f
					 :directory '(:relative "math")))
		      )
		  (compile-if-needed fn))
		)
	)
      )
(exit)
