(message "

        Welcome to the Luke Tierney \"Lisp-stat\" project. I want to give you
as much of the flavor of Luke's book as I can (without violating copyright
law), and I'll do that by demonstrating with the code he uses in the book
(Luke Tierney, Lisp-Stat, Wiley, 1990: ISBN 0271-6232).

        This code is freely available from the xlisp-stat sites on the internet.
(e.g. www.stats.ucla.edu, maintained by Jan DeLeeuw). I have merely corrected
some errors in that version, and added comments as I've seen fit.

                                                       Andy Long, 1996

        The general procedure will be as follows: I will describe the purpose
of the code, show you the code, and then send the code to the \"listener\",
which sends it to the \"interpeter\" (as the main program is called). The
listener window is the first one that comes up.

")

(message "

        Chapter 1 is an introduction. The book will make use of some data,
which we can load in using the load-data command:

(load-data \"tutorial\")

Load data is a function that Luke wrote, to provide access to the data in the
directory \"Data\", under the main xlisp-stat directory. You can put your own
data in that directory, and the load-data command will access it for you.
")
(load-data "tutorial")

(message "

        Having loaded in the data, we can do some elementary operation on it.
For example, a variable named \"precipitation\" has been defined (which we can
examine by simply typing its name to the listener)

precipitation

so we may want to try such operations as

(mean precipitation)
(standard-deviation precipitation)

")
precipitation
(mean precipitation)
(standard-deviation precipitation)

(message "")
(message "

        If we would rather work with a standardized form of precipitation, then
we could use the more complex, and perhaps curious-looking,

(/ (- precipitation (mean precipitation))
   (standard-deviation precipitation))

which might be read as 
\"the quotient of
		the difference of precipitation and the mean of precipitation
                by the standard deviation of precipitation\"

This is an example of \"reverse polish notation\", which may seem unweildy at
first, but which becomes more natural with time. In the calculus programs I've
written, I create a means around using \"rpn\", as I find that students rebel
against its use.

")
(/ (- precipitation (mean precipitation))
   (standard-deviation precipitation))

(message "")
(message "

        We might want to incorporate this procedure into a function, called
standardize, which we create using the \"defun\" (define function) command:

(defun standardize (x)
  (/ (- x (mean x)) (standard-deviation x))
  )

Now it's a lot easier to compute the standardization of precipitation:

(standardize precipitation)

(and students are going to like that a lot more!).

")
(defun standardize (x)
  (/ (- x (mean x)) (standard-deviation x))
  )
(standardize precipitation)
(message "")
(message "

        This is all the code from Chapter 1. Also discussed in Chapter 1 are
the following:

        1.1 Environments for statistical computing
        1.2 The Lisp-Stat Environment
            1.2.1 Why Lisp?
            1.2.2 Using Lisp-Stat
            1.2.3 Some Design and Portability Issues
            1.2.4 The Future of Lisp-Stat

        Next up: Chapter 2 - A Lisp-Stat Tutorial

")






