;; ========
(message "                                Section  3.10.1")
;; ========

(defun newton-search (f df guess)
  (if (good-enough-p guess f df)
      guess
    (newton-search f df (improve guess f df))
    ))

(defun newton-search (f df guess)
  (do ((guess guess (improve guess f df)))
      ((good-enough-p guess f df) guess)))

(defun improve (guess f df)
  (- guess (/ (funcall f guess) (funcall df guess))))

(defun good-enough-p (guess f df)
  (< (abs (funcall f guess)) .001))

(newton-search #'sin #'cos 3)

(defun newton-search (f df guess)
  (flet (
	 (improve (guess f df)
		  (- guess 
		     (/ (funcall f guess) 
			(funcall df guess))))
	 (good-enough-p (guess f df)
			(< (abs (funcall f guess)) .001))
	 )
	(do ((guess guess (improve guess f df)))
	    ((good-enough-p guess f df) guess)
	    )
	))

(defun newton-search (f df guess)
  (flet (
	 (improve (guess)
		  (- guess 
		     (/ (funcall f guess) 
			(funcall df guess))))
	 (good-enough-p (guess)
			(< (abs (funcall f guess)) .001))
	 )
	(do ((guess guess (improve guess)))
	    ((good-enough-p guess) guess)
	    )
	))

(defun make-derivative (f h)
  #'(lambda (x)
      (let (
	    (fx+ (funcall f (+ x h)))
	    (fx- (funcall f (- x h)))
	    (2h (* 2 h))
	    )
	(/ (- fx+ fx-) 2h)
	)))

(newton-search #'sin (make-derivative #'sin .001) 3)

(def x (gamma-rand 30 4.5))

(def f (let (
	     (s (sum (log x)))
	     (n (length x))
	     )
	 #'(lambda (a)
	     (- (* (- a 1) s) (* n (log-gamma a))))
	 ))

(def df (make-derivative f .001))

(def ddf (make-derivative df .001))

(mean x)

(newton-search df ddf (mean x))

;; ========
(message "                                Section  3.10.2") 
;; ========

(message "
(defun deriv (exp var)
  (cond
   ((constantp exp) 0)
   ((variablep exp) (if (same-variable-p exp var) 1 0))
   ((sump exp)
    (make-sum (deriv (addend exp) var)
	      (deriv (augend exp) var)))
   ((productp exp)
    (make-sum (make-product (multiplier exp)
			    (deriv (multiplicand exp) var))
	      (make-product (deriv (multiplier exp) var)
			    (multiplicand exp))))
   (t (error \"Can't differentiate this expression\"))
   ))

")
(defun deriv (exp var)
  (cond
   ((constantp exp) 0)
   ((variablep exp) (if (same-variable-p exp var) 1 0))
   ((sump exp)
    (make-sum (deriv (addend exp) var)
	      (deriv (augend exp) var)))
   ((productp exp)
    (make-sum (make-product (multiplier exp)
			    (deriv (multiplicand exp) var))
	      (make-product (deriv (multiplier exp) var)
			    (multiplicand exp))))
   (t (error "Can't differentiate this expression"))
   ))




(message "
(defun addend (e) (second e))

(defun augend (e) (third e))

(defun make-sum (a1 a2) (list '+ a1 a2))

(defun multiplier (e) (second e))

(defun multiplicand (e) (third e))

(defun make-product (m1 m2) (list '* m1 m2))

")
(defun addend (e) (second e))

(defun augend (e) (third e))

(defun make-sum (a1 a2) (list '+ a1 a2))

(defun multiplier (e) (second e))

(defun multiplicand (e) (third e))

(defun make-product (m1 m2) (list '* m1 m2))




(message "
(defun constantp (e) (numberp e))

(defun variablep (e) (symbolp e))

(defun same-variable-p (v1 v2)
  (and (variablep v1) (variablep v2) (eq v1 v2)))

(defun sump (e)
  (and (listp e) (= (length e) 3) (eq (first e) '+)))

(defun productp (e)
  (and (listp e) (= (length e) 3) (eq (first e) '*)))
")
(defun constantp (e) (numberp e))

(defun variablep (e) (symbolp e))

(defun same-variable-p (v1 v2)
  (and (variablep v1) (variablep v2) (eq v1 v2)))

(defun sump (e)
  (and (listp e) (= (length e) 3) (eq (first e) '+)))

(defun productp (e)
  (and (listp e) (= (length e) 3) (eq (first e) '*)))

(message "
(deriv '(+ x 3) 'x)

(deriv '(* x y) 'x)

(deriv '(* (* x y) (+ x 3)) 'x)

")
(deriv '(+ x 3) 'x)

(deriv '(* x y) 'x)

(deriv '(* (* x y) (+ x 3)) 'x)




(message "
(defun make-sum (a1 a2)
  (cond
   ((and (numberp a1) (numberp a2)) (+ a1 a2))
   ((numberp a1) (if (= a1 0) a2 (list '+ a1 a2)))
   ((numberp a2) (if (= a2 0) a1 (list '+ a1 a2)))
   (t (list '+ a1 a2))
   ))

(defun make-sum (a1 a2)
  (cond
   ((and (numberp a1) (numberp a2)) (+ a1 a2))
   ((numberp a1) (if (= a1 0) a2 (list '+ a1 a2)))
   ((numberp a2) (if (= a2 0) a1 (list '+ a1 a2)))
   (t (list '+ a1 a2))
   ))
")
(defun make-sum (a1 a2)
  (cond
   ((and (numberp a1) (numberp a2)) (+ a1 a2))
   ((numberp a1) (if (= a1 0) a2 (list '+ a1 a2)))
   ((numberp a2) (if (= a2 0) a1 (list '+ a1 a2)))
   (t (list '+ a1 a2))
   ))

(defun make-sum (a1 a2)
  (cond
   ((and (numberp a1) (numberp a2)) (+ a1 a2))
   ((numberp a1) (if (= a1 0) a2 (list '+ a1 a2)))
   ((numberp a2) (if (= a2 0) a1 (list '+ a1 a2)))
   (t (list '+ a1 a2))
   ))

(message "
(defun make-product (m1 m2)
  (cond
   ((and (numberp m1) (numberp m2)) (* m1 m2))
   ((numberp m1)
    (cond ((= m1 0) 0)
	  ((= m1 1) m2)
	  (t (list '* m1 m2))))
   ((numberp m2)
    (cond ((= m2 0) 0)
	  ((= m2 1) m1)
	  (t (list '* m1 m2))))
   (t (list '* m1 m2))
   ))
")
(defun make-product (m1 m2)
  (cond
   ((and (numberp m1) (numberp m2)) (* m1 m2))
   ((numberp m1)
    (cond ((= m1 0) 0)
	  ((= m1 1) m2)
	  (t (list '* m1 m2))))
   ((numberp m2)
    (cond ((= m2 0) 0)
	  ((= m2 1) m1)
	  (t (list '* m1 m2))))
   (t (list '* m1 m2))
   ))

(message "
(defun deriv (exp var)
  (cond
   ((constantp exp) 0)
   ((variablep exp) (if (same-variable-p exp var) 1 0))
   ((sump exp)
    (make-sum (deriv (addend exp) var)
	      (deriv (augend exp) var)))
   ((productp exp)
    (make-sum (make-product (multiplier exp)
			    (deriv (multiplicand exp) var))
	      (make-product (deriv (multiplier exp) var)
			    (multiplicand exp))))
   ((unary-p exp)
    (make-product (make-unary-deriv (unary-function exp)
				    (unary-argument exp))
		  (deriv (unary-argument exp) var)))
   (t (error \"Can't differentiate this expression\"))
   ))
")
(defun deriv (exp var)
  (cond
   ((constantp exp) 0)
   ((variablep exp) (if (same-variable-p exp var) 1 0))
   ((sump exp)
    (make-sum (deriv (addend exp) var)
	      (deriv (augend exp) var)))
   ((productp exp)
    (make-sum (make-product (multiplier exp)
			    (deriv (multiplicand exp) var))
	      (make-product (deriv (multiplier exp) var)
			    (multiplicand exp))))
   ((unary-p exp)
    (make-product (make-unary-deriv (unary-function exp)
				    (unary-argument exp))
		  (deriv (unary-argument exp) var)))
   (t (error "Can't differentiate this expression"))
   ))

(message "
(defun make-unary-deriv (fcn arg)
  (case fcn
	(exp (make-unary 'exp arg))
	(sin (make-unary 'cos arg))
	(cos (make-product -1 (make-unary 'sin arg)))
	(t (error \"Can't differentiate this expression\"))
	))

")
(defun make-unary-deriv (fcn arg)
  (case fcn
	(exp (make-unary 'exp arg))
	(sin (make-unary 'cos arg))
	(cos (make-product -1 (make-unary 'sin arg)))
	(t (error "Can't differentiate this expression"))
	))

(message "
(defun unary-p (e)
  (and (listp e) (= (length e) 2)))

(defun unary-function (e) (first e))

(defun unary-argument (e) (second e))

(defun make-unary (fcn arg) (list fcn arg))
")
(defun unary-p (e)
  (and (listp e) (= (length e) 2)))

(defun unary-function (e) (first e))

(defun unary-argument (e) (second e))

(defun make-unary (fcn arg) (list fcn arg))

(message "
(deriv '(exp (* 3 x)) 'x)

(deriv '(sin (* 3 x)) 'x)

(defun make-unary-deriv (fcn arg)
  (apply-unary-rule (get-unary-rule fcn) arg))

(def *mylist* '((x 1) (y \"Hello\") (abc a w (1 2 3))))

(assoc 'x *mylist*)

(assoc 'y *mylist*)

(assoc 'abc *mylist*)

(assoc 'z *mylist*)
")
(deriv '(exp (* 3 x)) 'x)

(deriv '(sin (* 3 x)) 'x)

(defun make-unary-deriv (fcn arg)
  (apply-unary-rule (get-unary-rule fcn) arg))

(def *mylist* '((x 1) (y "Hello") (abc a w (1 2 3))))

(assoc 'x *mylist*)

(assoc 'y *mylist*)

(assoc 'abc *mylist*)

(assoc 'z *mylist*)



(message "
(def *derivatives* nil)

(cons 'a '(b c))

(defun add-unary-rule (f rule)
  (setf *derivatives* (cons (list f rule) *derivatives*)))

(defun get-unary-rule (f)
  (let ((rule (assoc f *derivatives*)))
    (if rule
	rule
      (error \"Can't differentiate this expression\"))
    ))

(defun apply-unary-rule (entry arg)
  (funcall (second entry) arg))

")
(def *derivatives* nil)

(cons 'a '(b c))

(defun add-unary-rule (f rule)
  (setf *derivatives* (cons (list f rule) *derivatives*)))

(defun get-unary-rule (f)
  (let ((rule (assoc f *derivatives*)))
    (if rule
	rule
      (error "Can't differentiate this expression"))
    ))

(defun apply-unary-rule (entry arg)
  (funcall (second entry) arg))


(message "
(add-unary-rule 'exp #'(lambda (x) (make-unary 'exp x)))

(add-unary-rule 'sin #'(lambda (x) (make-unary 'cos x)))

(deriv '(exp x) 'x)

(deriv '(exp (* -1 (* x x))) 'x)

(deriv '(sin (* 3 x)) 'x)

\"(deriv '(* (cos (* 3 x))) 'x)\"
\"Error: Can't differentiate this expression\"
\"Happened in: #<FSubr-IF: #eb8e8>\"

")
(add-unary-rule 'exp #'(lambda (x) (make-unary 'exp x)))

(add-unary-rule 'sin #'(lambda (x) (make-unary 'cos x)))

(deriv '(exp x) 'x)

(deriv '(exp (* -1 (* x x))) 'x)

(deriv '(sin (* 3 x)) 'x)

"(deriv '(* (cos (* 3 x))) 'x)"
"Error: Can't differentiate this expression"
"Happened in: #<FSubr-IF: #eb8e8>"


(message "
(add-unary-rule 'cos 
		#'(lambda (x) 
		    (make-product -1 (make-unary 'cos x))))

(deriv (deriv '(sin (* 3 x)) 'x) 'x)
")
(add-unary-rule 'cos 
		#'(lambda (x) 
		    (make-product -1 (make-unary 'cos x))))

(deriv (deriv '(sin (* 3 x)) 'x) 'x)






