;; =================
(message "

            Section  6.2.1: Basic Structure and Terminology

") 
;; =================

(message "

	Messages are sent using the function send:

	(send <object> <selector> <arg 1> ... <arg n>):

If an object does not have its own method, then a successive search will be
made of the ancestors of an object until such a method is found (or, in case
none exists, an error will be signaled). The top object is *object*:

(send *object* :own-slots)
(send *object* :slot-value 'proto-name)
(send *object* :slot-value 'instance-slots)
*object*
")
(send *object* :own-slots)
(send *object* :slot-value 'proto-name)
(send *object* :slot-value 'instance-slots)
*object*

;; =================
(message "

                    Section  6.2.2: Constructing New Objects

") 
;; =================

(message "
(setf x (send *object* :new))
(send x :own-slots)
(send x :add-slot 'data (normal-rand 50))
(send x :add-slot 'title)
(send x :own-slots)
(send x :slot-value 'data)
(send x :slot-value 'title)
(send x :slot-value 'title \"a data set\")
(send x :slot-value 'title)
(send x :slot-value 'proto-name)
(send x :slot-names)
")
(setf x (send *object* :new))
(send x :own-slots)
(send x :add-slot 'data (normal-rand 50))
(send x :add-slot 'title)
(send x :own-slots)
(send x :slot-value 'data)
(send x :slot-value 'title)
(send x :slot-value 'title "a data set")
(send x :slot-value 'title)
(send x :slot-value 'proto-name)
(send x :slot-names)

(message "
(send x :has-slot 'title)

(send x :has-slot 'x)

(send x :has-slot 'proto-name)

(send x :has-slot 'title :own t)

(send x :has-slot 'proto-name :own t)
")
(send x :has-slot 'title)
(send x :has-slot 'x)
(send x :has-slot 'proto-name)
(send x :has-slot 'title :own t)
(send x :has-slot 'proto-name :own t)

(message "
;; ael: error
\"(send x :slot-value 'proto-name 'new-name)\"
\"Error: object does not own slot - PROTO-NAME\"
\"Happened in: #<Byte-Code-Closure: #159708>\"
")
;; ael: error
"(send x :slot-value 'proto-name 'new-name)"
"Error: object does not own slot - PROTO-NAME"
"Happened in: #<Byte-Code-Closure: #159708>"

;; =================
(message "

                    Section  6.2.3: Defining New Methods

") 
;; =================

(message "
(defmeth x :describe (&optional (stream t))
  (format stream \"This is ~a~%\"
	  (send self :slot-value 'title))
  (format stream \"The sample mean is ~g~%\"
	  (mean (send self :slot-value 'data)))
  (format stream \"The sample standard-deviation is ~g~%\"
	  (standard-deviation
	   (send self :slot-value 'data))))

(send x :describe)
")
(defmeth x :describe (&optional (stream t))
  (format stream "This is ~a~%"
	  (send self :slot-value 'title))
  (format stream "The sample mean is ~g~%"
	  (mean (send self :slot-value 'data)))
  (format stream "The sample standard-deviation is ~g~%"
	  (standard-deviation
	   (send self :slot-value 'data))))

(send x :describe)


(message "
(defmeth x :describe (&optional (stream t))
  (format stream \"This is ~a~%\" (slot-value 'title))
  (format stream \"The sample mean is ~g~%\"
	  (mean (slot-value 'data)))
  (format stream \"The sample standard-deviation is ~g~%\"
	  (standard-deviation (slot-value 'data))))

(defmeth x :title (&optional (title nil set))
  (if set (setf (slot-value 'title) title))
  (slot-value 'title))

(defmeth x :data (&optional (data nil set))
  (if set (setf (slot-value 'data) data))
  (slot-value 'data))

(defmeth x :describe (&optional (stream t))
  (let ((title (send self :title))
	(data (send self :data)))
    (format stream \"This is ~a~%\" title)
    (format stream \"The sample mean is ~g~%\" (mean data))          
    (format stream \"The sample standard-deviation is ~g~%\"
	    (standard-deviation data))))

(send x :own-slots)
")
(defmeth x :describe (&optional (stream t))
  (format stream "This is ~a~%" (slot-value 'title))
  (format stream "The sample mean is ~g~%"
	  (mean (slot-value 'data)))
  (format stream "The sample standard-deviation is ~g~%"
	  (standard-deviation (slot-value 'data))))

(defmeth x :title (&optional (title nil set))
  (if set (setf (slot-value 'title) title))
  (slot-value 'title))

(defmeth x :data (&optional (data nil set))
  (if set (setf (slot-value 'data) data))
  (slot-value 'data))

(defmeth x :describe (&optional (stream t))
  (let ((title (send self :title))
	(data (send self :data)))
    (format stream "This is ~a~%" title)
    (format stream "The sample mean is ~g~%" (mean data))          
    (format stream "The sample standard-deviation is ~g~%"
	    (standard-deviation data))))

(send x :own-slots)

(message "
(send x :method-selectors)
")
(send x :method-selectors)

;; =================
(message "

               Section  6.2.4: Printing Objects

")
;; =================

(message "
(defmeth x :print (&optional (stream t))
(format stream \"#<~a>\" (send self :title)))

x
")
(defmeth x :print (&optional (stream t))
(format stream "#<~a>" (send self :title)))
x

