;; =================
(message "

        Section  6.3.1: Constructing Prototypes and Instances

")
;; =================

(message "

	Prototypes are generated with the defproto macro:

(defproto data-set-proto '(data title))

(send data-set-proto :slot-value 'title \"a data set\")

(defmeth data-set-proto :describe (&optional (stream t))
  (format stream \"This is ~a~%\" (slot-value 'title))
  (format stream \"The sample mean is ~g~%\"
	  (mean (slot-value 'data)))
  (format stream \"The sample standard-deviation is ~g~%\"
	  (standard-deviation (slot-value 'data))))

(defmeth data-set-proto :title (&optional (title nil set))
  (if set (setf (slot-value 'title) title))
  (slot-value 'title))

")
(defproto data-set-proto '(data title))

(send data-set-proto :slot-value 'title "a data set")

(defmeth data-set-proto :describe (&optional (stream t))
  (format stream "This is ~a~%" (slot-value 'title))
  (format stream "The sample mean is ~g~%"
	  (mean (slot-value 'data)))
  (format stream "The sample standard-deviation is ~g~%"
	  (standard-deviation (slot-value 'data))))

(defmeth data-set-proto :title (&optional (title nil set))
  (if set (setf (slot-value 'title) title))
  (slot-value 'title))



(message "
(defmeth data-set-proto :data (&optional (data nil set))
  (if set (setf (slot-value 'data) data))
  (slot-value 'data))

(defmeth data-set-proto :print (&optional (stream t))
  (format stream \"#<~a>\" (send self :title)))

(setf x (send data-set-proto :new :data (chisq-rand 20 5)))

(send x :describe)
")
(defmeth data-set-proto :data (&optional (data nil set))
  (if set (setf (slot-value 'data) data))
  (slot-value 'data))

(defmeth data-set-proto :print (&optional (stream t))
  (format stream "#<~a>" (send self :title)))

(setf x (send data-set-proto :new :data (chisq-rand 20 5)))

(send x :describe)




(message "
(defmeth data-set-proto :isnew (data &key title)
  (send self :data data)
  (if title (send self :title title)))

(defmeth data-set-proto :plot () (histogram (send self :data)))

(defun make-data-set (x &key (title \"a data set\") (print t))
  (let ((object (send data-set-proto :new x :title title)))
    object))

(defun describe-data (data) (send data :describe))

(defun plot-data (data) (send data :plot))
")
(defmeth data-set-proto :isnew (data &key title)
  (send self :data data)
  (if title (send self :title title)))

(defmeth data-set-proto :plot () (histogram (send self :data)))

(defun make-data-set (x &key (title "a data set") (print t))
  (let ((object (send data-set-proto :new x :title title)))
    object))

(defun describe-data (data) (send data :describe))

(defun plot-data (data) (send data :plot))




;; =================
(message "

             Section  6.3.2: Prototypes Inheriting from Prototypes

")
;; =================

(message "
(defproto time-series-proto
  '(origin spacing) () data-set-proto)

(send time-series-proto :title)

(send time-series-proto :title \"a time series\")

(defmeth time-series-proto :origin (&optional (origin nil set))
(if set (setf (slot-value 'origin) origin))
(slot-value 'origin))

(defmeth time-series-proto :spacing (&optional (sp nil set))
(if set (setf (slot-value 'spacing) sp))
(slot-value 'spacing))
")
(defproto time-series-proto
  '(origin spacing) () data-set-proto)

(send time-series-proto :title)

(send time-series-proto :title "a time series")

(defmeth time-series-proto :origin (&optional (origin nil set))
(if set (setf (slot-value 'origin) origin))
(slot-value 'origin))

(defmeth time-series-proto :spacing (&optional (sp nil set))
(if set (setf (slot-value 'spacing) sp))
(slot-value 'spacing))





(message "
(send time-series-proto :origin 0)

(send time-series-proto :spacing 1)

(let* ((e (normal-rand 21))
       (data (+ (select e (iseq 1 20))
		(* .6 (select e (iseq 0 19))))))
  (setf y (send time-series-proto :new data)))

(send y :describe)
")
(send time-series-proto :origin 0)

(send time-series-proto :spacing 1)

(let* ((e (normal-rand 21))
       (data (+ (select e (iseq 1 20))
		(* .6 (select e (iseq 0 19))))))
  (setf y (send time-series-proto :new data)))

(send y :describe)




;; =================
(message "

        Section  6.3.3: Overriding and Modifying Inherited Methods

")
;; =================

(message "
(defmeth time-series-proto :plot ()
  (let* ((data (send self :data))
	 (time (+ (send self :origin)
		  (* (iseq (length data))
		     (send self :spacing)))))
    (plot-points time data)))

(defmeth time-series-proto :describe (&optional (stream t))
  (call-next-method stream)
  (format stream
	  \"The autocorrelation is ~g~%\"
	  (autocorrelation (send self :data))))

(defun autocorrelation (x)
  (let ((n (length x))
	(x (- x (mean x))))
    (/ (mean (* (select x (iseq 0 (- n 2)))
		(select x (iseq 1 (- n 1)))))
       (mean (* x x)))))

(send y :describe)
")
(defmeth time-series-proto :plot ()
  (let* ((data (send self :data))
	 (time (+ (send self :origin)
		  (* (iseq (length data))
		     (send self :spacing)))))
    (plot-points time data)))

(defmeth time-series-proto :describe (&optional (stream t))
  (call-next-method stream)
  (format stream
	  "The autocorrelation is ~g~%"
	  (autocorrelation (send self :data))))

(defun autocorrelation (x)
  (let ((n (length x))
	(x (- x (mean x))))
    (/ (mean (* (select x (iseq 0 (- n 2)))
		(select x (iseq 1 (- n 1)))))
       (mean (* x x)))))

(send y :describe)




(message "
(defmeth data-set-proto :isnew (data &rest agrs)
  (apply #'call-next-method :data data args))

(defmeth time-series-proto :describe (&optional (stream t))
  (call-method data-set-proto :describe stream)
  (format stream
	  \"The autocorrelation is ~g~%\"
	  (autocorrelation (send self :data))))
")
(defmeth data-set-proto :isnew (data &rest agrs)
  (apply #'call-next-method :data data args))

(defmeth time-series-proto :describe (&optional (stream t))
  (call-method data-set-proto :describe stream)
  (format stream
	  "The autocorrelation is ~g~%"
	  (autocorrelation (send self :data))))

