(message "
			Object-Oriented Programming

	Object-oriented programming is supported by the existance of an object
data type, *object*.

")

;; =================
(message "

                 Section  6.1: Some motivation

")
;; =================

(message "
	We could create functions to describe a family of data sets: for
example, we could create

(defun describe-data (data)
  (cond
   ((single-sample-p data) ...)
   ((multiple-sample-p data) ...)
   ((paired-sample-p data) ...)
   (t (error \"Don't know how to describe this data set yet.\"))))

if we knew that we had only three data set types (single, multiple, and
paired).
")

(message "
	But if we now introduce a time-series type, we are obligated to go back
over old ground, modifying describe data and any other function which doesn't
yet know about the existance of time-series.

	The alternative is to arrange for the data sets themselves to know how
to describe themselves, and to simply ask them to describe themselves. Thus we
will make a simple call to the :describe method of a data set, for
example. This is called MESSAGE-PASSING.

	This way, we don't risk breaking the code for existing data sets when
modifying a function to take a new type into account.
")

(message "
	Furthermore, we introduce the idea of inheritance: we can define a new
object in terms of an old one, adding additional methods as necessary,
changing inherited ones too, if necessary.

	These ideas will be illustrated in the examples that follow.

");; =================
(message "

            Section  6.2.1: Basic Structure and Terminology

") 
;; =================

(message "

	Messages are sent using the function send:

	(send <object> <selector> <arg 1> ... <arg n>):

If an object does not have its own method, then a successive search will be
made of the ancestors of an object until such a method is found (or, in case
none exists, an error will be signaled). The top object is *object*:

(send *object* :own-slots)
(send *object* :slot-value 'proto-name)
(send *object* :slot-value 'instance-slots)
*object*
")
(send *object* :own-slots)
(send *object* :slot-value 'proto-name)
(send *object* :slot-value 'instance-slots)
*object*

;; =================
(message "

                    Section  6.2.2: Constructing New Objects

") 
;; =================

(message "
(setf x (send *object* :new))
(send x :own-slots)
(send x :add-slot 'data (normal-rand 50))
(send x :add-slot 'title)
(send x :own-slots)
(send x :slot-value 'data)
(send x :slot-value 'title)
(send x :slot-value 'title \"a data set\")
(send x :slot-value 'title)
(send x :slot-value 'proto-name)
(send x :slot-names)
")
(setf x (send *object* :new))
(send x :own-slots)
(send x :add-slot 'data (normal-rand 50))
(send x :add-slot 'title)
(send x :own-slots)
(send x :slot-value 'data)
(send x :slot-value 'title)
(send x :slot-value 'title "a data set")
(send x :slot-value 'title)
(send x :slot-value 'proto-name)
(send x :slot-names)

(message "
(send x :has-slot 'title)

(send x :has-slot 'x)

(send x :has-slot 'proto-name)

(send x :has-slot 'title :own t)

(send x :has-slot 'proto-name :own t)
")
(send x :has-slot 'title)
(send x :has-slot 'x)
(send x :has-slot 'proto-name)
(send x :has-slot 'title :own t)
(send x :has-slot 'proto-name :own t)

(message "
;; ael: error
\"(send x :slot-value 'proto-name 'new-name)\"
\"Error: object does not own slot - PROTO-NAME\"
\"Happened in: #<Byte-Code-Closure: #159708>\"
")
;; ael: error
"(send x :slot-value 'proto-name 'new-name)"
"Error: object does not own slot - PROTO-NAME"
"Happened in: #<Byte-Code-Closure: #159708>"

;; =================
(message "

                    Section  6.2.3: Defining New Methods

") 
;; =================

(message "
(defmeth x :describe (&optional (stream t))
  (format stream \"This is ~a~%\"
	  (send self :slot-value 'title))
  (format stream \"The sample mean is ~g~%\"
	  (mean (send self :slot-value 'data)))
  (format stream \"The sample standard-deviation is ~g~%\"
	  (standard-deviation
	   (send self :slot-value 'data))))

(send x :describe)
")
(defmeth x :describe (&optional (stream t))
  (format stream "This is ~a~%"
	  (send self :slot-value 'title))
  (format stream "The sample mean is ~g~%"
	  (mean (send self :slot-value 'data)))
  (format stream "The sample standard-deviation is ~g~%"
	  (standard-deviation
	   (send self :slot-value 'data))))

(send x :describe)


(message "
(defmeth x :describe (&optional (stream t))
  (format stream \"This is ~a~%\" (slot-value 'title))
  (format stream \"The sample mean is ~g~%\"
	  (mean (slot-value 'data)))
  (format stream \"The sample standard-deviation is ~g~%\"
	  (standard-deviation (slot-value 'data))))

(defmeth x :title (&optional (title nil set))
  (if set (setf (slot-value 'title) title))
  (slot-value 'title))

(defmeth x :data (&optional (data nil set))
  (if set (setf (slot-value 'data) data))
  (slot-value 'data))

(defmeth x :describe (&optional (stream t))
  (let ((title (send self :title))
	(data (send self :data)))
    (format stream \"This is ~a~%\" title)
    (format stream \"The sample mean is ~g~%\" (mean data))          
    (format stream \"The sample standard-deviation is ~g~%\"
	    (standard-deviation data))))

(send x :own-slots)
")
(defmeth x :describe (&optional (stream t))
  (format stream "This is ~a~%" (slot-value 'title))
  (format stream "The sample mean is ~g~%"
	  (mean (slot-value 'data)))
  (format stream "The sample standard-deviation is ~g~%"
	  (standard-deviation (slot-value 'data))))

(defmeth x :title (&optional (title nil set))
  (if set (setf (slot-value 'title) title))
  (slot-value 'title))

(defmeth x :data (&optional (data nil set))
  (if set (setf (slot-value 'data) data))
  (slot-value 'data))

(defmeth x :describe (&optional (stream t))
  (let ((title (send self :title))
	(data (send self :data)))
    (format stream "This is ~a~%" title)
    (format stream "The sample mean is ~g~%" (mean data))          
    (format stream "The sample standard-deviation is ~g~%"
	    (standard-deviation data))))

(send x :own-slots)

(message "
(send x :method-selectors)
")
(send x :method-selectors)

;; =================
(message "

               Section  6.2.4: Printing Objects

")
;; =================

(message "
(defmeth x :print (&optional (stream t))
(format stream \"#<~a>\" (send self :title)))

x
")
(defmeth x :print (&optional (stream t))
(format stream "#<~a>" (send self :title)))
x

;; =================
(message "

        Section  6.3.1: Constructing Prototypes and Instances

")
;; =================

(message "

	Prototypes are generated with the defproto macro:

(defproto data-set-proto '(data title))

(send data-set-proto :slot-value 'title \"a data set\")

(defmeth data-set-proto :describe (&optional (stream t))
  (format stream \"This is ~a~%\" (slot-value 'title))
  (format stream \"The sample mean is ~g~%\"
	  (mean (slot-value 'data)))
  (format stream \"The sample standard-deviation is ~g~%\"
	  (standard-deviation (slot-value 'data))))

(defmeth data-set-proto :title (&optional (title nil set))
  (if set (setf (slot-value 'title) title))
  (slot-value 'title))

")
(defproto data-set-proto '(data title))

(send data-set-proto :slot-value 'title "a data set")

(defmeth data-set-proto :describe (&optional (stream t))
  (format stream "This is ~a~%" (slot-value 'title))
  (format stream "The sample mean is ~g~%"
	  (mean (slot-value 'data)))
  (format stream "The sample standard-deviation is ~g~%"
	  (standard-deviation (slot-value 'data))))

(defmeth data-set-proto :title (&optional (title nil set))
  (if set (setf (slot-value 'title) title))
  (slot-value 'title))



(message "
(defmeth data-set-proto :data (&optional (data nil set))
  (if set (setf (slot-value 'data) data))
  (slot-value 'data))

(defmeth data-set-proto :print (&optional (stream t))
  (format stream \"#<~a>\" (send self :title)))

(setf x (send data-set-proto :new :data (chisq-rand 20 5)))

(send x :describe)
")
(defmeth data-set-proto :data (&optional (data nil set))
  (if set (setf (slot-value 'data) data))
  (slot-value 'data))

(defmeth data-set-proto :print (&optional (stream t))
  (format stream "#<~a>" (send self :title)))

(setf x (send data-set-proto :new :data (chisq-rand 20 5)))

(send x :describe)




(message "
(defmeth data-set-proto :isnew (data &key title)
  (send self :data data)
  (if title (send self :title title)))

(defmeth data-set-proto :plot () (histogram (send self :data)))

(defun make-data-set (x &key (title \"a data set\") (print t))
  (let ((object (send data-set-proto :new x :title title)))
    object))

(defun describe-data (data) (send data :describe))

(defun plot-data (data) (send data :plot))
")
(defmeth data-set-proto :isnew (data &key title)
  (send self :data data)
  (if title (send self :title title)))

(defmeth data-set-proto :plot () (histogram (send self :data)))

(defun make-data-set (x &key (title "a data set") (print t))
  (let ((object (send data-set-proto :new x :title title)))
    object))

(defun describe-data (data) (send data :describe))

(defun plot-data (data) (send data :plot))




;; =================
(message "

             Section  6.3.2: Prototypes Inheriting from Prototypes

")
;; =================

(message "
(defproto time-series-proto
  '(origin spacing) () data-set-proto)

(send time-series-proto :title)

(send time-series-proto :title \"a time series\")

(defmeth time-series-proto :origin (&optional (origin nil set))
(if set (setf (slot-value 'origin) origin))
(slot-value 'origin))

(defmeth time-series-proto :spacing (&optional (sp nil set))
(if set (setf (slot-value 'spacing) sp))
(slot-value 'spacing))
")
(defproto time-series-proto
  '(origin spacing) () data-set-proto)

(send time-series-proto :title)

(send time-series-proto :title "a time series")

(defmeth time-series-proto :origin (&optional (origin nil set))
(if set (setf (slot-value 'origin) origin))
(slot-value 'origin))

(defmeth time-series-proto :spacing (&optional (sp nil set))
(if set (setf (slot-value 'spacing) sp))
(slot-value 'spacing))





(message "
(send time-series-proto :origin 0)

(send time-series-proto :spacing 1)

(let* ((e (normal-rand 21))
       (data (+ (select e (iseq 1 20))
		(* .6 (select e (iseq 0 19))))))
  (setf y (send time-series-proto :new data)))

(send y :describe)
")
(send time-series-proto :origin 0)

(send time-series-proto :spacing 1)

(let* ((e (normal-rand 21))
       (data (+ (select e (iseq 1 20))
		(* .6 (select e (iseq 0 19))))))
  (setf y (send time-series-proto :new data)))

(send y :describe)




;; =================
(message "

        Section  6.3.3: Overriding and Modifying Inherited Methods

")
;; =================

(message "
(defmeth time-series-proto :plot ()
  (let* ((data (send self :data))
	 (time (+ (send self :origin)
		  (* (iseq (length data))
		     (send self :spacing)))))
    (plot-points time data)))

(defmeth time-series-proto :describe (&optional (stream t))
  (call-next-method stream)
  (format stream
	  \"The autocorrelation is ~g~%\"
	  (autocorrelation (send self :data))))

(defun autocorrelation (x)
  (let ((n (length x))
	(x (- x (mean x))))
    (/ (mean (* (select x (iseq 0 (- n 2)))
		(select x (iseq 1 (- n 1)))))
       (mean (* x x)))))

(send y :describe)
")
(defmeth time-series-proto :plot ()
  (let* ((data (send self :data))
	 (time (+ (send self :origin)
		  (* (iseq (length data))
		     (send self :spacing)))))
    (plot-points time data)))

(defmeth time-series-proto :describe (&optional (stream t))
  (call-next-method stream)
  (format stream
	  "The autocorrelation is ~g~%"
	  (autocorrelation (send self :data))))

(defun autocorrelation (x)
  (let ((n (length x))
	(x (- x (mean x))))
    (/ (mean (* (select x (iseq 0 (- n 2)))
		(select x (iseq 1 (- n 1)))))
       (mean (* x x)))))

(send y :describe)




(message "
(defmeth data-set-proto :isnew (data &rest agrs)
  (apply #'call-next-method :data data args))

(defmeth time-series-proto :describe (&optional (stream t))
  (call-method data-set-proto :describe stream)
  (format stream
	  \"The autocorrelation is ~g~%\"
	  (autocorrelation (send self :data))))
")
(defmeth data-set-proto :isnew (data &rest agrs)
  (apply #'call-next-method :data data args))

(defmeth time-series-proto :describe (&optional (stream t))
  (call-method data-set-proto :describe stream)
  (format stream
	  "The autocorrelation is ~g~%"
	  (autocorrelation (send self :data))))

;; =================
(message "

             Section  6.4.1: Creating Objects and Prototypes

")
;; =================

(message "
(setf x (make-object))

(send x :make-prototype 'data-set-proto '(data title))
")
(setf x (make-object))
(send x :make-prototype 'data-set-proto '(data title))

;; =================
(message "

              Section  6.4.2: Additional Methods and Functions

")
;; =================

(message "
(send time-series-proto :precedence-list)

(send time-series-proto :parents)

(kind-of-p time-series-proto data-set-proto)

(kind-of-p data-set-proto time-series-proto)

(send x :reparent time-series-proto)
")
(send time-series-proto :precedence-list)
(send time-series-proto :parents)
(kind-of-p time-series-proto data-set-proto)
(kind-of-p data-set-proto time-series-proto)
(send x :reparent time-series-proto)



;; =================
(message "

                 Section  6.4.3: Shared Slots

") 
;; =================

(message "

(defproto myproto () '(instances))

(defmeth myproto :new (&rest args)
  (let ((object (apply #'call-next-method args)))
    (setf (slot-value 'instances)
	  (cons object (slot-value 'instances)))
    object))

(defmeth myproto :dispose (object)
  (setf (slot-value 'instances)
	(remove object (slot-value 'instances))))

")

(defproto myproto () '(instances))

(defmeth myproto :new (&rest args)
  (let ((object (apply #'call-next-method args)))
    (setf (slot-value 'instances)
	  (cons object (slot-value 'instances)))
    object))

(defmeth myproto :dispose (object)
  (setf (slot-value 'instances)
	(remove object (slot-value 'instances))))

;; =================
(message "

                 Section  6.4.4: The Object Documentation System

") 
;; =================

(message "
(send data-set-proto :documentation :title)

(send data-set-proto :documentation 
      :title \"sets or returns the title\")

(send data-set-proto :help :title)

(send data-set-proto :doc-topics)

(send data-set-proto :help)

(send data-set-proto :documentation
      'proto \"A generic data set.\")

(send data-set-proto :help)
")
(send data-set-proto :documentation :title)
(send data-set-proto :documentation 
      :title "sets or returns the title")
(send data-set-proto :help :title)
(send data-set-proto :doc-topics)
(send data-set-proto :help)
(send data-set-proto :documentation
      'proto "A generic data set.")
(send data-set-proto :help)

;; =================
(message "

                    Section  6.4.5: Saving Objects

") 
;; =================

(message "
(defmeth data-set-proto :save ()
  `(send data-set-proto :new
	 ',(send self :data)
	 :title ',(send self :title)))
(send x :save)
")
(defmeth data-set-proto :save ()
  `(send data-set-proto :new
	 ',(send self :data)
	 :title ',(send self :title)))
(send x :save)

;; =================
(message "

               Section  6.5.1: Compound Data Objects

") 
;; =================

(message "
(defproto point-proto '(x y) '() compound-data-proto)

(defmeth point-proto :x (&optional (x nil set))
  (if set (setf (slot-value 'x) x))
  (slot-value 'x))

(defmeth point-proto :y (&optional (y nil set))
  (if set (setf (slot-value 'y) y))
  (slot-value 'y))

(defmeth point-proto :isnew (x y)
  (send self :x x)
  (send self :y y))

(defmeth point-proto :data-length () 2)
")
(defproto point-proto '(x y) '() compound-data-proto)

(defmeth point-proto :x (&optional (x nil set))
  (if set (setf (slot-value 'x) x))
  (slot-value 'x))

(defmeth point-proto :y (&optional (y nil set))
  (if set (setf (slot-value 'y) y))
  (slot-value 'y))

(defmeth point-proto :isnew (x y)
  (send self :x x)
  (send self :y y))

(defmeth point-proto :data-length () 2)

(message "
(defmeth point-proto :data-seq ()
  (list (send self :x) (send self :y)))

(defmeth point-proto :make-data (seq)
  (send point-proto :new (select seq 0) (select seq 1)))

(defmeth point-proto :print (&optional (stream t))
  (format stream
	  \"#<a point located at x = ~d and y = ~d>\"
	  (send self :x)
	  (send self :y)))

(setf a (send point-proto :new 3 5))
(+ a a)
(* a 2)
(log a)
")
(defmeth point-proto :data-seq ()
  (list (send self :x) (send self :y)))
(defmeth point-proto :make-data (seq)
  (send point-proto :new (select seq 0) (select seq 1)))
(defmeth point-proto :print (&optional (stream t))
  (format stream
	  "#<a point located at x = ~d and y = ~d>"
	  (send self :x)
	  (send self :y)))
(setf a (send point-proto :new 3 5))
(+ a a)
(* a 2)
(log a)

;; =================
(message "

	Section  6.5.2: Linear Regression Models

") 
;; =================

#|
(defproto regression-model-proto
  '(x y intercept weights included
      predictor-names response-name case-labels
      sweep-matrix basis
      total-sum-of-squares
      residual-sum-of-squares))

This must not be the standard regression-model-proto - or if it is, we don't
want to redefine it here, as it will mess up future regression-models.
|#

;; =================
(message "

	Section  6.5.3: Nonlinear Regression Models

") 
;; =================

(message "
(defproto nreg-model-proto
  '(mean-function theta-hat epsilon count-limit verbose)
  nil
  regression-model-proto)
")
(defproto nreg-model-proto
  '(mean-function theta-hat epsilon count-limit verbose)
  nil
  regression-model-proto)

;; =================
(message "

	Section  6.6: Multiple Inheritance

") 
;; =================

(message "
(defproto food)
(defproto spice () () food)
(defproto fruit () () food)
(defproto cinnamon () () spice)
(defproto apple () () fruit)
(defproto pie () () (list apple cinnamon))

(send pie :precedence-list)

;; ael intentional error:
\"(defproto spice-cake () () (list spice pie))\"
\"Error: inconsistent precedence order\"
\"Happened in: #<FSubr-DEFPROTO: #10c128>\"

(defproto spice-cake () () (list pie spice))
")
(defproto food)
(defproto spice () () food)
(defproto fruit () () food)
(defproto cinnamon () () spice)
(defproto apple () () fruit)
(defproto pie () () (list apple cinnamon))
(send pie :precedence-list)
"(defproto spice-cake () () (list spice pie))"
"Error: inconsistent precedence order"
"Happened in: #<FSubr-DEFPROTO: #10c128>"
(defproto spice-cake () () (list pie spice))
;; =================
(message "

	Section  6.8.1: A Rectangular Data Set

") 
;; =================

(message "
(defproto rect-data-proto '(vlabels clabels) () data-set-proto)

(defmeth rect-data-proto :isnew
  (data &key title variable-labels case-labels)
  (let ((n (length (first data)))
	(m (length data)))
    (send self :data data)
    (if title (send self :title title))
    (send self :variable-labels
	  (if variable-labels
	      variable-labels
	    (mapcar #'(lambda (x) (format nil \"X~a\" x))
		    (iseq 0 (- m 1)))))
    (send self :case-labels
	  (if case-labels
	      case-labels
	    (mapcar #'(lambda (x) (format nil \"~a\" x))
		    (iseq 0 (- n 1)))))))
")
(defproto rect-data-proto '(vlabels clabels) () data-set-proto)

(defmeth rect-data-proto :isnew
  (data &key title variable-labels case-labels)
  (let ((n (length (first data)))
	(m (length data)))
    (send self :data data)
    (if title (send self :title title))
    (send self :variable-labels
	  (if variable-labels
	      variable-labels
	    (mapcar #'(lambda (x) (format nil "X~a" x))
		    (iseq 0 (- m 1)))))
    (send self :case-labels
	  (if case-labels
	      case-labels
	    (mapcar #'(lambda (x) (format nil "~a" x))
		    (iseq 0 (- n 1)))))))




(message "
(defmeth rect-data-proto :variable-labels
  (&optional (labels nil set))
  (if set (setf (slot-value 'vlabels) labels))
  (slot-value 'vlabels))

(defmeth rect-data-proto :case-labels
  (&optional (labels nil set))
  (if set (setf (slot-value 'clabels) labels))
  (slot-value 'clabels))

(send rect-data-proto :title \"a rectangular data set\")

(load-data stackloss)
(setf stack
      (send rect-data-proto :new (list air temp conc loss)
	    :variable-labels
	    (list \"Air\" \"Temp\" \"Conc\" \"Loss\")))

(defmeth rect-data-proto :plot ()
  (let ((vars (send self :data))
	(labels (send self :variable-labels)))
    (case (length vars)
	  (1 (histogram vars :variable-labels labels))
	  (2 (plot-points vars :variable-labels labels))
	  (3 (spin-plot vars :variable-labels labels))
	  (t (scatterplot-matrix vars :variable-labels labels)))))

(send stack :describe)

(send stack :plot)
")
(defmeth rect-data-proto :variable-labels
  (&optional (labels nil set))
  (if set (setf (slot-value 'vlabels) labels))
  (slot-value 'vlabels))
(defmeth rect-data-proto :case-labels
  (&optional (labels nil set))
  (if set (setf (slot-value 'clabels) labels))
  (slot-value 'clabels))
(send rect-data-proto :title "a rectangular data set")
(load-data stackloss)
(setf stack
      (send rect-data-proto :new (list air temp conc loss)
	    :variable-labels
	    (list "Air" "Temp" "Conc" "Loss")))
(defmeth rect-data-proto :plot ()
  (let ((vars (send self :data))
	(labels (send self :variable-labels)))
    (case (length vars)
	  (1 (histogram vars :variable-labels labels))
	  (2 (plot-points vars :variable-labels labels))
	  (3 (spin-plot vars :variable-labels labels))
	  (t (scatterplot-matrix vars :variable-labels labels)))))
(send stack :describe)
(send stack :plot)




;; =================
(message "

	Section  6.8.2: An Alternate Data Representation

") 
;; =================

(message "
(defproto observation-proto '(label))

(defmeth observation-proto :isnew (&rest args)
  (setf (slot-value 'label) (string (gensym \"OBS-\")))
  (apply #'call-next-method args))

(defmeth observation-proto :label () (slot-value 'label))

(defproto stack-obs-proto
  '(air temp conc loss) () observation-proto)

")
(defproto observation-proto '(label))

(defmeth observation-proto :isnew (&rest args)
  (setf (slot-value 'label) (string (gensym "OBS-")))
  (apply #'call-next-method args))

(defmeth observation-proto :label () (slot-value 'label))

(defproto stack-obs-proto
  '(air temp conc loss) () observation-proto)




(message "
(defmeth stack-obs-proto :air () (slot-value 'air))

(defmeth stack-obs-proto :temp () (slot-value 'temp))

(defmeth stack-obs-proto :conc () (slot-value 'conc))

(defmeth stack-obs-proto :loss () (slot-value 'loss))

(defmeth stack-obs-proto :log-loss () (log (send self :loss)))

(defproto data-set-proto '(data))
")
(defmeth stack-obs-proto :air () (slot-value 'air))

(defmeth stack-obs-proto :temp () (slot-value 'temp))

(defmeth stack-obs-proto :conc () (slot-value 'conc))

(defmeth stack-obs-proto :loss () (slot-value 'loss))

(defmeth stack-obs-proto :log-loss () (log (send self :loss)))

(defproto data-set-proto '(data))




(message "
(defmeth data-set-proto :values (var)
  (mapcar #'(lambda (x) (send x var))
	  (slot-value 'data)))

(defmeth data-set-proto :plot (&rest args)
  (let ((vals (mapcar #'(lambda (var) (send self :values var))
		      args))
	(labels (send self :values :label))
	(vlabs (mapcar #'string args)))
    (case (length args)
	  (0 (error \"too few arguments\"))
	  (1 (histogram vals 
			:point-labels labels
			:variable-labels vlabs))
	  (2 (plot-points vals
			  :point-labels labels
			  :variable-labels vlans))
	  (3 (spin-plot vals
			:point-labels labels
			:variable-labels vlabs))
	  (t (scatterplot-matrix vals 
				 :point-labels labels
				 :variable-labels vlabs)))))
")
(defmeth data-set-proto :values (var)
  (mapcar #'(lambda (x) (send x var))
	  (slot-value 'data)))

(defmeth data-set-proto :plot (&rest args)
  (let ((vals (mapcar #'(lambda (var) (send self :values var))
		      args))
	(labels (send self :values :label))
	(vlabs (mapcar #'string args)))
    (case (length args)
	  (0 (error "too few arguments"))
	  (1 (histogram vals 
			:point-labels labels
			:variable-labels vlabs))
	  (2 (plot-points vals
			  :point-labels labels
			  :variable-labels vlans))
	  (3 (spin-plot vals
			:point-labels labels
			:variable-labels vlabs))
	  (t (scatterplot-matrix vals 
				 :point-labels labels
				 :variable-labels vlabs)))))




(message "
(defmeth data-set-proto :describe (var)
  (let ((vals (send self :values var)))
    (format t \"Variable Name: ~a~%\" var)
    (format t \"Mean: ~g~%\" (mean vals))
    (format t \"Median: ~g~%\" (median vals))
    (format t \"Standard Deviation: ~g~%\" (standard-deviation vals))))

(setf stack-list
      (mapcar 
       #'(lambda (air temp conc loss label)
	   (send stack-obs-proto :new
		 :air air
		 :temp temp
		 :conc conc
		 :loss loss
		 :label (format nil \"~d\" label)))
       air temp conc loss (Iseq 0 20)))

(setf stack-loss (send data-set-proto :new :data stack-list))

(send stack-loss :describe :loss)

(send stack-loss :describe :log-loss)

(send stack-loss :plot :log-loss :air :temp)
")
(defmeth data-set-proto :describe (var)
  (let ((vals (send self :values var)))
    (format t "Variable Name: ~a~%" var)
    (format t "Mean: ~g~%" (mean vals))
    (format t "Median: ~g~%" (median vals))
    (format t "Standard Deviation: ~g~%" (standard-deviation vals))))
(setf stack-list
      (mapcar 
       #'(lambda (air temp conc loss label)
	   (send stack-obs-proto :new
		 :air air
		 :temp temp
		 :conc conc
		 :loss loss
		 :label (format nil "~d" label)))
       air temp conc loss (Iseq 0 20)))
(setf stack-loss (send data-set-proto :new :data stack-list))
(send stack-loss :describe :loss)
(send stack-loss :describe :log-loss)
(send stack-loss :plot :log-loss :air :temp)
