(message "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             Derivatives: diff
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        You may need to look at a derivative from time to time: the \"diff\"
command is useful for that:

        (func f(x) x ^ 3 - 2 * x + 3) ; f(x)=x^3-2*x+3
        (diff f x) ; What's its derivative look like?
")
        (func f(x) x ^ 3 - 2 * x + 3) ; f(x)=x^3-2*x+3
        (diff f x) ; What's its derivative look like?
(message "")
(message "
        If you want to use the derivative function, then you need to give it a
name. diff takes an optional argument, the name of the derivative:

        (diff f x fp) ; define the derivative function fp.

Then we can plot both of them on the same plot:

        (new-plot) ;; initializes the plotting environment
        (title \"Plot of f's Derivative\")
        (plot f fp -3 2)
")
        (new-plot)
        (diff f x fp) ; define the derivative function fp.
        (title "Plot of f's Derivative")
        (plot f fp -3 2)
(message "")
(message "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             Derivative plots
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        The special command diffplot plots f and some derivatives:

        (title \"Derivative plots\")
        (diffplot f -3 2)

This command will plot f and its derivative on the interval (-3 2).
")
        (title "Derivative plots")
        (diffplot f -3 2)

(message "

        If you want more derivatives, add an additional number (up to 3):

        (diffplot f -3 2 2) ; that last \"2\" means the second derivative, too.
        (diffplot f -3 2 3) ; that last \"3\" means the second and third
                            ; derivatives.
")
        (diffplot f -3 2 2)
        (diffplot f -3 2 3)

(message "")
(message "
        Numerical derivatives can also be defined, when you have an extremely
complicated function (or one of your own, which xlisp-stat has not been told to
recognize):

       (ndiff f fp &optional (delta-x 1e-6))
  
computes a numerical derivative function fp of a function f. It uses a forward
difference. delta-x can be adjusted for your particular function: it is the
delta-x in the approximation with a difference quotient.

       (new-plot)
       (title \"Numerical differentiation\")
       (func f(x) erf x)
       (ndiff f fp)
       (plot f fp)
")
       (new-plot)
       (title "numerical differentiation")
       (func f(x) erf x)
       (ndiff f fp)
       (plot f fp)
(message "")

(message "
       Diff will also compute partial derivatives (if you're in higher
dimensions):

       (func f(x y) x ^ 2 + sin(y))
       (diff f x)
       (diff f y)
       (diff f x fx) ;; this defines function fx as the partial wrt x
       (diff f y fy)

       (title \"Partial differentiation\")
       (plot fx fy)
")
       (func f(x y) x ^ 2 + sin(y))
       (diff f x)
       (diff f y)
       (diff f x fx) ;; this defines function fx as the partial wrt x
       (diff f y fy)
       (title "Partial differentiation")
       (plot fx fy)
(message "")
(message "
       Diff understands certain things which make your life easier: for
example, if you define functions f and g, and then h by their composition, then
h will be differentiated properly:

       (func f(x) sin x)
       (func g(x) x ^ 2)
       (func h(x) (f (g x)))

       (diff h x hp)
       (title \"A Composition\")
       (plot h hp -3 3)
")
       (func f(x) sin x)
       (func g(x) x ^ 2)
       (func h(x) (f (g x)))

       (diff h x hp)
       (title "A Composition")
       (plot h hp -3 3)
(message "")
(message "
       Similarly for their product, sum, quotient, etc.

       (func h(x) (f x) * (g x))
       (diff h x hp)
       (title \"A Product\")
       (plot h hp)
")
       (func h(x) (f x) * (g x))
       (diff h x hp)
       (title "A Product")
       (plot h hp)
(message "")

(message "
        If you're interested in computing higher derivatives, then you'll be
interested in the diff-n command: it allows you to decide the order that you
need:

 (func f(x) x ^ 5)
 (diff-n f x 1)
 (diff-n f x 3)
 (diff-n f x 5)
 (diff-n f x 6)
")
 (func f(x) x ^ 5)
 (diff-n f x 1)
 (diff-n f x 3)
 (diff-n f x 5)
 (diff-n f x 6)
(message "")
(message "
        Diff-n also handles partial derivatives of higher order, although you
can not yet do mixed partials this way - you'll have to use diff:

 (func f(x y) x ^ 2 + sin(y))
 (diff-n f x 2)
 (diff-n f y 2)
 (diff-n f y 2 fyy) ;; second partial of f with respect to y
 (what fyy)
")
 (func f(x y) x ^ 2 + sin(y))
 (diff-n f x 2)
 (diff-n f y 2)
 (diff-n f y 2 fyy) ;; second partial of f with respect to y
 (what fyy)
(message "")
