(message "

As an example, we can examine a data set that records the number of months
prior to an interview when individuals remember a stressful event (originally
from Haberman, [2, p. 2,]): 


> (def months-before (iseq 1 18))
MONTHS-BEFORE
> (def event-counts '(15 11 14 17 5 11 10 4 8 10 7 9 11 3 6 1 1 4))
EVENTS-RECALLED
")

(def months-before (iseq 1 18))
(def event-counts '(15 11 14 17 5 11 10 4 8 10 7 9 11 3 6 1 1 4))

(message "
The data are multinomial, and we can fit a log-linear Poisson model to see if
there is any time trend: 

> (def m (poissonreg-model months-before event-counts))
Iteration 1: deviance = 26.3164
Iteration 2: deviance = 24.5804
Iteration 3: deviance = 24.5704
Iteration 4: deviance = 24.5704

Weighted Least Squares Estimates:

Constant                  2.80316   (0.148162)
Variable 0             -0.0837691   (0.0167996)

Scale taken as:                 1
Deviance:                 24.5704
Number of cases:               18
Degrees of freedom:            16
")
(def m (poissonreg-model months-before event-counts))

(message "
")
(message "
Residuals for the fit can be obtained using the :residuals message: 

> (send m :residuals)
(-0.0439191 -0.790305 ...)
")
(send m :residuals)
(message "")

(message "
A residual plot can be obtained using 

(send m :plot-residuals)
")
(send m :plot-residuals)

(message "
The :fit-values message returns , the linear predictor without any
offset. The :fit-means message returns fitted mean response values.
Thus the expression 

(let ((p (plot-points months-before event-counts)))
  (send p :add-lines months-before (send m :fit-means)))

constructs a plot of raw counts and fitted means against time. 
")
(let ((p (plot-points months-before event-counts)))
  (send p :add-lines months-before (send m :fit-means)))

(message "")

(message "
To illustrate fitting binomial models, we can use the leukemia survival data
of Feigl and Zelen [4, Section 2.8.3,] with the survival time converted to a
one-year survival indicator: 

> (load-data \"leukemia\")
> (def surv-1 (if-else (> times-pos 52) 1 0))
SURV-1
> surv-1
(1 1 1 1 0 1 1 0 0 1 1 0 0 0 0 0 1)
")
(load-data "leukemia")
(def surv-1 (if-else (> times-pos 52) 1 0))
surv-1
(message "")

(message "
The dependent variable is the base 10 logarithm of the white blood cell counts
divided by 10,000: 

> transformed-wbc-pos
(-1.46968 -2.59027 -0.84397 -1.34707 -0.510826 0.0487902 0 0.530628 -0.616186
 -0.356675 -0.0618754 1.16315 1.25276 2.30259 2.30259 1.64866 2.30259)
")
transformed-wbc-pos

(message "
A binomial model for these data can be constructed by 

> (def lk (binomialreg-model transformed-wbc-pos surv-1 1))
Iteration 1: deviance = 18.2935
Iteration 2: deviance = 18.0789
Iteration 3: deviance = 18.0761
Iteration 4: deviance = 18.0761

Weighted Least Squares Estimates:

Constant                 0.372897   (0.590934)
Variable 0              -0.985803   (0.508426)

Scale taken as:                 1
Deviance:                 18.0761
Number of cases:               17
Degrees of freedom:            15
")
(def lk (binomialreg-model transformed-wbc-pos surv-1 1))
(message "")

(message "
This model uses the logit link, the canonical link for the binomial
distribution. As an alternative, the expression 

(binomialreg-model transformed-wbc-pos surv-1 1 :link probit-link)

returns a model using a probit link. 
")
(binomialreg-model transformed-wbc-pos surv-1 1 :link probit-link)


(message "
The :cooks-distances message helps to highlight the last observation
for possible further examination: 

> (send lk :cooks-distances)
(0.0142046 0.00403243 0.021907 0.0157153 0.149394 0.0359723 0.0346383
 0.0450994 0.174799 0.0279114 0.0331333 0.0347883 0.033664 0.0170441 
 0.0170441 0.0280411 0.757332)
")
(send lk :cooks-distances)
(message "")

(message "
This observation also stands out in the plot produced by 

(send lk :plot-bayes-residuals)
")
(send lk :plot-bayes-residuals)
(message "")

(message "
Four functions are provided to help construct indicator vectors for categorical
variables. As an illustration, a data set used by Bishop, Fienberg, and Holland
examines the relationship between occupational classifications of fathers and
sons. The classes are

a - professional, high-administrative
s - managarial,executive, high supervisory
i - low inspectional, supervisory
n - routine nonmanual, skilled manual
u - semi- and unskilled manual

We can set up the occupation codes as 

(def occupation '(a s i n u))
")
(def occupation '(a s i n u))

(message "
and construct the son's and father's code vectors for entering the data row by
row as

(def son (repeat occupation 5))
(def father (repeat occupation (repeat 5 5)))

The counts can then be entered as 

(def counts '(50  45   8  18   8 
              28 174  84 154  55 
              11  78 110 223  96
              14 150 185 714 447
               3  42  72 320 411))
")
(def son (repeat occupation 5))
(def father (repeat occupation (repeat 5 5)))
(def counts '(50  45   8  18   8 
              28 174  84 154  55 
              11  78 110 223  96
              14 150 185 714 447
               3  42  72 320 411))
(message "")

(message "
To fit an additive log-linear model, we need to construct level indicators.
This can be done using the function indicators: 

> (indicators son)
((0 1 0 0 h the fifth
levels of the variable son. By default, the first level is dropped. To
obtain indicators for all five levels, we can supply the :drop-first
keyword with value nil:

> (indicators son :drop-first nil)
((1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0)
 (0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0)
 (0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0)
 (0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0)
 (0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1))
")
(indicators son :drop-first nil)

(message "

To produce a readable summary of the fit, we also need some labels: 

> (level-names son :prefix 'son)
(\"SON(S)\" \"SON(I)\" \"SON(N)\" \"SON(U)\")

By default, this function also drops the first level. This can again be changed by supplying the
:drop-first keyword argument as nil: 

> (level-names son :prefix 'son :drop-first nil)
(\"SON(A)\" \"SON(S)\" \"SON(I)\" \"SON(N)\" \"SON(U)\")
")
(level-names son :prefix 'son)
(level-names son :prefix 'son :drop-first nil)
(level-names son :prefix "Son")

(message "
Using indicator variables and level labels, we can now fit an additive model as 

> (def mob-add
       (poissonreg-model
        (append (indicators son) (indicators father)) counts
        :predictor-names (append (level-names son :prefix 'son)
                                 (level-names father :prefix 'father))))

Iteration 1: deviance = 1007.97
Iteration 2: deviance = 807.484
Iteration 3: deviance = 792.389
Iteration 4: deviance = 792.19
Iteration 5: deviance = 792.19

Weighted Least Squares Estimates:

Constant                  1.36273   (0.130001)
SON(S)                    1.52892   (0.10714)
SON(I)                    1.46561   (0.107762)
SON(N)                    2.60129   (0.100667)
SON(U)                    2.26117   (0.102065)
FATHER(S)                 1.34475   (0.0988541)
FATHER(I)                 1.39016   (0.0983994)
FATHER(N)                 2.46005   (0.0917289)
FATHER(U)                 1.88307   (0.0945049)

Scale taken as:                 1
Deviance:                  792.19
Number of cases:               25
Degrees of freedom:            16
")
(def mob-add
       (poissonreg-model
        (append (indicators son) (indicators father)) counts
        :predictor-names (append (level-names son :prefix 'son)
                                 (level-names father :prefix 'father))))
(message "")

(message "
Examining the residuals using 

(send mob-add :plot-residuals)

shows that the first cell is an outlier -- the model does not fit this cell
well.
")
(send mob-add :plot-residuals)

(message "
To fit a saturated model to these data, we need the cross products of the
indicator variables and also a corresponding set of labels. The indicators are
produced with the cross-terms function 

> (cross-terms (indicators son) (indicators father))
((0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
 (0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0)
 ...)
")
(cross-terms (indicators son) (indicators father))

(message "")
(message "
and the names with the cross-names function: 

> (cross-names (level-names son :prefix 'son)
               (level-names father :prefix 'father))
(\"SON(S).FATHER(S)\" \"SON(S).FATHER(I)\" ...)

")
(cross-names (level-names son :prefix 'son)
               (level-names father :prefix 'father))
(message "")


(message "
The saturated model can now be fit by 

> (let ((s (indicators son))
        (f (indicators father))
        (sn (level-names son :prefix 'son))
        (fn (level-names father :prefix 'father)))
    (def mob-sat
         (poissonreg-model (append s f (cross-terms s f)) counts 
                           :predictor-names
                           (append sn fn (cross-names sn fn)))))

Iteration 1: deviance = 5.06262e-14
Iteration 2: deviance = 2.44249e-15

Weighted Least Squares Estimates:

Constant                  3.91202   (0.141421)
SON(S)                  -0.105361   (0.20548)
SON(I)                   -1.83258   (0.380789)
SON(N)                   -1.02165   (0.274874)
SON(U)                   -1.83258   (0.380789)
FATHER(S)               -0.579818   (0.236039)
FATHER(I)                -1.51413   (0.33303)
FATHER(N)                -1.27297   (0.302372)
FATHER(U)                -2.81341   (0.594418)
SON(S).FATHER(S)          1.93221   (0.289281)
SON(S).FATHER(I)          2.06417   (0.382036)
SON(S).FATHER(N)          2.47694   (0.346868)
SON(S).FATHER(U)          2.74442   (0.631953)
SON(I).FATHER(S)          2.93119   (0.438884)
SON(I).FATHER(I)          4.13517   (0.494975)
SON(I).FATHER(N)          4.41388   (0.470993)
SON(I).FATHER(U)          5.01064   (0.701586)
SON(N).FATHER(S)           2.7264   (0.343167)
SON(N).FATHER(I)          4.03093   (0.41346)
SON(N).FATHER(N)          4.95348   (0.385207)
SON(N).FATHER(U)          5.69136   (0.641883)
SON(U).FATHER(S)          2.50771   (0.445978)
SON(U).FATHER(I)          3.99903   (0.496312)
SON(U).FATHER(N)          5.29608   (0.467617)
SON(U).FATHER(U)          6.75256   (0.693373)

Scale taken as:                 1
Deviance:              3.37508e-14
Number of cases:               25
Degrees of freedom:             0
")
(let ((s (indicators son))
      (f (indicators father))
      (sn (level-names son :prefix 'son))
      (fn (level-names father :prefix 'father)))
  (def mob-sat
       (poissonreg-model (append s f (cross-terms s f)) counts 
			 :predictor-names
			 (append sn fn (cross-names sn fn)))))
(message "
     All done!
")


