(message "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             Monte-carlo integration:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	In monte-carlo integration of a function f, we rely on a good sampling
of the domain of f, and on the mean-value theorem for integrals: that the
average value of the function multiplied by the length/area/volume/etc. gives a
good approximation to the value of the integral.

	For a univariate function, the calculation of the integral of
the following function is carried out from 0 to 1. Note that we're

(a) using rand.u to provide us with 100 values on the interval [0,1],
(b) evaluating f at those values, and
(c) generating the mean of those values. Since we're on the interval [0,1], we
    don't need to divide by the interval length: it's one.

(func f(x) sqrt (1 - x ^ 2))
(calc (mean (f (rand.u 100))))
(simpson f 0 1) ;; compare to the \"correct\" numerical value:
")
(func f(x) sqrt (1 - x ^ 2))
(calc (mean (f (rand.u 100))))
(simpson f 0 1)
(message "")

(message "
	If we wanted to do the same problem, but on the interval [.4,.7], say,
then we'd need to take our random numbers from [.4,.7]:

(def xvals (calc (.4 + (.7 - .4) * (rand.u 100))))
(calc ((mean (f xvals)) * (.7 - .4)))
(simpson f .4 .7) ;; compare to the \"correct\" numerical value:
")
(def xvals (calc (.4 + (.7 - .4) * (rand.u 100))))
(calc ((mean (f xvals)) * (.7 - .4)))
(simpson f .4 .7) ;; compare to the \"correct\" numerical value:
(message "")

(message "
	Here's a little simulation, in which we compute the monte-carlo
integral for f from 0 to 1 100 times, and look at the histogram of the results.
(The mean of the trials gives us the value of a monte-carlo integration with
10000 samples!)

(setq trials (calc (sum (f (rand.u 100))) / 100))
(dotimes (i 100)
         (setq trials (combine trials (/ (sum (f (rand.u 100)) ) 100)))
         )
(mean trials)
(sd trials)
(histogram trials)
")
(setq trials (calc (sum (f (rand.u 100))) / 100))
(dotimes (i 100)
         (setq trials (combine trials (/ (sum (f (rand.u 100)) ) 100)))
         )
(mean trials)
(sd trials)
(histogram trials)
(message "")

(message "
	The situation is essentially the same for multivariate function
integration: here's an example: integration of f over [0,1]x[0,1] (of area 1):

(func f(x y ) exp (- (x ^ 2 + y ^ 2)))
(calc (mean (f (rand.u 100) (rand.u 100))))
(integrate f 0 1 0 1 10) ;; compare this to a rectangle rule approximation:
")
(func f(x y ) exp (- (x ^ 2 + y ^ 2)))
(calc (mean (f (rand.u 100) (rand.u 100))))
(integrate f 0 1 0 1 10)
