(message "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                     Object-Oriented Programming
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        One very attractive feature of xlisp-stat is that it permits
object-oriented programming. I'll demonstrate with an example, which I have
used profitably as a template in my own work. I stole this from 
	
       Original File name:      tools.lsp, extracted from XlispOOP.doc
       Original Author:         Tim Mikkelsen

       Xlisp-stat translator:   Andy Long, Fall 1995

")

(message "

        This example is a sort of inventory problem: you've got some tools, and
you want to pack them with descriptive features. You also want to know who has
them, how many of them there are, where they are, etc.

")

(message "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;       Define the superclasses and classes
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; make TOOLS superclass
;       with a different :ISNEW method
;       added methods are :BORROW and :RETURN
;       class variables are     NUMBER          contains # of tool instances
;                               ACTIVE-LIST     contains list of current objects
;       instance variables are  POWER           list - (AC BATTERY HAND)
;                               MOVEABLE        CAN-CARRY or CAN-ROLL or FIXED
;                               OPERATIONS      list
;                               MATERIAL        list - (WOOD METAL PLASTIC ...)
;                               PIECES          list
;                               LOCATION        HOME or person's name
")

(message "
        The tools proto: we define proto-types, which are the main objects,
using the defproto command: we give a name, followed by a list of attributes.

(defproto tools
  '(power 
    moveable 
    operations 
    material 
    pieces 
    location) 
  '(number active-list)
  nil
  \"A Tools Class, for demonstration purposes.\"
  )
(send tools :print)
")
(defproto tools
  '(power 
    moveable 
    operations 
    material 
    pieces 
    location) 
  '(number active-list)
   nil
   "A Tools Class, for demonstration purposes."
  )
(send tools :print)


(message "
        The tools methods: we define methods that the objects should be able to
interpret and process. These ones are methods to either create a slot-value
(for  number, or active-list) or to ask what those values are:

(defmeth tools :number (&optional new)
  (when new (setf (slot-value 'number) new))
  (slot-value 'number)
  )
(send tools :number 0)

(defmeth tools :active-list (&optional new)
  (when new (setf (slot-value 'active-list) new))
  (slot-value 'active-list)
  )
(send tools :active-list '()) ; initializes active-list

      The preceding 'send' statements initialize the tool class. We have to do
that before using isnew, the initialization routine, or else! I don't know how
to initialize the proto on the fly.
")
(defmeth tools :number (&optional new)
  (when new (setf (slot-value 'number) new))
  (slot-value 'number))
(send tools :number 0)

(defmeth tools :active-list (&optional new)
  (when new (setf (slot-value 'active-list) new))
  (slot-value 'active-list))
(send tools :active-list '()) ; initializes active-list

(message "")

(message "
        A couple more tool methods:

(defmeth tools :isnew (&optional objct)
  (setf (slot-value 'number) (1+ (slot-value 'number)))
  (setf (slot-value 'active-list) (cons objct (slot-value 'active-list)))
  (setf (slot-value 'location) 'home)
  objct)
  
(defmeth tools :borrow (by-whom)
  (let (
	(slot-value (slot-value 'location))
	)
    (if (eq slot-value 'home) ; then it can be checked out;
	(setf (slot-value 'location) by-whom) ; otherwise,
      (let ()
	(format t \"~%It's being used by ~s; try later.~%\" slot-value)
		slot-value))))
  
(defmeth tools :return ()
  (if (eq (slot-value 'location) 'home)
      (let ()
	(format t \"~%It's on the shelf.~%\")
	'home)
    (setf (slot-value 'location) 'home))
  )
")
(defmeth tools :isnew (&optional objct)
  (setf (slot-value 'number) (1+ (slot-value 'number)))
  (setf (slot-value 'active-list) (cons objct (slot-value 'active-list)))
  (setf (slot-value 'location) 'home)
  objct)
  
(defmeth tools :borrow (by-whom)
  (let (
	(slot-value (slot-value 'location))
	)
    (if (eq slot-value 'home) ; then it can be checked out;
	(setf (slot-value 'location) by-whom) ; otherwise,
      (let ()
	(format t "~%It's being used by ~s; try later.~%"
		slot-value)
	slot-value))))

(defmeth tools :return ()
  (if (eq (slot-value 'location) 'home)
      (let ()
	(format t "~%It's on the shelf.~%")
	'home)
    (setf (slot-value 'location) 'home))
  )

(message "")

(message "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; make HAND-TOOLS proto: now we'll make special types of tools
;       with a different :ISNEW method
;       new instance variable   WEIGHT          <number> of pounds
;       the rest of the slots are inherited from TOOLS: this is one of the nice
;       features of object-oriented programming!
; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto hand-tools '(weight) '(number tool-list) tools)

(defmeth hand-tools :number (&optional new)
  (when new (setf (slot-value 'number) new))
  (slot-value 'number))
(send hand-tools :number 0)

(defmeth hand-tools :tool-list (&optional new)
  (when new (setf (slot-value 'tool-list) new))
  (slot-value 'tool-list))
(send hand-tools :tool-list '())
")
(defproto hand-tools '(weight) '(number tool-list) tools)

(defmeth hand-tools :number (&optional new)
  (when new (setf (slot-value 'number) new))
  (slot-value 'number))
(send hand-tools :number 0)

(defmeth hand-tools :tool-list (&optional new)
  (when new (setf (slot-value 'tool-list) new))
  (slot-value 'tool-list))
(send hand-tools :tool-list '())

(message "")

(message "
        Another method: this is :isnew, which creates a new 'instance' of
hand-tools:

(defmeth hand-tools :isnew (pow op mat parts w-in name)
  (setf (slot-value 'power) pow)
  (setf (slot-value 'moveable) 'can-carry)
  (setf (slot-value 'operations) op)
  (setf (slot-value 'material) mat)
  (setf (slot-value 'pieces) parts)
  (setf (slot-value 'weight) w-in)
  (setf (slot-value 'location) 'home)
  (send hand-tools :number (1+ (send hand-tools :number)))
  (send hand-tools :tool-list (append (send hand-tools :tool-list) name))
  )
")
(defmeth hand-tools :isnew (pow op mat parts w-in name)
  (setf (slot-value 'power) pow)
  (setf (slot-value 'moveable) 'can-carry)
  (setf (slot-value 'operations) op)
  (setf (slot-value 'material) mat)
  (setf (slot-value 'pieces) parts)
  (setf (slot-value 'weight) w-in)
  (setf (slot-value 'location) 'home)
  (send hand-tools :number (1+ (send hand-tools :number)))
  (send hand-tools :tool-list (append (send hand-tools :tool-list) name))
  )

(message "")

(message "
        Make SHOP-TOOLS proto with a different :ISNEW method; no new instance
variables. The rest is inherited from TOOLS 
(defproto shop-tools '() '(number tool-list) tools)

(defmeth shop-tools :number (&optional new)
  (when new (setf (slot-value 'number) new))
  (slot-value 'number))
  
(send shop-tools :number 0)
(defmeth shop-tools :tool-list (&optional new)
  (when new (setf (slot-value 'tool-list) new))
  (slot-value 'tool-list))
  
(send shop-tools :tool-list)
(defmeth shop-tools :isnew (pow mov op mat parts name)
  ;; (call-next-method)
  (setf (slot-value 'power) pow)
  (setf (slot-value 'moveable) mov)
  (setf (slot-value 'operations) op)
  (setf (slot-value 'material) mat)
  (setf (slot-value 'pieces) parts)
  (setf (slot-value 'location) 'home)
  (send shop-tools :number (1+ (send shop-tools :number)))
  (send shop-tools :tool-list (append (send shop-tools :tool-list) name)))
")
(defproto shop-tools '() '(number tool-list) tools)

(defmeth shop-tools :number (&optional new)
  (when new (setf (slot-value 'number) new))
  (slot-value 'number))
  
(send shop-tools :number 0)
(defmeth shop-tools :tool-list (&optional new)
  (when new (setf (slot-value 'tool-list) new))
  (slot-value 'tool-list))
  
(send shop-tools :tool-list '())
(defmeth shop-tools :isnew (pow mov op mat parts name)
  ;; (call-next-method)
  (setf (slot-value 'power) pow)
  (setf (slot-value 'moveable) mov)
  (setf (slot-value 'operations) op)
  (setf (slot-value 'material) mat)
  (setf (slot-value 'pieces) parts)
  (setf (slot-value 'location) 'home)
  (send shop-tools :number (1+ (send shop-tools :number)))
  (send shop-tools :tool-list (append (send shop-tools :tool-list) name)))

(message "")

(message "
        We create instances of the various tool classes:

(setq hand-drill (apply #'send hand-tools :new
					; make an instance - HAND-DRILL
			'(ac) 
			'(drill polish grind screw)
			'(wood metal plastic)
			'(drill drill-bits screw-bits buffer)
			'2.5
			'(hand-drill)
			nil))
(send tools :isnew hand-drill)

(setq hand-saw (send hand-tools :new
			'(elbow grease) 
			'(cut)
			'(wood metal plastic)
			'(handle blade)
			'1.2
			'(hand-saw)))
(send tools :isnew hand-saw)
")
(setq hand-drill (apply #'send hand-tools :new
					; make an instance - HAND-DRILL
			'(ac) 
			'(drill polish grind screw)
			'(wood metal plastic)
			'(drill drill-bits screw-bits buffer)
			'2.5
			'(hand-drill)
			nil)
      )
(send tools :isnew hand-drill)

(setq hand-saw (send hand-tools :new
			'(elbow grease) 
			'(cut)
			'(wood metal plastic)
			'(handle blade)
			'1.2
			'(hand-saw)
			)
      )
(send tools :isnew hand-saw)

(message "")

(message "
(setq table-saw (apply #'send shop-tools :new
					; make an instance - TABLE-SAW
		       '(ac)
		       'fixed
		       '(rip cross-cut)
		       '(wood plastic)
		       '(saw blades fence)
		       '(table-saw)
		       'nil))
(send tools :isnew table-saw)

(setq radial-arm (apply #'send shop-tools :new
					; make an instance = RADIAL-ARM
			'(ac)
			'can-roll
			'(rip cross-cut)
			'(wood plastic)
			'(saw blades dust-bag)
			'(radial-arm)
			nil))
(send tools :isnew radial-arm)
")
(setq table-saw (apply #'send shop-tools :new
					; make an instance - TABLE-SAW
		       '(ac)
		       'fixed
		       '(rip cross-cut)
		       '(wood plastic)
		       '(saw blades fence)
		       '(table-saw)
		       'nil))
(send tools :isnew table-saw)

(setq radial-arm (apply #'send shop-tools :new
					; make an instance = RADIAL-ARM
			'(ac)
			'can-roll
			'(rip cross-cut)
			'(wood plastic)
			'(saw blades dust-bag)
			'(radial-arm)
			nil))
(send tools :isnew radial-arm)


(message "")

(message "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; The following session shows how to use the tool definitions. 
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(send hand-drill :show) ;; show the current status of the hand-drill
(send hand-drill :borrow 'fred)
(send table-saw :return)
(send hand-drill :borrow 'joe)
(send hand-drill :return)
(send hand-drill :borrow 'joe)

       Fred will be able to borrow the HAND-DRILL.  When an attempt is made
to return the  TABLE-SAW,  it was  already  at home.  A second  attempt  to
borrow the HAND-DRILL  indicates that you'll need to try later, because it is
already lent out to Fred.  Lastly, the HAND-DRILL was returned successfully,
so Joe was finally able to borrow it.
")
(send hand-drill :show)
(send hand-drill :borrow 'fred)
(send table-saw :return)
(send hand-drill :borrow 'joe)
(send hand-drill :return)
(send hand-drill :borrow 'joe)

(message "")

(message "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; The following session shows the structure of the TOOLS object:
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(send tools :show)
")
(send tools :show)

(message "")

(message "
 The two TOOLS sub-classes HAND-TOOLS and SHOP-TOOLS structure looks like:

(send hand-tools :show)
(send shop-tools :show)
")
(send hand-tools :show)
(send shop-tools :show)

(message "")

(message "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Some other interesting options:
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(send hand-drill :own-slots)
(send hand-drill :precedence-list)
(send hand-drill :own-methods)
(send hand-drill :add-slot 'expensive 'yes)
(send hand-drill :own-slots)
")
(send hand-drill :show)
(send table-saw :show)
(send radial-arm :show)
(send hand-drill :own-slots)
(send hand-drill :precedence-list)
(send hand-drill :own-methods)
(send hand-drill :add-slot 'expensive 'yes)
(send hand-drill :own-slots)

(message "")

(message "
(defmeth hand-drill :expensive (&optional new)
  (when new (setf (slot-value 'expensive) new))
  (slot-value 'expensive))

(send shop-tools :add-slot 'expensive 'nil)

(defmeth shop-tools :expensive (&optional new)
  (when new (setf (slot-value 'expensive) new))
  (slot-value 'expensive))

(send shop-tools :own-slots)
(send table-saw :own-slots)
(send shop-tools :expensive 'yes)
(send table-saw :expensive)
")
(defmeth hand-drill :expensive (&optional new)
  (when new (setf (slot-value 'expensive) new))
  (slot-value 'expensive)
  )

(send shop-tools :add-slot 'expensive 'nil)

(defmeth shop-tools :expensive (&optional new)
  (when new (setf (slot-value 'expensive) new))
  (slot-value 'expensive)
  )

(send shop-tools :own-slots)
(send table-saw :own-slots)
(send shop-tools :expensive 'yes)
(send table-saw :expensive)

(message "")

(message "
(def whats (send tools :active-list))
whats
(dotimes (i (send tools :number)) (send (select whats i) :show))

(defmeth tools :slots ()
  (let (
	(slot-vals nil)
	)
    (dolist (i (send self :slot-names))
	     (setq slot-vals (adjoin (list (slot-value i)) slot-vals))
	     )
    slot-vals
    )
  )
")
(def whats (send tools :active-list))
whats
(dotimes (i (send tools :number)) (send (select whats i) :show))

(defmeth tools :slots ()
  (let (
	(slot-vals nil)
	)
    (dolist (i (send self :slot-names))
	     (setq slot-vals (adjoin (list (slot-value i)) slot-vals))
	     )
    slot-vals
    )
  )

(message "")

(message "
(show-slots hand-drill)

(def crum (send hand-drill :slots))
(def crumnames (reverse (send hand-drill :slot-names)))
(first (element crum (search (list 'pieces) crumnames)))

(send hand-drill :slot-value 'pieces)
")
(show-slots hand-drill)

(def crum (send hand-drill :slots))
(def crumnames (reverse (send hand-drill :slot-names)))
(first (element crum (search (list 'pieces) crumnames)))

(send hand-drill :slot-value 'pieces)

(message "
        Let me see if I can summarize:

1) You define the class using a defproto:
------------------------------------------------------------------------------
DEFPROTO                                                        [function-doc]
Syntax (defproto name &optional ivars cvars (parent *object*) doc)
Makes a new object prototype with instance variables IVARS, 'class'
variables CVARS and parents PARENT. PARENT can be a single object or
a list of objects. IVARS and CVARS must be lists.
------------------------------------------------------------------------------
There are slots for 

	instances (e.g. hand-tools)
	class variables (everyone inherits these)

2) you define sub-classes similarly, etc.

3) everyone below inherits what's above.
")

(message "
        Above, in the method :slots, I define a way of actually accessing any
particular slot. It seems to me that we ought to avoid giving any particular
sub-object the same-named slot: I haven't been so careful above.

_____________________________________________________________________________

Some of the changes from the xlisp2.0 code:

1) We need to replace class by defproto, and change setq to defmeth.
2) remove :answer
3) Methods, as defined by Tim, contain the extra ().
")
