(message "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             Plotting functions: plot
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        Plotting is fairly easy: there are a variety of options (like using the
\"title\" command to give your plot a meaningful name, or the \"domain\" or
\"range\" commands to set the window if you want to). You can see your options
by typing (help new-plot). (\"help\" is the usual way of getting help on a
command, such as new-plot.) But plot does a pretty good job with little
information.

        (func f(x) x^3-2*x+3) ; f(x)=x^3-2*x+3
        (new-plot) ; initializes the plotting environment.
        (title \"Plot of f\")
        (plot f) ; plot f on default domain (-1,1)
")
        (func f(x) x^3-2*x+3)
        (new-plot)
        (title "Plot of f")
        (plot f)

(message "")
(message "
        If you're only interested in plotting f from -3 to 2, then just say so!

        (plot f -3 2) ; plot f from -3 to 2.
")
        (plot f -3 2) ; plot f from -3 to 2.

(message "")

(plot f -3 2) ; plot f from -3 to 2.
(send my-plot :location 300 200)
(message "
        You can play with the plot: click the mouse button inside the plot
window: if you do this quickly, it should zoom out. If you draw a box with the
mouse you'll zoom in on the area within the box (hold down the mouse button at
one point, and pull the mouse to another point: you're making two corners of a
box!).

        Type \"h\" (without the quotes) while the plot is \"active\" and you'll
see some additional options that are available with a single key-stroke. For
example, to remove the plot, type \"q\". This is much simpler than clicking on
the little box in the upper-left corner, n'est-ce pas?
")
(message "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             Plotting several functions
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        You may want to plot several functions of your own choosing together:

        (title \"Multiple plots\")
        (func g(x) x^2)
        (func h(x) sin(x))
        (func ff(x) - exp(x))
        (plot g h ff -3 2)
")
        (title "Multiple plots")
        (func g(x) x ^ 2)
        (func h(x) sin x)
        (func ff(x) - exp(x))
        (plot g h ff -3 2)

(message "")
(message "
        It is also possible to plot things other than functions with plot. Plot
was designed to do something useful for any reasonable thing passed: here are
some more examples:

 (title \"standard functions, like sine\")
 (plot sin)

 (title \"Multivariate Functions\")
 (func f(x y) (cos x + sin y))
 (plot f)

 (title \"matrix plots\")
 (plot (mat (4 4) (rand.u 16)))

 (title \"plot vectors of real or complex components\")
 (lines off)
 (def sample-data (rand.n 20)) ; make random sample data:
 (plot sample-data)
 (def sample-data (fft sample-data)) ; look at it's fft (complex vector info)
 (plot sample-data)
")
 (title "standard functions, like sine")
 (plot sin)

 (title "Multivariate Functions")
 (func f(x y) (cos x + sin y))
 (plot f)

 (title "matrix plots")
 (plot (mat (4 4) (rand.u 16)))

 (title "plot vectors of real or complex components")
 (lines off)
 (def sample-data (rand.n 20))
 (plot sample-data)
 (def sample-data (fft sample-data))
 (plot sample-data)

(message "That's all for plot.lsp")
