(message "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             Roots
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        You may need to find roots, of f or its derivatives: allow Newton!
Note the use of the domain function: it sets the domain for all future
plots, so don't forget! Let's find a zero of our cubic function f:

        (func f(x) x^3-2*x+3)
        (domain -3 2)        
        (title \"Newton's Method\")
        (newtons-method-demo f -1)
;; Newton can sometimes have a hard time! This is a funny show: try starting at
;; 0, rather than -1, sometime:
;;        (newton f 0)
")
        (func f(x) x^3-2*x+3)
        (domain -3 2)        
        (title "Newton's Method")
        (newtons-method-demo f -1)

(message "")
(message "
        If you don't want the graphical output, just the value, use the
function roots: type instead

        (roots f 1)

Or maybe you want the roots of the derivative function:

        (diff f x fp) ; define the derivative function fp.
        (roots fp (-2 2)) ; note: we're giving two different initial guesses
")
        (roots f 1)
        (diff f x fp)
        (roots fp (-2 2))

(message "")
(message "
        Or try muller's method (which requires 3 initial guesses):

        (muller f -2 0 2 :num-roots 3)

Muller's method will find complex roots, as well as real ones (Newton can NEVER
find a complex root). Since f is a cubic, it has three roots, which muller will
discover. (Note the use of the :num-roots option - this is a typical way of
telling xlisp-stat that you want to give an additional piece of information.)

        Newton found the real one above, but here are all three of them:
")
        (muller f -2 0 2 :num-roots 3)
(message "")

(message "
	If you have a multivariate function and want to find the roots, you
might try \"steep\" (i.e., steepest descent): here is the help command for it:

	?steep
")
?steep
(message "")

(message "
Or you could use a numerical procedure, such as newtonmax(which may take
awhile! Be patient...):

(func h(a b c d) (a-161)^2+(b-944)^2+(c-204)^2+(d-987)^2)
(newtonmax
 #'(lambda(x) (- (square (apply #'h x))))
 '(100 100 100 100)
 :count-limit 500
 :verbose nil
 )
;; true solution: '(161 944 204 987)
")
(func h(a b c d) (a-161)^2+(b-944)^2+(c-204)^2+(d-987)^2)
(newtonmax
 #'(lambda(x) (- (square (apply #'h x))))
 '(100 100 100 100)
 :count-limit 500
 :verbose nil
 )
(format t "True solution: (161 944 204 987)")
(message "That's all folks!")
