;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; tutorial Doc:    Creation Date:2/29/96        Author: Andy Long
;;
;;
(defun tutorial()
  "
;; Description/Arguments:

       (tutorial)
  
     This gives a pretty good idea of the capabilities of xlisp-stat, as I've
implemented it. You won't find everything here, but you'll probably find more
than you want.

;; Notes: This just loads in the file student.lsp, in the tutorials section.

;; See Also: student.lsp
"
  (read-line)
  (load (strcat *andy-default-path* "tutorial" separator "student.lsp")
	:print t))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                tutorials
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun tutorials()
  "
;; Description/Arguments:

       (tutorials)
  
   Offers a choice of tutorials to help the novice get started.

;; See Also: tutorial
"
  (let* (
	 (RXridge (send menu-proto :new "Tutorials"))
	 (dir (strcat *andy-default-path* "tutorial" separator))
	       
	 (intro (send menu-item-proto :new "Introduction"
		      :action
		      (lambda() (load (strcat dir "student.lsp") :print t))))
	 
	 (func (send menu-item-proto :new "Defining functions"
		     :action
		     (lambda() (load (strcat dir "func.lsp") :print t))))
	 
	 (diff (send menu-item-proto :new "Differentiation"
		     :action
		     (lambda() (load (strcat dir "diff.lsp") :print t))))
	 
	 (calc (send menu-item-proto :new "Calc - a calculator"
		     :action
		     (lambda() (load (strcat dir "calc.lsp") :print t))))
	 
	 (plot (send menu-item-proto :new "Plots"
		     :action
		     (lambda() (load (strcat dir "plot.lsp") :print t))))
	 
	 (2-d-func (send menu-item-proto :new "2-d functions"
			 :action
			 (lambda() (load (strcat dir "2-d-func.lsp") :print t))))
	 
	 (data (send menu-item-proto :new "Data"
		     :action
		     (lambda() (load (strcat dir "data.lsp") :print t))))
	 
	 (deriv (send menu-item-proto :new "Derivatives"
		      :action
		      (lambda() (load (strcat dir "deriv.lsp") :print t))))
	 
	 (tangent (send menu-item-proto :new "Tangents"
			:action
			(lambda() (load (strcat dir "tangent.lsp") :print t))))
	 
	 (taylor (send menu-item-proto :new "Taylor series"
		       :action
		       (lambda() (load (strcat dir "taylor.lsp") :print t))))
	 
	 (fourier (send menu-item-proto :new "Fourier Analysis"
			:action
			(lambda() (load (strcat dir "fourier.lsp") :print t))))
	 
	 (parametr (send menu-item-proto :new "Parametric functions"
			 :action
			 (lambda() (load (strcat dir "parametr.lsp") :print t))))
	 
	 (animate (send menu-item-proto :new "Animation"
			:action
			(lambda() (load (strcat dir "animate.lsp") :print t))))
	 
	 (multplot (send menu-item-proto :new "Multiple plots"
			 :action
			 (lambda() (load (strcat dir "multplot.lsp") :print t))))
	 
	 (roots (send menu-item-proto :new "Roots"
		      :action
		      (lambda() (load (strcat dir "roots.lsp") :print t))))
	 
	 (integrat (send menu-item-proto :new "Integration"
			 :action
			 (lambda() (load (strcat dir "integrat.lsp") :print t))))
	 
	 (complex (send menu-item-proto :new "Complex Features"
			:action
			(lambda() (load (strcat dir "complex.lsp") :print t))))
	 
	 (diffeq (send menu-item-proto :new "Differential equations"
		       :action
		       (lambda() (load (strcat dir "diffeq.lsp") :print t))))
	 
	 (eigen (send menu-item-proto :new "Eigenvalues and eigenvectors"
		       :action
		       (lambda() (load (strcat dir "eigen.lsp") :print t))))
	 
	 (linalg (send menu-item-proto :new "Linear algebra"
		       :action
		       (lambda() (load (strcat dir "linalg.lsp") :print t))))
	 
	 (pca-svd (send menu-item-proto :new "PCA versus SVD"
			:action
			(lambda() (load (strcat dir "pca-svd.lsp") :print t))))
	 
	 (glim (send menu-item-proto :new "Generalized Linear Modelling "
		     :action
		     (lambda() (load (strcat dir "glim.lsp") :print t))))
	 
	 (oop (send menu-item-proto :new "Object-Oriented Programming "
		     :action
		     (lambda() (load (strcat dir "oop.lsp") :print t))))
	 
	 (moreinfo (send menu-item-proto :new "More info"
			 :action
			 (lambda() (load (strcat dir "moreinfo.lsp") :print t))))
         (dash0     (send dash-item-proto :new))
         (killmenu  (send menu-item-proto :new "Remove this menu"
                          :action #'(lambda () 
				      (send RXridge :remove)
				      ))
		    )
	 )
    (send RXridge :append-items
	  intro
	  func
	  diff
	  calc
	  plot
	  2-d-func
	  data
	  deriv
	  tangent
	  taylor
	  fourier
	  parametr
	  animate
	  multplot
	  roots
	  integrat
	  complex
	  diffeq
	  eigen
	  linalg
	  pca-svd
	  glim
	  oop
	  moreinfo
	  dash0
	  killmenu)
    (send RXridge :install)
    RXridge
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                all-tutorials
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun all-tutorials (&optional start-over)
  "
;; Description/Arguments:

       (all-tutorials &optional resume)
  
   Runs through all (low-level) tutorials to help the novice get started.

;; See Also: tutorial
 (all-tutorials) ;; will pick up where you last got kicked out!
 (all-tutorials t) ;; will start fresh
"
  (let* (
	 (programs '(
		     "intro"
		     "func"
		     "diff"
		     "calc"
		     "plot"
		     "2-d-func"
		     "data"
		     "deriv"
		     "tangent"
		     "taylor"
		     "fourier"
		     "parametr"
		     "animate"
		     "multplot"
		     "roots"
		     "integrat"
		     "complex"
		     "diffeq"
		     "eigen"
		     "linalg"
		     "pca-svd"
		     "glim"
		     "oop"
		     #+macintosh	    "mac-help"
		     "moreinfo"
		     ))
	 (programs
	  (if (and (not start-over) (boundp '*current-tutorial*))
	      (select programs
		      (iseq
		       (search *current-tutorial* programs :test #'equalp)
		       (1- (length programs))
		       )
		      )
	    programs
	    )
	  )
	 )
    (dotimes
     (i (length programs))
     (let ((choice (select programs i)))
       (setq *current-tutorial* (list choice))
       (load (strcat *andy-default-path* "tutorial" separator choice ".lsp")
	     :print t)
       )
     )
    )
  )
